<?php
require_once("../../prepend.inc");

/**
 * Edit a Field Table.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Admin
 * @author Phil Calder
 */

global $SERVICES;

$ua =& $SERVICES->getService('Authentication');
$ua->adminRestrict();

$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');

$SERVICES->getService('Language');
$SERVICES->language->getLanguage('Admin');

$SERVICES->getService('Config');

$output =& $SERVICES->getService('Output');

function getFormFields($item) {
	global $L;
	//field definitions
	$fields =  array(
		array('label'=>'', 'field_key'=>'field_id', 'type'=>'hidden'), 
		array('label'=>'', 'field_key'=>'field_table', 'type'=>'hidden'), 
		array('label'=>'', 'field_key'=>'field_key', 'type'=>'hidden'), 
		array('label'=>$L['adminCustomFieldsConfirmDeleteCustomFieldKey'], 'field_key'=>'display_key', 'type'=>'free', 'text'=>$item['field_key']), 
		array('label'=>$L['adminCustomFieldsConfirmDeleteCustomField'], 'field_key'=>'confirm', 'type'=>'text')
	);
	
	foreach ($fields as $k => $v) {
		$fields[$k]['value'] = $item[$v['field_key']];	
	}
	
	return $fields;
}

function doSubmit() {
	global $SERVICES, $PHP_SELF, $L, $output, $add;
	$item = $_POST['item'];
	
	if ($_POST['submit']) {
		//server-side data validation goes here
		$fields = getFormFields($item);	//get field definitions
		$output->validateFields($item, $fields, $error_msg); //validate $item based on the definitions in $fields; $item values will get updated
		
		// custom validation - make sure the input matches the field_key the user wants to delete
		if ($item['field_key'] && ($item['field_key'] != $item['confirm'])) {
			if (!is_array($error_msg)) {
				$error_msg = array();	
			}
			$error_msg['confirm'] = $L['adminCustomFieldsConfirmDeleteTextDoesNotMatch'] . ' (' .$item['field_key'] . ')';
		} // custom validation
		
		if (is_array($error_msg)) {		//if server-side validation errors, return to form
			main( $item['field_id'], $item, $error_msg);
			exit();
		}
		
		if ($add->removeItem($item)) {
			doRedirect($item['field_table']);
			exit();
		} else {
			$content = $output->getBeginTable();
			$content .= $output->getTextRow($L['adminCustomFieldsErrorCannotDeleteCustomField']);
			$content .= $output->getTextRow('<a href="custom_fields_table_edit.php?table=' . $item['field_table'] . '">' . $L['Back'] . '</a>');
			$content = $output->getEndTable();
			quickPage($content);
			exit();
		}
	}
} //doSubmit

function doRedirect($table) {
	global $SERVICES;
	$SERVICES->getService('Config');
	$url = $SERVICES->config->lookup('modules.Admin.module_edit.table_edit.link') . '?table=' . $table;
	
	header('Location: ' . $url);
	exit();
}

function main($id = '', $item = '', $error_msg = '') {
	global $PHP_SELF, $SERVICES, $output, $L,$table, $edit, $delete, $type, $add, $action, $field_id;
	
	if (!$item) {  //item could be passed back from server-side data validation failure
		if (empty($action)) {
			$action = $_GET['action'];
		}
		
		if (empty($field_id)) {
			$action = $_GET['field_id'];
		}
		
		if ( (empty($action) || $action != 'delete') || empty($field_id)) {
			$content = $output->getTextRow($output->getLinkHref(array('href'=>$SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.link'), 'text'=>$SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.label'))));
			header('Location: admin_module_index.php');
			exit();
		}
		
		if ($action == 'delete') {
			//view or edit existing item
			$item = $add->getField($field_id);
		} else { 	//else create **** new item ****
			$item = array('field_table' => $table);
		}
	}
	
	$content = '';
	$content .= $output->getBeginTable();
		
	$title_menu = array(
		$output->getLinkHref(
			array(
				'href' => $SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.link'), 
				'text' => $SERVICES->config->lookup('modules.Admin.modulelink.module_editor_index.label')
			)
		), 
		$output->getLinkHref(
			array(
				'href' => $SERVICES->config->lookup('modules.Admin.module_edit.table_select.link') . $output->updateQueryString(array('delete'=>'')), 
				'text' => $SERVICES->config->lookup('modules.Admin.module_edit.table_select.label')
			)
		), 
		$output->getLinkHref(
			array(
				'href' => $SERVICES->config->lookup('modules.Admin.module_edit.table_edit.link') . $output->updateQueryString(array('delete'=>'')), 
				'text' => $SERVICES->config->lookup('modules.Admin.module_edit.table_edit.label')
			)
		)
	);
	
	// $content .= $output->getTitleRow($L['adminCustomFieldsFieldDeleteLabel'], $title_menu);
	
	$content .= $output->getTextRow($L['adminCustomFieldsDescriptionConfirmDeleteCustomField']);
	$content .= $output->getFieldSpacerRow();
	
	//nested data table ------------------
	$content .= $output->getBeginNestedEditTable();
	//form
	$content .= $output->getBeginForm($PHP_SELF);  
	
	foreach (getFormFields($item) as $k => $v) {
		$content .= $output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
	}
	
	$content .= $output->getFieldSpacerRow();
	$content .= $output->getSubmitRow();
	$content .= $output->getEndForm();
	//end form
	$content .= $output->getEndNestedTable();
	//end nested data table ------------------
	$content .= $output->getEndTable();
	//
	
	$title_text = $L['adminCustomFieldsFieldDeleteLabel'];
	$padding = true;
	
	$window_layout = $output->getBeginTable('mainTbl', true);
	$window_layout .= $output->getTitleRow($title_text);
	$window_layout .= $output->getMenuRow($title_menu);
	$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
	$window_layout .= $content;
	$window_layout .= '</td>' . $output->getEndRow(true);
	$window_layout .= $output->getEndTable(true);
	
	$output->addToBody($window_layout);
	$output->display();
	exit();
	
} // main

if (isset($_POST['submit']) || isset($_GET['delete'])) {
	doSubmit();
}

main();
?>