<?php
require_once("../../prepend.inc");
/**
 * Build a new module. This form is for creating standard fields in a new module table..
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Admin
 * @author Phil Calder
 */
 
/**
 * For a given table, allows the user to select 'standard' system fields to include, 
 * and specify their own 'standard' fields.
 * These fields are added to the given table and Data Definitions entered for these fields
 */

$ua =& $SERVICES->getService('Authentication');
$ua->adminRestrict();
 
$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');

$output =& $SERVICES->getService('Output');

$SERVICES->getService('Modules');
$mod =& $SERVICES->modules->getModule('Admin');

$default_table = ( ($default_table = $mod->getTemplateModuleName() . '__item') ? $default_table : 'TemplateNewModule__item');

$compulsary_fields = array( // Phil 20060825
	'id',
	'created', 
	'creator_id', 
	'last_update',
	'active'
);

function getFormFields($item) {
	global $add, $default_table, $table, $compulsary_fields;
	
	$default_fields = $add->getFields($default_table);
	$standard_system_fields = array();
	
	foreach ($default_fields as $k => $v) {
		//if ($v['field_key'] != 'id') { // Phil 20060825 removed if // prevous:ignore the 'id' field (it is created by default)
			// $standard_system_fields[$v['field_key']] = $v['field_key'];
			// $standard_system_fields[$k] = $v['field_key'];
			// $standard_system_fields[$k] = $k;
		//}
		if (!in_array($v['field_key'], $compulsary_fields)) {
			$standard_system_fields[$k] = $v['field_key'];
		}
	}
	// print '<p>---</p>'; print_r($default_fields);
	// print '<p>---</p>'; print_r($standard_system_fields);
	
	$field_type_dropdown_values = $add->getFieldTypeDropdownValues();
	$default_fields = $add->getFields($default_table);
	
	$fields = array(
		array(
			'label'=>'',
			'type'=>'hidden',
			'field_key'=>'field_table', 
			'value' => $table
		)
	);	
	
	$fields[] = array(
		'label'=>'Common Fields',
		'type'=>'multi_select',
		'field_key'=>'_system_standard_fields', 
		'dropdown_values'=>$standard_system_fields,
		// 'value' => implode(',', array_keys($standard_system_fields))
		// 'value' => $standard_system_fields
		'value' => array_keys($standard_system_fields)
	
	);
	
	$last_key = '1';
	
	for($k = $last_key; $k < ($last_key+10); $k++) {
		$fields[] = array(
			'label'      =>  'Field Name',
			'type'       =>  'text',
			 'field_key'  =>  $k . '][field_key', 
			'value'      =>  '',
			'notnull'    => 0, 
			'explanation' => '&nbsp;'
		);
		$fields[] = array(
			'label'      =>  'Field Type',
			'type'       =>  'dropdown',
			'field_key'  =>  $k . '][type', 
			'dropdown_values' => $field_type_dropdown_values
			// 'notnull'    => 1
		);
		$fields[] = array(
			'label'      =>  'Include',
			'type'       =>  'checkbox',
			'field_key'  =>  $k . '][checked',
			'value'      =>  1// ,
			// 'notnull'    => 1
		);
	}
	
	return $fields;
}

function doSubmit() {
	global $add, $item, $default_table, $compulsary_fields;
	// print '<p>doSubmit</p>'; 
	// print_r($item); print '<hr>';
	
	$table = $item['field_table'];
	bcrm_unset($item, 'field_table');
	
	$_system_standard_fields = $item['_system_standard_fields'];
	
	
	foreach ($compulsary_fields as $k => $v) {
		if (!in_array($v, $_system_standard_fields)) {
			$_system_standard_fields[] = $v;
		}
	}
	// print '<p>_system_standard_fields:</p>'; print_r($_system_standard_fields); print '<hr>';
	
	$default_fields = $add->getFields($default_table);
	// print '<p>(default_table: ' . $default_table . ') default_fields:</p>'; print_r($default_fields); print '<hr>';
		
	if (sizeof($item)) {
		// Create item table
		// print '<p>Create blank table: ' . $table . '</p>';
		$add->createStandardTable($table);
		$add->createSeenTable($table);
	}
	
	// print '<div>_system_standard_fields:</div>'; print_r($_system_standard_fields); print '<hr>';
	
	// ===== Standard 'system' Fields =====
	if (is_array($_system_standard_fields)) {
		foreach($_system_standard_fields as $k => $v) {
			// $default_fields[$k]['field_table'] = $table;
			// $default_fields[$k]['field_is_custom'] = 0; // standard field
			// bcrm_unset($default_fields, $k, 'field_id');
			$default_fields[$v]['field_table'] = $table;
			$default_fields[$v]['field_is_custom'] = 0; // standard field
			bcrm_unset($default_fields, $v, 'field_id');
			
			// Add data definition field
			// print '<p>Add _system_standard_fields data definition field, [k=' . $k . ', v=' . $v .']: ';
			// print_r($default_fields[$k]); 
			// print_r($default_fields[$v]); 
			// print '</p>';
			
			// Add field to item table
			// print '<p>Add _system_standard_fields field to item table: ' . $table . '</p>';
			// 
			// print '<hr></p>';
			
			// Phil 20060825 created field stored as an int but treated as a date_time file
			// This flag tells the DataDefinition Service to store the field as an INT.
			if ($v == 'created') {
				$default_fields[$v]['_special_date_time'] = 1;
			}
			
			$retval = $add->setItem($default_fields[$v]);
			
			// print '<div>retval: ' . $retval . '</div>';
			// 
			// print '<hr>';
		}
	}
	
	// ===== User create standard fields =====
	foreach($item as $k => $v) {
		
		if (
		isset($v['checked']) && $v['checked'] 
		&& isset($v['field_key']) && $v['field_key'] 
		&& isset($v['type']) && $v['type']
		) {
			
			bcrm_unset($v, 'checked');
			$v['field_table'] = $table;
			$v['field_is_custom'] = 0; // standard field
			
			// format the name (remove spaces etc)
			$v['field_key'] = strtolower($v['field_key']);
			$v['field_key'] = str_replace(' ', '_', $v['field_key']);
			
			// Add data definition field
			// print '<p>Add data definition field: ';
			// print_r($v); 
			// print '</p>';
			
			// Add field to item table
			// print '<p>Add field to item table: ' . $table . '</p>';
			
			// print '<hr></p>';
			
			$retval = $add->setItem($v);
			
			// print '<div>retval: ' . $retval . '</div>';
			// print '<hr>';
		}
	}
	
	// ===== Data Options =====
	$default_data_options = $add->getOptions($default_table);
	
	foreach ($default_data_options as $k => $v) {
		$item = array();
		$item['field_table'] = $table; // eg 'ModuleName__item'
		$item['type']        = $k; // eg 'edit'
		$item['method']      = $v['method'];
		$item['values']      = '' . (is_array($v['values']) ? implode(',', $v['values']) : '');
		$item['exclude_hidden'] = ($v['options']['exclude_hidden'] ? 1 : 0);
		$item['exclude_calculated'] = ($v['options']['exclude_calculated'] ? 1 : 0);
		$item['dropdown_values'] = $v['options']['dropdown_values'];
		// $item['search_title']   = $v['options']['search_title']; // Deprecated
		// $item['event_date']     = $v['options']['event_date'];
		
		$retval = $add->insertDataOption($item);
	}
	
	global $SERVICES;
	$SERVICES->getService('Config');
	$url = $SERVICES->config->lookup('modules.Admin.module_edit.table_edit.link');
	
	header('Location: ' . $url . '?table=' . $table);
	exit();
	
}

function main() {
	global $add, $output, $table, $L, $submit;
	
	if (isset($submit) && $submit) {
		doSubmit();
	}
	
	if (empty($table)) {
		quickPage($L['adminModuleTableBuilderNoTableError']);		
		exit();
	}
	
	// print '<p>table: ' . $table . '</p>'; 
	// print_r($default_fields); 
	// print '<hr>';
	// print_r($fields);
	
	$content = '';
	$content .= $output->getBeginTable('viewTbl');
	// $content .= $output->getTitleRow($L['adminModuleTableBuilderStandardFieldSelectLabel']);
	$content .= $output->getTextRow($L['adminModuleTableBuilderStandardFieldSelectDescription']);
	$content .= $output->getFieldSpacerRow();
	
	//nested data table ------------------
	$content .= $output->getBeginNestedEditTable();
	
	//form
	$content .= $output->getBeginForm($PHP_SELF);  
	//or, to exclude javascript input validation for your form, call it like this:   $content .= $output->getBeginForm($PHP_SELF, 0);
	
	foreach (getFormFields($item) as $k => $v) {
		//$v['value'] = $item[$v['field_key']];
		$content .= $output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
	}
	
	$content .= $output->getFieldSpacerRow();
	$content .= $output->getSubmitRow();
	
	$content .= $output->getEndForm();
	//end form

	$content .= $output->getEndNestedTable();
	//end nested data table ------------------
	
	$content .= $output->getEndTable();
	//
	
	$title_text = $L['adminModuleTableBuilderStandardFieldSelectLabel'];
	$padding = true;
	
	$window_layout = $output->getBeginTable('mainTbl', true);
	$window_layout .= $output->getTitleRow($title_text);
	// No menu // $window_layout .= $output->getMenuRow($title_menu);
	$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
	$window_layout .= $content;
	$window_layout .= '</td>' . $output->getEndRow(true);
	$window_layout .= $output->getEndTable(true);
	
	$output->addToBody($window_layout);
	$output->display();
	exit();
}

main();
?>