<?php
/**
 * Admin module class.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Admin
 * @author Phil Calder, Nathan Humphries
 */

/**
 * Admin module class.
 *
 * @package Admin
 * @author Phil Calder, Nathan Humphries
 */
class Admin {
	
	var $template_module_name = '';
	var $prefixed_template_module_name = '';
	
	function Admin() {
		global $SERVICES;
		$this->error = array();
		$this->db =& $SERVICES->getService('Database');
	}
	
	/**
	 * Returns the name of the template module.
	 * This is the module that will be copied when a new module is created.
	 * @return string $module_name The name of the template module
	 */
	function setTemplateModuleName($template_module_name = '') {
		if (!strlen($this->template_module_name) && !strlen($template_module_name)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			$template_module_name = $SERVICES->config->lookup('modules.Admin.module_edit.tempate_module_name');
		}
		
		if (empty($template_module_name)) {
			exit('No module name');
		}
		
		$this->template_module_name = $template_module_name;
		$this->prefixed_template_module_name = $this->template_module_name . '__';
	}
	
	/**
	 * Returns the name of the template module.
	 * This is the module that will be copoied when a new module is created.
	 * @return string $module_name The name of the template module
	 */
	function getTemplateModuleName() {
		if (!isset($this->template_module_name) || !strlen($this->template_module_name)) {
			$this->setTemplateModuleName(); // defaults to 'TemplateNewModule' (in Admin module config)
		}
		// print '<div>Admin::getTemplateModuleName ' . $this->template_module_name . '</div>';
		return $this->template_module_name;
	}
	
	/**
	 * Written for deprecated Admin index page
	 *
	 * @return array of links the current user can see on the admin page, 
	 * key is a module name, values are arrays of links.
	 */
	function getLinksList() {
		global $SERVICES, $USER, $errorlog;
		$SERVICES->getService('Config');
		$list = $SERVICES->config->getModuleConfig('Admin'); // Added 20050317 ('Admin');
		// print_r($list); print '<hr>';
		$list = $list['modulelink'];
		// $list is an array of module links
		// a module link is an array with a "priority" and an "href"
		$ua =& $SERVICES->getService('Authentication');
		// $modules = $ua->getAccessLevelPrivileges($USER['access_level']);
		$modules = $ua->getPrivileges();
		// Set defaul module ordering here
		$answer = array(
			'AdminRoot' => '',
			'UserAdmin' => '',
			'Admin' => '',
			'Front' => ''
		);
		// CODE FOR SORTING ACCORDING TO PRIORITY:
		if (is_array($list)) {
			foreach ($list as $k => $v) {
				if (is_array($v)) {
					// print_r($v); print '<hr>';
					$module = $v['module'];
					$index = 100 + intVal($v['priority']);
					$index .= $k;  // so that comparison is lexigraphical & indexes are unique
					if (
					!$module 
					|| (isset($modules[$module]) 
					&& ($modules[ $module ] 
					& $ua->getAdminLevel()))
					) {
						// user has access to the module, so return the link
						$answer[$module][ $index ] = $v; // Phil 20040924 - now sorted first by module
					}
				} else {
					$SERVICES->getservice('Log');
					$SERVICES->log->logError(
						'error occured while loading Admin links, module link does not have "href" and "priority" attributes.',
						'Admin'
					);
				}
			}
		}
		foreach ($answer as $k => $module) {
			if (is_array($answer[$k])) {
				ksort($answer[$k]);	
			}
		}
		// ksort($answer); // Dont sort these here - see the declaration for answer above
		return $answer;
	}
	
	// --- Functions used to create a new module ---
	
	/**
	 * Removes invalid characters from proposed module name
	 */
	function processModuleName($module_name) {
		// $module_name = str_replace(' ', '', $module_name);
		$module_name = ereg_replace("[^a-zA-Z0-9_]", "", $module_name);
		// $module_name = strtolower($module_name); // Not necessary (ie can have MyModuleName as a name)
		$module_name = ucfirst($module_name);
		if (empty($module_name)) {
			$module_name = 'CustomModule';
		}
		return $module_name;
	}
	
	function runNewModuleChecks($item, $src_basedir, $dest_basedir) {
		$error_msg = array();
		
		//before anything else, do a few checks, we dont want to override any current modules.
		if ($module_id = $this->getSystemModuleId($item['module_name']) ){
			$error_msg[] = 'Module Found in Database - A new module cannot have the same name as an existing one.<br>';
		}
		
		if (!is_dir($src_basedir . '/' . $this->getTemplateModuleName())) {
			$error_msg[] = 'Module Directory for Template Not Present.<br>';
		}
		
		if (is_dir($dest_basedir . '/' . $item['module_name'])) {
			$error_msg[] = 'Module Directory Exists - A new module cannot have the same directory destination as an existing one.<br>';
		}
		
		if (sizeof($error_msg)) {
			return $error_msg;
		}
		
		return true;
	}
	
	function addModuleToDatabase($item, $admin_id = '') {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Attempting to add module: ' . $item['module_name']. '',
			'Admin'
		);
		
		$data['name'] = $item['module_name'];
		$data['is_custom'] = 1; //default
		if (!$item['is_custom'] && $SERVICES->config->lookup('site.development')) {
			$data['is_custom'] = 0; //Development servers can create standard modules
		}
		// print '<div>Admin::addModuleToDatabase</div>'; print_r($data); print '<hr>'; exit();
		
		//set and get system modules
		$this->setSystemModule($data);
		$module_id = $this->getSystemModuleId($item['module_name']);
		
		$data1['module_id'] = $module_id;
		$data1['group_id'] = $admin_id;
		$data1['priv'] = 7; //default full access 
		//print_r($data1);
		
		$this->setSystemModuleGroupPermissions($data1, $admin_id);
	}
	
	/**
	 * @author Nathan Humphries
	 */
	function createModuleFileStructure($item, $prefix = '') {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Attempting to create file structure for module: ' . $item['module_name']. '',
			'Admin'
		);
		
		$SERVICES->getService('Config');
		//do the file creation and editing setup
		$src_dirname = $SERVICES->config->lookup('modules.basedir') . '/' . $this->getTemplateModuleName();
		
		// Standard module
		if (!($item['is_custom']==1) && $SERVICES->config->lookup('site.development')) { // Phil 20051031 check is_custom = 1
			$dest_dirname = $SERVICES->config->lookup('modules.basedir') . '/' . $item['module_name'];
		} else {
			$dest_dirname = $SERVICES->config->lookup('modules.custom_basedir') . '/' . $item['module_name'];
			// TODO - Make sure the custom basedir directory exists. If not, create it.
		}
		
		$src_table_prefix = ($prefix ? '' . $prefix : '' . $this->prefixed_template_module_name);
		$dest_table_prefix = $item['module_name'] . '__';
		
		//reset file info string
		$file_info = '';
		
		/////////////////////////////
		// SET src and dest things, grab the permissions from the src, and attempt to dupe it to the new structure
		/////////////////////////////
		
		$src_dirname_perms = substr(sprintf('%o', fileperms($src_dirname)), -4);
		//$src_dirname_perms = substr(sprintf('%o', fileperms($src_dirname)), -4);
		
		// Phil - I commented these out because they arent even used, 
		// and because posix_* functions are not available on Windows
		// $src_dirname_owner = fileowner ($src_dirname);
		// $src_dirname_owner = posix_getpwuid($src_dirname_perms);
		   
		// Phil - have added function exists here, at least until replaced with calls to Filesystem Service.
		$src_dirname_filegroup = '';       
		if (function_exists('posix_getgrgid')) {
			$src_dirname_filegroup = posix_getgrgid(filegroup ($src_dirname)); 
			$src_dirname_filegroup = $src_dirname_filegroup['name'];
		}
		
		//echo 'Permissions on src_dirname: ' . $src_dirname_perms;
		//echo '<br>';
		//echo 'Group on src_dirname: ' . $src_dirname_filegroup;
		//echo '<br>';
		
		//$str = '02' .  substr($src_dirname_perms,1);
		//echo 'Permissions on src_dirname: ' . $str; 
		//echo '<br>';
		
		// --------
		if (!is_dir(dirname($dest_dirname))) { // Phil 20050427
			if (!@mkdir(dirname($dest_dirname),02777) ) {
				$file_info = '
				<br>
				Failed to make directory.
				<br>
				Aborting file creation.
				<br>
				[' . dirname($dest_dirname) . ']
				<br>
				';
				return $file_info;
				
			} else {
				if (function_exists('chgrp')) { // Phil - added function check
				@chgrp($dest_dirname, $src_dirname_filegroup);
			}
			}
		}
		// --------
		
		//make new dirname
		if (!@mkdir($dest_dirname,02777) ) {
			
			$file_info .= '
			<br>
			<br>
			Failed to make directory.
			<br>
			Aborting file creation.
			<br>
			[' . $dest_dirname . ']
			<br>
			';
			return $file_info;
		
		} else {
			
			//these two lines are expierimental
			//substr strips off the 0
			//chmod($dest_dirname,$str); 
			if (function_exists('chgrp')) { // Phil - added function check
				@chgrp($dest_dirname, $src_dirname_filegroup);
			}
			//chown($dest_dirname, $src_dirname_owner);
			
		}
		
		//echo 'Permissions on src_dirname: ' . $dest_dirname;
		//echo '<br>';
		//echo 'Group on src_dirname: ' . $src_dirname_filegroup;
		//echo '<br>';
		
		///create file structure
		$info = $this->createFileStructure($item['module_name'], $src_dirname, $dest_dirname);
		
		$file_info .= $info[1];
		
		$file_info .= '<br><br>Copied and updated <b>' . $info[0] . '</b> files';
		///
		// 	
		return $file_info;
	}
	
	
	function copyDefaultTables($src_table_prefix, $dest_table_prefix) {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Copying default tables for: ' . $dest_table_prefix. '',
			'Admin'
		);
		//create database structure (Defaults to copying all 'TemplateNewModule__*' tables to 'ModuleName__*')
		
		$template_tables = $this->getDbTableNames($src_table_prefix);
		
		$created_tables = '';
		$not_created_tables = '';
		
		foreach ($template_tables as $k => $v) {
			
			$table_sql = $this->db->getShowCreateTable($v);
			//echo '<br>1sql: ' . $table_sql . '<br>';
			$create_table_sql = str_replace($src_table_prefix, $dest_table_prefix, $table_sql) . ';';
			//echo '<br>2sql: <br>' . $create_table_sql;
			$result = $this->db->sqlQuery($create_table_sql);
			
			if ($this->db->hasError()) {
				$this->db->printError();
			}
			
			if ($result) {
				$created_tables .= '' . str_replace($src_table_prefix, $dest_table_prefix, $v) . '<br>';
				
				$this->copyDefaultFields($v, str_replace($src_table_prefix, $dest_table_prefix, $v));
				$this->copyDefaultDataOptions($v, str_replace($src_table_prefix, $dest_table_prefix, $v));
				
			} else {
				$not_created_tables .= '' . str_replace($src_table_prefix, $dest_table_prefix, $v) . '<br>';
			}
			
		}
		
		$table_info = 'Copied Tables OK: <br>New Tables made: <br>' . $created_tables . '<br><br>';
		
		if (strlen($not_created_tables)) {
			
			$table_info .= '<br>Problem copying the following tables: <br>' . $not_created_tables . '<br><br>';
		
		}
		
		return $table_info;
		
	}
	
	function copyDefaultFields($src_table, $dest_table) {
		global $SERVICES;
		$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');
		
		$default_fields = $add->getFields($src_table);
		
		// print '<p>default_fields for ' . $src_table . ':</p>'; print_r($default_fields); print '<hr>';
		
		// =====
		foreach($default_fields as $k => $v) {
			
			$default_fields[$k]['field_table'] = $dest_table;
			$default_fields[$k]['field_is_custom'] = 0; // standard field
			bcrm_unset($default_fields, $k, 'field_id');
			
			$retval = $add->setItem($default_fields[$k]);
			
		}		
	}
	
	function copyDefaultDataOptions($src_table, $dest_table) {
		//print '<p>src: ' . $src_table . ', dest: ' . $dest_table . '</p>';
		// ===== Data Options =====
		global $SERVICES;
		$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');
		$default_data_options = $add->getOptions($src_table);
		
		if (is_array($default_data_options)) {
			
			foreach ($default_data_options as $k => $v) {
				
				$item = array();
				$item['field_table'] = $dest_table; // eg 'ModuleName__item'
				$item['type']        = $k; // eg 'edit'
				$item['method']      = $v['method'];
				$item['values']      = '' . (is_array($v['values']) ? implode(',', $v['values']) : '');
				$item['exclude_hidden'] = ($v['options']['exclude_hidden'] ? 1 : 0);
				$item['exclude_calculated'] = ($v['options']['exclude_calculated'] ? 1 : 0);
				$item['dropdown_values'] = $v['options']['dropdown_values'];
				// $item['search_title']   = $v['options']['search_title']; // Deprecated
				// $item['event_date']     = $v['options']['event_date'];
				
				$retval = $add->insertDataOption($item);
				
			}		
		}
	}
	
	function buildModuleTables($item) {
		global $L;
		$table = $item['module_name'] . '__item';
		return '<a href="../Admin/admin_module_table_builder.php?table=' . $table . '">' . $L['adminModuleBuildTablesDynamically'] . '</a>';
	}
	
	/**
	 * this function gets the admin user group id
	 *
	 * Used by the new module creation script.
	 */
	function getAdminGroupId() {
		global $SERVICES;
		$groups =& $SERVICES->getService('Groups');
		$admin_group_id = $groups->getAdministratorsGroupId();
		return $admin_group_id; 
	}
	
	function setSystemModule($data) {
		// print '<div>before:</div>'; print_r($data); print '<hr>';
		if (!isset($data['id']) || !$data['id']) { // Phil 20051018
			$minimum = ( (isset($data['is_custom']) && $data['is_custom']) ? '300' : '200');
			$new_id = $this->_getModuleInsertId($minimum);
			if ($new_id) {
				$data['id'] = $new_id;
			}
		}
		// print '<div>after:</div>'; print_r($data); print '<hr>';
		$this->db->insert('System__modules', $data);
		if ($this->db->hasError()) {
			$this->db->printError();
		}
	}
	
	/**
	 * @param integer $minimum The minimum id to use
	 * @return mixed An integer or false. Integer will be the first integer >= the minimum which is not already assigned to a module 
	 * @author Phil Calder
	 * 20051018
	 */
	function _getModuleInsertId($minimum = 300) {
		// $query = 'SELECT id FROM `System__modules` ORDER BY id ASC';
		if ($result = $this->db->select('id', 'System__modules', '', 'id ASC')) {
			$current_modules_ids = array();
			
			while ($row = @mysql_fetch_assoc($result)) {
				$current_modules_ids[] = $row['id'];
			}
			
			while ( in_array($minimum, $current_modules_ids) ) {
				$minimum++;
			}
			
			return $minimum;
		}
		return false;
	}
	
	function removePermissionsFor($module_name) {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Attempting to remove all privs for: ' . $item['module_name']. '',
			'Admin'
		);
		// print '<p>module_name = ' . $module_name . '</p>';
		
		$result = false;
		
		if ($id = $this->getSystemModuleId($module_name)) {
			// print '<p>id = ' . $id . '</p>';
			$result = $this->db->delete(
				'System__module_group', 
				'module_id = ' . $id
			);
		}
		return $result;
	}
	
	function removeModuleFromDb($module_name) {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Attempting to remove database record for module: ' . $item['module_name']. '',
			'Admin'
		);
		// print '<p>module_name = ' . $module_name . '</p>';
		
		$result = false;
		if ($id = $this->getSystemModuleId($module_name)) {
			// print '<p>id = ' . $id . '</p>';
			$result = $this->db->delete(
				'System__modules', 
				'id = ' . $id
			);
		}
		return $result;
	}
	
	function getSystemModuleId($module_name) {
		$where = "name='$module_name'";
		$result = $this->db->select(
			'id', 
			'System__modules', 
			$where
		);
		
		if($this->db->hasError()) {
			$this->db->printError();
		}
		
		while ($row = $this->db->getRow($result)) {
			$id = $row['id'];
		}
		
		return $id;
	}
	
	function setSystemModuleGroupPermissions($data) {
		$this->db->insert('System__module_group', $data);
		if($this->db->hasError()) {
			$this->db->printError();
		}
	}
	
	function createFileStructure($new_module, $src_dirname, $dest_dirname) {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction(
			'Attempting to create file structure for: ' . $new_module . '',
			'Admin'
		);
		
		if (!is_readable($src_dirname) ) {
			exit('<br><br>Files/Directories Not Readable....Cannot Proceed!');
		}
		
		$file_info = '';
		$dir = opendir( $src_dirname );
		$i = 0; // Count number of files copied
		
		while ($name = readdir($dir)) {
			
			if ($name != '.' && $name != '..') {
				
				if (is_dir($src_dirname . '/' . $name)) {
					
					@mkdir($dest_dirname . '/' . $name);
					
					if (!is_writeable($dest_dirname . '/' . $name) ) {
						
						return false;
						
					}
					
					// create directory
					$this->createFileStructure($new_module, $src_dirname . '/' . $name, $dest_dirname . '/' . $name);
					
				} else if (is_writeable($dest_dirname)) {
					
					// copy file
					$i++;
					
					if (!copy($src_dirname . '/' . $name, $dest_dirname . '/' . $name)) {
						
						$file_info .= '<br>failed to copy ' . $name . '...<br>';
						return false;
						
					} else {             
						
						$file_info .= '<br><b>-copied:</b> ' . $name . ' <b>to:</b> ' . $dest_dirname . ''; 
						$this->editAndReplaceTemplateRefs($new_module,$dest_dirname,$name);
						
					}
					
				} else {
					
					$file_info .= '<br>Error: Cannot write to Destination Directory: ' . $dest_dirname . '/';
					
				}
				
			}
			
		}
		
		$info = array($i,$file_info);
		return $info; //returns an array, [0] => 'files copied', [1]=>file info
		
	}
	
	function editAndReplaceTemplateRefs($new_module,$dest_dirname,$name) {
		global $SERVICES;
		
		$SERVICES->getService('Output');
		$modules =& $SERVICES->getService('Modules');
		
		// Reload custom modules (necessary to refresh since we have just added a module)
		$_custom_modules = $modules->getModules(1, true); 
		// print '<div>custom modules</div>'; print_r($_custom_modules); print '<hr>';
		
		$file_array = file($dest_dirname . '/' . $name);
		
		//reset variables
		$current_file = '';
		$edited_file = '';
		$complete_file = '';
		
		//Assume its 'Template' and 'template' that we are replacing
		
		//module name
		$new_module_name = str_replace('/','',$new_module);
		$new_module_name = str_replace('\\','',$new_module); // Windows
		
		//substr($new_module, 0, 1)  . substr($new_module, 1)
		
		$new_module_name = strtolower( substr($new_module_name, 0, 1) ) . substr($new_module_name, 1);
		//remove '/' commonly found at front of module name
		$module_name_lower = $new_module_name;
		$module_name_upper = ucfirst($new_module_name);
		
		// Loop through our array, show HTML source as HTML source; and line numbers too.
		foreach ($file_array as $line_num => $line) {
			$current_file .= $line;
		}
		
		if (is_writeable($dest_dirname . '/' . $name)) {
			//move all upper case TemplateNewModule to ucfirst(new_module name)
			
			$edited_file = str_replace(ucfirst($this->template_module_name), $module_name_upper, $current_file);
			
			$one = $this->template_module_name[0];
			$one = strtolower($this->template_module_name[0]);
			$lcfirst = $one . substr($this->template_module_name, 1);
			$complete_file = str_replace($lcfirst, $module_name_lower, $edited_file);
			
			// print '<div>filename: ' . $name . '</div>';
			// print '<div>is custom ' . $module_name_upper . ' = ' . ($modules->isCustom($module_name_upper) ? 'yes' : 'no') . '</div>';
			if ( ($name == 'module.config') && $modules->isCustom($module_name_upper) ) { // Phil 20050427, replace /modules/ with /modules_custom/
				$modules_basename = basename($SERVICES->config->lookup('modules.basedir'));
				$custom_basename = basename($SERVICES->config->lookup('modules.custom_basedir'));
				// print '<div>Replacing /' . $modules_basename . '/ with /' .  $custom_basename . '/</div>';
				$complete_file = str_replace(
					'/' . $modules_basename . '/', 
					'/' . $custom_basename . '/',
					$complete_file
				);
			}
			
			$file_name_and_path = $dest_dirname . '/' . $name;
			if (!($file_handler = fopen($file_name_and_path, "w+b") )) {
				quickPage( "Cannot open file ($name) for edit - check permissions." );
				exit();
			}
			if (fwrite($file_handler, $complete_file) === FALSE) {
				quickPage( "Cannot write to file ($name) - check permissions." );
				exit;
			} 
			if ($file_handler) {
				fclose($file_handler);
			}
			
		} else {
			quickPage( 'Permissions Incorrect, could not edit: ' . $name );
			exit();
		}
		
	}
	
	function getDbTableNames($table_prefix = '') {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		
		$tables = $db->getTables($table_prefix);
		
		return $tables;
	}
	
	/**
	 * exports the current database
	 * 
	 * @param  string $export_path the path to save the structure files...
	 * @param  boolean $structure flag to export table structure
	 * @param  boolean $data flag to export data with tables
	 * @return mixed boolean true on success, or a string error message
	 */
	function exportDatabase($export_path, $incl_structure = true, $incl_data = true) {
		global $SERVICES, $L;
		$SERVICES->getService('FileSystem');
		
		$SERVICES->filesystem->setDirectory($export_path);
		$db_filename = $export_path . '/dump.sql';
		if (!$SERVICES->filesystem->openFile($db_filename,'a')) {
			// quickPage('Failed to make structure dump file.');
			// exit();
			return $L['adminBackupCreateDbDumpFailed'] . ' ' . $db_filename;
		}
		
		$exclusions = array(
			'Logger__actions', 
			'Logger__errors', 
			'Logger__usage'
		);
		
		// foreach($SERVICES->modules->getModules() as $k => $v) {
			// print '<div>k=' . $k . ', v=' . $v . '</div>';
			// foreach ($this->getDbTableNames($v . '__') as $kk => $vv) {
			$db_tables = $this->getDbTableNames();
			// print '<div>DEBUG:</div>';
			// print_r($db_tables);
			foreach ($db_tables as $kk => $vv) {
				// print '<div>kk=' . $kk . ', vv=' . $vv . '</div>';
				$structure  = '';
				$data = '';
				
				// Structure
				if ($incl_structure) {
					$structure = $this->getDatabaseTableStructure($vv);
					$SERVICES->filesystem->writeToFile($structure);
				}
				
				// Data
				if ($incl_data) {
					
					if (!$SERVICES->database->quickget('count(*)',$vv)) {
						$zero_count[] = $vv;
						// print '<div>Zero count, kk=' . $kk . ', vv=' . $vv . '</div>';
						continue;
					}
					if (in_array($vv, $exclusions)) {
						// print '<div>Excluding table, kk=' . $kk . ', vv=' . $vv . '</div>';
						continue;
					}
					$data_result = $SERVICES->database->standardSelect('*',$vv); // Bybasses custom table processing
					// formulate insert statement
					if ($data_result) {
						
						$data .= '#' . "\n";
						$data .= '# DATA FOR TABLE: ' . $vv . "\n";
						$data .= '#' . "\n\n";					
						$data .= 'INSERT INTO ' . $vv . ' VALUES ';
						
						$SERVICES->filesystem->writeToFile($data);
						
						$comma_flag = '';
						
						// Phil 20060712 - Added 2nd and 3rd params, dont strip slashes
						while ($row = $SERVICES->database->standardGetRow($data_result)) { // -1, false
							//$data .= '(\'' . implode('\',\'',$row) . '\'),'; 
							if (is_array($row)) {
								foreach ($row as $row_k => $row_v) {
									$row[$row_k] = addslashes($row_v);
								}
							}
							$SERVICES->filesystem->writeToFile($comma_flag . "\n" . '(\'' . implode('\',\'',$row) . '\')');
							$comma_flag = ',';
						}
						$SERVICES->filesystem->writeToFile(';' . "\n\n");
					}
				}
			}
		// }
		// print '<div>closing file</div>';
		$SERVICES->filesystem->closeFile();
		return true;
	}
	
	function compressExport($export_path) {
		global $SERVICES, $L;
		$SERVICES->getService('FileSystem');
		
		if (!file_exists($export_path) || !is_dir($export_path) || !is_readable($export_path)) {
			// $SERVICES->filesystem->printDebug(get_class($this) . '::compressExport: ' . $L['adminBackupCompressExportFailed'] . ' ' . $export_path);
			return $L['adminBackupCompressExportFailed'] . ' ' . $export_path;
		}
		$filename = basename($export_path);
		// $SERVICES->filesystem->printDebug(get_class($this) . '::compressExport: Calling filesystem zipDirectory(' . $filename . ', ' . $export_path . ')');
		return $SERVICES->filesystem->zipDirectory($filename, $export_path);
	}
	
	/**
	 * get table structure from the currently used db.
	 * 
	 * @param  string $table table to get structure from
	 * @return $structure returns a structure statement as if it was from an sql file
	 */
	function getDatabaseTableStructure($table){
		global $SERVICES;
		//get database table structure
		$structure = '#' . "\n";
		$structure .= '# STRUCTURE FOR TABLE: ' . $table . "\n";
		$structure .= '#' . "\n\n";
		$structure .= $SERVICES->database->getShowCreateTable($table) . ";\n\n";
		
		return $structure;
	}
}
?>