<?php
/** 
 * @license http://www.browsercrm.com/licence.php BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 **/

includeWebMAT('admin_dropdown.inc');

/**
 * Layout manager for the Contact View
 * 
 * @package Contact
 * @author Phil Calder
 **/
class ContactViewWebMATAdminDropdown extends WebMATAdminDropdown{ // Note: used to extend admin nested dropdown
	
	/**
	 * Constructor
	 * Calls parent, then sets user and 'New' menu links
	 */
	function ContactViewWebMATAdminDropdown($post, $get, $mini_data_definition = false, $access_level = 'read') {
		$this->WebMATAdminDropdown($post, $get, $mini_data_definition, $access_level); // Phil 20050908 Set default access level to read
		$this->setUserId();
		$this->new_menu_links[] = $this->getResetUserLink();
	}
	
	function main() {
		global $L, $SERVICES;
		
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Front');
		$SERVICES->language->getLanguage('Contact');
		parent::main();
	}
	
	function setUserId() {
		global $USER, $SERVICES;
		
		if (
			$this->userIdIsFixed() 
			&& isset($this->form_values['user_id']) 
			&& ($this->form_values['user_id'] != $USER['user_id']) 
			&& ($ua =& $SERVICES->getService('Authentication'))
			&& $ua->adminAccess($USER['user_id'], 'Admin')
		) {
			
			$this->user_id = $this->form_values['user_id'];
			// $this->user_id = $this->getUserId();
			
		} else {
			$this->user_id = $USER['user_id'];
		}
	}
	
	function setFixedUserId() {
		$this->user_id_fixed = true;
	}
	
	function userIdIsFixed() {
		return $this->user_id_fixed;
	}
	
	function getUserId() {
		return $this->user_id;
	}
	
	function getItems() {
		$items = $this->getItemsFor();
		if (!sizeof($items)) {
			$this->copyDefaultSetup();
			$items = $this->getItemsFor();
		}
		return $items;
	}
	
	/**
	 * Gets a list of items to display in the selected column, or all columns.
	 *
	 * @param integer $user_id (optional) 
	 * @return array $columns key is the column id, value is the column name
	 */
	 function getItemsFor($user_id = '') {
		global $USER, $SERVICES;
		
		$ua =& $SERVICES->getService('Authentication');
		
		if (empty($user_id)) {
			global $USER;
			$user_id = ($USER['user_id'] ? $USER['user_id'] : 0);
		} else if ($user_id == 'default') {
			$user_id = '0';
		}
		
		// $extra_where = ' AND active = 1 AND user_id = ' . $user_id;
		// $items = $this->dropdown->getItems('1' . $extra_where);
		// Phil 20070314 active included by default, no need for the unused '1' or AND
		$extra_where = 'user_id = ' . $user_id;
		$items = $this->dropdown->getItems($extra_where);
		
		
		$pieces = array();
		
		foreach ($items as $k => $v) {
			// print '<div>k=' . $k . '</div>'; print_r($v); print '<hr>';
			if (!$user_id || $ua->readAccess($user_id, $v['name'])) {
				// if ($this->copy_default) {
					
					// $pieces[$k] = array(
						// 'name' => $v['name'],
						// 'priority' => $v['priority']
					// );
					$pieces[$k] = $v;
					if (isset($pieces[$k]['user_id'])) {
						unset($pieces[$k]['user_id']);
					}
					
				// } else {
				// 	$pieces[$k] = $v['name']; // This values could be the language item instead of the key: $L[$v['label']]
				// }
			}
		}
		return $pieces;
	}
	
	function copyDefaultSetup() {
		global $USER;
		
		$this->copy_default = true;
		$items = $this->getItemsFor('default');
		$this->copy_default = false;
		
		foreach ($items as $k => $v) {
			
			if (is_array($v)) {
				$this->dropdown->setValue(
					array(
						'name'=>$v['name'], 
						'user_id' => $USER['user_id'], 
						'priority'=>$v['priority']
					)
				);
			} else {
				$this->dropdown->setValue(
					array(
						'name'=>$v, 
						'user_id' => $USER['user_id'], 
					)
				);
			}
		}
	}
	
	/**
	 * @deprecated version 4.130
	 * Now calls and returns {@link getConfigPieces}
	 */
	function getPieces() {
		return $this->getConfigPieces(); // Phil 20050523
	}
	
	/**
	 * @author Phil Calder 20050801
	 */
	function getConfigPieces() {
		global $SERVICES, $USER;
		$SERVICES->getService('Config');
		$ua =& $SERVICES->getService('Authentication');
		$config_pieces = $SERVICES->config->lookup('modules.Front.contactviewconfig');
		
		// Phil 20070501 - these pieces now attached to the 'main' contact tabset
		// if (is_array($compulsary_config_pieces = $SERVICES->config->lookup('modules.Front.compulsarycontactviewconfig'))) {
			// $config_pieces = bcrm_array_merge($compulsary_config_pieces, $config_pieces);
		// }
		
		if (!is_array($config_pieces)) {
			$config_pieces = array();
		}
		
		foreach($config_pieces as $k => $config_piece) {
			// print '<div>k=' . $k . '</div>'; // print_r($config_piece); print '<hr>';
			if (!isset($config_piece['module']) && is_array($config_piece)) { // Incase there is more than one view per module
				foreach ($config_piece as $kk => $sub_piece) {
					if (!$ua->readAccess($USER['user_id'], $sub_piece['module'])) {
						// print '<div>a) No access to ' . $k . '</div>';
						bcrm_unset($config_pieces, $k);
					} else {
						bcrm_unset($config_pieces, $k);
						$config_pieces[ $sub_piece['module'] . '_' . $kk ] = $sub_piece;
					}
				}
			} else { // Only one piece for this module
				if (!$ua->readAccess($USER['user_id'], $config_piece['module'])) {
					bcrm_unset($config_pieces, $k);
				}
			}
		}
		
		// if (isset($config_pieces['file'])) { // Only one, we need an array
			// $links = array($config_pieces);
		// }
		
		$sortarray = array();
		
		foreach ($config_pieces as $k => $config_piece) {
			$sortarray[] = (isset($config_piece['priority']) ? $config_piece['priority'] : '');
		}
		array_multisort($sortarray,$config_pieces);
		
		return $config_pieces;
	}
	
	function getAvailablePieces() {
		global $SERVICES, $L;
		$SERVICES->getService('Language');
		
		$links = $this->getPieces();
		$values = array();
		
		foreach ($links as $k => $v) {
			$SERVICES->language->getLanguage($v['module']);
			// $values[$k] = $v; // This values could be the language item instead of the key: $L[$v['label']]
			// print '<p>label: ' . $v['label'] .' [' . $L[$v['label']] . ']</p>';
			$values[$k] = (strlen($L[$v['label']]) ? $L[$v['label']] : $v['label']);
		}
		
		return $values;
	}

	/**
	 * Creates a cut down Data Dictionary
	 *
	 * @param array $item the current item from the dropdown table.
	 * @return array an array of items with formating infromation
	 */
	function getFormFields($item) {
		global $L, $USER;
		
		if (!isset($item['user_id'])) {
			$item['user_id'] = $this->getUserId(); // $USER['user_id'];
		}
		
		if (!$this->mini_data_definition) {
			global $SERVICES;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage('Front');
			$SERVICES->language->getLanguage('Contact');
			$fields =  array(
				'id' => array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'),
				'user_id' => array('label'=>'', 'field_key'=>'user_id', 'type'=>'hidden'),
				$this->dropdown_config['field'] => array('label'=>$L[$this->module_name_lower . 'DropdownValueName'], 'field_key'=>$this->dropdown_config['field'], 'type'=>'text', 'notnull' => 1),
				'priority' => array('label'=>$L['DisplayOrder'], 'field_key'=>'priority', 'type'=>'integer', 'min' => 0, 'max' => 999, 'default_value' => 100, 'notnull' => 1), 
				array('label'=>$L['CountAlerts'], 'field_key'=>'count_alerts', 'type'=>'checkbox'), 
				array('label'=>$L['CountItems'], 'field_key'=>'count_items', 'type'=>'checkbox'), 
				array('label'=>$L['ShowItems'], 'field_key'=>'show_items', 'type'=>'integer')
			);
			$fields[$this->dropdown_config['field']]['type'] = 'dropdown';
			$fields[$this->dropdown_config['field']]['dropdown_values'] = $this->getAvailablePieces();
		} else {
			$fields = $this->mini_data_definition;
		}
		
		foreach ($fields as $k => $v) {
			if (isset($item[$v['field_key']])) {
				$fields[$k]['value'] = $item[$v['field_key']];
			} else {
				$fields[$k]['value'] = $fields[$k]['default_value'];
			}
		}
		// print '<div>getFormFields item</div>'; print_r($item); print '<hr>';
		return $fields;
	}
	
	/**
	 * Builds the nested table for the list view
	 *
	 * @return string the table rows for the nested table. If there are no values for this dropdown, the string will be empty.
	 */
	function getNestedTable() {
		global $USER;
		$table_rows = ''; // Phil - initialised our return value.
		
		// Only items for the current user
		$this->dropdown->setWhereStatement(
			array(
				'user_id' => $this->getUserId() // $USER['user_id']
			)
		);
		
		$values = $this->dropdown->getValues('&nbsp;&nbsp;&nbsp;');
		
		if (is_array($values)) {
			// print_r($values); print '<hr>';
			global $SERVICES;
			$ua =& $SERVICES->getService('Authentication');
			
			$items = array();
			foreach ($values as $k => $v) {
				if ($ua->readAccess($USER['user_id'], $v)) {
					// print '<p>user ' . $USER['user_id'] . ' can read ' . $v . '</p>';
					$items[] = array( 'id' => $k, $this->dropdown_config['field'] => $v);
				}//  else {
					// print '<p>user ' . $USER['user_id'] . ' CANNOT read ' . $v . '</p>';
				// }
			}
			
			foreach ($items as $item) {
				$table_row = array();
				foreach ($this->getFormFields($item) as $field) {
					if ($field['field_key'] == $this->dropdown_config['field']) {
						// $table_row[] = $this->output->getStringField($field); // Phil
						//$table_row[] = $this->getStringField($field);
						$table_row[] = array (
							'content' => $this->output->getStringField($field),
							'tag' => 'th'
						);
					}
				}
				$table_row = $this->getEditDeleteLinks($table_row,$item['id']);
				$table_rows .= $this->output->getTableRow($table_row);
			}
		}
		return $table_rows;
	}
	
	/**
	 * Appends an edit link & deletelink to the supplied array.
	 *
	 * @param array $link_array the current row_array
	 * @param string $id the current item id
	 * @return array the row_array with the links appended
	 */
	function getItemLinks($row_array, $id) {
		global $L, $PHP_SELF;
		$row_array[] = $this->output->getLinkHref( array('href' => $PHP_SELF . '?id=' . $id, 'text' => $L['LinkEdit']) );
		$text = '';
		$row_array[] = $this->output->getLinkDelete($PHP_SELF . '?delete=' . $id, $text);
		return $row_array;
	}
	
	function getResetUserLink() {
		global $L, $SERVICES;
		if (strstr($SERVICES->getGlobal('PHP_SELF'), 'default')) {
			$url = '../Contact/admin_contact_view_default_setup.php?reset=1';
		} else {
			$url = '../Contact/admin_contact_view_setup.php?reset=1';
		}
		return $this->output->getLinkConfirm(
			$url,
			$L['contactResetUserContactView'], 
			$L['contactResetUserContactViewAreYouSure'], 
			$this->toolbar_link_mode // whether to display as part of the toolbar
		);
	}
	
	function resetForUser() {
		// print '<p>resetForAll</p>';
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		if ($ua->adminAccess($USER['user_id'], 'UserAdmin')) {
			// print '<p>reseting</p>';
			$user_id = $USER['user_id'];
			$db =& $SERVICES->getService('Database');
			$db->delete(
				'Contact__view_dropdown', 
				"user_id = '$user_id'"
			);
			if ($db->hasError()) {
				$db->printError();	
			}
			$this->copyDefaultSetup();
		}
	}
	
	function getIdFor($module, $piece) {
		$key = $module . ($piece ? '_' . $piece : '');
		// print '<div>looking for key: ' . $key . '</div>';
		// $config_pieces = (array)$this->getConfigPieces();
		$items = (array)$this->getItems();
		// print_r($items); print '<hr>';
		// foreach ($config_pieces as $k => $v) {
			// if ($k == $key) {
				// print '<div>found ' . $k . '</div>'; print_r($v); print '<hr>';
				// return;
			// }
		// }
		$tmp_id = false;
		foreach ($items as $k => $v) {
			if ($key == $v['name']) {
				// print '<div>found ' . $key . '</div>'; print_r($v); print '<hr>';
				return $v['id'];
			} else if ($piece == $v['name']) {
				$tmp_id = $v['id'];
			}
		}
		return $tmp_id;
	}
}
?>