<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */
 
/**
 * @package Contact
 * @author Phil Calder
 */
class ContactItemWebMATList extends WebMATList {
	/**
	 * Sets a custom design time filter for import_id (if the relevant form value is set) or duplicates
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		
		$this->new_menu_links[] = $this->getNewPersonLink();
		if (isset($this->form_values['import_id']) && $this->form_values['import_id']) {
			if (!$this->form_values['filter']) {
				$this->form_values['filter'] = 'fnone'; // Phil 20050711
			}
			$this->db_functions->setDesignTimeFilters(array('import_id'=>$this->form_values['import_id']));
		} else if (isset($this->form_values['duplicates']) && $this->form_values['duplicates']) {
			if (!$this->form_values['filter']) {
				$this->form_values['filter'] = 'fnone';
			}
			$this->db_functions->setDesignTimeFilters(array('is_duplicate_checked'=>array('where'=>'is_duplicate_checked <> 1')));
			// TODO: if we have a contact_id (or id) filter, eg: ?duplicates=1&contact_id=1
			// find all duplicates for the given contact and display in this list (along with the given contact)
			// add an option to process multi to merge selected contacts
			global $L;
			$this->language_overwrite['title'] = $this->getTitleText() . ': ' . $L['contactContactDuplicates'];
		}
		// Phil 20050711
		if (isset($this->form_values['contact_company']) && $this->form_values['contact_company']) {
			if (!$this->form_values['filter']) {
				$this->form_values['filter'] = 'fnone'; // Phil 20050711
			}
			$this->db_functions->setDesignTimeFilters(
				array( // Phil 20050123 see WebMATDatabaseTable::getDesignTimeFiltersWhere - can use any key in table 'columns', so no need to use __special__
					'id'=>array('where'=>'Contact__item.contact_company=' . $this->form_values['contact_company'])
				)
			);
		}
	}
	
	/**
	 * Builds the View link
	 * 
	 * Differs from the standard 'view' link. This is a 'contact_view'.
	 * 
	 * @param string $id the id of the item you want to view
	 * @return string containing the link
	 */
	function getViewLink() {
		global $SERVICES, $L;
		$contact_view = 'contact_view';
		if ( ($this->getFileSetHref($contact_view)) && ($this->isDataOption('edit')) ) {
			$link_href = $this->getFileSetHref($contact_view);
			$link = $this->output->getLinkView($link_href);
		} else {
			$link = parent::getViewLink();
		}
		return $link;
	}
	
	/**
	 * Returns a new company link
	 */
	function getNewLink() {
		return $this->getNewCompanyLink();
	}
	
	function getNewCompanyLink() {
		return $this->mod->getNewCompanyLink();
	}
	
	function getNewPersonLink() {
		return $this->mod->getNewPersonLink();
	}
	
	/**
	 * Returns new company and new person links
	 *
	 * @author Phil Calder
	 */
	function getNewLinks() {
		// Modules can override this method to include more than one new link
		return array(
			'newCompany'   => $this->getNewCompanyLink(), 
			'newPerson' => $this->getNewPersonLink()
		);
	}
	
	/**
	 * Based on contact fields: id
	 *
	 * @return mixed return value will be false or an array
	 * @author Phil Calder
	 * @todo Remove duplication from this function and getSelectContacts() (ie write function getContactFields())
	 */
	function getSelectContacts() {
		$this->output->setWhitespaceCompressionSwitch(false);
		if ($this->getFileSetHref('process_multi')) {
			$contact_fields = array('id');
			global $L;
			return array('select_contacts' => $L['labelSelectContacts'], 'contact_fields' => implode(',', $contact_fields));
		}
		return false;
	}
	
	/**
	 * Based on contact field: id
	 * 
	 * @return mixed return value will be false or an array
	 * @author Phil Calder
	 * @todo Remove duplication from this function and getSelectContacts() (ie write function getContactFields())
	 */
	function getMailmergeContacts() {
		$this->output->setWhitespaceCompressionSwitch(false);
		if ($this->getFileSetHref('process_multi')) {
			$contact_fields = array('id');
			if (count($contact_fields)) {
				global $L;
				return array('mailmerge' => $L['labelEmailmergeContacts'], 'contact_fields' => implode(',', $contact_fields));
			}
		}
		return false;
	}	
	
	/**
	 * Performs a check to make sure the user has delete rights for this module
	 *
	 * Delete rights are based on a Contact user preference
	 *
	 * @return string the mass delete link
	 */
	function getMassDeleteItem() {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		if (! ($allow_delete = $SERVICES->preferences->getUserPref('contactAllowContactDelete'))) {
			return false;
		}
		if ($link_href = $this->getFileSetHref('mass_delete')) {
			global $L;
			return array('mass_delete' => $L['labelMassDelete']);
		}
		return false;
	}	
	
	/** 
	 * @deprecated version 5
	 */
	function addSecondaryFields() {
		return false;
	}
	
	/**
	 * @deprecated version 5
	 */
	function getShowAllAddressesLink($show_all_addresses_link = 1) {
		return false;
		// return $this->mod->getShowAllAddressesLink($show_all_addresses_link);
	}
	
}
?>