<?php
require_once("../../prepend.inc");
/** 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */

includeWebMAT('multi_edit.inc');

/**
 * Contact multi-edit page
 *
 * @author Phil Calder
 * @package Contact
 */
class ContactItemWebMATMultiEdit extends WebMATMultiEdit {
	var $contact_permissions = array();
	
	/**
	 * any processing required before adding the form
	 *
	 * Overridden to ensure that Contact display name is non-editable text
	 */
	function main($method = false, $values = false, $error_msgs = false) {
		//any other processing before the Form is created (ie. adding extra items)
		$this->dd->changeAttributes(
			array(
				'field_key' => 'display_name', 
				'type' => 'free'
			)
		);
		//$this->output->setShowContactSelector(false);
		parent::main($method, $values, $error_msgs);
	}
	
	/**
	* Store item permissions for use in {@link afterDBWrite}
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference
		//any other processing before posting to database
		// print_r($item);
		$db_item = $this->db_functions->getItem($item['id']);
		$this->contact_permissions = array(
			'view_group_id'  => $db_item['view_group_id'], 
			'edit_group_id'  => $db_item['edit_group_id'], 
			'owner_group_id' => $db_item['owner_group_id'], 
			'admin_group_id' => $db_item['admin_group_id'], 
			'lock_permissions' => $db_item['lock_permissions']
		);
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		$this->mod->updateContactDisplayName($item['id']);
		if ($this->checkCascadingPermissions($item, $fields, $error_msg, $id)) {
			$this->runCascadingPermissions($item, $fields, $error_msg, $id);
		}
		parent::afterDbWrite($item, $fields, $error_msg, $id);
	}
	
	function checkCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		// Apply cascading permissions where applicable
		global $SERVICES;
		$SERVICES->getService('Config');
		
		$this->new_permissions = array(
			'view_group_id'  => $item['view_group_id'], 
			'edit_group_id'  => $item['edit_group_id'], 
			'owner_group_id' => $item['owner_group_id'], 
			'admin_group_id' => $item['admin_group_id'], 
		);
		
		if (is_array($this->contact_permissions)) {
			// print '<div>contact id is: ' . $item['id'] . '</div>'; print_r($item);
			// print '<div>saved</div>'; print_r($this->contact_permissions); print '<hr>';
			// print '<div>new</div>'; print_r($new_permissions); print '<hr>';
				
			$old_lock_permissions = $this->contact_permissions['lock_permissions'];
			bcrm_unset($this->contact_permissions, 'lock_permissions');
			
			$db_item = $this->db_functions->getItem($item['id']);
			$this->contact_lock_permissions = $db_item['lock_permissions'];
			
			if (!$this->contact_lock_permissions) {
				return false;
			} else if ($this->contact_lock_permissions && !$old_lock_permissions) {
				// The edited contact has had the lock_permissions field set
				return true;
			}
			
			foreach ($this->contact_permissions as $k => $v) {
				if ( isset($this->new_permissions[$k]) && ($this->new_permissions[$k] != $v) ) {
					// The permissions have been changed
					return true;
				}
			}
		}	
		return false;
	}
	
	function runCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) {
		global $SERVICES;
		// print '<p>need to cascade permissions (' . $k . ' has changed or lock has been set)</p>';
		
		$SERVICES->getService('Log');
		$SERVICES->log->logAction('Contact: Run cascade permissions for contact item ' . $item['id'] . '');
		
		$modules = $SERVICES->config->lookup('modules');
		// print_r(array_keys($modules));
		if (is_array($modules)) {
			foreach ($modules as $k => $v) {
				// print '<hr><div>module: ' . $k . '</div>'; 
				if (is_object($mod =& $SERVICES->modules->getModule($k))) {
					// print '<div>is an object: ' . get_class($mod) . '</div>'; 
					// print_r(array_keys($v));
					$file_sets = $v['file_set'];
					// $file_sets = $SERVICES->config->lookup('modules'.$k.'.file_set');
					if (is_array($file_sets)) {
						// print '<div>file_sets: </div>'; print_r(array_keys($file_sets));
						foreach($file_sets as $fk => $fv) {
							// print '<div>file_set: ' . $fk . '</div>'; 
							if (
								method_exists($mod, 'getDatabaseTable') 
								&& ($db_functions =& $mod->getDatabaseTable($fk))
							) {
								// print '<div>got db_functions</div>';
								if (
									$this->contact_lock_permissions 
									|| isset($fv['display_by_group']) && $fv['display_by_group']
								) {
									// print '<div>Cascade permissions</div>';
									$db_functions->cascadePermissions($this->new_permissions, $item['id']);
								}
								unset($db_functions);
							}
							
						}
					}
				}
				unset($mod);
			}
		}		
	}
}
?>