<?php
/**
 * The import data class
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package ContactImport
 * @author Phil Calder
 */
 
/**
 * The import data class
 * 
 * @package ContactImport
 * @author Phil Calder
 */
class ContactImportDataWebMATEdit extends WebMATEdit {
	
	var $crm_module = '';
	var $standalone_companies = true; // Companies have their own contact record TODO: This --may-- become an option (similar to merge_companies)
	var $merge_companies = false; // Items with the same company info will point to the same Company record
	var $existing_companies = array(); // stores info about the companies imported so far
	var $attach_field_priority = 1; // determines who gets extra data (address), default is 1=person; also 2=company
	
	/**
	 * Performs per-module beforeDbWrite functionality
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeSetItem(&$item, $mod) {  //passed by reference
		global $SERVICES;
		if ($mod == 'Person') {
			global $L;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage('Person');
			
			$SERVICES->getService('Preferences');
			$name_format_pref = $SERVICES->preferences->getUserPref('personNameFormat', 'last_name , first_name');
			
			$field_separator = ' ';
			
			$display_name_fields = explode($field_separator, $name_format_pref);
			
			$display_name = '';
			foreach ($display_name_fields as $k => $v) {
				// $display_name .= ((isset($item[$v]) && $item[$v]) ? $field_separator . $item[$v] : (strlen(trim($display_name)) ? $v : ''));
				// Incorrectly included display name field (eg last_name) if not in item
				$display_name .= ((isset($item[$v]) && $item[$v]) ? $field_separator . $item[$v] : (strlen(trim($display_name)) ? ' ' : ''));
			}
			
			$display_name = trim($display_name, ' :,.-');
			
			// Phil 20060710
			// display name is calculated (first and last names), 
			// but if we are importing that field and the fields we calculate it from are empty, 
			// use the imported field
			if (!strlen($display_name) && strlen($item['display_name']) && !strlen($item['last_name']) && !strlen($item['first_name'])) {
				$display_name = $item['display_name'];
				$item['last_name'] = $item['display_name'];
			}
			
			$item['display_name'] = '' . (strlen($display_name) ? $display_name : $L['personDefaultDisplayName']);
			
		} else if ($mod == 'AddressPostal') {
			$postal_mod =& $SERVICES->modules->getModule($mod);
			if (is_object($postal_mod) && method_exists($postal_mod, 'getDisplayName')) {
				$item['display_name'] = $postal_mod->getDisplayName(false, $item);
			}
		} else if ($mod == 'AddressPhysical') {
			$physical_mod =& $SERVICES->modules->getModule($mod);
			if (is_object($physical_mod) && method_exists($physical_mod, 'getDisplayName')) {
				$item['display_name'] = $physical_mod->getDisplayName(false, $item);
			}
		}
		
	}
	
	function doSubmitBegin() {
		if ($this->form_values['item']['id']) {
			
			$item = $this->db_functions->getItem($this->form_values['item']['id']);
			
			$this->merge_companies = (isset($item['reuse_companies']) ? $item['reuse_companies'] : $this->merge_companies);
			//$this->merge_companies = true;
			
			// Phil: 20040218 - standalone_companies always true (requirement change from MCR)
			//$this->standalone_companies = (isset($item['standalone_companies']) ? $item['standalone_companies'] : $this->standalone_companies);
			// print '<div>standalone_companies: ' . $this->standalone_companies . '</div>';
		}
		
		$this->store = new SessionStore(1);
		$this->map = new Mapping($this->store);
		
		if ($this->submit) {
			$this->saveMapping();
		} else {
			return;
		}
		
		$this->doSubmitPermissions();
		
		$this->importData();
		
	}
	
	/**
	 * Halt if the user cannot import
	 */
	function doSubmitPermissions() {
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		if (!$ua->adminAccess($USER['user_id'], 'ContactImport')) {
			global $L;
			quickPage($L['contactImportNoImportRights'] . '<br> <br> ' . $this->getLinkReturnPage() );
			exit();
		}
	}
	
	/**
	 * remove mapping
	 */
	function doSubmitEnd($id) {
		$this->store->hardDeleteStore();
		parent::doSubmitEnd($id);
	}
	
	/**
	 * Import the data
	 */
	function importData() {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		
		//todo implement this in the UI if required
		// set the value to what's in the config, in case it's different than the default
		//$this->attach_field_priority = $SERVICES->config->lookup('modules.ContactImport.import.attach_field_priority');
		
		$count = $db->quickGet('count(*)', $this->map->getTableName());
		$this->setPageTimeLimit($count);
		
		$result = $db->select('*', $this->map->getTableName());
		
		// These variables initialised here to set up default module ordering for processing
		$data = array(); // stores the data we will import
		
		//todo togo?  this is set from dds below; is it needed to establish the order?
		$objects = array( // store the available Import modules (keys)
			'Contact' => array(), 
			'Company' => array(), 
			'Person' => array(), 
			//'AddressPostal'=>array(), 
			//'AddressEmail'=>array(), 
			//'AddressPhone'=>array(), 
			//'AddressURL'=>array(), 
			'Note' => array()
		);
		
		// --
		// Change the objects array based on data definition options (any module with an 'import' data option)
		$fields = new Fields();
		$dds = $fields->getImportModuleDataDictionaries();
		//print_r($dds); print '<hr>';
		if (is_array($dds)) {
			//$data = $objects = array();
			$objects = array();
			$data_keys = array_keys($dds);	
			foreach($data_keys as $k => $v) {
				// $data[$v] = array();
				$objects[$v] = array();
			}
		}
		// --
		// print_r($data); print '<hr>'; print_r($objects); print '<hr>';
		
		$mapping_from = $this->map->getColMapFrom();
		$mapping_to = $this->map->getColMapTo();
		
		
		//todo togo?
		// Get the table name ContactImport_field_mapping
		$db_functions =& $this->mod->getDatabaseTable("fieldMapping");
		
		// get mapping from ContactImport__field_mapping, based on the template_id
		$result_field_mapping = $db->select('*','ContactImport__field_mapping',"template_id=".$this->form_values['item']['id']);
		
		//get columns for the temp table that is being imported
		$mapping_from = $db->getColumns($this->store->getValue('table_name'));
		
		// collect the data in $mapping_to_values
		while ($row = $db->getRow($result_field_mapping)) {
			$mapping_to_values[$row['external_name']] = $row['crm_field_name'];
		}
		
		// prepare a $mapping_to contents.
		for ($i=0; $i<count($mapping_from); $i++) {
			if (!empty($mapping_to_values) && array_key_exists($mapping_from[$i], $mapping_to_values)) {
				$mapping_to[] = $mapping_to_values[$mapping_from[$i]];
			} else {
				$mapping_to[] = "";
			}
		}
		
		// get all the data from ContactImportTemplate
		$template_item = $this->db_functions->getItem($this->form_values['item']['id']);
		
		// get the permissions from ContactImport_item to be saved in ContactImport__field_mapping
		$template_permissions = $db->quickGet("ContactImport__item.owner_group_id, ContactImport__item.view_group_id, ContactImport__item.edit_group_id, ContactImport__item.admin_group_id", "ContactImport__item", "id = ".$template_item['id'], "ContactImport__item.id");
		
		
		// if save_type is new, means user want to save this as a new mapping for this template
		if ($this->form_values['save_type'] == "New") {
			foreach ($mapping_from as $k => $v) {
				if ($mapping_to[$k] != "") {
					$template_item = array(
						"id" => '',  
						"template_id"=>$this->form_values['item']['id'], 
						"external_name"=>$v, "crm_field_name"=>$mapping_to[$k], 
						"owner_group_id"=>$template_permissions['owner_group_id'], 
						"view_group_id"=>$template_permissions['view_group_id'], 
						"edit_group_id"=>$template_permissions['edit_group_id'], 
						"admin_group_id"=>$template_permissions['admin_group_id']
					);
					$db_functions->setItem($template_item);
				}
			}
		} else if ($this->form_values['save_type'] == "Replace") { // replace existing mapping for this template
			global $SERVICES;
			$db =& $SERVICES->getService('Database');
			// delete old values before inserting new
			$db->delete($db_functions->table_name, "template_id = ".$this->form_values['item']['id']);
			foreach ($mapping_from as $k => $v) {
				if ($mapping_to[$k] != "") {
					$template_item = array("id" => '',  "template_id"=>$this->form_values['item']['id'], "external_name"=>$v, "crm_field_name"=>$mapping_to[$k], "owner_group_id"=>$template_permissions['owner_group_id'], "view_group_id"=>$template_permissions['view_group_id'], "edit_group_id"=>$template_permissions['edit_group_id'], "admin_group_id"=>$template_permissions['admin_group_id']);
			
					$db_functions->setItem($template_item);
				}
			}
		}
		
		
		//start of actual import
		
		$db_functions = $this->getModuleDatabaseTable("ContactImport", "item");
		// print '<p>standalone_companies: ' . $this->standalone_companies . '</p>';
		// print '<p>merge_companies: ' . $this->merge_companies . '</p>';
		$module_suffix = '__';
		$table_suffix = '.';
		
		$row_num = 0;
		$this->count_contacts_added = 0; // Used for 'Display Summary'
		$this->count_people_added = 0; // Used for 'Display Summary'
		$this->count_companies_added = 0; // Used for 'Display Summary'
		$this->count_standalone_company_contacts = 0; // Used for 'Display Summary'
		
		$date = $SERVICES->getService('DateLib');
		$all_fields = $fields->getFields();
		
		while ($row = $db->getRow($result)) {

			// when dealing with attach_field_priority, we may need both.
			$person_contact_id = 0;
			$company_contact_id = 0;
			
			//build $data array for the current import row
			foreach ($mapping_from as $k => $v) {
				$i = 0; // item number - there may be more than one item per row (ie 2 Note.content fields - we will need to add 2 items)
				if ($row[$v]) {
					
					$module = substr($mapping_to[$k], 0, strpos($mapping_to[$k], $module_suffix));
					$table_field = substr($mapping_to[$k], (strpos($mapping_to[$k], $module_suffix) + strlen($module_suffix)));
					$table = substr($table_field, 0, strpos($table_field, $table_suffix));
					$field = substr($table_field, strpos($table_field, $table_suffix)+strlen($table_suffix));
					
					if (isset($objects[$module])) {
						if (isset($data[$row_num][$module][$table][$i][$field])) {
							$i++; // We already have stored this field - this (and any other fields from this table) will be assigned to a new record
						}
						
						// Gurdeep Singh[20050825] Added a code to check if the field data type is date, if it is date then check if it is not empty, if its not empty then format the date and assign $row[date_field] the formatted date value
						$date_field_name =  $mapping_from[$k];
						if ($dds[$module]->data_dictionary[$field]['type'] == "date" || $dds[$module]->data_dictionary[$field]['type'] == "date_time") {
							$formatted_date = $date->getMySqlDateFromUserPrefDate($row[$date_field_name], $this->form_values['item']['date_format'], $this->form_values['item']['date_separator'])."<br>";
							$row[$date_field_name] = $formatted_date;
						}
						
						// print '<div>assigning ' . $row[$v] . ' to: ' . $table . ' - ' . $field . '</div>'; // DEBUG
						// $data[$row_num][$module][$table][$i][$field] = $row[$v];
						// Phil 20060817
						$data[$row_num][$module][$table][$i][$field] = $this->processField($row[$v], $module, $field, $dds[$module]->data_dictionary[$field]);
						
						
						//todo togo?
						// Gurdeep Singh [20050817] Added a code to check for the field content and updated the field notes in AddressPhone__item
						if ( $notes_field = $SERVICES->config->lookup('modules.ContactImport.notes.' . $module . '.' . $table . '.' . $field) ) {
							$data[$row_num][$module][$table][$i][$notes_field] = $v;
						}
						//
						
					}
				}
			}
			
			//if have some data from this row to process
			/*
			if (is_array($data[$row_num])) { 
				
				$contact_data = (isset($data[$row_num]['Contact']['item'][0]) ? $data[$row_num]['Contact']['item'][0] : false);
				if (isset($data[$row_num]['Contact']['item'])) {
					unset($data[$row_num]['Contact']['item']);
				}
				
				if ($contact_id = $this->addContact($contact_data)) { // Add a new contact
					$count_contacts_added++;
					//print '<div>DEBUG: increment for ' . $contact_id . '</div>'; //TODO: remove
					$has_data = false; // If we dont add any data (items) to the contact, we will clean up (delete) the contact later
					
					print '<div>contact_id: ' . $contact_id . '</div>';
					
					// Traverse the modules/tables processing new objects
					foreach ($data[$row_num] as $mod => $tables) { 

						// useful for attach_field_priority - used further below
						if ($mod == 'Person') {
							$person_contact_id = $this->getNewContactId();
						}
						else if ($mod == 'Company') {
							$company_contact_id = $this->getNewContactId();
						}
						
						
						if (is_array($tables)) {
							foreach ($tables as $table => $items) { // Standard modules will have only the 'item' table, but this handles multiple tables for a module
								if (is_array($items)) {
									
									$db_functions =& $this->getModuleDatabaseTable($mod, $table);
									
									//if (is_object($db_functions) && method_exists($db_functions, 'setItem')) {
										// $new_items = array();
										// foreach ($items as $k => $v) {
											// $new_items[$k] = $v;
										// }
										
										
										$default_item = $this->getDefaultItem($mod, $table); // Default values will be used for fields that are not specified in this import
										
										print_r($items);
										
										//foreach ($new_items as $new_item_k => $new_item_v) {
										foreach ($items as $k => $v) {
											
											// only if not we're not in the 'Person' module or normal assigning
											if (($mod != 'Person') && ($this->attach_field_priority == 2)) {
												if ($company_contact_id > 0) {
													$v['contact_id'] = $company_contact_id;
												}
												else {
													$v['contact_id'] = $this->getNewContactId(); // The contact_id generated by our call to addContact (above)
												}
											}
											else {
												$v['contact_id'] = $this->getNewContactId(); // The contact_id generated by our call to addContact (above)
											}

											// $item = bcrm_array_merge($v, $default_item); // Default item values do not overwrite any values specified in the import
											$item = bcrm_array_merge($default_item, $v); // Default item values do not overwrite any values specified in the import
											$this->beforeSetItem($item, $mod); // Any processing before we proceed (ie module specific processing)
											
											if (($mod == 'Company') && $this->merge_companies) {
												
												$existing_company = false; // Assume this is not an existing company
												foreach ($this->existing_companies as $ec_k => $ec_v) {
													if ($ec_v['display_name'] == $item['display_name']) {
														$existing_company = $ec_v['contact_id']; // This company matches one we have already imported, store the contact_id for that company
														break;
													}
												}
												
												if ($existing_company) { // We will set this contacts contact_company field to point to the existing company
													// $item_id = $db_functions->setItem($item); // No need to set item - we are setting a pointer to an existing item
													$this->updateContactCompany($existing_company);
													// $has_data = true; // Not here
													// print '<div>existing companies - ' . $existing_company . ' - ' . $this->getNewContactId() . '</div>';	
												} else {
													
													// print '<div>new company for ' . $this->getNewContactId() . '</div>';	
													// Create standalone top-level company contacts
													// There must be a person record, and the standalone option must be set, otherwise we will assign this company directly to the current contact
													if (array_key_exists('Person', $data[$row_num]) && $this->standalone_companies) {
														// This is a new company
														// We need to create a new contact
														// Create the new company
														// assign the Company to the new contact, 
														// get our original contacts contact_company to point to this new contact
														// Update the name for our new Company contact
														
														// // $v['contact_company'] = $this->addNewContactForCompany();
														$new_contact_company = $this->addNewContactForCompany(); // Create new contact
														
														// for attach_field_priority
														$person_contact_id = $new_contact_company;

														$item['contact_id'] = $new_contact_company;
														$item_id = $db_functions->setItem($item); // Create a new Company item
														$this->updateContactCompany($new_contact_company, $new_contact_company); // gassign the Company to the new contact
														$this->updateContactCompany($new_contact_company); // get our original contacts contact_company to point to this new contact
														// we need now to set the contact_person
														if ($this->new_contact_id > 0) {
															$db_functions->setItem(array(
																'id' => $item_id,
																'contact_person' => $this->new_contact_id)
															);
														}
														$this->updateContactDisplayNameFields($new_contact_company); // Update the name for our new Company contact
														
														$has_data = true;
														$count_standalone_company_contacts ++;
														// print '<div>standalone companies - ' . $new_contact_company . ' - ' . $this->getNewContactId() . '</div>';	
														
													} else {
														$item_id = $db_functions->setItem($item); // Create a new Company item
														// print '<div>not standalone companies - ' . $new_contact_company . ' - ' . $this->getNewContactId() . '</div>';	
														if ($item_id) {
															$has_data = true;
															
															// There is no Person - this is a standalone top-level company!
															$count_standalone_company_contacts ++;
															$count_contacts_added--;
															// print '<div>DEBUG: decrement for ' . $item_id . '</div>'; // : remove
														}
													}
													
													// needed for attach_field_priority
													$company_contact_id = (($this->standalone_companies && $new_contact_company) ? $new_contact_company : $this->getNewContactId());
													
													// print '<div>Company id: ' . $item_id . ', new contact company: ' . $new_contact_company . ', ' . $this->getNewContactId() . '</div>';
													
													$this->existing_companies[$item_id] = array(
														'display_name' => $item['display_name'], 
														'contact_id'   => ( ($this->standalone_companies && $new_contact_company) ? $new_contact_company : $this->getNewContactId())
														// 'contact_id'   => $this->getNewContactId()
													);
													
													unset($new_contact_company); // Yes, we need to do this
													
												}
												
											} else {
												$item_id = $db_functions->setItem($item);
												// print '<div>** item id: ' . $item_id . ', mod: ' . $mod . '</div>';

												if ($item_id) {
													$has_data = true;
												}
											}
											unset($item_id);
										}
									//}
								}
							}
						}
					}
					
					if (!$has_data) {
						// print '<DIV>THERE IS NO DATA FOR THIS CONTACT ' . $this->getNewContactId() . '</DIV>';
						$this->removeUnusedContact($this->getNewContactId());
						$count_contacts_added--;
						// print '<div>DEBUG: decrement for unknown (no data)</div>'; //TODO: remove
					} else {
						$this->updateContactDisplayNameFields($contact_id);
					}
				
				}
				// print '<hr>';
				
				
			}
			*/
			
			
			if (is_array($data[$row_num])) { // We have some data from this row to process
				//print '<div>data[' . $row_num . ']:</div>'; print_r($data[$row_num]); print '<hr>'; 
				
				//init
				$contact_data = $company_data = $person_data = false;
				
				// Get Contact, Company, and Person data (if any)
				if (isset($data[$row_num]['Contact']['item'][0])) {
					$contact_data = $data[$row_num]['Contact']['item'][0];
					unset($data[$row_num]['Contact']);
				}
				if (isset($data[$row_num]['Company']['item'][0])) {
					$company_data = $data[$row_num]['Company']['item'][0];
					unset($data[$row_num]['Company']);
				}
				if (isset($data[$row_num]['Person']['item'][0])) {
					$person_data = $data[$row_num]['Person']['item'][0];
					unset($data[$row_num]['Person']);
				} else {
					//@todo  set from a flag - require person
					$person_data = array('display_name' => '[No Name]');
				}
				
				
				//@todo rework
				$attach_to = $this->attach_field_priority;
				if ( ($attach_to == 1) && $person_data) { // Person
					
				} else {
					$attach_to = 2; // Company
				}
				//echo 'attach: ' . $attach_to;
				// echo '<hr>';
				// print_r($contact_data);
				// echo '<hr>';
				
				
				if ($company_data) {
					// Returns existing contact for company or new contact for company
					$company_contact_id = $this->getCompany($company_data, $contact_data); 
				}
				
				if ($person_data) {
					// Returns new contact for person
					$person_contact_id = $this->getPerson($person_data, $contact_data, $company_contact_id); 
				}
				
				if ($company_contact_id) {
					$this->updateContactPerson($company_contact_id);
				}
				
				// print '<div>attach_to =  ' . $attach_to . '</div>';
				if ( ($attach_to == 2) && $company_contact_id) {
					$contact_id = $company_contact_id;
					// print '<div>contact_id = company_contact_id =  ' . $contact_id . '</div>';
				} else if ( ($attach_to == 1) && $person_contact_id) {
					$contact_id = $person_contact_id;
					// print '<div>contact_id = person_contact_id =  ' . $contact_id . '</div>';
				} else {
					$contact_id = $this->addContact($contact_data); // New contact
					$this->count_contacts_added++;
					// print '<div>contact_id = new contact = ' . $contact_id . '</div>';
				}
				
				
				// scenarios:
				// Contains both Company and Person data
				// -- need to determine based on $merge_companies and $attach_field_priority
				
				// Contains only Company OR Person data
				// several cases: 
				// merge_companies is false:
				// -- create 1 contact, assign all data (including company or person) to that contact
				// merge_companies is true and this row contains company data:
				// -- assign all data (including company or person) to exisitng company contact +++ needs work
				
				// Contains only Contact data (eg legacy_id)
				// Contains no Contact, Person, or Company fields (eg import only contains address fields)
				// -- in both cases, a default Contact is created for each row
				
				// Contains multiple Contact, Company or Person records (shouldnt be allowed? - what about a company with multiple sub contacts on one row?)
				// -- multiple Contact items wont work (probably ok) - see below: unset($data[$row_num]['Contact']['item']);
				// -- multiple people not implemented (only 1st person is created)
				
				// $standalone_companies - forced
				// $merge_companies - optional, see also $existing_companies
				// $attach_field_priority - determines who gets extra data (addresses etc), default is 1=person; also 2=company
				
				
				if ($contact_id) {
					
					// print '<div>DEBUG: increment for ' . $contact_id . ' data:</div>'; print_r($contact_data); print '<hr>';//TODO: remove
					$has_data = false; // If we dont add any data (items) to the contact, we will clean up (delete) the contact later
					// print '<div>contact_id: ' . $contact_id . '</div>';
					
					foreach ($data[$row_num] as $mod => $tables) { // Traverse the modules/tables processing new objects
						// print '<div>data[' . $row_num . '] mod: ' . $mod . '</div>';
						
						if (is_array($tables)) {
							foreach ($tables as $table => $items) { // Standard modules will have only the 'item' table, but this handles multiple tables for a module
								// print '<div>table: ' . $table . '</div>';
								if (is_array($items)) {
									$db_functions =& $this->getModuleDatabaseTable($mod, $table);
									if (is_object($db_functions) && method_exists($db_functions, 'setItem')) {
										$new_items = array();
										
										foreach ($items as $item_k => $item_v) {
											$new_items[$item_k] = $item_v;
										}
										
										$default_item = $this->getDefaultItem($mod, $table); // Default values will be used for fields that are not specified in this import
										
										foreach ($new_items as $new_item_k => $new_item_v) {
											// print '<div>new item: ' . $new_item_k . ' attach to  ' . $contact_id . '</div>'; print_r($new_item_v); print '<hr>';
											
											$new_item_v['contact_id'] = $contact_id;

											// $item = bcrm_array_merge($new_item_v, $default_item); // Default item values do not overwrite any values specified in the import
											$item = bcrm_array_merge($default_item, $new_item_v); // Default item values do not overwrite any values specified in the import
											$this->beforeSetItem($item, $mod); // Any processing before we proceed (ie module specific processing)
											
											$item_id = $db_functions->setItem($item);
											// print '<div>returned from ' . get_class($db_functions) . '::setItem item id: ' . $item_id . ', mod: ' . $mod . '</div>'; print_r($item); print '<hr>';

											if ($item_id) {
												$has_data = true;
											}
											
											unset($item_id);
										}
									}
								}
							}
						}
					}
					
					//if (!$has_data) {
						// print '<DIV>THERE IS NO DATA FOR THIS CONTACT ' . $this->getNewContactId() . '</DIV>';
						// $this->removeUnusedContact($this->getNewContactId());
						// $contacts_added--;
						// print '<div>DEBUG: decrement for unknown (no data)</div>'; //TODO: remove
					//} else {
					$this->updateContactDisplayName($contact_id);
					//}
				
				}
				// print '<hr>';
				
				
			}
			
			$row_num++;
			unset($row);
			unset($data); //save memory
		}
		
		unset($this->default_item); // just to save memory
		
		$this->displaySummary($this->count_contacts_added, $this->count_standalone_company_contacts, $this->getImportId());
		
	
	}
	
	function updateContactPerson($contact_id) {
		if ($contact_id) {
			
			if (!is_object($this->crm_module)) {
				global $SERVICES;
				$SERVICES->getService('Modules');
				$this->crm_module =& $SERVICES->modules->getModule('Contact');
			}
			
			$this->crm_module->updateContactPerson($contact_id);
			
		}
	}
	
	
	function displaySummary($count_contacts_added = 0, $count_standalone_company_contacts = 0, $import_id = 0) {
		global $L, $SERVICES;
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('ContactImport');
		
		//		echo 'here: ' . $this->count_contacts_added . ' | ' . $this->count_people_added . ' | ' . $this->count_companies_added . ' | ' 
		//			. $count_standalone_company_contacts;
		
		$content = '';
		$content .= $this->output->getBeginTable('viewTbl');
		
		if ($this->count_people_added || $this->count_companies_added
				 || $count_contacts_added || $count_standalone_company_contacts) {
			
			if ($this->count_people_added) {
				$content .= $this->output->getTextRow($L['contactImportNumPeopleAdded'] . ': ' . $this->count_people_added . '');
			}
			if ($this->count_companies_added) {
				$content .= $this->output->getTextRow($L['contactImportNumCompaniesAdded'] . ': ' . $this->count_companies_added . '');
			}
			
			if ($count_contacts_added) {
				$content .= $this->output->getTextRow($L['contactImportNumContactsAdded'] . ': ' . $count_contacts_added . '');
			}
			if ($count_standalone_company_contacts) {
				$content .= $this->output->getTextRow($L['contactImportNumTopLevelCompaniesAdded'] . ': ' . $count_standalone_company_contacts . '');
			}
			$content .= $this->output->getTextRow($L['contactImportImportId'] . ': ' . $import_id . '');
			$href = $this->output->getLinkHref(
				array(
					'href'=>'../Contact/index.php?import_id=' . $import_id . '&search_filter=off&view_all=1', 
					'text'=>$L['contactImportViewContacts']
				), 
				false
			);
			$content .= $this->output->getTextRow($href);
		} else {
			$content .= $this->output->getTextRow($L['contactImportNoContactsAdded']);
			//quickPage($L['contactImportNoContactsAdded']);
			//exit();
		}
		
		//$content .= $this->output->getFieldSpacerRow();
		$href = $this->output->getLinkHref(
				array(
					'href'=>'index.php', 
					'text'=>$L['contactImportBackToImport']
				), 
				false
			);
		$content .= $this->output->getTextRow($href);
		
		$content .= $this->output->getEndTable();
		
		$this->mod->display($content, $L['contactImportTitleComplete']);
			
		//quickPage($content);
		exit();
	}
	
	// Phil 20060817
	function processField($value, $module, $field_name, $dd_field) {
		// $row[$v], $module, $field, $dds[$module]->data_dictionary[$field])
		// print '<div>value: ' .$value . '<br>module: ' . $module . '<br>field_name: ' . $field_name . '<br>dd_field:</div>'; print_r($dd_field); print '<hr>';
		$field_type = $dd_field['type'];
		switch ($field_type) {
			case 'dropdown':
				if (is_array($dd_field['dropdown_values']) && array_key_exists($value, $dd_field['dropdown_values'])) {
					// print '<div>value: ' . $value . ' exists as a key in dropdown_values for ' . $field_name . ' [no change]</div>';
				} else if (is_array($dd_field['dropdown_values']) && in_array($value, $dd_field['dropdown_values'])) {
					// print '<div>value: ' . $value . ' exists as a value in dropdown_values for ' . $field_name . '</div>';
					foreach ($dd_field['dropdown_values'] as $k => $v) {
						if ($value == $v) {
							// print '<div>value: ' . $value . ' has dropdown_values key ' . $k . '</div>';
							$value = $k;
							
							return $value;
						}
					}
				} else {
					// print '<div>value: ' . $value . ' DOES NOT exist (key or value) in dropdown_values for ' . $field_name . '</div>';
					if (isset($dd_field['_dropdown_table_name']) && $dd_field['_dropdown_table_name']) {
						$value = $this->addDropdownValue($dd_field['_dropdown_table_name'], array('name' => $value)); // returns new key
					}
				}
			break;
			default:
			break;
		}
		
		return $value;
	}
	
	function addDropdownValue($table, $item) {
		global $SERVICES;
		if (is_object($add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition'))) {
			$table = $add->getDropdownTableName($table);
			// print '<div>addDropdownValue for table ' . $table . ' item:</div>'; print_r($item); print '<hr>';
			if (strlen($table)) {
				return $add->setDropdownValue($item, $table, 'name');
			}
		}
		return false;
	}
	
	function getNewContactId() {
		return $this->new_contact_id;
	}
	
	function addContact($item = false) {
		// return new contact id	
		$db_functions =& $this->getModuleDatabaseTable('Contact', 'item');
		
		$default_item = $this->getDefaultItem('Contact', 'item');
		
		$default_item['import_id'] = $this->getImportId();
		
		// print '<p>addContact:</p>'; print_r($item); print '<hr>';
		if ($item && is_array($item)) {
			// $default_item = bcrm_array_merge($item, $default_item);
			$default_item = bcrm_array_merge($default_item, $item);
		}
		
		$this->new_contact_id = $db_functions->setItem($default_item);
		
		if ($this->new_contact_id) {
			$this->updateContactDisplayNameFields($this->new_contact_id);
		}
		
		return $this->getNewContactId();
	}
	
	function updateContactDisplayName($contact_id) {
		if ($contact_id) {
			
			if (!is_object($this->crm_module)) {
				global $SERVICES;
				$SERVICES->getService('Modules');
				$this->crm_module =& $SERVICES->modules->getModule('Contact');
			}
			
			$this->crm_module->updateContactDisplayName($contact_id);
			
		}
	}
	
	function addNewContactForCompany() {
		// return new contact id	
		$db_functions =& $this->getModuleDatabaseTable('Contact', 'item');
		
		$default_item = $this->getDefaultItem('Contact', 'item');
		
		$default_item['import_id'] = $this->getImportId();
		
		$contact_company = $db_functions->setItem($default_item);
		
		return $contact_company;
	}
	
	function getCompany($company_data, $contact_data = false) {
		// print '<div>getCompany</div>';
		
		if ($this->merge_companies) {
			
			$existing_company = false; // Assume this is not an existing company
			if ($company_data['display_name']) {
				//echo 'here';print_r($company_data);
				
				/*
				foreach ($this->existing_companies as $ec_k => $ec_v) {
					if ($ec_v['display_name'] == $company_data['display_name']) {
						$existing_company = $ec_v['contact_id']; // This company matches one we have already imported, store the contact_id for that company
						// print '<div>breaking, existing company found: ' . $existing_company . ' ' . $ec_v['display_name'] . '</div>'; // print_r($ec_v); print '<hr>';
						return $existing_company;
					}
				}
				*/
				
				//search db for a matching company
				//for now, rule 1: based solely on display name
				//$db_functions =& $this->getModuleDatabaseTable('Company', 'item');
				global $SERVICES;
				$db =& $SERVICES->getService('Database');
				if ($id = $db->quickget('id', 'Company__item', "display_name = '" . addSlashes($company_data['display_name']) . "'"))
				{
					//print '<div>breaking, existing company found, name: ' . $company_data['display_name'] . '</div>'; // print_r($ec_v); print '<hr>';
					return $id;
				}
			}
		} 
		
		//new company
		$company_contact_id = $this->addContact($contact_data);
		//$this->count_contacts_added++;
		$this->count_companies_added++;
		
		// --
		// print '<div>Adding Company record for contact ' . $company_contact_id . '</div>';
		$mod = 'Company';
		$table = 'item';
		$db_functions =& $this->getModuleDatabaseTable($mod, $table);
		$default_item = $this->getDefaultItem($mod, $table);
		
		$company_data['contact_id'] = $company_contact_id;
		$item = bcrm_array_merge($default_item, $company_data); // Default item values do not overwrite any values specified in the import
		$this->beforeSetItem($item, $mod); // Any processing before we proceed (ie module specific processing)
		$item_id = $db_functions->setItem($item);
		// print '<div>Added Company record ' . $item_id . '</div>'; print_r($item); print '<hr>';
		// Update the Contact record
		$this->updateContactCompany($company_contact_id, $company_contact_id);
		$this->updateContactDisplayName($company_contact_id);
		
		//		$this->existing_companies[$item_id] = array(
		//			'display_name' => $item['display_name'], 
		//			'contact_id'   => $company_contact_id
		//		);
		
		// --
		return $company_contact_id;
		
	}
	
	function getPerson($person_data, $contact_data = false, $company_contact_id = false) {
		if ($company_contact_id) {
			$contact_data['contact_company'] = $company_contact_id;
			//$contact_data['contact_account'] = $company_contact_id;
		} else {
			$contact_data['contact_company'] = 0;
			//$contact_data['contact_account'] = 0;
		}
		
		//new person
		$person_contact_id = $this->addContact($contact_data);
		//$this->count_contacts_added++;
		$this->count_people_added++;
		
		// --
		// print '<div>Adding Person record.</div>'; print_r($contact_data); print '<hr>';
		$mod = 'Person';
		$table = 'item';
		$db_functions =& $this->getModuleDatabaseTable($mod, $table);
		$default_item = $this->getDefaultItem($mod, $table);
		
		$person_data['contact_id'] = $person_contact_id;
		$item = bcrm_array_merge($default_item, $person_data); // Default item values do not overwrite any values specified in the import
		$this->beforeSetItem($item, $mod); // Any processing before we proceed (ie module specific processing)
		$item_id = $db_functions->setItem($item);
		
		// print '<div>Added Person record ' . $item_id . '</div>'; print_r($item); print '<hr>';
		// Update the Contact record
		$this->updateContactDisplayName($person_contact_id);
		
		// --
		return $person_contact_id;
	}
	
	
	//	function removeUnusedContact($contact_id) {
	//		if ($contact_id) {
	//			$db_functions =& $this->getModuleDatabaseTable('Contact', 'item');
	//			$db_functions->deleteItem($contact_id);
	//		}
	//	}
	
	function updateContactDisplayNameFields($contact_id) {
		if ($contact_id) {
			if (!is_object($this->crm_module)) {
				global $SERVICES;
				$SERVICES->getService('Modules');
				$this->crm_module =& $SERVICES->modules->getModule('Contact');
			}
			$this->crm_module->updateContactDisplayName($contact_id);
			$this->crm_module->updatePostalAddressDisplayName($contact_id);
			$this->crm_module->updatePhysicalAddressDisplayName($contact_id);
		}
	}
	
	// function updateContactDisplayName($contact_id) {
		// if ($contact_id) {
			// if (!is_object($this->crm_module)) {
				// global $SERVICES;
				// $SERVICES->getService('Modules');
				// $this->crm_module =& $SERVICES->modules->getModule('Contact');
			// }
			// $this->crm_module->updateContactDisplayName($contact_id);
		// }
	// }
	
	function updateContactCompany($contact_company, $contact_id = '') {
		
		if (empty($contact_id)) {
			$contact_id = $this->getNewContactId();
		}
		
		if ( $contact_company && ($contact_id) ) {
			if ($contact_db_functions =& $this->getModuleDatabaseTable('Contact', 'item')) {
				$contact_db_functions->setItem(
					array(
						'id'=>$contact_id, 
						'contact_company' => $contact_company
					)
				);
			}
		}
	}
	
	function getImportId() {
		
		if (!isset($this->import_id) || !$this->import_id) {
			srand((double)microtime()*1000000);
			$import_id =  rand(0,1000000);
			$this->import_id = $import_id;
		}
		
		return $this->import_id;
		
		// exit('--getImportId-');
		
	}
	
	function getDefaultItem($module, $table) {
		if (!isset($this->default_item[$module][$table]) && !is_array($this->default_item[$module][$table])) {
			
			$db_functions =& $this->getModuleDatabaseTable($module, $table);
			$dd = new WebMATDataDictionary($db_functions->getDataDefinition());
			$this->default_item[$module][$table] = $dd->getDefaultItem();
		
		}
		
		return $this->default_item[$module][$table];
		
	}
	
	function &getModuleDatabaseTable($module, $table) {
		global $SERVICES;
		
		$SERVICES->getService('Modules');
		if ($mod =& $SERVICES->modules->getModule($module)) {
			$db_functions =& $mod->getDatabaseTable($table);
			if (is_object($db_functions)) {
				return $db_functions;
			}
		}
		return false;
				
	}
	
	function saveMapping() {
		$mapping_to = $this->map->getColMapTo();
		//$mapping_to = $this->form_values['item']['mapping'];
		
		$this->map->setColMapTo($mapping_to);
		$this->map->trashMapping();
		$this->store = new SessionStore(1);
		
	}

	function saveMapping_old() {
		return;
		$mapping_from = $this->map->getColMapFrom();
		
		$mapping_to = $this->map->getColMapTo();
		
		$new_mapping_keys = $this->form_values['item']['mapping'];
		
		$original_mapping_to = $mapping_to;
		foreach ($new_mapping_keys as $k => $v) {
			
			$mapping_to[$k] = $original_mapping_to[$v];
		}
		
		$this->map->setColMapTo($mapping_to);
		$this->map->trashMapping();
		$this->store = new SessionStore(1);
		
	}
	
	function setPageTimeLimit($count = 0) {
		
		$count = (max($count,0)); // make 0 if negative
		// Set the expected mumber of minutes the import will take
		// $num_minutes = round((1.5/1200)*$num_lines, 1);
		$time = (($count+15)*60);
		
		if (!ini_get('safe_mode') ) {
		   @set_time_limit($time);
//		   @ini_set('memory_limit', '64M');
		}		
		
	}
	
}
?>