<?php
/**
 * Load text file into a database
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package ContactImport
 */

/**
 * Loads the text file from form post into the database as a temp table Import__*
 *
 * @package ContactImport
 */
class FileLoader {

	/**
	 * Constructor method
	 * Sets fields to default values
	 */
    function FileLoader(&$inStore, $inFilename = 'none', $inLineTerm = '\r\n', $inFieldTerm = ',', $headRow = 0, $inEnc = '"', $inEsc = '\\', $use_load_data_infile = 0) {
		
    	global $SERVICES;	
			$this->db =& $SERVICES->getService('Database');
			$ua =& $SERVICES->getService('Authentication');
			
			$this->filename = $inFilename;
			
			if(!$inStore){
				$this->store = new SessionStore();
			} else {
				$this->store = $inStore;
			}
			$this->use_load_data_infile = $use_load_data_infile;
			if($inStore) {
				// IF we are passed a store, get the values from there
				$this->line_terminator = $this->store->getValue('line_terminator');
				$this->field_terminator = $this->store->getValue('field_terminator');
				$this->headerRow = $this->store->getValue('header_row');
				$this->enclosed = $this->store->getValue('enclosed');
				$this->escaped = $this->store->getValue('escaped');
			}
			
			if(!$this->line_terminator){
		    $this->line_terminator = $this->stripIfNeeded($inLineTerm);
		    $this->store->addValue('line_terminator', $this->line_terminator);
			}
	    if(!$this->field_terminator){
		    $this->field_terminator = $this->stripIfNeeded($inFieldTerm);
				$this->store->addValue('field_terminator', $this->field_terminator);
			}
			if(!$this->headerRow){
				$this->headerRow = $headRow;
				$this->store->addValue('header_row', $this->headerRow);
			}
			if(!$this->enclosed){
				$this->enclosed = $this->stripIfNeeded($inEnc);
				$this->store->addValue('enclosed', $this->enclosed);
			}
			if(!$this->escaped){
				$this->escaped = $this->stripIfNeeded($inEsc);
				$this->store->addValue('escaped', $this->escaped);
			}
			
			$this->setFileSystemService();
    }
    
    /**
	 * TrashFileLoader method
	 * Saves the current session storage
	 * WARNING: Must be called if the session variables are to be saved
	 */
    function trashFileLoader() {
    	$this->store->trashStore();
    }
		
		/**
		 * Methods to set the required settings
		 * Error checking to make sure non-optional fields are set
		 */
		function setLineTerminator($inLineTerm) {
			if (strlen($inLineTerm) == 0)
			return 'Line terminator field is not optional. Enter a value.';
			  
			$this->line_terminator = $this->stripIfNeeded($inLineTerm);
			$this->store->addValue('line_terminator', $this->line_terminator);
		}
		
		function setFieldTerminator($inFieldTerm) {
		  if (strlen($inFieldTerm) == 0)
		  	return 'Field terminator field is not optional. Enter a value.';
			
			$this->field_terminator = $this->stripIfNeeded($inFieldTerm);
			$this->store->addValue('field_terminator', $this->field_terminator);
		}
		  
		function setHeaderRow($headRow) {
			if($headRow)
			  $this->headerRow = 1;
			else
				$this->headerRow = 0;
				
			$this->store->addValue('header_row', $this->headerRow);
		}
		  
		function setEnclosed($inEnc) {
			$this->enclosed = $this->stripIfNeeded($inEnc);
			$this->store->addValue('enclosed', $this->enclosed);
		}
		
		function setEscaped($inEsc) {
			$this->escaped = $this->stripIfNeeded($inEsc);
			$this->store->addValue('escaped', $this->escaped);
		}
			
		function setFilename($inFilename = 'none'){
			global $L;
			if($inFilename == 'none')
				return $L['contactImportErrorNoImportFileSpecified'];
				
			$this->filename = $inFilename;
		}
		
		function setStore(&$inStore){
			if(!$inStore)
				$inStore = new SessionStore();
				
			$this->store = $inStore;
		}
		
		/**
		 * Creates the FileSystem object and set the directory name
		 */
		function setFileSystemService() {
			global $SERVICES;
			$SERVICES->getService('FileSystem');
			$file = $this->filename;
			$SERVICES->filesystem->setDirectory(dirname($file));
		}
		
		/**
		 * Methods to get the required settings
		 */
		function getLineTerminator() {
			return $this->line_terminator;
		}
		
		function getFieldTerminator() {
		  return $this->field_terminator;
		}
		  
		function getHeaderRow() {
		  return $this->headerRow;
		}
		  
		function getEnclosed() {
			return $this->enclosed;
		}
		
		function getEscaped() {
			return $this->escaped;
		}
			
		function getFilename(){
			return $this->filename;
		}
		
		function getTablePrefix() {
			return 'ContactImport__data';
		}
		
		function getTablename(){
			
			if (strpos($this->table_name, '__') === false) {
				$this->table_name = $this->getTablePrefix() . trim($this->table_name);
			}
			
			return $this->table_name;
		}
		
		function getStore(){
			return $this->store;
		}
	
		/**
		 * RunLoad method
		 * Loads the file into a new table, returns any errors
		 */
		function runLoad($inFilename = NULL){
			
			//We return the error message if we have one	
			// Open the file
			$toReturn = $this->openFile($inFilename);
			if ($toReturn) {
				return $toReturn;
			}
				
			// Open the file
			$toReturn = $this->createTable();
			if ($toReturn) {
				return $toReturn;
			}
				
			// Open the file
			$toReturn = $this->loadData();
			if ($toReturn) {
				return $toReturn;
			}
		}
	
		/**
		 * OpenFile method
		 * Opens the file, and creates the header row as required
		 */
		function openFile($inFilename = 'none'){
			global $SERVICES;
			if($inFilename != 'none'){
				$error = $this->setFilename($inFilename);
				if ($error)
					return $error;
			}
			
			// Phil DEBUG
			// print '<div>File: ' . $this->filename . '</div>';
			// print '<div>base File: ' . basename($this->filename) . '</div>';
			// print '<div>Directory: ' . $SERVICES->filesystem->getDirectory() . '</div>';
			// if (!file_exists($this->filename)) {
				// print '<div>File does not exist: ' . $this->filename . '</div>';
			// }
			// if (!is_readable($this->filename)) {
				// print '<div>File not readable: ' . $this->filename . '</div>';
			// }
			
			// Here we need to open the header row, if there is one.
			// If not, then we need to find out how many columns we have.
			// if (!$SERVICES->filesystem->openFile(basename($this->filename))) {
				// return $SERVICES->filesystem->printError();
			// }
			// Phil 20060623 Use complete filename
			if (!$SERVICES->filesystem->openFile($this->filename)) {
				return $SERVICES->filesystem->printError();
			}
			
			// Convert the line terminator to a character
			$line_term_char = $this->convertToChar($this->line_terminator);
			
			// Convert the field terminator to a character
			$field_term_char = $this->convertToChar($this->field_terminator);
			
			// Read the first line
			$line = $SERVICES->filesystem->getFileLine(8092);
				
			// If we have reached the end of file too soon
			if (feof($SERVICES->filesystem->current_file)) {
				return 'End Of File reached too soon. No data in file. Unable to import.';
			}
			
			$first_line = $this->csvExplode($line);
			
			$cur_header = 0;
			foreach($first_line as $column_header){
				if ($this->headerRow){
				
					// Gurdeep Singh 20050804 | commented the code below, and added a new code to replace " ?,'" with "_", used array to implement this
					//$this->field_headers[$cur_header] = str_replace(' ', '_', $column_header);
					$this->field_headers[$cur_header] = str_replace(Array(' ', '?', '.', '\''), '_', trim($column_header));

					// Check for empty header entries.
					if ($this->field_headers[$cur_header] == '') {

						//return 'Empty header field. File cannot be imported. ';
						$this->field_headers[$cur_header] = 'Field' . "$cur_header";
					}
					$this->field_headers[$cur_header] = '`' . $this->field_headers[$cur_header] . '`';
					$cur_header++;
			
				} else {
					$this->field_headers[$cur_header] = 'Field' . "$cur_header";
					$cur_header++;
				}
			}
			$SERVICES->filesystem->closeFile();
		}
	
		/**
		 * Creates the new table to import the text file into
		 */
		function createTable() {
			// IF the field header are not set, return with error
			if (sizeof($this->field_headers) < 1) {
				return 'Must open file first. Cannot import without headers.';
			}
			// SET the tablename before creating the table
			$this->setTablename();
			
			// Construct the query
			$sql_query = '';
			
			// Now add the user colunms 
			foreach($this->field_headers as $header_name) {
				$sql_query .= "$header_name LONGTEXT, ";		
			}
				
			$sql_query = trim(trim($sql_query, ' '), ',');
			
			// Builds the 'create table' statement
			// $sql_query = 'CREATE TABLE ' . $this->table_name . ' (' . $sql_query . ')';
			$sql_query = 'CREATE TABLE ' . $this->getTableName() . ' (' . $sql_query . ')';

			// Executes the query
			$result = $this->db->sqlQuery($sql_query);
			
			// Return with an error message if necessary		
			if($this->db->hasError()){
				$error_message = $this->db->getError();
				return $error_message['str'];
			} else {
				$this->db->rebuildCache();	
			}
		}
		
		
		function loadData() {
			if ($this->use_load_data_infile) {
				$this->loadDataDirect();
				if ($this->db->hasError()) {  //MCR 20060710
					$this->loadDataByLine();	//MCR 20060710
				}						//MCR 20060710
			} else {
				$this->loadDataByLine();
			}
			global $SERVICES;
			$SERVICES->filesystem->collectGarbage(basename($this->filename));
		}
		
		/**
		 * Open a file and adds the data into the datab
		 * 
		 * @return string an error message on fail
		 */
		function loadDataByLine() {
			global $SERVICES;
			// if (!$SERVICES->filesystem->openFile(basename($this->filename))) {
				// return $SERVICES->filesystem->printError();
			// }
			// Phil 20060624 Use complete filename
			if (!$SERVICES->filesystem->openFile($this->filename)) {
				return $SERVICES->filesystem->printError();
			}
			if(sizeof($this->getTableName()) < 1) {
				return 'Must select a table name first. Cannot import without name.';
			}
			$count = 0;
			while (!feof($SERVICES->filesystem->current_file)) {
				if ($this->headerRow && ($count == 0)) {
					$SERVICES->filesystem->getFileLine(8092);
				} else {
					$item = array();
					foreach ($this->csvExplode($SERVICES->filesystem->getFileLine(8092)) as $k => $v) {
						if ($this->field_headers[$k]) {
							$item[$this->field_headers[$k]] = addslashes($v);
						}
					}
					$this->db->insert($this->getTableName(), $item);
					if ($this->db->hasError()) {
						$this->db->printError();
					}
				}
				

				$count++;
			}
			
			$SERVICES->filesystem->closeFile();
			
		}
		
		/**
		 * Explodes a csv line into an array
		 * 
		 * @param string $str_buffer the line from the file
		 * @return array the line form a file phased as an array
		 */
		function csvExplode($str) {
			$delim = $this->field_terminator;
			$qual = $this->enclosed;
			$esc = $this->escaped;
			$term_char = $this->convertToChar($this->line_terminator);
			
			$str = trim(rtrim($str), $term_char);
			
			if ($esc != '\\') {
				$str = str_replace(array('\\', $esc), array('\\\\', '\\'), $str);
			}
			
			$skipchars = array( $qual, "\\" );
			$len = strlen($str);
			$inside = false;
			$word = '';
			for ($i = 0; $i < $len; ++$i) {
				if ($str[$i]==$delim && !$inside) {
					$out[] = $word;
					$word = '';
				} else if ($inside && in_array($str[$i], $skipchars) && ($i<$len && $str[$i+1]==$qual)) {
					$word .= $qual;
					++$i;
				} else if ($str[$i] == $qual) {
					$inside = !$inside;
				} else {
					$word .= $str[$i];
				}
			}
			$out[] = $word;
			return $out;
		}
		
		
		/**
		 * This function checks and builds the sql-string for loading the data
		 * LOAD DATA INFILE 'file_name.txt' [REPLACE | IGNORE] INTO TABLE table_name
		 *    [FIELDS
		 *        [TERMINATED BY '\t']
		 *        [OPTIONALLY] ENCLOSED BY "]
		 *        [ESCAPED BY '\\' ]]
		 *    [LINES TERMINATED BY '\n']
		 *    [(column_name,...)]
		 */
		function loadDataDirect(){
			// IF the table name are not set, return with error
			// if(sizeof($this->table_name) < 1)
			if(sizeof($this->getTableName()) < 1)
				return 'Must select a table name first. Cannot import without name.';
			
			$line_term = $this->line_terminator;
			$field_term = $this->field_terminator;
			$enc = $this->enclosed;
			$esc = $this->escaped;
    
			// Formats the data posted to this script
			// Just changes '\' to '\\'
			if ($line_term == "\\") $line_term = "\\"."\\";
			if ($field_term == "\\") $field_term = "\\"."\\";
			if ($enc == "\\") $enc = "\\"."\\";
			if ($esc == "\\") $esc = "\\"."\\";
			
			//MCR20060710
			if ($enc == "'") $enc = "\\"."'";
			if ($esc == "'") $esc = "\\"."'";
			
			
			// echo '<p>' . $this->filename . '</p>';
			// Windows
			// <p>C:/my_webs/browsercrmdev/data/tmp\tes14E.tmp</p>
			// File 'C:\my_webs\browsercrmdev\data\tmp	es14E.tmp' not found (Errcode: 22)
			$this->filename = str_replace('\\', '/', $this->filename);
			
			// Builds the query
			$import_query = "LOAD DATA INFILE '" . $this->filename . "'";
			// $import_query .= " INTO TABLE $this->table_name";
			$import_query .= " INTO TABLE " . $this->getTableName() . "";
			   
			if (strlen($field_term) > 0)
				$import_query .= " FIELDS TERMINATED BY '$field_term'";
			    
			if (strlen($enc) > 0)
				$import_query .= " ENCLOSED BY '$enc'";
			    
			if (strlen($esc) > 0)
				$import_query .= " ESCAPED BY '$esc'";
			    
			if (strlen($line_term) > 0)
				$import_query .= " LINES TERMINATED BY '$line_term'";
			
			// If we have a header row
			// Specify to ignore the first column
			if($this->headerRow)		
				$import_query .= ' IGNORE 1 LINES';

			//exit( $import_query);
			
			// Run the query to input the data into the table
			$result = $this->db->sqlQuery($import_query);
			
			// Error handling
			if($this->db->hasError()){
				$error_message = $this->db->getError();
				return $error_message['str'];
			}
		}
		
		/**
		 * Private helper method
		 * Converts a string representation of special characters
		 * into a single character representation
		 */
		function convertToChar($input){
			// \r\n == Windows Newline.
			if ($input == '\r\n')
				return "\r\n";
			// \n == Newline.
			if ($input == '\n')
				return "\n";
			// \r == Carriage return.
			if ($input == '\r')
				return "\r";
			// \t == Tab Character.
			if ($input == '\t')
				return "\t";
			// \\ == Backslash.
			if ($input == '\\')
				return "\\";
			// \" == Double Quote.
			if ($input == '\"')
				return "\"";
				
			return $input;
		}
	
		/**
		 * Private helper method
		 * Strips the escape character from input if necessary
		 * Returns the unchanged input otherwise
		 */
		function stripIfNeeded($input){
			if (strpos($input, '\\'.'\\') !== false)
				$input = stripslashes($input);
			
			if($input == '\"')
				$input = '"';
			
			return $input;
		}
		
		/**
		 * Private helper method
		 * Method to set a unique table name
		 */
		function setTablename(){
			
			global $USER, $ua;
			
			$query = 'SHOW TABLES LIKE "' . $this->getTablePrefix() . '%"';
			
			// Gets tables in Import database
			$result = $this->db->sqlQuery($query);
			
			if ($this->db->hasError()) {
				$this->db->getError();
			}
			
			// Get full db name
			// $db_index_name = 'Tables_in_' . $this->db->database;
			
			$results = array();
			while ($row = $this->db->getRow($result)) {
				//$results[] = $row[$db_index_name];
				$results[] = current($row); // Phil
			}
			
			// print '<div>query: ' . $query . '</div>';
			// print '<div>table_name: ' . $this->getTableName() . '</div>';
			// print_r($results);
			
			$i = 1;
			do {
				
				$this->table_name = $name.$i;
				$i++;
			
			} while (in_array($this->getTableName(), $results) || in_array(strtolower($this->getTableName()), $results));   //MCR strtolower for some Windows installations
					
			$this->store->addValue('table_name', $this->getTableName());
			$this->store->trashStore();
		}
		
}

?>