<?php
/**
 * The view page
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package ContactImport
 * @author Phil Calder
 */
class ContactImportMapWebMATView extends WebMATView {

	var $showed_num_rows = false;
	var $rows_display;

	function run() {
		$this->initMapping();
	}


	function getTableRow($field) {
		$content = '';
		if (!$this->showed_num_rows) {
			global $L;

			$content = $this->output->getTableRow(array($L['contactImportLabelNumberOfRecords'], $this->getNumLines()));
			$this->showed_num_rows = true;
		}
		return $content;
	}

	function getNumLines() {
		$num = $this->db->quickget(
			'count(*)',
			$this->map->table_name
		);
		return $num;
	}

	function initMapping() {
		global $rows_display;
		$this->return_string = true;

		global $SERVICES;
		$this->db =& $SERVICES->getService('Database');

		// Get the stored variables
		$this->store = new SessionStore(1);
		// Get our mapping
		$this->map = new Mapping($this->store);

		// print_r($this->store); print '<hr>'; print_r($this->map);
		/**
		 * Now we load any variables that we need to
		 * If the load fails, we construct the variables, and store them
		 */
		// Load the mapping from and to
		$mapping_from = $this->map->getColMapFrom();
		if(!isset($mapping_from) || sizeof($mapping_from) < 1){
			$errors_list[] = $this->map->loadMapFrom();
			$this->map->trashMapping();
			$this->store = new SessionStore(1);
			$mapping_from = $this->map->getColMapFrom();
		}
		// print_r($mapping_from);
		// print_r($errors_list);

		$mappingTo = $this->map->getColMapTo();
		if(!isset($mappingTo) || sizeof($mappingTo) < 1){
			$errors_list[] = $this->map->loadMapTo();
			$this->map->trashMapping();
			$this->store = new SessionStore(1);
			$mappingTo = $this->map->getColMapTo();
		}

		// Load our list of recognised date styles
		$date_styles = $this->store->getValue('date_styles');
		if(!isset($date_styles) || sizeof($date_styles) < 1){
			$date_styles = array();
			$date_styles['d/m/Y'] = '01 12 2003  (d/m/Y)';
			$date_styles['j/m/y'] = '1 12 2003  (j/m/Y)';
			$date_styles['m/d/Y'] = '12 01 2003  (m/d/Y)';
			$date_styles['m/j/Y'] = '12 1 2003  (m/j/Y)';

			$date_styles['d/m/y'] = '01 12 03  (d/m/y)';
			$date_styles['j/m/y'] = '1 12 03  (j/m/y)';
			$date_styles['m/d/y'] = '12 01 03  (m/d/y)';
			$date_styles['m/j/y'] = '12 1 03  (m/j/y)';

			$date_styles['Y/m/d'] = '2003 12 01  (Y/m/d)';
			$date_styles['Unix Timestamp'] = 'Unix Timestamp';

			$this->store->addValue('date_styles', $date_styles);
		}

		// Select the columns to hide
		$not_hidden = $this->store->getValue('not_hidden');
		if(!isset($not_hidden) || sizeof($not_hidden) < 1){
			$not_hidden = array();

			foreach($mapping_from as $col){
				$num = $this->db->quickget(
					'count(*)',
					$this->map->table_name,
					"`$col`<>'' and `$col` is not null"
				);
				if($num == 0){
					$not_hidden[$col] = 0;
				} else {
					$not_hidden[$col] = 1;
				}
			}
			$this->store->addValue('not_hidden', $not_hidden);
		}

		// Gurdeep Singh [20050816] Added the code below to get total rows in the CSV file
		$import_table = $this->map->getTableName();
		$num_items = $this->db->quickGet('count(*)', $import_table);

		// Gurdeep Singh [20050816] Added the code to check the value for the form item Show all rows. If selected then set $rows_display_all variable to total rows
		if (isset($this->form_values['item']['show_all_rows'])) {
			$rows_display_all = $num_items;
		}

		// Gurdeep Sing [20050816] Added the code to check the value for hide_empty_cols. If selected then set the value for $hide_empty to 1
		if (isset($this->form_values['item']['hide_empty_cols'])) {
			$hide_empty = 1;
		}

		// Gurdeep Singh [20050816] commented the code below
		/**
		 * Now we change our viewing settings given the input
		 */
/*		if(isset($submit_refresh)){
			if(!isset($rows_display) && !isset($rows_display_all))
				$rows_display = 5;

			if(!isset($hide_empty))
				$hide_empty = 0;
		}
*/
		// Gurdeep Singh [20050816] Added the code below to get the user entry for number of rows to be displayed and set $rows_display variable to the user entered value
		if (isset($this->form_values['item']['num_rows']) && $this->form_values['item']['num_rows'] != '' && !isset($this->form_values['item']['show_all_rows'])) {
			$rows_display = $this->form_values['item']['num_rows'];

			$this->store->addValue('rows_display', $rows_display);
		}

		// Gurdeep Singh [20050816] Modified the code to check, if $rows_display ids isset then use it to display number of rows, else display 5 rows by default
		if(isset($rows_display) && $rows_display != ''){
			$this->store->addValue('rows_display', $rows_display);
		} else {
			$this->store->addValue('rows_display', 5);
		}

		// Gurdeep Sing [20050816] Modified the code to check the value for show_all_rows field in the form, if the checkbox is selected then use the value to display all the rows
		if(isset($this->form_values['item']['show_all_rows'])) {
			$this->store->addValue('rows_display', $rows_display_all);
		}

		// Gurdeep Singh [20050816] Modified the code to hide empty columns
		if(isset($hide_empty) && $hide_empty > 0){
			$import_table = $this->db->quickGet('*', $import_table);
			$this->store->addValue('hide_empty', $hide_empty);
		} else {
			$hide_empty = $this->store->getValue('hide_empty');
		}

		// Save the new stored variables
		$this->store->trashStore();
		$this->store = new SessionStore(1);

		// print '<div>store:</div>'; print_r($this->store); print '<hr>';

		$this->map = new Mapping($this->store);

		// If we have a mapping, we must check if we are using a custom field
		// Otherwise we should save it anyway
		if (isset($mapping)) {

			// Get the old mapping
			if (!isset($back_button)) {
				$old_mapping = $this->map->getMapping();
			}

			// Set the new mapping
			$this->map->setMapping($mapping);

			$current_address = array();
			$cur_address_types = array();
			$cur_address_num = 1;
			$no_addresses = true;

			// print '<div>mapping:</div>'; print_r($mapping); print '<hr>';

			// Loop through the mapping, to check a couple of things
			foreach ($mapping as $key => $value) {

				// If this is an address field, record this fact
				$array_value = explode('_', $value);
				if(sizeof($array_value) > 1 && $array_value[0] == 'a'){
					// IF we have seen this address field before
					if(in_array($value, $cur_address_types)){
						$cur_address_types = array();
						$cur_address_num++;
					}

					// Otherwise just add the relevant data to our arrays
					$cur_address_types[] = $value;
					$current_address[$key] = $cur_address_num;
					$no_addresses = false;
				}

				// If a Custom field has been selected to map from
				// We must run the create_custom script
				if(strstr($value, "Custom: ")){
					$this->map->trashMapping();
					$value = explode(": ", $value);
					header("location:create_custom.php?mapping=mapping&mapping_from=$key&type=" . $value[1]);
					exit();
				}
			}
			// If we have no adresses, set it up for this
			if ($no_addresses && isset($submit_get_labels)) {
				unset($submit_get_labels);
				$submit_unchecked = true;
			}

		} else {
			$mapping = $this->map->getMapping();
		}

		// If we have a SetValues Mapping, we must store it
		if (isset($setValMapping)) {

			// First we must add any new id mappings
			if (isset($setValMapping_New)) {
				foreach($setValMapping_New as $key => $value) {
					if ($value && $value != '') {
						$setValMapping[$key] = $value;
					}
				}
			}

			// Set the new value mapping
			$this->map->setValueMapping($setValMapping);

			// If given id field values have been selected
			foreach($setValMapping as $key => $value){

				// We must run the create_custom script
				if(strstr($value, "New ")){
					$this->map->trashMapping();
					$value = explode(" ", $value);
					header("location:create_custom.php?mapping=set_map&mapping_from=$key&type=" . $value[1]);
					exit();
				}
			}
		}

		// if we have a date standard/seperator, we must store it
		if(isset($date_standard))
			$this->map->setDateFormat($date_standard);

		if(isset($date_seperator))
			$this->map->setDateSeperator($date_seperator);

		// If we have some address labels, store them
		if(isset($address_labels)){
			$this->map->setAddressLabels($address_labels);
		} else {
			$address_labels = $this->map->getAddressLabels();
		}

		// If the reset button has been pressed
		// Reset our mappings, and our date standard
		if($reset_button){
			$this->map->setMapping(array());
			$this->map->setDateFormat('dd mm yy');
			$this->map->setDateSeperator('/');
		}

		// Save our values, if necessary
		$this->map->trashMapping();

	}

	function getMappingFieldSelector($index, $value = '') {
		$mapping_to = $this->map->getColMapTo();
		$fields = new Fields();
		$dropdown_values = $fields->getFieldDropdownValues();

		$field = array(
			'label'=>'',
			'field_key'=>'mapping][' . $index,
			'type'=>'dropdown',
			'dropdown_values'=>$dropdown_values, // $mapping_to,
			'value' => $value,
			'notnull'=>0
			// 'value'=>$index // Phil - make the user select the value to map to - dont try to 'guess'
		);

		return $this->output->getHtmlField($field);

	}

	function getImportMappingSelectorTable() {
		global $L;

		$content = '';
		$content .= $this->output->getBeginTable('bodyTbl');
		$content .= '<tbody id="data">';

		$mapping_from = $this->map->getColMapFrom();
		// labels
		$labels = array();
		foreach ($mapping_from as $k => $v) {
			$labels[$k] = $v;
		}

		$import_table = $this->map->getTableName();

		// print_r($items);
		// Phil 20060624 header labels are now arrays (Output requires this) with keys
		$header_labels = array();
		$header_labels['_label'] = array('label'=> $L['contactImportLabelLabel']);
		$header_labels['_field'] = array('label'=> $L['contactImportFieldLabel']);

		$table_fields = $this->db->getColumns($import_table);
		$num_items = $this->db->quickGet('count(*)', $import_table);

		$num_rows_preview = $this->store->getValue('rows_display');
		$limit = '';
		if($num_rows_preview != $L['importShowAllRows']) {
			$limit = 'limit 0,' . $num_rows_preview;
		} else {
			$num_rows_preview = $this->store->getValue('rows_display');
			$num_rows_preview = 100; // the maximum we will display
		}

		for($i=1; ($i <= $num_items) && ($i <= $num_rows_preview); $i++) {
			$header_labels[$i] = array('label'=>$L['contactImportItemLabel'] . ' ' . $i);
		}
		$content .= $this->output->getHeaderRow($header_labels);

		$mapping_from = $this->map->getColMapFrom();
		$result = $this->db->select("external_name, crm_field_name", "ContactImport__field_mapping", "template_id =".$this->form_values['id'], "id");

		while($row = $this->db->getRow($result)) {
			$external_name_values[$row['external_name']] = $row['external_name'];
			$crm_field_name_values[$row['external_name']] = $row['crm_field_name'];
		}

		foreach ($labels as $k => $v) {
			$row = array();
			$row_data = array();
			$row_field_selector = array();
			$row_header = array();
			// label
			$row_header[] = $v;
			if (!empty($external_name_values) && array_search($v, $external_name_values)) {
				// mapping to field (dropdown
				$row_field_selector[] = $this->getMappingFieldSelector($k, $crm_field_name_values[$v]);
			} else {
				$row_field_selector[] = $this->getMappingFieldSelector($k);
			}

			$result = $this->db->select($v, $import_table, '', '', $limit);
			if (isset($this->form_values['item']['hide_empty_cols'])) {
				$hide_empty = 1;
			}

			while ($db_row = $this->db->getRow($result)) {
				$row_data[] = current($db_row);
			}

			// Gurdeep Singh [20050818] Added the code to remove empty rows
			if ($hide_empty == 1) {
				for ($i=0; $i<count($row_data); $i++) {
					if ($row_data[$i] != "") {
						if (!empty($row_data)) {
							$row = array_merge($row_header, $row_field_selector, $row_data);
							$content .= $this->output->getTableRow($row);
						}
						break;
					} else {
						continue;
					}
				}
			} else {
				$row = array_merge($row_header, $row_field_selector, $row_data);
				$content .= $this->output->getTableRow($row);
			}
		}
		$content .= '</tbody>';
		$content .= $this->output->getEndTable();

		return $content;
	}

	function getImportMappingContentTable() {
		global $L;

		$rows_display = $this->store->getValue('rows_display');

		$content = '';
		$submit_refresh_url = '';
		
		$content .= $this->output->getBeginForm($submit_refresh_url);
		$content .= $this->output->getBeginTable('viewTbl');
		
		if (isset($this->form_values['item']['num_rows']) && $this->form_values['item']['num_rows'] != "") {
			$number_of_rows_to_display = $this->form_values['item']['num_rows'];
		} else {
			$this->form_values['item']['num_rows'] = 5;
		}

		$import_table = $this->map->getTableName();
		$num_items = $this->db->quickGet('count(*)', $import_table);

		$content .= $this->output->getHtmlFieldRow(
			array(
				'label'=> $L['contactImportNumRows'],
				'field_key'=> 'num_rows',
				'type'=> 'free',
				'value'=> ($num_items ? $num_items : '0')
			)
		);
		// Number of rows to be  displayed
		$content .= $this->output->getHtmlFieldRow(
			array(
				'label'=> $L['contactImportNumRowsToDisplay'],
				'field_key'=> 'num_rows',
				'type'=> 'text',
				'value'=> $rows_display
			)
		);
		// show all rows
		$content .= $this->output->getHtmlFieldRow(
			array(
				'label'=> $L['contactImportShowAllRows'],
				'field_key'=> 'show_all_rows',
				'type'=> 'checkbox',
				'value'=> $this->form_values['item']['show_all_rows']
			)
		);
		// hide empty columns
		$content .= $this->output->getHtmlFieldRow(
			array(
				'label'=> $L['contactImportHideEmptyColumns'],
				'field_key'=> 'hide_empty_cols',
				'type'=> 'checkbox',
				'value'=> $this->form_values['item']['hide_empty_cols']
			)
		);
		
		// Gurdeep Singh [20050816] code to make the submit button
		$content .= $this->output->getSubmitRow();
		$content .= $this->output->getFieldSpacerRow();
		
		
		$content .= $this->output->getEndTable();
		$content .= $this->output->getEndForm();
		
		// import mapping table
		$submit_url = 'save_mapping.php?id='.$this->form_values['id'];
		$content .= $this->output->getBeginForm($submit_url);
		
		$content .= $this->getImportMappingSelectorTable();
		
		$content .= '<br />';
		$content .= $this->output->getBeginTable('viewTbl');
		
		// Load our list of recognised date styles
		$date_styles = $this->store->getValue('date_styles');
		$field = array(
			'label'=>$L['contactImportDateFormat'],
			'field_key'=>'date_format',
			'type'=>'dropdown',
			'dropdown_values'=>$date_styles, // $mapping_to,
			'value' => 'Y/m/d',
			'notnull'=> 0
			// 'value'=>$index // Phil - make the user select the value to map to - dont try to 'guess'
		);
		$content .= $this->output->getHtmlFieldRow($field);

		$content .= $this->output->getHtmlFieldRow(
			array(
				'label'=> $L['contactImportDateSeparator'],
				'field_key'=> 'date_separator',
				'type'=> 'text',
				'value' => '/',
				'size' => 8
			)
		);
		$content .= $this->output->getHtmlFieldRow(
			array(
				'label' => '',
				'type'  => 'hidden',
				'field_key' => 'id',
				'value' => $this->form_values['id']
			)
		);
		$content .= $this->output->getFieldSpacerRow();
		$content .= $this->output->getSubmitRow();
		
		$content .= $this->output->getEndTable();
		$content .= $this->output->getEndForm();
		
		return $content;
	}

	function main($return_string = false) {
		global $L;
		//$content = parent::main(1); // return content as a string
		$content .= $this->getImportMappingContentTable();

		if (!$return_string) {
			//$this->output->addToBody($content);
			//$this->output->display();
			$this->mod->display($content, $L['contactImportTitleMapFields']);
			return true;
		} else {
			return $content;
		}

	}
	
	

}
?>