<?php
/**
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Documents
 * @author Matthew Corich
 */

// 20050627 Senthil Change class name DocumentsSelector to DocumentsItemWebMATSelector for Attach Document

class DocumentsItemWebMATSelector extends WebMATSelector {
	/**
	 * @var array the version items for each of the tiems
	 * @access private
	 */
	var $version_items = array();
	
	/**
	 * @var array the version items for each of the tiems
	 * @access private
	 */
	var $mime_items = array();


	/**
	 * init function sets all the class variables
	 * 
	 * CHANGED removes the extra footer information
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->output->setFooterBlockSwitch(false);

		// since this is just a selector, we don't want the config stuff.
		$this->output->req_config_js = false;
	}


	/**
	 *
	 */
	function getTitleRow() {
		return '';
	}


	/**
	 * creates the selector link
	 */
	function buildSelectorLink($id, $field, $item) {

		$version_id = $item['version_id'];
		$mime_id = $item['mime_type_id'];
		// build the filename
		$filename = $this->version_items[$version_id]['file_name'];
		if ($this->version_items[$version_id]['file_extension'] != '') {
			$filename .= '.'. $this->version_items[$version_id]['file_extension'];
		}

		$size = $this->version_items[$version_id]['size'];
		$type = $this->mime_items[$mime_id]['name'];

		//parent function 
		$text = $this->getStringField($field);

		$href = 'JAVASCRIPT: setReturn(\'' . $version_id . '\', \'' . addslashes($filename) . '\', \'' .$size . '\', \'' . $type . '\'); window.close()';

		return $this->output->getLinkHref( array('href' => $href, 'text' => $text) );
	}


	/**
	 * Overrides the parent function so we can use afterGetItems
	 *
	 * @return boolean true or false (default is true)
	 */
	function useGetItemsIterator() {
		return false;
	}


	/**
	 *
	 */
	function afterGetItems(&$items) {

		if (count($items)) {
			$version_ids = array();
			foreach ($items as $k => $v) {				
				$version_ids[] = $v['version_id'];
				$mime_ids[] = $v['mime_type_id'];
			}
			$version_db_functions =& $this->mod->getDatabaseTable('version');
			$dtf = array(
				'id' => array('where' => 'id IN (' . implode(',', $version_ids) . ')')
			);
			$version_db_functions->setDesignTimeFilters($dtf);
			$this->version_items = $version_db_functions->getItems();

			$mime_db_functions =& $this->mod->getDatabaseTable('mime_type');
			$dtf = array(
				'id' => array('where' => 'id IN (' . implode(',', $mime_ids) . ')')
			);
			$mime_db_functions->setDesignTimeFilters($dtf);
			$this->mime_items = $mime_db_functions->getItems();
		}
	}


	/**
	 *
	 */
	function getJSSetReturn() {
		$script = '
		function setReturn(id, name, size, type) {
			parent.document.forms["mainform"].elements["item[version_id]"].value = id;
			parent.document.forms["mainform"].elements["item[filename]"].value = name;
			parent.document.forms["mainform"].elements["item[size]"].value = size;
			parent.document.forms["mainform"].elements["item[type]"].value = type;
			parent.document.forms["mainform"].submit();
		}';
		
		return $script;
	}


	/**
	 *
	 */
	function getJSChangeLink() {
		return '';
	}


	/**
	 * Bosco 20050817 | Overridden to remove Config Filter Links after selecting Document 
	 */
	function getTableLinks($item) {
		return array();
	}
}

?>