<?php
/**
 * The main class for this module
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Documents
 */

includeModule('Documents', 'inc/database_tables.inc');

class Documents extends WebMATModule {
	/**
	 * @var array $browser_mime_types browser viewable mime_type ids
	 */
	var $browser_mime_types = false;
	
	function Documents() {
		$this->module_name = 'Documents';
		$this->WebMATModule();
	}
	
	
	/**
	 * Gets the mime_type_id if exists or creates a new mime type
	 * 
	 * @param string $mime_type the mime type
	 * @param string $file_name the file name
	 * @return integer the mime_type_id
	 */
	function getMimeType($mime_type, $file_name) {
		global $SERVICES;
		$SERVICES->getService('Config');
		$dropdown_config = $SERVICES->config->lookup('modules.Admin.modulelink.documents_mime_type');
		$dropdown = new WebMATDropDown($dropdown_config['module'], $dropdown_config['table'], $dropdown_config['field']);
		$mime_types = $dropdown->getValues();
		if ($id = array_search($mime_type, $mime_types)) {
			return $id;
		} else {
			$item = array();
			$item['mime_type_category_id'] = $this->getMimeTypeCategory($mime_type);
			$item['priority'] = 100;
			$item['active'] = 1;
			if ($mime_type) {
				$item['name'] = $mime_type;
			} else {
				if ($id = array_search($item['extention'], $mime_types)) {
					return $id;
				} else {
					$item['name'] = $item['extention'];
				}
			}
			return $dropdown->setValue($item);
		}
	}
	
	
	/**
	 * Gets the mime_type category if exists or creates a new mime type category
	 * 
	 * @param string $mime_type the mime type
	 * @return integer the mime_type_category_id
	 */
	function getMimeTypeCategory($mime_type) {
		global $SERVICES;
		$main_type = substr($mime_type, 0, strpos($mime_type, '/'));
		$SERVICES->getService('Config');
		$dropdown_config = $SERVICES->config->lookup('modules.Admin.modulelink.documents_mime_type_category');
		$dropdown = new WebMATDropDown($dropdown_config['module'], $dropdown_config['table'], 'main_type');
		$main_types = $dropdown->getValues();
		if ($id = array_search($main_type, $main_types)) {
			return $id;
		} else {
			$item = array();
			$item['priority'] = 100;
			$item['active'] = 1;
			if ($mime_type) {
				$item['name'] = $main_type;
				$item['main_type'] = $main_type;
			} else {
				if ($id = array_search('other', $main_types)) {
					return $id;
				} else {
					$item['name'] = 'other';
				$item['main_type'] = 'unknowen';
				}
			}
			return $dropdown->setValue($item);
		}
	}
	
	
	/**
	 * checks if a mime type is browser viewable
	 * 
	 * @param $mime_type_id the mime type id
	 * @return boolean TRUE if browser viewable or else FALSE
	 */
	function isBrowserViewable($mime_type_id) {
		if (!$this->browser_mime_types) {
			$this->browser_mime_types = array();
			global $SERVICES;
			$db =& $SERVICES->getService('Database');
			$fields = 'Documents__mime_type.id';
			$tables = 'Documents__mime_type_category, Documents__mime_type';
			$where = 'Documents__mime_type.mime_type_category_id = ' .
				'Documents__mime_type_category.id AND view_in_browser =1';
			$result = $db->select($fields, $tables, $where);
			while ($row = $db->getRow($result)) {
				$this->browser_mime_types[] = $row['id'];
			}
		}
		return in_array($mime_type_id, $this->browser_mime_types);
	}
}
?>