<?php
/**
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Documents
 */

/**
 * Display document summary and version list
 */
class DocumentsVersionWebMATList extends WebMATList {
	/**
	 * formats the size of the file
	 * 
	 * @param array $dd_item the items with dd structure
	 */
	function calculateValues(& $dd_item, $item) {
		$size = $item['size'];
		$type = 'bytes';
		switch ($size) {
			case ($size < 1024):
			break;
			case ($size > 1024):
			$size = round($size/1024);
			$type = 'KBs';
			case ($size < 1024):
			break;
			case ($size > 1024):
			$size = round($size/1024);
			$type = 'MBs';
		}
		$dd_item['size']['value'] = $size . '&nbsp;' . $type; 
	}
	
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->markParentSeen();
	}
	
	function markParentSeen() {
		$parent_db_functions =& $this->db_functions->getParentDbFunctions();
		$parent_db_functions->setItemSeenStatus($this->form_values['parent_id'], 1);
	}
	
	/**
	 * Changed to add the parent id to the query_string
	 * 
	 * @param integer $contact_id unused
	 * @return string the new link
	 */
	function getNewLink($contact_id = '') {
		if ( count($this->db_functions->getEditableParents()) ) {
			global $L;
			$link_href = str_replace('%parent_id%',$this->form_values['parent_id'], $this->getFileSetHref('new'));
			return $this->output->getLinkHref( array('href' => $link_href, 'text' => $L['documentsLinkUpdate']) );
		}
		return '';
	}
	
	/**
	 * Builds the edit link
	 *
	 * Uses 'custom' edit link text (Edit Properties)
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getEditLink($id = false) {
		global $L;
		$link_href = $this->getFileSetHref($this->edit_link);
		return $this->output->getLinkEdit($this->getModulePath() . $link_href, $L['documentsLabelEdit']);
	}
	
	function editTableLinks(& $links, $id, $item) {
	   	//before this line, the parent_items is holding the first item from item table;
		//which is causing the problem on getParentItems with different master_ids 
		$this->db_functions->parent_items = null;  //this needs to be considered..
		
	    //before getParentItems 
		$parent_item = $this->db_functions->getParentItems();
		//print_r($parent_item); echo '<hr>';print_r($parent_item['mime_type_id']);
		if (!$this->mod->isBrowserViewable($item['mime_type_id'])) {
			$links['documentsViewDocument'] = '';
		}
		if (!count($this->db_functions->getEditableParents())) {
			bcrm_unset($links, 'edit');
			bcrm_unset($links, 'delete');
		} else {
			if ($parent_item['version_id'] == $id) {
				$links['delete'] = '';
			}
		}
		
	}
	
	
	/**
	 * Used to display the document information
	 * 
	 * @return a sub table containing the document summary
	 */
	function getDescriptionContent() {
		// function addToDescription() {
		// Phil 20060306 Dont need list config stuff for document summary
		$this->output->req_config_js = false;
		
		global $SERVICES, $L;
		$content = '';
		//$parent_item = $this->db_functions->getParentItems();
		$db_functions = $this->db_functions->getParentDbFunctions();
		
		$parent_item = $db_functions->getItem($this->form_values['parent_id']);
		//print_r($this->form_values);
		
		//print_r($parent_item);
		
		$ua =& $SERVICES->getService('Authentication');

		$dropdown_config = $SERVICES->config->lookup('modules.Admin.modulelink.documents_mime_type');
		$dropdown = new WebMATDropDown($dropdown_config['module'], $dropdown_config['table'], $dropdown_config['field']);
		$mime_type = $dropdown->getValue($parent_item['mime_type_id']);
		
		$rows[] = array('<strong>' . $L['documentsName'] . '</strong>', $parent_item['name']);
		//$rows[] = array('<strong>' . $L['documentsMimeType'] . '</strong>', $mime_type['name']);
		// $rows[] = array('<strong>' . $L['documentsAssignedTo'] . '</strong>', 
			// $ua->users->getUsername($parent_item['user_id']));
			
		if ($parent_item['owner_group_id']) {
			$group = $ua->groups->getGroup($parent_item['owner_group_id']);
		} else {
			$group['name'] = $L['Unrestricted'];
		}
			$rows[] = array('<strong>' . $L['documentsAssignedTo'] . '</strong>', $group['name']);
			
		if ($parent_item['view_group_id']) {
			$group = $ua->groups->getGroup($parent_item['view_group_id']);
		} else {
			$group['name'] = $L['Unrestricted'];
		}
		$rows[] = array('<strong>' . $L['documentsVisibleTo'] . '</strong>', $group['name']);
			
		if ($parent_item['edit_group_id']) {
			$group = $ua->groups->getGroup($parent_item['edit_group_id']);
		} else {
			$group['name'] = $L['Unrestricted'];
		}
		$rows[] = array('<strong>' . $L['documentsEditableBy'] . '</strong>', $group['name']);
			
		if ($parent_item['admin_group_id']) {
			$group = $ua->groups->getGroup($parent_item['admin_group_id']);
		} else {
			$group['name'] = $L['Unrestricted'];
		}
		$rows[] = array('<strong>' . $L['AdminGroup'] . '</strong>', $group['name']);

		// --
		// $content .= $this->output->getBeginNestedTable();
		// $this->no_filters = true;
		// $content .= $this->getSubTableHeader(array('summary'=>array('label'=>$L['documentsVersionDocumentSummary'])), 'none');
		// $this->no_filters = false;
		// $content .= $this->output->getEndNestedTable();
		// -- 
		// $content .= $this->output->startDataBlock();
		$content .= $this->output->getBeginNestedTable('viewTbl');
		foreach ($rows as $row) { 
			$content .= $this->output->getTableRow($row);
		}
		$content .= $this->output->getEndNestedTable();
		// $content .= $this->output->endDataBlock();
		// --
		
		return $content;
	}
	
}
?>