<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * Used by module Export functionality (process multi dropdown option)
 *
 * Note: QuickExport directly instanciates a WebMATExport object.
 * Does not use this class
 * @package Export
 */
class ExportItemWebMATEdit extends WebMATEdit {
	
	var $export_table;
	var $export_data_option = 'list';
	
	/**
	 * Set export table and alter data definition
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->setExportTable($this->form_values['table']);
		$this->alterDataDefinition();
	}
	
	/** 
	 * Title: Export Options
	 */
	function getTitleText() {
		global $L;
		return $L['exportOptions'];
	}

	/**
	 * Disabled
	 */
	function getNextPageRow() {
		return '';
	}
	
	function beforeForm(&$id, &$item, &$error_msg) {
		global $SERVICES,$L;
		$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');
		$items = $add->getOptionByType($this->export_data_option, $this->export_table);
		// print_r($items); print '<hr>';
		// if (empty($items)) {
			// $values = implode(",",$add->getFieldNames($this->export_table));
		// }else {
			// $values = $items['option_values'];
		// }
		// Phil 20051121 The code above did not allow for 'exclude' setting of data option
		// ie: when method is 'exclude', the 'option_values' fields are the ones we dont want
		if (empty($items)) { // no such data option
			$values = implode(",",$add->getFieldNames($this->export_table));
		} else {
			
			if ( !isset($items['method']) || (isset($items['method']) && ($items['method'] == 'include')) ) {
				// include option values
				$values = $items['option_values'];
			} else {
				// exclude option values
				$values = implode(",",$add->getFieldNames($this->export_table));
				// print '<div>exclude ' . $items['option_values'] . ' from:</div>'; print_r($values); print '<hr>';
				if (is_array($all_values = $add->getFieldNames($this->export_table))) {
					$exclude_values = explode(',', $items['option_values']);
					// print '<div>all values:</div>'; print_r($all_values); print '<hr>';
					// print '<div>exclude values:</div>'; print_r($exclude_values); print '<hr>';
					foreach ($all_values as $k => $v) {
						if (in_array($v, $exclude_values)) {
							bcrm_unset($all_values, $k);
						}
					}
					// print '<div>values:</div>'; print_r($all_values); print '<hr>';
					$values =  implode(",",$all_values);
				}
			}
		}
		
		$lang =& $SERVICES->getService('Language');
		$export_module = $this->export_table; // TODO remove the '__item'
		// print '<div>values:</div>'; print_r($values); print '<hr>';
		// TODO: //$tmp_fields = $add->getFields($this->export_table);
		//print '<div>fields:</div>'; print_r($tmp_fields); print '<hr>';
		// print '<div>export_table: ' .$this->export_table . ' module: ' . $this->module_name . ', ' . $this->getModuleName() . '</div>';
		// foreach ($tmp_fields as $k => $v) {
			// print '<div>field ' . $v['field_key'] . ' label: ' . $lang->getLanguageText($v['label'], $export_module) . '</div>';
		// }
		
		$item['table_fields'] = $values;
		$module = substr($this->export_table,0,strpos($this->export_table,'__'));
		$item['file_name'] = $module . "_" . date('Ymd'); 
		// Bosco 20051221 | Added to set js 
		$this->output->addToHead($this->getDisplayJs());
		$this->output->addInitJS('showHideDelimiters()');
		$file_type_key = $this->mod->getFileTypeKey();
		if (!empty($file_type_key)) {
			$this->dd->changeAttributes(array('field_key' => $file_type_key, 'tag_extras' => 'onChange=showHideDelimiters();'));
		}
	}
	
	/**
	 * Returns js function which hides the delimiter fileds
	 *
	 */
	function getDisplayJs() {
		// Bosco 20060310 | Changed code to Show / Hide Delimiter rows
		$delim_fields = $this->mod->getDelimiterFieldNames();
		$delim_count = count($delim_fields);	
		$js = '<script type="text/javascript" language="javascript">
				var delimiter_fields = new Array();' . "\n";
				
				foreach ($delim_fields as $k => $v) {
					$js .= 'delimiter_fields[' . $k .'] = \'' . str_replace(' ', '', ucwords(str_replace('_', ' ', $v))) . '\';' . "\n";	
				}	
				
		$js .= 		'function showHideDelimiters() {
					var obj1 = document.getElementById(\'idExcelVersion\');
					if (obj1 && (typeof obj1 == \'object\')) {
						if (document.forms.mainform.elements[\'item[file_type]\'].value == 2) {
							for(i=0;i<' . $delim_count . ';i++) {
								var elem = \'id\' + delimiter_fields[i];
								document.getElementById(elem).style.display = \'none\';
							}
							obj1.style.display = \'\';
						}
						else {
							for(i=0;i<' . $delim_count . ';i++) {
								var elem = \'id\' + delimiter_fields[i];
								document.getElementById(elem).style.display = \'\';
							}
							obj1.style.display = \'none\';
						}
					}
				}
			</script>';
		return $js;
	}
	
	function alterDataDefinition() {
		global $SERVICES, $L;
		$add =& $SERVICES->getService('DataDefinition', '', 'AdminDataDefinition');
		$lang =& $SERVICES->getService('Language');
		// print '<div>lang_name: ' . $this->export_table . '</div>';
		$lang_name = substr($this->export_table, 0, strpos($this->export_table, '__'));
		// print '<div>lang_name: ' . $lang_name . '</div>';
		// $lang_name = strtolower($lang_name[0]) .  substr($lang_name, 1);
		// print '<div>lang_name: ' . $lang_name . '</div>';
		$lang->getLanguage('_Global');	
		if ($lang_name) {
			$lang->getLanguage($lang_name);
		}
		// $available_fields = array_flip($add->getFieldNames($this->export_table));
		// print_r($add->getFieldNames($this->export_table)); print '<hr>';
		// foreach ($available_fields as $k => $v) {
			// $available_fields[$k] = $k;
		// }
		
		$field_names = (array)$add->getFieldNames($this->export_table, true, false, true); // include labels
		// print_r($field_names); print '<hr>';
		
		$available_fields = array();
		foreach ($field_names as $k => $v) {
			if (isset($v['field_key']) && $v['field_key']) {
				$available_fields[$v['field_key']] = ($v['label'] ? $v['label'] . ' - ' : '') . $v['field_key'];
			}
		}
		// print_r($available_fields); print '<hr>';
		
		
		
		$extra_fields = array(
			'label' => $L['exportChooseFields'], //'Select Fields',
			'field_key' => 'table_fields', 
			'type' => 'multi_select', 
			'field_priority' => 100, 
			'dropdown_values' => $available_fields,
			'explanation' => $L['exportChooseFieldsToExport'], //'Select the field(s) you want to export', 
			'csv'  => 1,
			'value' =>''
			// 'get_content' => 1
		);
		
		$this->dd->data_dictionary['table_fields'] =$extra_fields;
		
		$L['exportConvertValues'] = 'Convert Values';
		$L['exportConvertValuesExplanation'] = 'Export readable values'; 
		
		// Phil 20051124
		$convert_values = array(
			'label' => $L['exportConvertValues'], //'Convert Values',
			'field_key' => 'convert_values', 
			'type' => 'checkbox', 
			'field_priority' => 700, 
			'explanation' => $L['exportConvertValuesExplanation'], //'Export readable values', 
			'default_value' =>'1'
		);
		
		$this->dd->data_dictionary['convert_values'] =$convert_values;
		// Bosco 20051201 | Added to replace language keys with values
		foreach ($this->dd->data_dictionary as $k => $v) {
			if ($v['type'] == 'dropdown' && is_array($v['dropdown_values']) && !empty($v['dropdown_values'])) {
				foreach ($v['dropdown_values'] as $key => $val) {
					if (!empty($L[$val])) {
						$this->dd->data_dictionary[$k]['dropdown_values'][$key] = $L[$val];
					}
				}
			}
			// Bosco 20051221 | Added to assign ID for delimiter field rows	
			foreach ($this->mod->getDelimiterFieldNames() as $key => $val) {
				if ($v['field_key'] == $val) {
					// Bosco 20060310 | Changed code to Show / Hide Delimiter rows
					//$this->dd->data_dictionary[$k]['row_id'] = 'idRowDelim' . $key;
					$this->dd->data_dictionary[$k]['additional_output']['before'] = '<tbody id="id' . str_replace(' ', '', ucwords(str_replace('_', ' ', $val))) . '">';
					$this->dd->data_dictionary[$k]['additional_output']['after'] = '</tbody>';
				}
			}
			$excel_field_key = $this->mod->getExcelVersionKey();
			if (!empty($excel_field_key)) {
				// Bosco 20060310 | Changed code to Show / Hide Delimiter rows
				//$this->dd->data_dictionary[$excel_field_key]['row_id'] = 'idExcelVersion'; 
				//$this->dd->data_dictionary[$excel_field_key]['row_tag_extras'] = 'style="display: none;"'; 
				$this->dd->data_dictionary[$excel_field_key]['additional_output']['before'] = '<tbody id="idExcelVersion" style="display:none;">';
				$this->dd->data_dictionary[$excel_field_key]['additional_output']['after'] = '</tbody>';
			}
		}	
	}

	function setExportTable($table) {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		if ($db->tableExists($table)) {
			$this->export_table = $table;
		} else {
			$this->redirectBack();
		}
	}
	
	function redirectBack() {
		$this->redirectTo($_SESSION['return_page']);
		exit();
	}
	
	function redirectTo($return_page) {
		header('location: ' . $return_page);
		exit();
	}

	function run() {
		if (isset($this->form_values['submit'])) {
			$this->exportData();
			exit;
		}
	}

	function exportData() {
		global $SERVICES;
		$exp = getWebMATObject('WebMATExport', 'Export');
		
		$fields = $this->form_values['item']['table_fields'];
		$table = $this->form_values['table'];
		$module = substr($table,0,strpos($table,'__'));
		$mod =& $SERVICES->modules->getModule($module);
		$db_functions =& $mod->getDatabaseTable($this->form_values['table']);
		
		// print_r($db_functions->getDataDefinition()); print '<hr>'; exit();
		// print_r($fields); print '<hr>'; exit();
		
		//$this->setDataDefinitionFields($db_functions,$fields);
		$mod->dd = getWebMATObject(
			'WebMATDataDictionary',
			$mod->getModuleName(),
			array(
				'data_definition' => $db_functions->getDataDefinition(),
				'data_option' => '__all__' // Phil 20060110 changed to __all__ from 'export'. See WebMATDataDefinition
			) 
		);

		$ids =implode(
			',', 
			$_SESSION[$this->form_values['ids']]
		);
		$ids_where = array('id' => array('where' => 'id IN (' . $ids . ')'));
		$db_functions->setDesignTimeFilters($ids_where);

		$items = $db_functions->getItems();

		$mod->dd->setDictionaryIncludeFields($fields);

		if ($this->form_values['item']['include_header']) {
			$exp->include_header = true;
			$exp->setHeaderRow($mod->dd->getDataDictionary());
		} else {
			$exp->include_header = false;
		}
		// Phil 20051124
		$convert_values = (isset($this->form_values['item']['convert_values']) && $this->form_values['item']['convert_values'] ? true : false);
		foreach ($items as $k => $v) {
			$dd_item = $mod->dd->getDataStructureWithData($v);
			// Phil 20051124 Optional 2nd parameter to getRow defaults to true 
			// and will convert field values according to Output::getStringField
			$data[] = $exp->getRowData($dd_item, $convert_values);
		}
		$exp->setData($data);
		// Bosco 20051220 | CSV for Windows
		if (isset($this->form_values['item']['file_type']) && $this->form_values['item']['file_type'] == 2) {
			$exp->setCsvForExcelDelimiters();
		}
		$exp->exportDataToCsv();
	}
}
?>