<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Front
 */

/**
 * @package Front
 */
function refreshAlerts() {
	echo 'here';
}
	
class AlertProcessor {
	
	/**
	 * Class constructor
	 */
	function AlertProcessor() {
	}
	
	/**
	 * Sets and processes stored queries (if any). 
	 *
	 * @param boolean $force_refresh When true, ignores any caching 
	 * @param boolean $include_page When true, {@link display} will outut a complete html page
	 */
	function run($force_refresh = false, $include_page = true) {
		// See if the session values(s) are stored (hasStoredQueries)
		// if not, process the relevant modules and store session values (setStoredQueries)
		// then:
		// run the queries (processStoredQueries)
		// update the alert frame by displaying content (generateContent, display)
		$content = '';
		// if (!$force_refresh && $this->isAtLogin()) {
		if (0) { // Phil 20080830 Lets process alerts, even during login
			// Dont process anything here - let the menu page handle the first load
		} else {
			$this->preProcessing();
			if ($force_refresh || !$this->hasStoredQueries()) {
				$this->setStoredQueries();
			}
			if ($this->hasStoredQueries()) {
				$this->processStoredQueries();
				$content = $this->generateContent();
			}
		}
		print $content;
		// $this->display($content, $include_page);
	}
	
	/**
	 * Use this to determine whether to skip alert processing
	 *
	 * @return boolean Whether this page load is within a certain interval of the user last logging in
	 */
	function isAtLogin() {
		global $USER;
		// Refresh interval could be false, or possibly ''
		// We will use a default of 60 seconds
		$refresh_interval = $this->getRefreshInterval();
		$refresh_interval = max(60, intVal($refresh_interval));
		$current_time = time();
		$last_login = $USER['last_login'];
		// Phil 200605 Recent login changes will mean this is unlikely, but worth checking
		if (empty($last_login)) {
			return true;
		}
		if ($current_time && $refresh_interval && $last_login) {
			return !( ($current_time - $refresh_interval) >= $last_login);
		}
		return false;
	}
	
	/**
	 * Based on a config setting, run pre-processing for selected modules
	 *
	 * @return boolean Whether there are stored queries or not.
	 * @see getStoredQueries
	 */
	function preProcessing() {
		// Phil 20060510 Prevent email checking at login
		if (!$this->isAtLogin()) {
			$preprocess_modules = array( // todo: make this a config lookup
				'Mail' => array(
					'includeModule' => '/check_overdue_email.php'
				)
			);
		}
		if (is_array($preprocess_modules)) {
			foreach ($preprocess_modules as $k => $v) {
				if (is_array($v)) {
					if (isset($v['includeModule']) && $v['includeModule']) {
						includeModule($k, $v['includeModule']);
					}
				}
			}
		}
	}
	
	/**
	 *
	 * @return boolean Whether there are stored queries or not.
	 * @see getStoredQueries
	 */
	function hasStoredQueries() {
		return (is_array($this->getStoredQueries()) && count($this->getStoredQueries()) ? true : false);
	}
	
	/**
	 * @reurn array of stored queries. An empty array means there are no stored queries.
	 */
	function getStoredQueries() {
		global $USER; // Phil 20060607 Using $USER instead of $_SESSION
		return (array)$USER['alert_queries'];
	}
	
	/**
	 * Loops through the modules to process and calls {@link setStoredQuery}
	 * @see getModulesToProcess
	 * @see setStoredQuery
	 */
	function setStoredQueries() {
		if (is_array($process_modules = $this->getModulesToProcess())) {
			foreach ($process_modules as $k => $v) {
				$this->setStoredQuery($k, $v);
			}
		}
	}
	
	/**
	 * Loops through the modules to process and calls {@link setStoredQuery}
	 * @param string $modue_name
	 * @param mixed $values. Optional, default is false. Can provide an array of default values (keys: table, element)
	 */
	function setStoredQuery($module_name, $values = false) {
		global $SERVICES;
		if (!is_object($mod =& $SERVICES->modules->getModule($module_name))) {
			return false;
		}
		
		if (!is_array($values)) {
			$values = $this->getDefaultValues($module_name);
		}
		
		if (!is_object($db_functions =& $mod->getDatabaseTable($values['table']))) {
			return false;
		}
		
		if (method_exists($db_functions, 'getCountAlertsQuery')) {
			$query = $db_functions->getCountAlertsQuery();
			if (is_array($query) && array_key_exists('fields', $query) && array_key_exists('tables', $query) ) {
				$store_query = "SELECT " . $query['fields'] . " FROM " . $query['tables'] . "";
				if (array_key_exists('where', $query)) {
					$store_query .= " WHERE  " . $query['where'] . "";
				}
				global $USER; // Phil 20060607 Using $USER instead of $_SESSION
				$USER['alert_queries'][$module_name]= array(
					'query' => $store_query, 
					'element' => $values['element']
				);
				return true;
			}
		}
		return false;
	}
	
	/** 
	 * @return array of default values
	 * @see setStoredQuery
	 */
	function getDefaultValues($module_name) {
		// Usually get these values from the module config 
		return array('table' => 'item', 'element' => 'alert_' . $module_name);
	}
	
	function processStoredQueries() {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		$db->hide_print_errors; // Disables error printing
		$this->processed_alerts = array();
		if (is_array($queries = $this->getStoredQueries())) {
			foreach ($queries as $k => $v) {
				$count = false;
				
				if ($result = $db->sqlQuery($v['query'])) {
					$row = $db->getRow($result);
					if (is_array($row)) {
						if (sizeof($row) == 1) {
							$v = array_values($row);
							$count = stripslashes($v[0]);
						} else {
							$count = $db->getNumRows($result);
						}
					} else if ($row !== false) {
						$count = (int)stripslashes($row);
					}
					
					if ($count !== false) {
						$this->processed_alerts[$k] = $count;
					}
				}
			}
		}
	}
	
	/**
	 * @return array key = module, value = number of alerts
	 */
	function getProcessedAlerts() {
		return (array)$this->processed_alerts;
	}
	
	/**
	 * @return boolean Whether there are menu alerts for the 'About' page
	 */
	function getProcessedAboutAlert() {
		includeModule('Front', 'inc/front_about.inc');
		$retval = false;
		if (class_exists('FrontAbout')) {
			$about = new FrontAbout();
			$retval = $about->getMenuAlerts();
		}
		return $retval;
	}
	
	/**
	 * @return string Javascript content to display in the head of a web page 
	 * Attempts to call a javascript function in a parent.header frame to populate menu alerts
	 * @author Phil Calder, Jack Farley
	 */
	function generateContent() {
		// error_reporting(E_ALL);
		$alerts = $this->getProcessedAlerts();
		$aboutAlert = $this->getProcessedAboutAlert(); // Phil 20060209
		$content = '';
		
		// print_r($alerts); print '<hr>';
		if ($aboutAlert) {
			$alerts['About'] = 1;
		}
		
		$return_alerts = array();
		foreach ($alerts as $k => $v) {
			// if ($v) {
				$return_alerts[] = array(
					'mod' => $k, 
					'alerts' => (int)$v
				);
			// }
		}
		
		global $SERVICES;
		$output =& $SERVICES->getService('Output');
		return $output->jsonEncode($return_alerts);
		
		// pre version 5 code below (deprecated)
		// 20050106 Jack
		$content .= '
		<script language="JavaScript" type="text/javascript">
			var alertArr = new Array(';
		foreach( $alerts as $module => $numAlerts) {
			$content .= "new Array('$module', $numAlerts),";
		}
		$content .= ' null);
		';
		
		// The 'About' alert is a special case that changes the link colour for the 'About' link
		if ($aboutAlert) {
			$content .= 'var aboutAlert=' . 'true' . ';';
		} else {
			$content .= 'var aboutAlert=' . 'false' . ';';
		}
		
		// Phil - Run doAlerts 
		// in the header frame (ie from the refresh page), 
		// or in the current window/document (ie from the menu frame)
		$content .= '
		if (parent.headerframe) {
			if (parent.headerframe.doAlerts) {
				parent.headerframe.doAlerts(alertArr, aboutAlert);
			}
		} else {
			if (window.doAlerts) {
				doAlerts(alertArr, aboutAlert);
			} else if (document.doAlerts) {
				doAlerts(alertArr, aboutAlert);
			}
		}
		</script>
		';
		return $content;	
	}
	
	/**
	 * Creates and outputs html
	 *
	 * A simple page. All content in the 'head'. 
	 * Contains a meta refresh based on a user preference, and any content passed to this method.
	 *
	 * @param string $content Additional head content
	 * @param boolean $include_page Optional, default is true. When true will output a complete html page, otherwise just the content parameter string
	 * @author Phil Calder, Jack Farley
	 *
	 */
	function display($content, $include_page = true) {
		$page = '';
		if ($include_page) {
			$page = '<!DOCTYPE HTML PUBLIC "-//W3C//Dtd HTML 4.01 Transitional//EN">
			<html>
			<head>
			<title>Refresh</title>
			';
			
			if ($delay = $this->getRefreshInterval()) {
				$page .= '
				<meta http-equiv="refresh" content="' . $delay . '">
				';
			}
		}
		
		$page .= $content;
		if ($include_page) {
			$page .= '
			</head>
			<body></body>
			</html>';
		}
		print $page;
	}
	
	/**
	 * @return an array of potential 'alerting' modules
	 */
	function getAllAlertingModules() {
		global $SERVICES;
		$modules = array();
		if (is_object($config =& $SERVICES->getService('Config'))) {
			$modules = $config->lookup('modules.Front.header.alert');
		}
		return $modules;
	}
	
	/**
	 * 
	 * @return an array of modules the current user has in their header/ menu frame module list (and 'Rest' sub menu)
	 */
	function getUsersHeaderModules() {
		if (is_object($layout = $this->getLayoutObject())) {
			// $menu_modules = $layout->getItems(); // The modules the user has selected
			$menu_modules = $layout->getConfigPieces(); // The complete list of available modules
			// print '<div>' . get_class($this) . '::getUsersHeaderModules() menu modules:</div>'; print_r($menu_modules); print '<hr>';
		}
		return $menu_modules;
	}
	
	function getLayoutObject() {
		global $SERVICES;
		includeModule('Front', 'inc/dashboard_layout.inc');
		// hardcode the dropdown config name
		$post = array();
		$get['dropdown_config'] = 'system_dashboard_dropdown';
		// 'layout' is the dropdown manager (used for editing the dashboard - the menu frame as of version 4.150). 
		// We use it here to get the items to display.
		if ( class_exists('DashboardWebMATAdminDropdown') && ($layout = new DashboardWebMATAdminDropdown($_POST, $_GET)) ) {
			return $layout;
		}
		return false;
	}
	
	function getModulesToProcess() {
		$all_alerting = $this->getAllAlertingModules();
		if (is_array($all_alerting)) {
			$all_alerting_keys = array_keys($all_alerting);
		} else {
			return false;
		}
		$users_modules = $this->getUsersHeaderModules(); // All menu modules available to this user
		
		$display_alerts_for = array(); // All modules in the users Menu (not including 'Other' modules (ie the 'Rest')
		if (is_object($layout = $this->getLayoutObject())) {
			$user_items = (array)$layout->getItemsFor(); // current user
			foreach ($user_items as $ui_k => $ui_v) {
				// if (isset($ui_v['no_alerts']) && !$ui_v['no_alerts']) {
					$display_alerts_for[$ui_v['name']] = ($ui_v['no_alerts'] ? false : true);
				// }
			}
		}
		
		// Whether to display alerts for the 'Rest' of the modules (not in the Menu)	
		$display_other_alerts = $layout->getPrefDisplayOtherAlerts(); 
		// print '<div>' . get_class($this) . '::getUsersHeaderModules() user items:</div>'; print_r($user_items); print '<hr>';
		
		$process_modules = array();
		if (is_array($users_modules)) {
			foreach ($users_modules as $k => $v) { 
				// if (in_array($k, $all_alerting_keys)) {
					// $process_modules[$k] = $all_alerting[$k];
				// }
				if (in_array($k, $all_alerting_keys)) {
					// print '<div>k=' . $k . '</div>';
					if (array_key_exists($k, $display_alerts_for)) {
						if ($display_alerts_for[$k]) {
							// print '<div>(menu) process_modules=' . $k . '</div>';
							$process_modules[$k] = $all_alerting[$k];
						} 
					} else if ($display_other_alerts) {
						$process_modules[$k] = $all_alerting[$k];
						// print '<div>(rest) process_modules=' . $k . '</div>';
					}
				}
			}
		} else {
			return false;
		}
		// print '<div>process_modules:</div>'; print_r($process_modules); print '<hr>';
		return $process_modules;
	}
	
	/**
	 * @return mixed refresh interval: false if we dont refresh, 
	 *  otherwise the number of seconds between refreshes
	 */
	function getRefreshInterval() {
		global $SERVICES;
		$refresh_interval = false;
		if (is_object($SERVICES->getService('Preferences'))) {
			$refresh_interval = intVal($SERVICES->preferences->getUserPref('frontRefresh'));
		}
		if (intVal($refresh_interval) > 0) {
			$refresh_interval = $refresh_interval;
		} else {
			$refresh_interval = false;
		}
		return $refresh_interval;
	}
}
?>