<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('admin_dropdown.inc');

/**
 * Controls the dashboard layout for a user
 *
 * As of version 4.5 the 'dashboard' layout applies to the application menu frame.
 * 
 * @author Phil Calder
 */
class DashboardWebMATAdminDropdown extends WebMATAdminDropdown { // Note: used to extend admin nested dropdown
	
	function DashboardWebMATAdminDropdown($post, $get, $mini_data_definition = false) {
		
		$this->WebMATAdminDropdown($post, $get, $mini_data_definition);
		$this->setUserId();
		global $L, $SERVICES;
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Front');
		$this->output->toolbar_link_mode = true;
		$this->new_menu_links[] = $this->getResetUserLink();
	}
	
	// function main() {
		// parent::main();
	// }
	
	/**
	 * Custom lookup for dropdown config values
	 */
	function customSetDropdownConfig($config_values = array()) {
		global $SERVICES;
		// == Customised == //
		$this->dropdown_config = $SERVICES->config->lookup('modules.Admin.modulelink.system_dashboard_dropdown');
		
		if (!isset($this->dropdown_config['field']) || !$this->dropdown_config['field']) { //Sets dropdown_config['field'] to name if null
			$this->dropdown_config['field'] = 'name';
		}
		
		// Override
		if (is_array($config_values)) {
			foreach ($config_values as $k => $v) {
				$this->dropdown_config[$k] = $v;
			}
		}
	}
	
	function setDropdownConfig($config_values = array()) {
		//parent::setDropdownConfig();
		$this->customSetDropdownConfig(); // Phil 20050316
		$this->dropdown_config['module'] = 'System'; // Because we are in the front module, but want to store the db table as 'System__'
	}
	
	function setUserId() {
		global $USER, $SERVICES;
		
		if (
			$this->userIdIsFixed() 
			&& isset($this->form_values['user_id']) 
			&& ($this->form_values['user_id'] != $USER['user_id']) 
			&& ($ua =& $SERVICES->getService('Authentication')) 
			&& $ua->adminAccess($USER['user_id'], 'Admin')
		) {
			$this->user_id = $this->form_values['user_id'];
			// $this->user_id = $this->getUserId();
		} else {
			$this->user_id = $USER['user_id'];
		}
	}
	
	function userIdIsFixed() {
		return (isset($this->user_id_fixed) ? $this->user_id_fixed : false);
	}
	
	function getUserId() {
		return $this->user_id;
	}
	
	/** 
	 * Gets items for the user, or copies default setup then get items if the user has no items
	 */
	function getItems() {
		$items = $this->getItemsFor();
		if (!sizeof($items)) {
			$this->copyDefaultSetup();
			$items = $this->getItemsFor();
		}
		return $items;
	}
	
	/**
	 * Gets a list of items for the selected user
	 *
	 * @param mixed $user_id (optional) Can be 'default', or a user_id. If not specified, use the current users id.
	 * @return array $pieces key is the column id, value is an array with keys 'name', 'priority', 'no_alerts'
	 */
	 function getItemsFor($user_id = '') {
		global $USER, $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		
		if (empty($user_id)) {
			$user_id = ($USER['user_id'] ? $USER['user_id'] : 0);
		} else if ($user_id == 'default') {
			$user_id = '0';
		}
		
		$extra_where = ' AND active = 1 AND user_id = ' . $user_id;
		$items = $this->dropdown->getItems('1' . $extra_where);
		$pieces = array();
		
		foreach ($items as $k => $v) {
			if (!$user_id || $ua->readAccess($user_id, $v['name'])) {
				$pieces[$k] = array(
					'name' => $v['name'],
					'priority' => $v['priority'], 
					'no_alerts' => $v['no_alerts']
				);
			}
		}
		return $pieces;
	}
	
	/**
	 * Gets the 'User' preference
	 *
	 * @return boolean Whether to show alerts in the Menu for modules not in the users menu setup
	 */
	function getPrefDisplayOtherAlerts() {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		return $prefs->getUserPref('frontMenuDisplayOtherAlerts');
	}
	
	/**
	 * Sets the 'User' preference
	 *
	 * @param boolean $value Whether to show alerts in the Menu for modules not in the users menu setup
	 */
	function setPrefDisplayOtherAlerts($value) {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		$value = ($value ? '1' : '0');
		return $prefs->setUserPref('frontMenuDisplayOtherAlerts', $value);
	}
	
	/**
	 * Copies the 'default' users setup for the current user 
	 */
	function copyDefaultSetup() {
		global $USER;
		$items = $this->getItemsFor('default');
		foreach ($items as $k => $v) {
			if (is_array($v)) {
				$this->dropdown->setValue(
					array(
						'name'=>$v['name'], 
						'user_id' => $USER['user_id'], 
						'priority'=>$v['priority'], 
						'no_alerts' => $v['no_alerts']
					)
				);
			} else {
				$this->dropdown->setValue(
					array(
						'name'=>$v, 
						'user_id' => $USER['user_id'], 
						'no_alerts' => $v['no_alerts']
					)
				);
			}
		}
	}
	
	// Phil 20050523
	function getConfigPieces() {
		global $SERVICES, $USER;
		$SERVICES->getService('Config');
		$ua =& $SERVICES->getService('Authentication');
		$config_pieces = $SERVICES->config->lookup('modules.Front.dashboardconfig');
		
		if (!is_array($config_pieces)) {
			$config_pieces = array();
		}
		
		foreach($config_pieces as $k => $config_piece) {
			// print '<div>k=' . $k . '</div>'; // print_r($config_piece); print '<hr>';
			if (!isset($config_piece['module']) && is_array($config_piece)) { // Incase there is more than one view per module
				foreach ($config_piece as $kk => $sub_piece) {
					if (!$ua->readAccess($USER['user_id'], $sub_piece['module'])) {
						// print '<div>a) No access to ' . $k . '</div>';
						bcrm_unset($config_pieces, $k);
					} else {
						bcrm_unset($config_pieces, $k);
						$config_pieces[ $sub_piece['module'] . '_' . $kk ] = $sub_piece;
					}
				}		
			} else { // Only one piece for this module
				if (!$ua->readAccess($USER['user_id'], $config_piece['module'])) {
					// print '<div>b) No access to ' . $k . '</div>';
					bcrm_unset($config_pieces, $k);
				}
			}
		}
		
		// if (isset($config_pieces['file'])) { // Only one, we need an array
			// $links = array($config_pieces);
		// }
		
		$sortarray = array();
		
		foreach ($config_pieces as $k => $config_piece) {
			$sortarray[] = (isset($config_piece['priority']) ? $config_piece['priority'] : '');
		}
		array_multisort($sortarray,$config_pieces);
		
		return $config_pieces;
	}
	
	function getAvailablePieces() {
		global $SERVICES, $L;
		$SERVICES->getService('Language');
		$links = $this->getPieces();
		$values = array();
		foreach ($links as $k => $v) {
			$SERVICES->language->getLanguage($v['module']);
			$values[$k] = $L[$v['label']];
		}
		return $values;
	}

	/**
	 * Creates a cut down Data Dictionary
	 *
	 * @param array $item the current item from the dropdown table.
	 * @return array an array of items with formating infromation
	 */
	function getFormFields($item) {
		global $L, $USER;
		if (!isset($item['user_id'])) {
			$item['user_id'] = $this->getUserId(); // $USER['user_id'];
		}
		
		if (!$this->mini_data_definition) {
			global $SERVICES;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage('Front');
			$boolean_dropdown = array(
				// '0' => $L['frontMenuShowAlerts'], 
				'1' => $L['frontMenuHideAlerts']
			);
			
			$fields =  array(
				'id' => array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'),
				'user_id' => array('label'=>'', 'field_key'=>'user_id', 'type'=>'hidden'),
				$this->dropdown_config['field'] => array('label'=>$L['frontDashboardValueName'], 'field_key'=>$this->dropdown_config['field'], 'type'=>'text', 'notnull' => 1),
				'priority' => array('label'=>$L['DisplayOrder'], 'field_key'=>'priority', 'type'=>'integer', 'min' => 0, 'max' => 999, 'default_value' => 100, 'notnull' => 1), 
				'no_alerts' => array('label'=>$L['frontMenuAlerts'], 'field_key'=>'no_alerts', 'type'=>'dropdown', 'dropdown_values' => $boolean_dropdown)
			);
				
			$fields[$this->dropdown_config['field']]['type'] = 'dropdown';
			$fields[$this->dropdown_config['field']]['dropdown_values'] = $this->getAvailablePieces();
			
		} else {
			$fields = $this->mini_data_definition;
		}
		
		foreach ($fields as $k => $v) {
			if ($item[$v['field_key']]) {
				$fields[$k]['value'] = $item[$v['field_key']];
			} else {
				$fields[$k]['value'] = $fields[$k]['default_value'];
			}
		}
		
		return $fields;
	}
	
	function getBeforeDataContent() {
		global $L;
		$table_row = array(
			$L['frontDashboardValueName'], 
			$L['DisplayOrder'], 
			$L['frontMenuAlerts'], 
			'', 
			''
		);
		return $this->output->getTableRow($table_row);
	}
	
	function getAfterDataContent() {
		global $L;
		$pref = $this->getPrefDisplayOtherAlerts();
		if ($pref) {
			$label = $L['frontMenuShowAlerts'];
			$change_value = '0';
		} else {
			$label = $L['frontMenuHideAlerts'];
			$change_value = '1';
		}
		$change_link = $this->output->getLinkHref(
			array(
				'href' => $PHP_SELF . '?show_other_alerts=' . $change_value, 
				'text' => $L['labelChange']
			) 
		);
		
		$table_row = array(
			$L['frontMenuDisplayOtherAlerts'], 
			'', 
			$label, 
			$change_link, 
			''
		);
		return $this->output->getTableRow($table_row);
	}
	
	/**
	 * Builds the nested table for the list view
	 *
	 * @return string the table rows for the nested table. If there are no values for this dropdown, the string will be empty.
	 */
	function getNestedTable() {
		global $USER, $L;
		$table_rows = ''; // Phil - initialised our return value.
		// Only items for the current user
		$this->dropdown->setWhereStatement(
			array(
				'user_id' => $this->getUserId() // $USER['user_id']
			)
		);
		// $values = $this->dropdown->getValues('&nbsp;&nbsp;&nbsp;');
		$values = $this->dropdown->getValuesArray();
		// print '<div>values:</div>'; print_r($values); print '<hr>'; exit();
		
		if (is_array($values)) {
			global $SERVICES, $USER;
			$ua =& $SERVICES->getService('Authentication');
			$items = array();
			foreach ($values as $k => $v) {
				if ($ua->readAccess($USER['user_id'], $v['name'])) {
					$items[] = $v;
				}
			}
			// print '<div>items:</div>'; print_r($values); print '<hr>'; exit();
			
			foreach ($items as $item) {
				$table_row = array();
				foreach ($this->getFormFields($item) as $field) {
					if ($field['field_key'] == $this->dropdown_config['field']) {
						$table_row['label'] = $this->getStringField($field);
					} else if ($field['field_key'] == 'priority') {
						$table_row['priority'] = $this->getStringField($field);
					} else if ($field['field_key'] == 'no_alerts') {
						$table_row['no_alerts'] = ($field['value'] ? $L['frontMenuHideAlerts'] : $L['frontMenuShowAlerts']);
					}
				}
				$table_row = $this->getEditDeleteLinks($table_row, $item['id']);
				$table_rows .= $this->output->getTableRow($table_row);
			}
		}
		return $table_rows;
	}
	
	/**
	 * Appends an edit link & deletelink to the supplied array.
	 *
	 * @param array $link_array the current row_array
	 * @param string $id the current item id
	 * @return array the row_array with the links appended
	 */
	function getItemLinks($row_array, $id) {
		global $L, $PHP_SELF;
		$row_array[] = $this->output->getLinkHref( 
			array(
				'href' => $PHP_SELF . '?id=' . $id, 
				'text' => $L['LinkEdit']
			) 
		);
		
		$text = '';
		$row_array[] = $this->output->getLinkDelete($PHP_SELF . '?delete=' . $id, $text);
		return $row_array;
	}
	
	function getResetUserLink() {
		global $L, $SERVICES;
		if (strstr($SERVICES->getGlobal('PHP_SELF'), 'default')) {
			$url = '../../modules/Front/admin_dashboard_default_layout.php?reset=1';
		} else {
			$url = '../../modules/Front/admin_dashboard_layout.php?reset=1';
		}
		return $this->output->getLinkConfirm(
			$url,
			$L['frontResetUserMenu'], 
			$L['frontResetUserMenuAreYouSure'], 
			$this->toolbar_link_mode // whether to display as part of the toolbar
		);
	}
	
	function resetForUser() {
		global $SERVICES, $USER;
		$user_id = $USER['user_id'];
		$ua =& $SERVICES->getService('Authentication');
		if ($user_id && $ua->writeAccess($user_id, 'Front')) {
			$db =& $SERVICES->getService('Database');
			$db->delete(
				'System__dashboard_dropdown', 
				"user_id = '$user_id'"
			);
			if ($db->hasError()) {
				$db->printError();	
			}
			$this->copyDefaultSetup();
		}
	}
	
	/**
	 * @deprecated
	 * @see getConfigPieces
	 */
	function getPieces() {
		return $this->getConfigPieces(); // Phil 20050523
	}
	
	/**
	 * Process an edit/delete and updates the database
	 *
	 * Extended to handle changing {@link setPrefDisplayOtherAlerts}
	 */
	function doSubmit() {
		if (isset($_GET['show_other_alerts'])) {
			$value = ($_GET['show_other_alerts'] ? '1' : '0');
			$this->setPrefDisplayOtherAlerts($value);
		}
		parent::doSubmit();
	}
	
	/**
	 * Users have access to this class provided they have at least read access to the module
	 *
	 * @param string $module Optional module name (default is current module)
	 * @param integer $level Optional (default is read)
	 */
	function accessRestrict($module = '', $level = 'read') {
		global $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		$ua->readRestrict($module); //halt page execution for users who do not have Admin access for this module
	}
}
?>