<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Front 
 */

/**
 * Handles common tasks for Front module pages
 * @author Phil Calder
 * @package Front 
 */
class FrontManager {
	
	/**
	 * Constructor
	 *
	 * Calls {@link init}
	 */
	function FrontManager() {
		$this->init();
	}
	
	/**
	 * Initialises required services and Front language
	 */
	function init() {
		global $SERVICES;
		$SERVICES->getService('Config');
		$SERVICES->getService('Modules');
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Front');
	}
	
	/**
	 * Checks whether module exists and that the curren tuser has read access to it
	 */
	function hasAccessTo($module) {
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		$modules =& $SERVICES->getService('Modules');
		if ($modules->exists($module) && $ua->readAccess($USER['user_id'],$module)) {
			return true;
		}
		return false;
	}
	
	/** 
	 * @return array Module names to exclude from 'Front' classes (menu and settings)
	 * @author Phil Calder
	 */
	function getModulesToExclude() {
		$exclude_other_menu_modules = array(
			'TemplateNewModule', 
			'Template', 
			'Template_main', 
			'Template_alternative', 
			'ClientLoginManager'
		);
		return $exclude_other_menu_modules;
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed string: The module label for the given module (or the module name if there is no label)
	 * @author Phil Calder
	 */
	function getModuleLabel($module_name) {
		// print '<div>module_name: ' . $module_name . '</div>';
		global $SERVICES;
		if ($key = $SERVICES->config->lookup('modules.Front.modulelink.'	. $module_name . '.label')) {
			// print '<div>key: ' . $key . '</div>';
			if ($label = $SERVICES->language->getLanguageText($key, $module_name)) {
				// print '<div>label: ' . $label . '</div>';
				return $label;
			}
		}
		return $module_name;
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or a string: The link to the given module
	 * @author Phil Calder
	 */
	function getModuleLink($module_name) {
		global $SERVICES;
		if ($key = $SERVICES->config->lookup('modules.Front.modulelink.' . $module_name . '.link')) {
			return $key;
		}
		return false;
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of query-filters (alert, mine, unseen). Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getQfilters($module_name) {
		global $SERVICES;
		if ($qfilters = $SERVICES->config->lookup('modules.Front.qfilters.' . $module_name)) {
			if (is_array($qfilters)) {
				return $qfilters;
			}
		}
		return false;
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of config-filters. Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getNonstandardFilters($module_name) {
		$filters = array();
		switch ($module_name) {
			case 'SystemBookmarks':
				// 
				global $SERVICES;
				if (is_object($modules =& $SERVICES->getService('Modules'))) {
					if (is_object($sysbk_mod =& $modules->getModule('SystemBookmarks'))) {
						$filters = $sysbk_mod->getMenuBookmarkLinks();
						// print_r($filters);
						foreach ($filters as $k => $v) {
							$filters[$k]['link'] = $filters[$k]['href'];
							$filters[$k]['label'] = $filters[$k]['text'];
							bcrm_unset($filters[$k], 'href');
							bcrm_unset($filters[$k], 'text');
						}	
					}
				}
			break;
			default:
			break;
		}
		return $filters;
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of config-filters. Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getFilters($module_name) {
		global $SERVICES;
		$filter_exceptions = array('SystemBookmarks');
		if (in_array($module_name, $filter_exceptions)) {
			return $this->getNonStandardFilters($module_name);
		}
	
		if ($filters = $SERVICES->config->lookup('modules.' . $module_name . '.file_set.item.filter')) {
			if (is_array($filters)) {
			
				// Filtering on a dropdown table
				// todo: optimise this! (based on WebMATFilters)
				if (isset($filters[0]) && is_string($filters[0]) && strstr($filters[0], '__table(')) {
					$filter_info = array();
					$filter_info['type'] = 'dropdown';
					$temp =  explode('.', str_replace(array('__table(', ')__'), '', $filters[0]) );
					if (is_array($temp) && count($temp) > 1) {
						$filter_info['module'] = $temp[0];
						$filter_info['table'] = $temp[1];
						if (!isset($temp[2]) || !$temp[2]) {
							$filter_info['name'] = 'name';
						} else {
							$filter_info['name'] = $temp[2];
						}
						$filter_info['in_data_option'] = 1;
						$dropdown = getWebMATObject(
							'WebMATDropdown', 
							$filter_info['module'], // module
							array(
								'module' => $filter_info['module'], 
								'table' => $filter_info['table'], 
								'field_name' =>$filter_info['name']
							)
						);
						$dropdown_values = $dropdown->getValues(' > ');
						foreach ($dropdown_values as $k => $v) {
							$filter_info['options'][$k] = array('label' => $v);
						}
						unset($dropdown);
						
						if (is_array($filter_info['options'])) {
							$filter_links = array();
							$index_url = $this->getModuleLink($module_name);
							foreach ($filter_info['options'] as $k => $v) {
								if (is_array($v)) {
									$filter_links[$k] = array(
										'label' => addslashes($v['label']), // Phil 20070824 Prevent js errors
										'link'  => $index_url . '?filter=f' . $k . $this->getModuleSpecificFilterURLSuffix($module_name)
									);
								}
							}
						}
						if (isset($filters['display_no_filter_link']) && $filters['display_no_filter_link']) {
							$filter_links['display_no_filter_link'] = 1;
						}
						$filters = $filter_links;
					} else {
						$filters = array();
					}
				}
				return $filters;
			}
		}
		return false;
	}
	
	function getModuleSpecificFilterURLSuffix($module_name) {
		if ( ($module_name == 'Documents') || ($module_name == 'Files') ) {
			return '&view_type=custom';
		}
		return '';
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of action links (aeg 'new'). Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getActions($module_name) {
		global $SERVICES;
		if ($actions = $SERVICES->config->lookup('modules.Front.actions.'	. $module_name)) {
			if (is_array($actions)) {
				return $actions;
			}
		}
		return false;
	}
	
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of other_links. Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getOtherLinks($module_name) {
		global $SERVICES;
		if ($other_links = $SERVICES->config->lookup('modules.Front.other_links.'	. $module_name)) {
			if (is_array($other_links)) {
				return $other_links;
			}
		}
		return false;
	}
	
	function getRefreshLink() {
		global $USER, $L, $SERVICES;
		$content = '<a href="' . $SERVICES->config->lookup('modules.Front.refresh_alerts.link') . '" title="' . $L['LinkRefreshExplanation'] . '"><img src="' . $USER['skin_web_path'] . 'img/refresh.gif" border="0" alt="Refresh" /></a>';
		return $content;
	}
	
	/**
	 * @return string The default target frame name
	 * @author Phil Calder
	 */
	function getDefaultTarget() {
		return 'contentframe';
	}
	
	/**
	 * @return mixed A config lookup for the help url (string or false)
	 * @author Phil Calder
	 */
	function getHelpLink() {
		global $SERVICES;
		return $SERVICES->config->lookup('help.url');	
	}
}

/**
 * Handles functionality for the Front module 'menu' page
 * @author Phil Calder
 * @package Front 
 */
class FrontMenuManager extends FrontManager {
	// function FrontMenuManager() {
	// }
	
	/**
	 * Initialises required services
	 *
	 * Calls parent method, initialises Authentication service, sets users menu array
	 * @author Phil Calder
	 */
	function init() {
		parent::init();
		global $SERVICES;
		$SERVICES->getService('Authentication');	
		$this->getUsersMenus(); // Actually, this sets the $menus class variable
	}
	
	/**
	 * @return array An array of modules the current user has in their menu frame module list
	 * @author Phil Calder
	 */
	function getUsersMenuModules() {
		global $SERVICES;
		$menu_modules = array();
		// 'layout' is the dropdown manager (used for editing the dashboard - the menu frame as of version 4.150).
		// We use it here to get the modules to display.
		includeModule('Front', 'inc/dashboard_layout.inc');
		// hardcode the dropdown config name
		$post = array();
		$get['dropdown_config'] = 'system_dashboard_dropdown';
		if (class_exists('DashboardWebMATAdminDropdown') && is_object($layout = new DashboardWebMATAdminDropdown($post, $get))) {
			$menu_modules = $layout->getItems();
		}
		if (is_array($menu_modules)) {
			// print_r($menu_modules);
			foreach ($menu_modules as $k => $v) {
				if (!$this->hasAccessTo($v['name'])) {
					bcrm_unset($menu_modules, $k);
				}
			}
		}
		return $menu_modules;
	}
	
	/**
	 * @return array All the modules we display in the menu that are not in the users main menu row
	 * @author Phil Calder
	 */
	function getOtherMenuModules($users_modules = false) {
		global $SERVICES;
		// 'layout' is the dropdown manager (used for editing the dashboard - the menu frame as of version 4.150).
		// We use it here to get the modules to display.
		includeModule('Front', 'inc/dashboard_layout.inc');
		// hardcode the dropdown config name
		$post = array();
		$get['dropdown_config'] = 'system_dashboard_dropdown';
		
		$other_modules = array();
		if (class_exists('DashboardWebMATAdminDropdown') && is_object($layout = new DashboardWebMATAdminDropdown($post, $get))) {
			$all_modules = $layout->getConfigPieces();
			if (!is_array($users_modules)) {
				$users_modules = array();
				$users_items = $layout->getItems(); // Phil 20060825 this now returns an array we need to process
				if (is_array($users_items)) {
					foreach ($users_items as $k => $v) {
						if (isset($v['name']) && $v['name']) {
							$users_modules[] = $v['name'];
						}
					}
				}
			}
			// print '<div>' . get_class($this) . '::getOtherMenuModules all_modules/users_modules</div>'; print_r($all_modules); print '<hr>'; print_r($users_modules); print '<hr>';
			$other_modules = array_diff($tmp_keys = array_keys($all_modules), $users_modules);
			if (is_array($other_modules)) {
				foreach ($other_modules as $k => $v) {
					if (!$this->hasAccessTo($v)) {
						bcrm_unset($other_modules, $k);
					}
				}
			}
		}
		
		// Initialise the 'Other' menu
		$other_menu = array(
			'label'   => '>', 
			'link'    => 'modules/Front/modules.php?is_menu=1', // Phil 20060125
			'alert'   => false,
			'newIcon' => false,
			'idInc'   => true, // JACK - required to show alerts on these items
			'menu'    => array(
				'actions' => array()
			)
		);
		
		$exclude_other_menu_modules = $this->getModulesToExclude();
		if (is_array($other_modules) && count($other_modules)) {
			foreach ($other_modules as $k => $v) {
				// 'label' and 'link'
				if ( (!in_array($v, $exclude_other_menu_modules)) && ($label = $this->getModuleLabel($v)) && ($link = $this->getModuleLink($v)) ) {
					$other_menu['menu']['actions'][$v] = array(
						'link' => $link,
						'label' => $label //, 
						// 'mod' => (string)$v // Never implemented: planning to use for FrontHome sub menu alerts
					);
				}
			}
		}
		// Add a Menu Setup link
		$other_menu['menu']['actions']['_menu_setup'] = (array)$this->getMenuSetupLink();
		//return $other_menu;
		return $other_menu['menu']['actions'];
	}
	
	/** 
	 * @return array $menu_setup An array with 'link' and 'label' keys for a Menu Setup link
	 */
	function getMenuSetupLink() {
		global $L;
		$link = array(
			'link' => 'modules/Front/admin_dashboard_layout.php',
			'label' => $L['frontDashboardDropdownValues']
		);
		return $link;
	}
	
	/**
	 * Builds the menu array for the current user
	 * 
	 * Sets this as a class variable for internal use
	 *
	 * @return array An array of modules and (potential) sub menu items
	 * @author Phil Calder
	 */
	function getUsersMenus() {
		global $L;
		$menus = array();
		
		// Phil 20060208 Hardcoding a link for the desktop
		$menus['FrontHome']['label'] = $L['LabelDesktop'];
		$menus['FrontHome']['link'] = 'modules/Front/desktop.php';
		// Phil 20061028 Adding Filters (Mine | All)
		$menus['FrontHome']['menu'] = array();
		$menus['FrontHome']['menu']['filters'] = array(
			array(
				'link' => 'modules/Front/desktop.php?piece_filter_type=', 
				'label' => 'LinkAll'
			),
			array(
				'link' => 'modules/Front/desktop.php?piece_filter_type=mine', 
				'label' => 'labelQfilterMine'
			)
		);
		// Phil 20070416 Add Display options (Standard Alternative)
		$menus['FrontHome']['menu']['actions'] = array(
			array(
				'link' => 'modules/Front/desktop.php?alt_data_option=', 
				'label' => 'LinkStandardView'
			),
			array(
				'link' => 'modules/Front/desktop.php?alt_data_option=1', 
				'label' => 'LinkAlternativeView'
			)
		);
		//MCR add Other Menu to 'Desktop' menu
		$menus['FrontHome']['menu']['other'] = $this->getOtherMenuModules();
		
		
		// get all the modules the current user has in their 'menu'
		if (is_array($modules = $this->getUsersMenuModules())) {
			foreach ($modules as $k => $v) {
				$name = $v['name'];
				$menus[$name] = array();
				// 'label'
				if ($label = $this->getModuleLabel($name)) {
					$menus[$name]['label'] = $label;
				}
				// 'link'
				if ($link = $this->getModuleLink($name)) {
					$menus[$name]['link'] = $link;
					// . '?filter=fnone&qfilter=&search_filter=off'
				}
				// 'alert' - will be populated by doAlerts (javascript)
				// 'menu'
				$menus[$name]['menu'] = array();
				
				// 'qfilters'
				if (is_array($qfilters = $this->getQfilters($name))) {
					$menus[$name]['menu']['qfilters'] = $qfilters;
					// print_r($qfilters); print '<hr>';
				}
				
				// 'actions'
				if (is_array($actions = $this->getActions($name))) {
					$menus[$name]['menu']['actions'] = $actions;
				}
				
				// 'filters'
				if (is_array($filters = $this->getFilters($name))) {
					// -- Phil 20060131 Add no filter link ('All')
					if (isset($filters['display_no_filter_link']) && $filters['display_no_filter_link']) {
						global $L;
						$index_url = $this->getModuleLink($name);
						$label = $L['LinkAll'];
						$filters[] = array(
							'label' => $label, 
							'link'  => $index_url . '?filter=fnone'
						);
					} 
					$menus[$name]['menu']['filters'] = $filters;
				}
				
				// other links - defined in config
				if (is_array($other_links = $this->getOtherLinks($name))) {
					$menus[$name]['menu']['other'] = $other_links;
				}
				// bookmarks
				if (is_array($bookmarks = $this->getMenuBookmarks($name))) {
					if (count($bookmarks)) {
						foreach ($bookmarks as $kk => $v) {
							$bookmarks[$kk]['link'] = $bookmarks[$kk]['href'];
							$bookmarks[$kk]['label'] = $bookmarks[$kk]['text'];
							bcrm_unset($bookmarks[$kk], 'href');
							bcrm_unset($bookmarks[$kk], 'text');
						}
						$menus[$name]['menu']['bookmarks'] = $bookmarks;
					}
				}
			}
		}
		//$menus['Rest'] = $this->getOtherMenuModules();
		$this->menus = $menus;
		return $menus;
	}

	//MCR
	function getMenuBookmarks($module_name)
	{
		global $SERVICES;
		if (is_object($system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks'))) {
			$links = $system_bookmarks_mod->getBookmarkLinks(false, false, $module_name);
			return $links;
		}
	}
	
	/**
	 * @param string $module_name The name of the module
	 * @return mixed false or an array: An array of query-filters. Relevant keys are 'label' and 'link'.
	 * @author Phil Calder
	 */
	function getQfilters($module_name) {
		// This was originally overridden to return only alerts
		// as of v5 returns all qfilters
		$qfilters = parent::getQFilters($module_name);
		// print_r($qfilters); print '<hr>';
		return (is_array($qfilters) ? $qfilters : false);
		// if (is_array($qfilters) && isset($qfilters['alert'])) {
			// return array($qfilters['alert']);
		// }
		// return false;
	}
	
	/**
	 * @return string The default target frame name
	 * @author Phil Calder
	 */
	function getDefaultTarget() {
		return 'contentframe';
	}
	
	function getNotice() {
		$content = '';
		return '<td>' . $content . '</td>';
	}
	
	function getSearchForm() {
		global $SERVICES, $USER, $L, $skin_path;
		$ua =& $SERVICES->getService('Authentication');
		if ($ua->readAccess($USER['user_id'],'Search')) {
			// Phil 20070528
			$search =& $SERVICES->modules->getModule('Search');
			return $search->getSearchForm(false, true, true); // 3rd param - called from sidebar
		}
		return '';
	}
	
	function getTopSearchForm() {
		global $SERVICES, $USER, $L, $skin_path;
		$ua =& $SERVICES->getService('Authentication');
		if ($ua->readAccess($USER['user_id'],'Search')) {
			
			// Phil 20070528
			//$search =& $SERVICES->modules->getModule('Search');
			//return $search->getSearchForm(false, true, true); // 3rd param - called from sidebar
			
			$search_type = $SERVICES->preferences->getUserPref('searchType');
			return '
			<table border="0" cellspacing="0" cellpadding="0">
			<tr><td>
			<form action="modules/Search/index.php" target="contentframe" style="margin:0;" id="searchForm" onClick="parent.window.Layout.focusContentPanel()">
			<select name="item[search_type]" class="dull" id="search-select">
			<option value="initial"' . ($search_type == 'initial' ? ' selected' : '') . '>' . $L['frontSearchInitialMatch'] . '</option>
			<option value="contains"' . ($search_type == 'contains' ? ' selected' : '') . '>' . $L['frontSearchContaining'] . '</option>
			</select>
			<input name="item[search_keywords]" type="text" id="search-input" value="" class="dull" />
			</td><td>
			<span style="float:right;" id="searchbutton"></span>
			</form>
			</td></tr>
			</table>
			';
			//onfocus="searchInput(1);" onblur="searchInput(0);" 
			//<input name="button" type="button" id="SearchBoxButton" onclick="document.getElementById(\'searchForm\').submit(); return false;" onmousedown="this.className += \' down\';" onmouseup="this.className = \'CommonForm\';" onmouseout="this.className = \'CommonForm\';" value="' . $L['frontSearchGo'] . '" />
			 
			// //		  &nbsp; <a href="modules/Search/?advanced=1" target="contentframe" title="' . $L['frontAdvancedSearchLinkTitle'] . '">' . $L['frontAdvancedSearch'] . '</a>
	
		}
		return '';
	}
	
	
	function getApplicationTitle() {
		global $L;
		return $L['ApplicationTitle'];
	}
	
	function getMenuTitle() {
		global $L;
		return $this->getApplicationTitle() . ' ' . $L['frontLabelMenu'];
	}
	
	function getPromoText() {
		global $L, $SERVICES;
		// Do not alter the next line (commented out)
		// return ''; // disable promo links
		
		$content = '';
		if ($SERVICES->showPromoMessages()) {
			// Phil if it's our demo use these
			if ($SERVICES->getGlobal('http_host') == 'demo.browsercrm.com') {
				//20070824 changed 'download' to 'register' for v5
				$content .= '<div><a href="' . $SERVICES->getContactUrl('register') . '" target="_blank">' . $L['frontPromo1'] . '</a></div>';
				$content .= '<div><a href="' . $SERVICES->getContactUrl('hosted') . '" target="_blank">' . $L['frontPromo4'] . '</a></div>';
			} else {
				//echo $SERVICES->getNewsUrl('promo');
				if ($SERVICES->config->lookup('login.skip_newsfeed')) {
					return $content;
				}
				$promo = str_replace('__l__', $SERVICES->getContactUrlQueryString(),$SERVICES->getDataFromUrl($SERVICES->getNewsUrl('promo')));
				if ($promo == '1') {
					$promo = '';
				}
				$content .= $promo;
			}
		}
		return $content;
	}
	
	//MCR 20070314 added $dir_replace and functionality 
	//for new layout where this html sits at doc_root level
	function getLinks($dir_replace = '') {
		global $SERVICES, $L;
		$content = '';
		if (!$SERVICES->config->lookup('login.skip_newsfeed')) {			
			// Phil 20060720 Display upgrade link if applicable
			$current_version = $SERVICES->config->lookup('app.version');
			$new_version = str_replace('__l__', $SERVICES->getContactUrlQueryString(),$SERVICES->getDataFromUrl($SERVICES->getNewsUrl('version')));
			// $new_version = '4.605.01'; // TODO: remove (DEBUG)
			if ($current_version && $new_version && (strlen($new_version) >=5)) {
				if (floatval($new_version) > floatval($current_version)) {
					$upgrade_url = $SERVICES->getContactUrl('upgrade_version');
					$upgrade_title_text = $L['frontLinkUpgradeToVersion'] . ': ' . $new_version;
					$content .= '<a href="' . $upgrade_url . '" target="_blank" title="' . $upgrade_title_text . '" class="error">' . $L['frontLinkUpgrade'] . '</a>';
				}
			}
		}
		
		// $content .= '<a href="#" onclick="javascript:xajax_refreshAlerts();">Test xajax</a>';
		// $content .= '<a href="#" onclick="javascript:MainMenu.setAlerts();">Test setAlerts function</a>';
		
		$content .= '<a href="' . str_replace($dir_replace, '', $SERVICES->config->lookup('modules.Front.modules.link')) . '" target="contentframe" onClick="parent.window.Layout.focusContentPanel()">' . $L['LinkSettings'] . '</a>';
		
		//$content .= '<a href="javascript:Layout.showAbout();" id="aboutLink">' . $L['frontAbout'] . '</a>';
		$content .= '<a href="' . str_replace($dir_replace, '', $SERVICES->config->lookup('modules.Front.about.link')) . '" target="contentframe" onClick="parent.window.Layout.focusContentPanel()">' . $L['frontAbout'] . '</a>';
		
		// $content .= '<a href="' . $SERVICES->config->lookup('modules.Front.help.link') . '" onclick="javascript:loadhelp(\'header\', \'\')" target="contentframe">' . $L['frontHelp'] . '</a>';
		//$content .= '<a href="#" onclick="javascript:loadhelp(\'header\', \'\')">' . $L['frontHelp'] . '</a>';
		$content .= '<a href="javascript:Layout.showHelp();">' . $L['frontHelp'] . '</a>';
		$content .= '<a href="' . $SERVICES->getContactUrl('supportrq') . '" title="' . $L['frontWebSupportExplanation'] . '"target="_blank">' . $L['frontWebSupport'] . '</a>';
		//MCR 20060110 str_replace to assure clean url
		$content .= '<a href="' . str_replace($dir_replace, '', str_replace('index.php', '', $SERVICES->config->lookup('modules.Front.logout.link'))) . '">' . $L['frontLogOut'] . '</a>';
	
		return $content;
	}
	
	function getAlertRow() {
		if (!is_array($this->menus)) {
			$this->getUsersMenus();	
		}
		$content = '';
		foreach ($this->menus as $title => $settings) {
			$content .= '
			<td class="item" onmouseover="Item(\'' . $title . '\', 1);" onmouseout="Item(\'' . $title . '\', 0);" onclick="Select(\'' . $title . '\'); gotoPg(\'' . ( isset($settings['link']) ? $settings['link'] . '?qfilter=alert' : 'false' ) . '\');" title="No Alerts">
			 <div id="alert_' . $title . '" class="item-normal">
			  &nbsp;
			 </div>
			</td>
			';
		}
		return $content;
	}
	
	function getTitleRow() {
		if (!is_array($this->menus)) {
			$this->getUsersMenus();	
		}
		$content = '';
		foreach ($this->menus as $title => $settings) { 
			 $content .= '
			 <td class="item-normal" onmouseover="Item(\'' . $title . '\', 1);" onmouseout="Item(\'' . $title . '\', 0);" onclick="Select(\'' . $title . '\'); gotoPg(\'' . ( isset($settings['link']) ? $settings['link'] : 'false' ) . '\');" id="title_' . $title . '"><a href="' . ( isset($settings['link']) ? $settings['link'] : 'javascript: return false;' ) . '" target="' . $this->getDefaultTarget() . '">' . ( isset($settings['label']) ? $settings['label'] : $title ) . '</a></td>
			 ';
		}
		return $content;
	}
	
	// //template - what is required for ext (simple example)
	// function XXXgetMenu() {
		// $content .= '
			// tb.add(
				// new Ext.Toolbar.MenuButton({
				// text: \'' . ( isset($settings['label']) ? $settings['label'] : $title ) . '\',
				// handler: onButtonClick,
				// cls: \'x-btn-text-icon blist\',
				// menu : {items: [
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'},
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'},
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'}
						// ]}
				// })
			// ),
			// tb.add(
				// new Ext.Toolbar.MenuButton({
				// text: \'' . ( isset($settings['label']) ? $settings['label'] : $title ) . '\',
				// handler: onButtonClick,
				// cls: \'x-btn-text-icon blist\',
				// menu : {items: [
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'},
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'},
						// {text: \'<u>Admin</u>\', href: \'./modules/Tasks/index.php\', hrefTarget: \'contentframe\'}
						// ]}
				// })
			// )
			// ;
			// ';
		// 
		// return $content;
	// }
	
	//MCR 20070314
	//convert either of the 2 formats in use
	function remapLink($link) {
		if (substr($link, 0, 6) == '../../') {
			$link = str_replace('../../', '', $link);
		} elseif ((substr($link, 0, 3) == '../') && (substr($link, 0, 10) != '../modules')) {
			$link = str_replace('../', 'modules/', $link);
		}
		return $link;
	}
	
	//this is currently the content for the 
	function getMenuButtonFunctions() {
		global $L;
		if (!is_array($this->menus)) {
			$this->getUsersMenus();	
		}
		$target = $this->getDefaultTarget();
		$content = '';
		
		$content .= "
		function focusContentPanel() {
			parent.window.Layout.focusContentPanel();
		}
		
		";
		
		foreach ($this->menus as $title => $settings) {
			$content .= "
			function onButtonClick" . $title . "(btn){ 
				parent.window.Layout.focusContentPanel();
				frames['contentframe'].location.href = '" . ( isset($settings['link']) ? $this->remapLink($settings['link']) : 'false' ) . "';
			}";
		}
		return $content;
	}
	//MCR 20070314
	function getMenu() {
		global $L, $USER;
		if (!is_array($this->menus)) {
			$this->getUsersMenus();	
		}
		$target = $this->getDefaultTarget();
		$content = '';
		
		//pop FrontHome (Desktop) menu
		//$menus['FrontHome']
		
		
		
		//$this->menus = array();
		$count = 0;
		foreach ($this->menus as $title => $settings) {
			//$content .= '<div id="items_' . $title . '" class="hidden">';
			//echo $title . ', ';
			// Phil - dont display a MenuButton unless we have a menu
			$has_menu = (is_array($settings) && is_array($settings['menu']) && count($settings['menu']));
			$class_name = ( $has_menu ? 'MenuButton' : 'Button');
			
			$content .= '
			mainmenu.add(
				new Ext.Toolbar.' . $class_name . '({
				handler: onButtonClick' . $title . ',
				id: \'mainmenuItem' . $title . '\',
				';
				
			if ($title == 'FrontHome') {
				$content .= 'icon: \'' . $USER['skin_web_path'] . 'img/desktop.png\',
				text: \'' . ( isset($settings['label']) ? $settings['label'] : $title ) . '\',
				cls: \'x-btn-text-icon blist\'';
				
				//text: \'&nbsp;\',
				// tooltip: \'<b>Desktop</b><br/>Icon only button with tooltip\',
			} else {
				// shows alert icon
				//$content .= 'icon: \'skins/plain/img/menualert.gif\',
				//text: \'' . ( isset($settings['label']) ? $settings['label'] : $title ) . '\',
				//cls: \'x-btn-text-icon blist\'';
				
				// not alerting - set in /index.inc.php
				$content .= 'text: \'' . ( isset($settings['label']) ? $settings['label'] : $title ) . '\',';
				if ($settings['icon']) {
					//$content .= 'icon: \'' . $settings['icon'] . '\',';
				}
				//$content .= 'cls: \'x-btn-text blist\'';  //last param, no comma
				if (in_array($title, array('Person', 'Company'))) {
					$content .= 'cls: \'x-btn-text blist\'';  //last param, no comma
				} else {
					$content .= 'cls: \'x-btn-text-icon alert-off blist\'';  //last param, no comma
				}
				
				
				
			}
			//skins/plain/img/menualert.gif
			
			if ($has_menu) {
					
				$content .= ',
				menu : {items: [';
				foreach ($settings['menu'] as $k => $submenu) {
					if (is_array($submenu) && count($submenu)) {
						// print '<div>target=" ' . $target . '" k=' . $k . '</div>'; print_r($submenu); print '<hr>';
						foreach ($submenu as $subkey => $values) {
							// print $subkey . '<br>'; print_r($values); print '<hr>';
							if ($values['link']) {
								if (isset($values['label']) && $values['label']) {
									$label = $values['label'];
									if ($L[$values['label']]) {
										$label = $L[$values['label']];
									}
								} else {
									$label = $subkey;
								}
								// $idTxt = '';
								// // WORK OUT IF A ID IS REQUIRED
								// if (isset($settings['idInc'])) {
									// $idTxt = ' id="link_' . $subkey . '" title="No Alerts"';
								// } else if ($k == "qfilters") {
									// $idTxt = ' id="link_' . $title	 . '"';
								// }
								// $content .= '
									// <a href="' . $values['link'] . '"' . $idTxt . ' target="' . $this->getDefaultTarget() . '" onclick="Select(\'' . $title . '\');">' . $label . '</a>
								// ';
								//$values['icon'] = 'new_company.png';
								//$content .= "
								//{text: 'Admin', href: './modules/Tasks/index.php', hrefTarget: 'contentframe'},";
								
								// id: 'submenuItem" . $values['mod'] . "', 
								$content .= "
								{text: '" . $label . "'," .
								//($values['icon'] ? "icon: '" . $values['icon'] . "'," : "")	. 
								 "href: '" . $this->remapLink($values['link']) . "',
								 hrefTarget: '" . $target . "', 
								 handler: focusContentPanel
								},";
							}
						}
						$content = trim($content, ',');
					}
					$content .= ", '-',";
				}
				$content = trim($content, ',');
				$content = trim($content, '\'');
				$content = trim($content, '-');
				$content = trim($content, '\'');
				$content = trim($content, ' ');
				$content = trim($content, ',');
				$content .= '
					]},';
			}
			$content = trim($content, ',');
			$content .= '
				})
			),';
			$count++;
			//echo $count;
			//break;;
		}
		$content = trim($content, ',');
		
		$content .= ';';
		
		// function setAlerts(id, value) {
			// layout.getRegion(\'north\').tb.items.get(\'menuItem_Contacts\').disable();
		// }
		// ';
		
		return $content;
	}
	
	
	//pre v5
	/*
	function getMenuRow() {
		global $L;
		if (!is_array($this->menus)) {
			$this->getUsersMenus();	
		}
		$content = '';
		foreach ($this->menus as $title => $settings) {
			$content .= '<div id="items_' . $title . '" class=getGettingStartedContent"hidden">';
			// Phil 20050111
			if (is_array($settings) && is_array($settings['menu']) && count($settings['menu'])) {
				foreach ($settings['menu'] as $k => $submenu) {
					$content .= '
					  <span  class="' . $k . '">
					';
					if (is_array($submenu) && count($submenu)) {
						// print '<div>k=' . $k . '</div>'; print_r($submenu); print '<hr>';
						foreach ($submenu as $subkey => $values) {
							// print $subkey . '<br>'; print_r($values); print '<hr>';
							if ($values['link']) {
								if (isset($values['label']) && $values['label']) {
									$label = $values['label'];
									if ($L[$values['label']]) {
										$label = $L[$values['label']];
									}
								} else {
									$label = $subkey;
								}
								$idTxt = '';
								// WORK OUT IF A ID IS REQUIRED
								if (isset($settings['idInc'])) {
									$idTxt = ' id="link_' . $subkey . '" title="No Alerts"';
								} else if ($k == "qfilters") {
									$idTxt = ' id="link_' . $title	 . '"';
								}
								$content .= '
									<a href="' . $values['link'] . '"' . $idTxt . ' target="' . $this->getDefaultTarget() . '" onclick="Select(\'' . $title . '\');">' . $label . '</a>
								';
							}
						}
					}
					$content .= '
					  </span>
					';
				}
				
			}
			$content .= '</div>';
		}
		return $content;
	}
	*/
	
} // FrontMenuManager

/**
 * Handles functionality for the Front module 'modules' page
 * @author Phil Calder
 * @package Front 
 */
class FrontModulesManager extends FrontManager {
	
	/**
	 * var $output
	 * @access private
	 */
	var $output;

	/**
	 * var $is_menu
	 * @access private
	 * @author Matthew Ross
	 */
	var $is_menu = false;
	
	function FrontModulesManager() {
		$this->init();
	}	
	
	function init() {
		parent::init();
		global $SERVICES;
		$this->output =& $SERVICES->getService('Output');
		
		if (isset($_GET['is_menu']) && $_GET['is_menu'] == 1) {
			$this->is_menu = true;
		}
	}

	/**
	 * @return string The default target frame name
	 * @author Phil Calder
	 */
	function getDefaultTarget() {
		return '_self';
	}

	/**
	 * @return array of links the current user can see on the modules page,
	 * key is a module name, values are arrays of links.
	 */
	function getLinksList() {
		global $SERVICES, $USER, $errorlog;
		$list = $SERVICES->config->lookup('modules.Admin.modulelink');
		// $list is an array of module links
		// a module link is an array with a "priority" and an "href"
		$ua =& $SERVICES->getService('Authentication');
		$modules = $ua->getPrivileges();
		// Set default module ordering here
		$answer = array(
			'Front' => '', 
			'Admin' => '', 
			'UserAdmin' => '',
		);

		// CODE FOR SORTING ACCORDING TO PRIORITY:
		if (is_array($list)) {
			foreach ($list as $k => $v) {
				if (is_array($v)) {
					// print_r($v); print '<hr>';
					$module = $v['module'];
					$index = 100 + intVal($v['priority']);
					$index .= $k;  // so that comparison is lexigraphical & indexes are unique
					if ( !$module || (isset($modules[$module]) && ($modules[ $module ] & $ua->getAdminLevel())) ) {
						// user has access to the module, so return the link
						$answer[$module][ $index ] = $v; // Phil 20040924 - now sorted first by module
					}
				} else {
					$SERVICES->getService('Log');
					$SERVICES->log->logError(
						'error occured while loading Module links, module link does not have "href" and "priority" attributes.',
						'Modules'
					);
				}
			}
		}
		foreach ($answer as $k => $module) {
			if (is_array($answer[$k])) {
				ksort($answer[$k]);
			}
		}
		return $answer;
	}
	
	function getDropdown($values, $settings = false) {
		if (!is_array($settings)) {
			$settings = array(
				'name' => 'dropdown',
				'href_key' => 'link',
				'label_key' => 'label', 
				'incl_empty' => true
			);
		}
		
		$content = '';
		if (is_array($values) && !empty($values)) {
			$options = '';
			if ( $settings['incl_empty'] && ($values['options'] && count($values['options'])) ) {
				$options .= '<option value="javascript:return false;"></option>';
			}
			foreach ($values['options'] as $k => $v) {
				if (is_array($v)) {
					$options .= '<option value="' . $v[$settings['href_key']] . '">' . $v[$settings['label_key']] . '</option>';
				}
			}
			if (strlen($options)) {
				$content .= '<select id="' . $settings['name'] . '" name="' . $settings['name'] . '" onChange="window.location=this.value;">' . $options . '</select>';
			} else {
				$content = false;
			}
		}	
		return $content;
	}
	
	function getAdminLinksOfModule($module_name, $module_links = array()) {
		global $SERVICES, $L;
		if (is_array($module_links) && !empty($module_links)) {
			$other_links_rows = '';
			foreach ($module_links as $mp => $mv) {	
				if (is_array($mv) && !empty($mv)) {
					//MCR 20060506 because links array can now be merged modules Front,Admin,UserAdmin
					if (!$module_name) {
						$admin_link_label = $L[$mv['label']];
					} else {
						$admin_link_label = ($label = $SERVICES->language->getLanguageText($mv['label'], $module_name)) ? $label : $mv['label'];
					}
					$other_links_rows .= $this->output->getLinkHref(array('href' => $mv['link'], 'text' => $admin_link_label)) . ' &nbsp; '; //<br>
				}
			}
			return $other_links_rows;
		} else {
			return '&nbsp;';
		}
	}
	
	function getListLink($module_name, $label_key = 'ListView') {
		global $L;
		$list_link = false;
		if ($index_url = $this->getModuleLink($module_name)) {
			$label = $L[$label_key];
			$list_link = $this->output->getLinkHref(
				array(
					'text' => $label, 
					'href'  => $index_url . '?filter=fnone&qfilter=&search_filter=off'
				)
			);
		}
		return $list_link;
	}
	
	/**
	 * Creates the module table content and returns
	 *
	 */	
	function getModuleContent($key, $module_name, $module_links = array(), $show_back = false) {
		// Phil 20060505
		global $SERVICES, $L;
		$module_content = '';
		$content = '';
		if (!empty($module_name)) {
			// Get Recent Items - Phil 20060207 On hold for now
			// -------------------------------------------------	
			// Get Preferences Links
			$pref_links = $this->getPrefLinks($module_name);
			$p_row = array();	
			$p_row[] = $SERVICES->language->getLanguageText('useradminPreferenceMany', 'UserAdmin');
			foreach ($pref_links as $pk => $pv) {
				if(is_array($pv) && !empty($pv)) {
					$pref_label = ($label = $SERVICES->language->getLanguageText($pv['label'], $pv['module'])) ? $label : $pv['label'];
					$preference_rows .= $this->output->getLinkHref(array('href' => $pv['link'], 'text' => $pref_label)) . '&nbsp; ';
				}
			}
			$p_row[] = $preference_rows;
			
			// Customise
			$no_module_editor_link = (array)$SERVICES->config->lookup('modules.Admin.module_editor.exclusions');
			// Get Module Editor Link
			if ($this->include_customise && !in_array($module_name, $no_module_editor_link) && strlen($module_editor_link = $this->output->getModuleEditorLink($module_name))) {
				$customise = array();
				$customise[] = $L['LinkCustomize'];
				$customise[] = $module_editor_link;
				// Add customise links to content
				// $module_content .= $this->output->getTableRow($customise);
			} else {
				$customise = array();
				$customise[] = '&nbsp;';
				$customise[] = '&nbsp;';
			}
			// Get Other Links	
			$other_links = array();
			$other_links[] = $L['LinkSettings'];
			
			// Get Admin Links of the Module
			$other_links[] = $this->getAdminLinksOfModule($module_name, $module_links);
			
			// Get Bookmarks // Phil 20060917 Not currently used
			// if (is_object($system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks'))) {
				// $links = $system_bookmarks_mod->getBookmarkLinks(false, false, $module_name);
				// $bookmarks = array(
					// 'options' => $links
				// );
				// $settings = array(
					// 'name' => $module_name . '_bookmarks', 
					// 'href_key' => 'href', 
					// 'label_key' => 'text', 
					// 'incl_empty' => true
				// );
				// 
				// if ( ($bookmark_dropdown = $this->getDropdown($bookmarks, $settings)) !== false) {
					// $bookmark_row = array(
						// $L['LabelBookmarks'], 
						// $bookmark_dropdown
					// );
				// }
			// }
		
			$title_menu = array(
				'settings' => 'off', 
				'print_view' => 'off', 
				'bookmark' => 'off', 
				'back' => ($show_back ? '' : 'off'), // Phil, see param
				'help' => -1, 
				'refresh' => 'off'
			);
			
			// Get Actions
			$actions = $this->getActions($module_name);	
			if (is_array($actions) && !empty($actions)) {
				foreach ($actions as $ak => $av) {
					if (is_array($av) && !empty($av)) {
						if (isset($av['label']) && isset($av['link'])) {
							$action_label = ($label = $SERVICES->language->getLanguageText($av['label'], $module_name)) ? $label : $av['label'];
							$title_menu[] = $this->output->getLinkHref(array('href' => $av['link'], 'text' => $action_label));
						}
					}
				}
			}
			
			// List link
			$list_link = $this->getListLink($module_name);
			$title_menu[] = $list_link;
			
			//Get QFilters	
			$qfilters = $this->getQFilters($module_name);
			if (is_array($qfilters) && !empty($qfilters)) {
				foreach ($qfilters as $qk => $qv) {
					if (isset($qv['label']) && isset($qv['link'])) {
						
						$q_label = $SERVICES->language->getLanguageText($qv['label'], $module_name);
						// $title_menu[] = $this->output->getLinkHref(array('href' => $qv['link'], 'text' => $q_label, 'target' => $qv['target'], 'tag_extras' => $qv['tag_extras']));
						$title_menu[] = $this->output->getLinkHref(array('href' => $qv['link'], 'text' => $q_label));

					}
				}
			}
			
			// Get Help Link
			$help_id = strtolower( substr($module_name, 0, 1)) . substr($module_name, 1) . 'Index';
			if(strlen($help_link = $this->output->getLinkHelp($help_id))) {
				$title_menu[] = $help_link;
			}
			
			if ($key !== false) {
				$content .= '<div id="' . $key . '"">'; //  style="visibility:hidden
			}
			
			$module_pieces = '';
			$module_pieces .= '<tr>';
			if (is_array($p_row)) {
				$module_pieces .= '<th>' . $this->getModuleLabel($module_name) . '</th>';
				$module_pieces .= '<td class="values" nowrap>' . $list_link . ($list_link ? ' &nbsp; ' : '') . $help_link . '</td>';
				$module_pieces .= '<td class="values" nowrap>' . $p_row[1] . '</td>';
				if (!$this->is_menu) {
					$module_pieces .= '<td class="values">' . $other_links[1] . '</td>';
					$module_pieces .= '<td class="values" nowrap>' . $customise[1] . '</td>';
				}
			}
			$module_pieces .= '</tr>';
			return $module_pieces;
		}
		return $content;
	}
	
	function getPrefLinks($module_name) {
		global $SERVICES, $USER;
		$pref_links = array();
		if (!empty($module_name)) {
			// user prefs
			if (is_array($user_pref_link = $SERVICES->config->lookup('modules.Front.modules.userprefs')) && !empty($user_pref_link['link'])) {
				$user_pref_link['link'] .= '?module=' . $module_name;
				$pref_links[] = $user_pref_link; 
			}
			if (!$this->is_menu) {
				// company Prefs
				if ($SERVICES->authentication->adminAccess($USER['user_id'], $module_name) && is_array($company_pref_link = $SERVICES->config->lookup('modules.Front.modules.companyprefs')) && !empty($company_pref_link['link'])) {
					$company_pref_link['link'] .= '?module=' . $module_name;
					$pref_links[] = $company_pref_link; 
				}
			}
		}
		return $pref_links;
	}
	
	function makeContent($links) {
		global $SERVICES, $L;
		$this->output->req_contact_js = true;
		
		// Do not alter the following line
		$this->include_customise = 1;
		
		$content = '';
		//need to get Langs before if block
		$module_privs = $SERVICES->authentication->getPrivileges();
		// print_r($module_privs); print '<hr>';
		$modules = array_keys($module_privs);
		sort($modules); 		
		foreach (array('Front', 'Admin', 'UserAdmin') as $k => $v) {
			$SERVICES->language->getLanguage($v);
			if ( ($key = array_search($v, $modules)) !== false) {
				unset($modules[$key]);
			}
		}
		
		// Phil 20060508 Dont display these modules in the Settings page
		if (!$this->include_customise) {
			$exclude_other_modules = $this->getModulesToExclude();
			foreach ($exclude_other_modules as $k => $v) {
				if ( ($key = array_search($v, $modules)) !== false) {
					unset($modules[$key]);
				}
			}
		}
		
		// Phil 20060918
		// Remove modules the user does not have at least read access to
		foreach ($modules as $k => $v) {
			// print '<div>modules[' . $k . '] = ' . $v . '</div>';
			if (isset($module_privs[$v]) && !$module_privs[$v]) {
				// print '<div>REMOVING modules[' . $k . '] = ' . $v . '</div>';
				bcrm_unset($modules, $k);
			}
		}
		
		// --
		$module = false;
		if (isset($_GET['module']) && $_GET['module']) {
			$module = $_GET['module'];
			$module_content = $this->getModuleContent(false, $module, $links[$module], true);
		
		} else {
			//$content = $this->makeContent($links);
			//$admin_links = bcrm_array_merge($links['Front'], $links['Admin'], $links['UserAdmin']);
			//$admin_links = bcrm_array_merge($links['Front'], $links['Admin']);
				// print_r($admin_links);
				// [100about] => Array
					// (
					    // [priority] => 0.2
					    // [link] => modules/Front/about.php
					    // [label] => frontAbout
					    // [module] => Front
					// )
			//TODO sort on priority
			//if (!$this->is_menu) {
				$front_admin_content = $this->getAdminLinksOfModule(false, $links['Front']);
			//}
			//TODO manually add link to just-Front-prefs here 
			//$front_admin_content .= ??
			
			//if we arrived here from Dashboard settings, we already have _GET['module']= Front
			//otherwise we add this custom link
			if (!$_GET['module'] == 'Front') {
				global $SERVICES;
				if (is_array($user_pref_link = $SERVICES->config->lookup('modules.Front.modules.userprefs')) && !empty($user_pref_link['link'])) {
					$user_pref_link['link'] .= '?module=Front'; // . $module_name;
					//$label = $SERVICES->language->getLanguageText($user_pref_link['label'], 'Front');
					$label = $L['frontSetGeneralPreferences'];
					$front_pref_link = '<a href="' . $user_pref_link['link'] . '">' . $label . '</a>';
					$front_admin_content = $front_pref_link . ' &nbsp; ' . $front_admin_content;
				}
			}
			
			if ( !$this->include_customise) {
				$remove = array(
					'modules/Admin/admin_module_index.php', 
					'modules/Admin/admin_module_create.php',
					'modules/Admin/admin_module_remove.php'
				);
				if (is_array($links['Admin']) && count($links['Admin'])) {
					foreach($links['Admin'] as $k => $v) {
						if ( isset($v['link']) && in_array($v['link'], $remove) ) {
							unset($links['Admin'][$k]);
						}
					}
				}
			}
			
			$admin_content = $this->getAdminLinksOfModule(false, $links['Admin']);
			$useradmin_content = $this->getAdminLinksOfModule(false, $links['UserAdmin']); 
			
			if (!$this->is_menu) {
				$window_layout .= '<tr><th>' . $L['frontUserSettings'] . '</th><td>' . $front_admin_content .'</td></tr>';
				$window_layout .= $this->output->getSpacerRow();
				if (trim($admin_content) && ($admin_content != '&nbsp;')) {
					$window_layout .= '<tr><th>' . $L['frontInstallationSettings'] . '</th><td>' . $admin_content .'</td></tr>';
					$window_layout .= $this->output->getSpacerRow();
				}
				if (trim($useradmin_content) && ($useradmin_content != '&nbsp;')) {
					$window_layout .= '<tr><th>' . $L['useradminModuleLabel'] . '</th><td>' . $useradmin_content .'</td></tr>';
					$window_layout .= $this->output->getSpacerRow();
				}
				//$window_layout .= $this->output->getTextRow('&nbsp;', 8);
				$window_layout .= $this->output->getEndTable();
				$window_layout .= '&nbsp;<br>';
				//$window_layout .= $this->output->getBeginTable('viewTbl');
				$window_layout .= '<table border="0" cellpadding="0" cellspacing="0" class="settingsTbl">';
				
			}
			
			//per module content
			$module_content = '';
			foreach ($modules as $k => $v) {
				$module_content .= $this->getModuleContent($k, $v, $links[$v]);
			}
		}
		
		if ($this->include_customise) {	
			$customize = $L['LinkCustomize'];
		} else {
			$customize = '&nbsp;';
		}
		
		$window_layout .= '<tr class=double><td>' . $L['adminModule'] . '</td><td>&nbsp;</td><td>' . $L['frontPreferences'] . '</td>';
		if (!$this->is_menu) {
			$window_layout .= '<td>' . $L['adminLinks'] . '</td><td>' . $customize . '</td>';
		}
		$window_layout .= '</tr>';	
		$window_layout .= $module_content;
		return $window_layout;
	}
	
	function main() {
		global $SERVICES, $USER, $L, $output;
		$links = $this->getLinksList();
		// $module = false;
		// if (isset($_GET['module']) && $_GET['module']) {
			// $module = $_GET['module'];
			// $content = $this->getModuleContent(false, $module, $links[$module], true);
		// } else {
			// $content = $this->makeContent($links);
		// }
		$content = $this->makeContent($links);
		$this->display($content);
	}
	
	/**
	 * Sends content to an Output display class or returns as a string
	 * 
	 * @param mixed $content A string or array containing content to display
	 * @author Phil Calder
	 * @see getReturnString
	 */
	function display($content = '') {
		global $SERVICES, $L;
	//	$window_layout = $this->output->getBeginTable('mainTbl', true);
		if (!$this->is_menu) {
			$window_layout .= $this->output->getTitleRow($L['LinkSettings']);
		} else {
			$window_layout .= $this->output->getTitleRow($L['frontDashboardModuleLinks']);
		}
	//	$window_layout .= $this->output->getBeginRow('', true) . '<td id="bodyCell" style="padding: 10px">';
	
		$window_layout .= $this->output->getBeginContainer('settings-container');
		//$window_layout .= $this->output->getBeginTable('settingsTbl');
		$window_layout .= '<table border="0" cellpadding="0" cellspacing="0" class="settingsTbl">';
		$window_layout .= $content;
		$window_layout .= $this->output->getEndTable();
		$window_layout .= $this->output->getEndContainer();
		
		//$window_layout .= '</td>' . $this->output->getEndRow(true);
//		$window_layout .= $this->output->getEndTable(true);
		
		
		$SERVICES->getService('Output');
		$SERVICES->output->addToBody($window_layout);
		$SERVICES->output->display();
		exit();
	}
} // FrontModulesManager
?>