<?php
/**
 * The main class for this module
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Logger
 */

includeModule('Logger', 'inc/database_tables.inc');

class Logger extends WebMATModule {
	function Logger() {
		$this->module_name = 'Logger';
		$this->WebMATModule();
		//for usage log
		global $SERVICES;
		$this->db =& $SERVICES->getService('Database');
	}
	
	
	/**
	 * Returns an array of session details beginnig at the specified start time
	 *
	 * @param $begin The begin date/time (optional)
	 * @return array of sessions
	 */
	function getSessionDetails($begin = '') {
		
		$sessions = $this->getSessions($begin);
		
		// print '<p>There are ' . sizeof($sessions) . ' sessions</p>';
		
		foreach ($sessions as $k => $session) {
			
			// print '<p>session_id: ' . $session['session_id'] . '</p>';
			if ($timestamp = $this->getSessionStartTime($session)) {
				
				// print '<p>timestamp: ' . $timestamp . ' for ' . $session['session_id'] . '</p>';
				$sessions[$k]['timestamp'] = $timestamp;
				
				unset($timestamp);
				
			}
			
			if ($page_count = $this->getSessionPageCount($session)) {
				
				// print '<p>page_count: ' . $page_count . ' for ' . $session['session_id'] . '</p>';
				$sessions[$k]['page_count'] = $page_count;
				
				unset($page_count);
				
			}
			
		}
	
		// print_r($sessions);
		
		return $sessions;
		
	}

	/** 
	 * Returns distinct session ids with their user ids starting at the specified start time
	 *
	 * @param $begin The start time (optional)
	 * @return array $sessions keys are session_id and user_id
	 */
	function getSessions($begin = '') {
		
		if ($begin == '') {
			
			$begin = (time() - 24*3600);
			
		}
		
		$result = $this->db->select(
			'DISTINCT session_id, user_id, remote_addr', 
			'Logger__usage', 
			'timestamp > ' . $begin, 'timestamp DESC , user_id'
		);
		
		$sessions = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$sessions[] = $row;
			
		}
		
		return $sessions;
		
	}
	
	/**
	 * Retuns a specific session
	 */
	function getSession($item) {
		
		if (! (isset($item['timestamp']) && isset($item['user_id']) && isset($item['session_id'])) ) {
			
			return false;
			
		}
		
		$where = 'timestamp > ' . $item['timestamp'] . ' AND session_id = "' . $item['session_id'] . '" AND user_id = ' . $item['user_id'] . '';
		
		// $where = 'timestamp > 0 AND session_id = "' . $item['session_id'] . '" AND user_id = ' . $item['user_id'] . '';
		// print '<p>WHERE: ' . $where . '<p>';
		
		$result = $this->db->select(
			'timestamp, module_name, page, remote_addr', 
			'Logger__usage', 
			$where, 
			'timestamp DESC'
		);
		
		$session = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$session[] = $row;
			
		}
		
		return $session;
		
	}
	
	/**
	 * Returns a session start time
	 */
	function getSessionStartTime($session) {
		
		if (is_array($session) && $session['session_id'] && $session['user_id']) {
			
			$timestamp = $this->db->quickGet(
				'timestamp', 
				'Logger__usage', 
				'session_id = "' . $session['session_id'] . '" AND user_id = "' . $session['user_id'] . '"', 
				'timestamp ASC'
			); //, 'LIMIT 1');
			
			// print '<p>timestamp: ' . $timestamp . '</p>';
			
			if ($timestamp > 0) {
				
				// print '<p>returning ' . $timestamp . '</p>';
				return $timestamp;
				
			}
			
		}
		
		// print '<p>returning false</p>';
		return false;
		
	}
	
	/**
	 * Returns a session page count
	 */
	function getSessionPageCount($session) {
		
		// error_reporting(E_ALL);
		
		if (is_array($session) && $session['session_id'] && $session['user_id']) {
			
			$timestamp = $this->db->quickGet(
				'COUNT(*)', 
				'Logger__usage', 
				'session_id = "' . $session['session_id'] . '" AND user_id = "' . $session['user_id'] . '"'
			);
			
			// print '<p>timestamp: ' . $timestamp . '</p>';
			if ($timestamp > 0) {
				
				// print '<p>returning ' . $timestamp . '</p>';
				return $timestamp;
				
			}
			
		}
		
		// print '<p>returning false</p>';
		return false;
		
	}
	
}
?>