<?php
require_once("../../prepend.inc");
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * Administrators can view the system usage log.
 *
 * @package Logger
 * @author Phil Calder
 */

$ua =& $SERVICES->getService('Authentication');
$ua->adminRestrict();

$output =& $SERVICES->getService('Output');

$SERVICES->getService('Modules');
$mod =& $SERVICES->modules->getModule('Logger');

$SERVICES->getService('Preferences');
$SERVICES->getService('DateLib');

/**
 * Displays an expanded view of a user 'session'
 */
function displayItem($item) {
	global $output, $mod, $SERVICES, $L, $period, $PHP_SELF;
	
	$session = array();
	if (isset($item['timestamp']) && isset($item['user_id']) && isset($item['session_id'])) {
		$session = $mod->getSession($item);
	}
	
	if ( sizeof($session) > 1 ) {
	} else {
		return false;
	}
	
	$content = '';
	// $menu = array('', $L['Date'], $L['Time'], '', $L['loggerModuleName'], $L['loggerPage']);
	$menu = array(
		'_tmp1' => array('label' => ''), 
		'date' => array('label' => $L['Date']), 
		'time' => array('label' => $L['Time']), 
		'_tmp2' => array('label' => ''), 
		'module' => array('label' => $L['loggerModuleName']), 
		'page' => array('label' => $L['loggerPage']),
		'remote_addr' => array('label' => $L['loggerRemoteAddr'])
	);
	
	$content .= $output->getHeaderRow($menu);
	
	foreach ($session as $line) {
		$item_row = array('', 'date'=>'', 'time'=>'', 'user_id'=>'', 'module_name'=>'', 'page'=>'');
		if (isset($line['timestamp'])) {
			$item_row['date'] = $SERVICES->datelib->getUserPrefDateFromTimestamp($SERVICES->offsetTimeZoneForRead($line['timestamp']));
			$item_row['time'] = $SERVICES->datelib->getUserPrefTimeFromTimestamp($SERVICES->offsetTimeZoneForRead($line['timestamp']));
		}
		
		$item_row['user_id'] = (isset($line['user_id']) ? $line['user_id'] : '');
		$item_row['module_name'] = (isset($line['module_name']) ? $line['module_name'] : '');
		$item_row['page'] = (isset($line['page']) ? $line['page'] : '');
		$item_row['remote_addr'] = (isset($line['remote_addr']) ? $line['remote_addr'] : '');
		$content .= $output->getTableRow($item_row);
		unset($item_row);
	}
	return $content;
}

function displayItems($expand = array()) {
	global $output, $mod, $SERVICES, $L, $period, $PHP_SELF;
	
	$default_max = 24*3600;
	
	if ($period) {
 		// $SERVICES->preferences->setUserPref('loggerPeriod', min($period, $default_max) );
		$SERVICES->preferences->setUserPref('loggerPeriod', $period);
	} else {
		// $period = min($SERVICES->preferences->getUserPref('loggerPeriod', $default_max), $default_max);
		$period = $SERVICES->preferences->getUserPref('loggerPeriod', $default_max);
	}
	
	if ($period < 1) { // entire log is -1
		$period = time() + 1;
	}
	
	$begin = time() - $period;
	
	$sessions = $mod->getSessionDetails($begin);
	
	$ua =& $SERVICES->getService('Authentication');
	
	$all_users = $ua->users->getUsersList();
	
	$content = '';
	
	$content .= $output->getTextRow($L['loggerSessionsInPeriod'] . ': ' . sizeof($sessions));
	
	// $menu = array('', $L['Date'], $L['Time'], $L['Username'], $L['loggerPageCount'], $L['loggerSessionID']);
	$menu = array(
	 	'_tmp1' => array('label' => ''), 
		'date' => array('label' => $L['Date']), 
		'time' => array('label' => $L['Time']), 
		'username' => array('label' => $L['Username']), 
		'page_count' => array('label' => $L['loggerPageCount']), 
		'session_id' => array('label' => $L['loggerSessionID']),
		'remote_addr' => array('label' => $L['loggerRemoteAddr'])
	);
	$content .= $output->getHeaderRow($menu);
	
	foreach ($sessions as $session) {
		
		$link = '<a href="' . $PHP_SELF . '?expand[timestamp]=' . (isset($session['timestamp']) ? $session['timestamp'] : '') . '&expand[user_id]=' . $session['user_id'] . '&expand[session_id]=' . $session['session_id'] . '">%link_text%</a>';
		$default = '<a href="' . $PHP_SELF . '">%link_text%</a>';
		
		$item_row = array('', 'date'=>'', 'time'=>'', 'user_id'=>'', 'page_count'=>'', 'session_id'=>'');
		
		if (isset($session['page_count']) && ($session['page_count'] > 1)) {
			if ( ($session['session_id'] == $expand['session_id']) && ($session['user_id'] == $expand['user_id']) ) {
				$item_row['date'] = str_replace('%link_text%', $SERVICES->datelib->getUserPrefDateFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])), $default);
				$item_row['time'] = str_replace('%link_text%', $SERVICES->datelib->getUserPrefTimeFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])), $default);
			} else {
				$item_row['date'] = str_replace('%link_text%', $SERVICES->datelib->getUserPrefDateFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])), $link);
				$item_row['time'] = str_replace('%link_text%', $SERVICES->datelib->getUserPrefTimeFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])), $link);
			}
			
		} else {
			$item_row['date'] = (isset($session['timestamp']) ? $SERVICES->datelib->getUserPrefDateFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])) : '');
			$item_row['time'] = (isset($session['timestamp']) ? $SERVICES->datelib->getUserPrefTimeFromTimestamp($SERVICES->offsetTimeZoneForRead($session['timestamp'])) : '');
		}
		
		if ($session['user_id']) {
			// $item_row['user_id'] = $session['user_id']; // $output->getLinkContact($item['contact_id']);
			$item_row['user_id'] = $all_users[$session['user_id']];
		}
		
		$item_row['page_count'] = (isset($session['page_count']) ? $session['page_count'] : '');;
		$item_row['session_id'] = $session['session_id'];
		$item_row['remote_addr'] = $session['remote_addr'];
		$content .= $output->getTableRow($item_row);
		
		if ( ($session['session_id'] == $expand['session_id']) && ($session['user_id'] == $expand['user_id']) ) {
			$retval = displayItem($expand);
			if ($retval) {
				$content .= $retval;
				$content .= $output->getHeaderRow($menu);
			}
		}
		
		unset($item_row);
		
	}
	
	return $content;
}


function doSubmit() {
	global $output, $mod, $SERVICES, $period;
	
	if (isset($period)) {
		$SERVICES->preferences->setUserPref('loggerPeriod', min($period, 24*3600) ); // User pref max is one day but can use greater value for this session
	} else {
		$period = $SERVICES->preferences->getUserPref('loggerPeriod',  24*3600);
	}
}

function getPeriodSelect() {
	global $period;
	
	$content = '<select name="period">';

	$options = array( 
		'60' => '1 minute', 
		'300' => '5 minutes', 
		'900' => '15 minutes', 
		'1800' => '30 minutes', 
		'3600' => '1 hour', 
		'18000' => '5 hours', 
		'43200' => '12 hours', 
		'86400' => '24 hours', 
		'172800' => '48 hours', 
		'604800' => '1 week', 
		'2419200' => '4 weeks', 
		'15768000' => '6 months', 
		'31536000' => '1 year', 
		'-1' => 'Entire Log' 
	);
	
	while (list($k,$v) = each($options)) {
		if ($k == $period){
			$content .= '<option value="' . $k . '" selected>' . $v. '</option>';
		} else {
			$content .= '<option value="' . $k . '">' . $v . '</option>';
		}
	}
	
	$content .= '</select>';
	return $content;
}

function main() {
	global $output, $mod, $SERVICES, $L, $period, $expand, $PHP_SELF;
	
	$content = '';
	
	$content .= $output->getBeginTable();
	$content .= $output->getTitleRow($L['loggerModuleLabel']);
	$begin_form = '<form name="log" action="' . $PHP_SELF . '" method="GET">';
	
	$select = getPeriodSelect();
	
	$submit = $output->getHtmlSubmit($L['ButtonSubmit']);
	$end_form = '</form>';
	
	$content .= $begin_form;
	$content .= $output->getTextRow($select . ' ' . $submit, 6);
	$content .= $end_form;
	$content .= $output->getBeginNestedListTable();
	$content .= displayItems($expand);
	$content .= $output->getEndNestedTable();
	$content .= $output->getEndTable();
	$output->addToBody($content);
	$output->display();	
	exit();

}

doSubmit();
main();

?>