<?php

// POPMail.inc

/* For the future:
	If there ever is an issue with some emails not being downloaded, while others are,
	check the uidl of the emails. It's a bit confusing if the uidl for an email is
	associated with the server. There is a very small chance 2 emails on 2 different
	servers have the same uidl, but there's still a chance. Just a thought.
*/

// require_once("../../prepend.inc"); // Phil: This only goes in php pages (not inc files)
global $SERVICES;
$SERVICES->getService('FileSystem');

//require_once('../Mail/other/mimeDecode.php');


class POPMail {
	
	/**
	* @var $fp the file pointer associated with the POP server connection
	* @access private
	*/
	var $fp;
	
	/**
	* @var $db_functions the db object for the pop_account table
	* @access private
	*/
	var $db_functions;
	
	/**
	* @var $item_db the db object for the item table
	* @access private
	*/
	var $item_db;
	
	/**
	* @var $uidl_db the db object for the uidl table
	* @access private
	*/
	var $uidl_db;
	
	/**
	* @var $mod the mod object for the Mail module
	* @access private
	*/
	var $mod;
	
	/**
	* @var $halt boolean, true - halt on errors, false - keep silent
	* @access private
	*/
	var $halt;
	
	/**
	* @var $error - the error messages
	* @access private
	*/
	var $error = '';
	
	/**
	* @var $default_port - the default port number
	* @access private
	*/
	var $default_port = 110;
	
	
	/**
	* POPMail constructor - sets initial variables
	*
	* @param $halt true, false whether to halt on error
	*/
	function POPMail($halt) {
		global $SERVICES, $USER;
		$this->mod =& $SERVICES->modules->getModule('Mail'); // Phil 20051018 use =& for singleton instance classes
		$this->db_functions =& $this->mod->getDatabaseTable('pop_account'); // Phil 20051018 use =& for singleton instance classes
		$this->halt = $halt;
		$this->uidl_db =& $this->mod->getDatabaseTable('uidl'); // Phil 20051018 use =& for singleton instance classes
		$this->item_db =& $this->mod->getDatabaseTable('item'); // Phil 20051018 use =& for singleton instance classes
	}
	
	/**
	* Connects to the POP server
	*
	* @param $server - the address of the POP server
	* @param $port - the port of the POP server (if not default
	* @param $user - the username for the POP server
	* @param $pass - the password for the POP server
	* @return true on success, false otherwise
	*/
	function connect($server, $port = 110, $user, $pass) {	
		global $L;
		$this->fp = @fsockopen($server, $port, $errno, $errstr);
		if ($this->fp === FALSE) {
			if ($errno != 0) { // Phil 20060602 The connection was opened, we need to close it.
				$this->quit(true);
			}
			$this->error .= '<p>' . $L['mailServerConnect'] . ' <b>' . $server . '</b></p>';
			return false;
		}
		socket_set_blocking($this->fp,-1);
		$reply = fgets($this->fp, 512);
		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br><b>' . $server .'</b> : ' . $L['mailErrorUnknown'] . '</p>';
			$this->quit(true); // Phil 20060602
			return false;
		}
		
		fwrite($this->fp, "USER $user\r\n");
		$reply = fgets($this->fp, 512);
		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['mailErrorUsername'] . ' ' . $L['mailErrorForAccount'] . ': <b>' . $user . '</b> ' . $L['mailErrorOn'] . ' ' . $L['mailErrorServer'] . ': <b>' . $server . '</b></p>';
			$this->quit(true); // Phil 20060602
			return false;
		}     
		
		fwrite($this->fp, "PASS $pass\r\n");
		$reply = fgets($this->fp, 512);
		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['mailErrorPassword'] . ' ' . $L['mailErrorForAccount'] . ': <b>' . $user . '</b> ' . $L['mailErrorOn'] . ' ' . $L['mailErrorServer'] . ': <b>' . $server . '</b></p>'; 
			//$this->error .= $L['mailError'] . ' <br><br>' . $L['mailErrorPassword'] . ' <b>' . $user . '</b> ' . $L['mailErrorServer'] . ': <b>' . $server . '</b><br>';
			$this->quit(true); // Phil 20060602
			return false;
		}
		return true;
	}
	
	/**
	* Returns all POP accounts the user has access to
	*
	* Phil 20060523 Added optional param order_by
	* @param array $order_by (Optional, default false). Will be passed to getItems if it is an array containing field and order keys.
	* @return $accounts the array of accounts the user has access to 
	*/	
	function getAccounts($order_by = false, $check_type = false) {
		global $USER;
		if (is_array($order_by) && (isset($order_by['field']) && $order_by['field']) && (isset($order_by['order']) && $order_by['order']) ) {
			$order_by_arg = $order_by;
		} else {
			$order_by_arg = false;
		}
		
		
		$types_to_check = '';
		switch ($check_type) {
			case 'manual': // type: manual - just include the accounts "manually", or "manually and automatically" check (type in (0,2)
				$types_to_check = '0,2';
			break;
			case 'auto': // type: auto - just include the accounts "automatically", or "manually and automatically" check (type in (0,1)
				$types_to_check = '0,1';
			break;
			default: // type: false - just exclude the accounts we "never" check (not type=3)
				$types_to_check = '0,1,2';
			break;
		}
		$this->db_functions->setDesignTimeFilters( 
			array(
				'check_type' => array(
					'where' => 'check_type IN (' . $types_to_check . ')'
				)
			) 
		);

		// get user group id 
		// $groups_set = $USER['group_string'];		
		// $accounts = $this->db_functions->getItems(); //false, false, 'view_group_id IN (' . $groups_set . ')');
		$accounts = $this->db_functions->getItems($order_by_arg);
		
		// print '<p>accounts (check_type: ' . $check_type . ')</p>'; print_r($accounts); print '<hr>';
		return $accounts;
	}
	
	/**
	* Returns all POP accounts the user has access to
	*
	* Phil 20060523 Added optional param order_by
	* @param array $order_by (Optional, default false). Will be passed to getItems if it is an array containing field and order keys.
	* @return $accounts the array of accounts the user has access to 
	*/	
	function getAccountsToCheck($order_by = false) {
		return $this->getAccounts($order_by, 'manual'); // 
	}
	
	/**
	* Sets the time the account was last checked for new mail
	*
	* @param $id - the id of the POP account
	*/
	function setLastChecked($id, $value = false) {
		// Phil 20070208 Added param $value (eg the return value from the previous time this function was called)
		
		$current_value = $this->db_functions->db->quickGet(
			'last_checked', 
			$this->db_functions->getTableName(), 
			"`id`='$id'"
		);
		
		if (!$value) {
			$this->db_functions->setItem(array('id' => $id, 'last_checked' => time()));	
		} else {
			$this->db_functions->setItem(array('id' => $id, 'last_checked' => (int)$value));	
		}
		
		return (int)$current_value;
	}
	
	
	/**
	* Returns a list of the uid's for all new messages
	*
	* @param $uidl - the uidl from the POP server
	* @param $server - the server address we are currently checking
	* @return An array with all uid's needed to fetch
	*/
	function getMessagesToFetch($uidl, $server) {	
		global $SERVICES;
		
		/* was
		$uidl_set = '"' . implode('","', array_values($uidl));
		$uidl_set .= '"';
		
		*/
		
		$uidl_set = '';
		foreach ($uidl as $k => $v) {
			//MCR 2005-07-12   addslashes required for broken (or hacked) uidls which may contain ' or "
			$uidl_set .= '\'' . addslashes($v) . '\',';
		}
		$uidl_set = trim($uidl_set, ',');
		// check item table for all uids not already there
		//TODO active causes duplicates to be downloaded
		//TODO direct db query to avoid active=1 where clause?
		
		$db =& $SERVICES->getService('Database');
		
		// Matt 20050715 //$result = $db->select('id, md5_hash, server', 'Mail__item', 'server = "' . $server . '" AND md5_hash IN (' . $uidl_set . ')');
		$result = $db->select('id, md5_hash, server', 'Mail__item', 'server = \'' . $server . '\' AND md5_hash IN (' . $uidl_set . ')');
		
		if ($db->hasError())	$db->printError();
		while ($row = $db->getRow($result)) {
			$item_list[$row['id']] = $row;
		}
		
		//$item_list = $this->item_db->getItems(false, false, 'md5_hash IN (' . $uidl_set . ')');// the list of items we already have

		// code to make sure we only get messages we don't have
		if (is_array($item_list)) {
			//print_r($item_list);
			$uidl = array_flip($uidl); // array_flip is evil
			foreach ($item_list as $kkk => $vvv) {
				//echo $vvv['id'] . ' ' . $vvv['active'] . '<br>';
				bcrm_unset($uidl, $vvv['md5_hash']);
			}	
			// uidl now holds list of all messages we want to retreive
			$uidl = array_flip($uidl);
		}
		
		return $uidl;
	}
	
	
	/**
	* Checks whether we are about to download a message that is already being downloaded
	*
	* @param $uid the uid of the message
	* @return true - don't get the message, false - get the message
	*/
	function checkConcurrentDownload($uid) {
		$uid = addslashes($uid); // Phil 20060421

		// check uidl table incase the message is being downloaded by another user		
		$uidl_inuse = $this->uidl_db->getItems(false, false, 'uidl = "' . $uid . '"'); // the list of items we already have
		
		if (is_array($uidl_inuse)) {
			// file is MAYBE being downloaded elsewhere
			// check if the file has been being downloaded for a long time
			// can only be 1 result
			foreach ($uidl_inuse as $id => $stuff) {						
				if (strtotime("+10 minutes", $stuff['created']) < time()) {   //MCR 20060421 10 mins up from 5
					// this email failed to download (timeout 10 mins)					
					$this->uidl_db->setItem(array('id' => $id, 'active' => 0));					
					return false; // ok to dl message
				} 				
			}
			return true; // don't dl message
		}
		return false; // dl message
	}
	
	/**
	* Checksout the message - it is being downloaded
	*
	* @return the id of the row inserted into the uidl table
	*/
	function checkoutMessage($uidl) {
		// set the uidl table - we are downloading this message
		
		$uidl_item = array('uidl' => $uidl);
		
		$uidl_id = $this->uidl_db->setItem($uidl_item);
		
		return $uidl_id;
	}
	
	/**
	* Checksin message - it has been downloaded
	*
	* @param $id - the id of the entry into uidl table
	*/
	function checkinMessage($id) {		
		$this->uidl_db->setItem(array('id' => $id, 'active' => 0));
	}
	
	/**
	* Gets the header information from the message for entry into the database
	*
	* @param $msg - the message array from the server
	* @param $account - the account array the message was from
	* @param $uid - the uid of the message
	* @return The header array with all information (to be written to DB)
	*/
	function setHeaders($msg, $account, $uid) {
		global $L;
		
		// extract header information
		$header = $this->parseHeader($msg);
		$header['md5_hash'] = $uid;
		$header['sent_status'] = 'mailInbox';
		$header['is_opened'] = 0;
		$header['action_required'] = 1;
		$header['is_closed'] = 0;				
		$header['view_group_id'] = $account['message_view_group_id'];
		$header['edit_group_id'] = $account['message_edit_group_id'];
		$header['owner_group_id'] = $account['message_owner_group_id'];
		$header['admin_group_id'] = $account['message_admin_group_id'];
		$header['receive_account_id'] = $account['id'];
		$header['server'] = $account['host'];
		// quick double check if a subject is provided
		if (!isset($header['subject']) || (strlen(trim($header['subject'])) == 0) || !$header['subject']) {
			$header['subject'] = $L['mailSubjectNoSubject'];
		}
		return $header;
	}
	
	/**
	* Writes address data to the item_email table
	*
	* @param $header - the header array with all data
	* @param $id - the id of the message (in the database)
	*/
	function setAddressItems($header, $id) {
		// write address data to item_email table
		$address = array('recipient', 'cc', 'sender');
		$addresses = array();
		foreach ($address as $field) {
			$temp = $this->mod->splitAddresses($header[$field]);
			if (!empty($temp)) {
				$addresses = bcrm_array_merge($addresses, $temp);
			}
		}	
		$address_db =& $this->mod->getDatabaseTable('item_email'); // Phil 20051018 use =& for singleton instance classes
		foreach ($addresses as $addr) {
			$addres['address'] = $addr['address'];
			$addres['item_id'] = $id;
			$address_db->setItem($addres);
		}
	}
	
	/**
	* Checks mail for an account
	*
	* @param $account - the account array
	*/
	function getMailForAccount(&$account) {
		global $SERVICES;
		// set last_checked in account table
		// Phil 20070208 Store the previous last checked time. May need to reset later.
		$last_checked_timestamp = $this->setLastChecked($account['id']);
		
		// Phil 20070125 Accounts now have a port number
		$port = (int)$account['port'];
		if (!$port) {
			$port = $this->default_port;
		}
		// Phil 20070125 Accounts can use SSL (requires php open ssl)
		// will fail to get messages if open ssl support not installed
		$ssl_host_prefix = 'ssl://';
		if ($account['use_ssl']) {
			$account['host'] = $ssl_host_prefix . str_replace($ssl_host_prefix, '', $account['host']);
		}
		
		// connect and check mail for each account
		// connect and login
		// print '<div>connect to host: ' . $account['host'] . ' port: ' . $port . '</div>'; // DEBUG
		if (!$this->connect($account['host'], $port, $account['username'], base64_decode($account['password']))) {
			return;
		}
		
		// get list of UID's from server
		$uidl = $this->getUIDL();
		
		if (!$uidl) {
			$this->quit(true); // Phil 20060602 (closes socket)
			return;
		}
		
		// $this->error .= '<p>Debugging POPMail</p>';
		// print '<hr>';
		// print '<div>uidl count: ' . count($uidl) . '</div>'; // print_r($uidl); print '<hr>'; // DEBUG
		// print '<div>host: ' . $account['host'] . '</div>';
		$messages = $this->getMessagesToFetch($uidl, $account['host']);
		
		// print '<div>num of messages to get: ' . count($messages) . '</div>';  print_r($messages); print '<hr>'; // DEBUG
		
		$delete = array_diff($uidl, $messages);
		
		// print '<div>account uses pop_delete: ' . ($account['pop_delete'] ? 'yes' : 'no') . '</div>';
		// print '<div>num of messages to remove [if using pop_delete]: ' . count($delete) . '</div>'; // DEBUG
		// print_r($uidl);
		
		// now fetch message numbers (key field) in $uidl
		if (is_array($messages)) { 
			
			// Phil 20070207 Max number of messages to download per account
			// TODO: config lookup and user pref: 10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 350, 400, 450, 500, All (1000?)
			$max_download_per_account = false; 
			global $SERVICES;
			if ($config =& $SERVICES->getService('Config')) {
				$max_download_per_account = $config->lookup('modules.Mail.file_set.pop_account.max_emails_to_download');
			}
			// print '<div>max_download_per_account' . $max_download_per_account . '</div>'; 
			$num_messages_downloaded = 0; // Initialise counter
			
			foreach ($messages as $kk => $vv) {
				if ($max_download_per_account && ($num_messages_downloaded >= $max_download_per_account)) {
					global $L;
					$check_link = ''; 
					if ($output =& $SERVICES->getService('Output')) {
						$check_link = '<br><br>' . $output->getLinkHref(array('href' => 'check_email.php', 'text' => 'mailCheckMail'));
					}
					// print '<p>last_checked_timestamp: ' . $last_checked_timestamp . '</p>';
					$unused_timestamp = $this->setLastChecked($account['id'], $last_checked_timestamp);
					// print '<p>last_checked_timestamp: ' . $last_checked_timestamp . ' (unused: ' . $unused_timestamp .')</p>';
					$this->error .= '<p>' . $L['mailErrorMoreEmailsToDownload'] . ' <br><br>' . $L['mailDownloaded'] . ': ' . $num_messages_downloaded . '. ' . $L['mailRemaining'] . ': ' . (count($messages) - $num_messages_downloaded) . $check_link . '</p>';
					// print '<p>Exiting message download loop after downloading ' . $num_messages_downloaded . ' messages [max: ' . $max_download_per_account . ', remaining: ' . (count($messages) - $num_messages_downloaded) . ']</p><hr>';
					break;
				}
				$num_messages_downloaded++;
				
				$msg = array();			
				$msg = $this->get($kk);
				
				
				if ($this->checkConcurrentDownload($vv)) {
					// print '<div>continue - concurrent download ' . $kk . ', ' . $vv . '</div>';
					continue;
				}
				
				$uidl_id = $this->checkOutMessage($vv);
				
				$header = $this->setHeaders($msg, $account, $vv);


				//$header['sender'] = htmlspecialchars($header['sender']);
				// write to db, don't set item_seen
				$id = $this->item_db->setItem($header, false);
				
				if ($id) { // Phil 20070207 Need to check we were able to add a db entry
					// write email content to file				
					
					if (is_array($msg)) {
						$file_msg = implode('', $msg);
					} else {
						$file_msg = '';
					}
					
					if ($SERVICES->filesystem->writeFileContents($id, 'mail', $file_msg)) {
						// success - set the message size
						$msg_size = $SERVICES->filesystem->getFileSize($id, 'mail');
						if ($msg_size > 0) {
							$this->item_db->setItem(array('id' => $id, 'message_size' => $msg_size), false);
						}
					} else {
						// failed to write // TODO: At least remove the uidl from the 'delete' array
						// print '<div>WARNING: could not save file for id: ' . $kk . ' uidl: ' . $vv . '</div>';
						// Remove the item entry and set an error code
						$result = $this->item_db->db->delete(
							$this->item_db->getTableName(), 
							"id='$id'", 
							'LIMIT 1'
						);
						global $L;
						$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['FileSystemErrorWritingToFile'] . '</p>';
						return;
					}
					$header['sender'] = stripslashes($this->htmlToSpecialChars($header['sender']));
					
					$this->setAddressItems($header, $id);
					
				}
				
				$this->checkinMessage($uidl_id);
				
			}
		}
		
		if ($account['pop_delete']) {
			// print '<p>account pop_delete set to true: ' . $account['pop_delete'] . '</p>';
			foreach ($delete as $k => $v) {
				// print '<p>delete from pop server: ' . $k . ' [uidl: ' . $v . ']</p>';
				$this->delete($k);
			}
		}
		$this->quit();
	}
	
	/* global function in ~/services/Services.inc - all references should be
	 * altered to use that function. - *HOWEVER*, we want to keep angled
	 * brackets and quotes, so keep using this one for now. change later!
	 */
	function htmlToSpecialChars($str){
		$trans_table = array_flip(get_html_translation_table(HTML_ENTITIES));
		return strtr($str, $trans_table);
	}
	
	
	/**
	* Checks mail for all accounts the user has access to
	*
	*/
	function main() {
		// $start = time(); // DEBUG
		global $SERVICES, $USER, $L;
		
		// Phil 20070124 Force check. Get all accounts that we check (Refresh or 'Check Email')
		$accounts = $this->getAccountsToCheck();
		// $accounts = $this->getAccounts();
		
		if (is_array($accounts)) {
			
			foreach ($accounts as $k => $v) {
				// print '<div>getting mail for ' . $k . '</div>'; print_r($v); print '<hr>'; // DEBUG
				$this->getMailForAccount($v);
				
			}
			
			if ($this->halt && $this->error) {
				quickPage($this->error . '<p><a href="' . $_SESSION['return_page'] . '">' . $L['Back'] . '</a></p>');
				exit();
			}
		} else {
			if ($this->halt) {
				quickPage('<p>' . $L['mailNoAccount'] . '<br><br><a href="../Mail/account_menu.php">' . $L['mailCreateAccount'] . '</a> | <a href="' . $_SESSION['return_page'] . '">' . $L['Back'] . '</a></p>');
				exit();
			} else {
				return;
			}
		}
		$finish = time(); // DEBUG
		// print '<div>Time: ' . date('s', $finish-$start) . ' seconds to check ' . count($accounts) . ' accounts</div>'; // DEBUG
		// exit(); // DEBUG
	}
	
	/**
	* Decodes the headers of a message
	*
	* #Note: There has been an instance where an email comes in with multple
	* 	headers, that is, 2+ subjects, From, etc. This creates an array
	*	and thus screws up the insert sql. Since this has only happened
	*	once, it does not seem to be a major issue, if it comes up again
	*	might be wortwhile to throw in some 'is_array' checks.
	*
	* @param $msg - the msg array
	* @return The decoded headers
	*/
	function parseHeader($msg) {
		/* FROM HEADER I NEED:
		TO
		CC	
		FROM
		SUBJECT
		DATE
		ATTACHMENTS (boolean) 
		BCRM-Thread-ID if exists*/
		
		$headers = array();
		
		if (is_array($msg)) {
			$msg = implode('', $msg);
		}
		
		$mime = new CustomMimeDecode($msg);
		$decode = $mime->decode(array('decode_bodies' => false));
		
		$attachments = 0;
		
		$trim = array('"', '\'', '"', '\\');
		$replace = array('', '', '', '');
		if (is_array($decode->headers['to'])) {
			$headers['recipient'] = stripslashes(implode(',', $decode->headers['to']));
		} else {
			$headers['recipient'] = stripslashes($decode->headers['to']);
		}
		if ($decode->headers['cc']) {
			if (is_array($decode->headers['cc'])) {
				$headers['cc'] = stripslashes(implode(',', $decode->headers['cc']));
			} else {
				$headers['cc'] = stripslashes($decode->headers['cc']);
			}
		}
		$headers['sender'] = stripslashes($decode->headers['from']);
		// odd ball case where 2 subjects were set in the mail header
		if (is_array($decode->headers['subject'])) {
			$headers['subject'] = $decode->headers['subject'][0];
		} else {
			$headers['subject'] = $decode->headers['subject'];
		}
		
		if (is_array($decode->headers['date'])) {
			$headers['date'] = $decode->headers['date'][0];
		} else {
			$headers['date'] = $decode->headers['date'];
		}
		$date = strtotime($headers['date']);
		// sometimes the strtotime function might screw up, this is
		// most often due to the fact that the timezone code (ex: GMT)
		// is at the end, along with the UTC (EX: +0500) code, in which 
		// case strtotime breaks and returns -1 ... please note, this is
		// a *specific* fix, and should not be taken as general
		// the specific case is when we get a date like:
		// Wed, 23 Mar 2005 19:02:02 +0000 GMT, where strtotime gets
		// confused because there's +0000 as well as GMT (usually only 1)
		if (($date < 0) && (strlen($headers['date']) > 0)) {
			$date = 0;
			// takin a chance that this is due to the timezone, but
			// let's check to ensure, only good way is to see if the
			// last bit is not numeric
			$posSpace = strrpos($headers['date'], ' ');
			if (!is_numeric(substr($headers['date'], $posSpace))) {
				$date = strtotime(substr($headers['date'], 0, $posSpace));
				// now do another check if it's a positive number,
				// if not, we have no idea, so just set it to [0] [MCR, no set to now]
				if ($date < 0) {
					$date = 0;
				}
			}
		}
		
		//MCR 20060517
		if ($date < 1) {
			$date = time();
		}
		
		$headers['date'] = $date;
		//$headers['recipient'] = $decode->headers['to'];
		
		
		if ($decode->parts) {
			foreach ($decode->parts as $key => $value) {
				if ($value->disposition ==  'attachment' || $value->disposition == 'inline') {
					$headers['attachments'] = 1;
					break;
				}
			}
		}
		return $headers;
	}
	
	/**
	* Get the uidl from the server
	*
	* @return The UIDL
	*/
	function getUIDL() {
		global $L;
		$UIDLArray = array();
		 
		fwrite($this->fp, "UIDL\r\n");
		$reply = fgets($this->fp, 512);
		
		$reply = $this->strip_clf($reply);
		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['mailErrorGettingMsg'] . '</p>';
			return false;
		}

		$line = "";
		$count = 1;
		$line = fgets($this->fp,512);
		while (!ereg("^\.\r\n",$line)) {
			if(ereg("^\.\r\n",$line)) {
				break;
			}
			list ($msg,$msgUidl) = preg_split('/\s+/',$line);
			$msgUidl = $this->strip_clf($msgUidl);			
			$UIDLArray[$msg] = $msgUidl;
			
			$count++;
			$line = fgets($this->fp,512);
		}
		return $UIDLArray;
	}
	
	/**
	* Strip newlines from text
	*
	* @param $text - the text to strip
	* @return The stipped text
	*/
	function strip_clf($text = "") {
         // Strips \r\n from server responses

	 	if (empty($text)) {
			return $text;
		} else {
			$stripped = str_replace("\r",'',$text);
			$stripped = str_replace("\n",'',$stripped);
			return $stripped;
		}
	}
	
	/**
	* Checks a reply from the server is OK
	*
	* @param $cmd - the reply from the server
	* @return true - the reply is ok, false - there was an error
	*/
	function is_ok($cmd = "") {
		//  Return true or false on +OK or -ERR/ 
		if (empty($cmd)) {
			return false;
		} else {
			return( ereg ("^\+OK", $cmd ) );
		}
	}
	
	/**
	* Retrieve a message from the server
	*
	* @param $msgNum - the server message number to retreive
	* @return The message array
	*/
	function get($msgNum) {
		global $L;
		//  Retrieve the specified msg number. Returns an array
		//  where each line of the msg is an array element.  
		
		$cmd = "RETR $msgNum";
		fwrite($this->fp, "$cmd\r\n");
		$reply = fgets($this->fp, 512);
		

		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['mailErrorGettingMsg'] . '</p>';
			return false;
		}

		$count = 0;
		$MsgArray = array();

		$line = fgets($this->fp,512);
		while (!ereg("^\.\r\n",$line)) {
			$MsgArray[$count] = $line;
			$count++;
			$line = fgets($this->fp,512);
			if (empty($line)) {
				break;
			}
		}
		
		return $MsgArray;
	}
	
	/**
	* Delete a message from the server
	*
	* @param $msgNum - the server message number to delete
	*/
	function delete($msgNum = "") {
		global $L;
		//  Flags a specified msg as deleted. The msg will not
		//  be deleted until a quit() method is called.
		// Phil 20060609 Added is_resource
		if (!isset($this->fp) || !is_resource($this->fp)) {
			return false;
		}
		fwrite($this->fp, "DELE $msgNum\r\n");
		$reply = fgets($this->fp, 512);
		
		if (!$this->is_ok($reply)) {
			$this->error .= '<p>' . $L['mailError'] . ' <br><br>' . $L['mailErrorDeleteMsg'] . '</p>';
			return false;
		}
		return true;
	}
	
	function quit($silent = false) {
		// Phil 200606xx Added param $silent
		global $L;
		// Phil 20060609 Added is_resource
		if (!isset($this->fp) || !is_resource($this->fp)) {
			return false;
		}        
		fwrite($this->fp, "QUIT\r\n");
		$reply = fgets($this->fp, 512);
		
		if (!$silent && !$this->is_ok($reply)) {
			$this->error .= $L['mailError'] . ' <br><br>' . $L['mailErrorDeleteOnQuitMsg'];
			fclose($this->fp); // Phil 20060602
			return false;
		}
		fclose($this->fp); // Phil 20060602
		return true;
	}

}

?>