<?php
/**
 * The edit page
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeModule('Mail', 'inc/item_edit.inc');

class MailForwardWebMATEdit extends MailItemWebMATEdit {


	/**
	 * any processing 
	 */
	function getItem($id = false, $item = false) {
		
		if (!$item) {
		
			$item = parent::getItem($id, $item);
		
			global $L;
	
			$this->checkoutMessage($item);
	
			// see if there are any previous actions
			$action_value = $this->mod->getPreviousActionStatement($item['id']);
			if ($action_value) {
				$item['previous_actions'] = $action_value;
			} else if (array_key_exists('previous_actions', $this->dd->data_dictionary)) {
				bcrm_unset($this->dd->data_dictionary, 'previous_actions'); // get rid of it - no need if nothing to display
			}
		}
		return $item;
	}


	/**
	 * Set the recipients (to, cc, bcc)
	 *
	 * @param array $item
	 */
	function setRecipients(& $item) {
		global $SERVICES;
		
		// reset all addys
		$item['recipient'] = '';
		$item['cc'] = '';
		$item['bcc'] = '';
		
		$default_bcc = $SERVICES->preferences->getUserPref('mailDefaultBccAddresses');
		if ($default_bcc) {
			$item['bcc'] = $default_bcc;
		}
	}
	
	
	/**
	 * sets the subject. intended to be overriden
	 *
	 * @param array $item
	 */
	function setSubject(& $item) {
		parent::setSubject($item);  //MCR 20051007
		
		// Add the RE or FW prefixes to the subject 
		
		//MCR 20051007  
		//TODO we can add something so that after 3 or 4 'RE'it just shows '...' or something
		
		//(only do this once so we don't end
		// up with RE: RE: RE: etc)
		//if (strpos($item['subject'], 'FWD') === FALSE) {
			//TODO add language
			$item['subject'] = 'FWD: ' . $item['subject'];
		//}
	}
	
	
	
	/**
	 * Operations before we actually write to the db, included in here is sending
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference

		parent::beforeDBWrite($item, $fields, $error_msg);

		// if we've made it this far, continue with a few 'forward' customizations
		// the parent is who the original id was, so include it
		$item['parent_id'] = $this->form_values['id'];
		$item['message_send_type'] = 'forward';
		$item['is_opened'] = 1;
		
		// Make sure we are going to get a new row in the table for this (this _IS_ afterall, an edit page)
		bcrm_unset($item, 'id');
	}


	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(& $links, $id, $item) {
		global $L, $SERVICES;
		
		// forwarding is essentially creating a new item, so we don't need any links
		foreach ($links as $k => $v) {
			if ($k != 'list') {
				bcrm_unset($links, $k);
			}
		}
		
		// but maybe we want to view the original?
		$links['mailViewParent'] = $SERVICES->output->getLinkHref(array('href' => $this->getFileSetHref('view_parent'), 'text' => $L['mailViewParent']));
					$SERVICES->output->getLinkHref(array('href' => $this->getFileSetHref('view_parent'), 'text' => '[+]', 'title' => $L['LinkOpenInNewWindow'], 'target' => '_blank'));
					
		
	}


	/**
	 * Over-ride the parent to disply a more appropriate title
	 */
	function getTitleText() {
		global $L;
		return $L['mailTitle_forwardedit'];
	}
}

?>