<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Memo
 * @author Minggang Chen, M Corich, Phil Calder
 */
 
/**
 * The main display class
 * 
 * @package Memo
 * @author Minggang Chen, M Corich, Phil Calder
 */
class MemoItemWebMATList extends WebMATList {
	
	var $groups_table = false;
	
	/**
	 * @author Phil Calder
	 */
	function setGroupsTableObject() {
		if (!is_object($this->groups_table) && class_exists('item_groupMemoDatabaseTable')) {
			// $this->groups_table = getWebMATObject(
				// 'WebMATDatabaseDetailTable',
				// 'Memo'
			// );
			$data_definition = array(
				'fields' => array(
					'item_id' => array('field_key' => 'item_id'),
					'group_id' => array('field_key' => 'group_id')
				), 
				'options' => array()
			
			);
			// print_r($data_definition); exit();
			$this->groups_table = new item_groupMemoDatabaseTable('Memo', $data_definition);
		}
	}
		
	/**
	 * @param array $dd_item the items with dd structure
	 */
	function calculateValues(& $dd_item, $item) {
		global $SERVICES, $L, $USER;
		
		// Phil 20061017 Process send to groups
		if ($item['id'] && array_key_exists('groups', $dd_item)) {
			$this->setGroupsTableObject();
			$groups = $this->groups_table->getItems('', '', 'item_id=' . $item['id']);
			$group_arr = array();
			if (is_array($groups)) {
				foreach ($groups as $k => $v) {
					if ($v['group_id']) {
						$group_arr[] = $v['group_id'];
					}
				}
			}
			$dd_item['groups']['value'] = $group_arr; 
		}
		
		$output =& $SERVICES->getService('Output');
		//$unseen_flag = $output->getFlagUnseen();
		//$unseen_flag = $output->getFlagUnseen();
		$unseen_flag = $output->getImageTag('img/flag_unread.png', $L[''], 'width="16" height="16" border="0" hspace="1"');

		
		$ua =& $SERVICES->getService('Authentication');
		$all_users = $ua->users->getUsersList();
		$SERVICES->getService('Preferences');
		$dd_item['creator_id']['value'] = $ua->users->getUsername($item['creator_id']);
		
		$title_length = $SERVICES->preferences->getUserPref('memoTitleLength', 100);
		if ($title_length != 'All' && $title_length != '') {
			if (strlen($item['subject']) > $title_length) {
				$item['subject'] = substr($item['subject'], 0, $title_length) . '...';
			}
		}
		
		$num_recipients_to_display = $SERVICES->preferences->getUserPref('memoRecipientLength', 5);
		$db_functions =& $this->mod->getDatabaseTable('item_seen');
		$recipients = $db_functions->getItems('', '', 'item_id=' . $item['id']);
		if ($recipients) {
			$count = 1;
			// Bosco 20050706 | Added to remove creator name from recipients list
			$recipients_count = count($recipients);
			foreach ($recipients as $k => $v) {
				if (($num_recipients_to_display != 'All') && ($count > $num_recipients_to_display)) {
					$dd_item['recipients']['value'] .= '...';
					break;
				} 	
				// Bosco 20050706 | Added condition to remove creator Name from recipients list	
				if ($v['user_id'] && !(($recipients_count != 1) && ($v['user_id'] == $item['creator_id']))) {
					if (!$v['seen']) {
						$dd_item['recipients']['value'] .= $unseen_flag . '';
					}
					$dd_item['recipients']['value'] .= $all_users[$v['user_id']]. ', ';
					
					// Add a line break for every 5 recipients
					// Output adds nowrap to the row because of images (the unseen flag)
					if ( $count && ( $count %5 == 0)) { 
						$dd_item['recipients']['value'] .= '<br>';
					}
					$count++;
				}
			}
			$dd_item['recipients']['value'] = trim($dd_item['recipients']['value'], ', ');
		}
		
		// Phil 20060117 - removed, see WebMATList::buildListTable
		// $link_href =  str_replace('%id%', $item['id'], $this->getFileSetHref('view'));
		// $dd_item['subject']['value'] = $this->output->getLinkHref( array('href' => $link_href, 'text' => $item['subject']) );
	}

	function editTableLinks(& $links, $id, $item) {
		global $USER;
		if ($id) {	//2005/05/25 Cecil  check if id is available
			$filter_info = $this->getFilterInfo();
			bcrm_unset($links, 'edit');
			// bcrm_unset($links, 'view');
			 
			if ($item['creator_id'] == $USER['user_id']) {
				$links['memoReply'] = '';
			}
			//2005/05/25 CECIL
			if ($this->db_functions->isArchived($id)) {
					$links['memoArchive'] = '';
			}
		}
	}
	
	function getPagingRow() {
		return $this->output->getPagingRow( array('memoindexListSize', 'memoTitleLength', 'memoRecipientLength') );
	}

	/**
	 * returns the second menu row
	 *
	 * @return string the second menu row
	 */
	function getCheckboxMenuExtraItems() {
		// Bosco 20050624 | Extended to remove 'Archive' option from checkbox menu row dropdown in 'Archived' config Filter
		$filter_info = $this->getFilterInfo();
		if ((!$this->form_values['filter'] && $filter_info['value'] != 2) || ($this->form_values['filter'] && $this->form_values['filter'] != 'f2')) {
			$this->addCheckboxMenuExtraItems(array('mass_archive' => 'Archive'));	
		}
		return parent::getCheckboxMenuExtraItems();
	}
	
	/**
	 * There is no standard 'edit' link in this module
	 *
	 * @return array An array of item links for the list page
	 * @author Phil Calder
	 */
	function getTableLinks($item) {
		$links = parent::getTableLinks($item);
		// bcrm_unset($links, 'view');
		bcrm_unset($links, 'edit');
		return $links;
	}
	
	/**
	 * Forces list to NOT use an iterator for getItems
	 *
	 * ie process and return the entire items array
	 * Added 200512xx. This modules list page relies on bulk processing 
	 * of items in the {@link afterGetItems} method
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function useGetItemsIterator() {
		return false;
	}
}
?>