<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package SystemBookmarks
 */

/**
 * @package SystemBookmarks
 */
class SystemBookmarksItemWebMATEdit extends WebMATEdit {
	
	/**
	 * changed to do nothing
	 */
	function disableReadOnly() {}
	
	/**
	 * sets the item class variable
	 */
	function setItemVariable() {
		if ($this->form_values['item']) {
			$this->item = $this->form_values['item'];
		} else {
			$this->delete = false;
		}
		if ($this->form_values['bookmark']) {
			$this->form_values['item'] = $_SESSION[$this->form_values['bookmark']];
			$this->form_values['item']['priority'] = 100;
			$this->form_values['item'] = bcrm_array_merge($this->form_values['item'],$this->dd->getDefaultItem()); //Syed 20050823 | includes default permissions settings
		}
	}
	
	/**
	 * gets the form fields
	 * 
	 * CHANGED to remove the set as dashboard and set as alert if the page type
	 * is not a list
	 * 
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string and table row for the form
	 */
	function getFieldRows($item, $error_msg) {
		if ($item['page_type'] != 'list') {
			$this->dd->setDictionaryExcludeFields(array('is_alert', 'is_dashboard','show_filter'));
		}
		return parent::getFieldRows($item, $error_msg);
	}
	
	/**
	 * unsets the item id if adding a bookmark
	 */
	function beforeDBWrite($item, $fields, $error_msg) {
		if ($this->form_values['bookmark']) {
			bcrm_unset($item, 'id');
		}
	}
	
	/**
	 * Gets the table links
	 * 
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		if ($this->form_values['bookmark']) {
			return array();
		} else {
			return parent::getTableLinks($item);
		}
	}
	
	/**
	 * adds the close window javascript if you are creating a bookmark
	 */
	function doSubmitEnd($id) {
		if ($this->form_values['bookmark']) {  
			echo '
			<script type="text/javascript">
				window.close()
			</script>';
		}
	}
	
	/**
	 * Ensure we dont redirect if we are not supposed to
	 *
	 * @see doSubmitEnd
	 * @author Phil Calder
	 */
	function doRedirect($id = '') {
		if (!$this->form_values['bookmark']) {   
			$this->redirectToList($id);
			exit();
		}
	}
	
	/**
	 * Ensure we dont redirect if we are not supposed to
	 *
	 * @see doSubmitEnd
	 * @author Phil Calder
	 */
	function redirectBack($id = '') {
		if (!$this->form_values['bookmark']) {   
			$this->redirectToList($id);
			exit();
		}
	}
	
	/**
 	 * overrides the parent method
	 */
	function getNextPageRow() {
		return '';
	}
	
	
	/**
     * overrides the parent method
     */
	function getLastViewedRow() {
		return '';
	}
	
	/**
     * overrides the parent method
     */
	function getLastViewedContent() {
		return '';
	}
	
	/**
	 * Automatically subscribe the user to this bookmark if they created it
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 * @author Phil Calder
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		$this->subscribeUserToBookmark($item, $id);
	}
	
	/**
	 * @param integer $id The bookmark id
	 * @author Phil Calder
	 */
	function subscribeUserToBookmark($item, $id) {
		$seen_item = array();
		$seen_item['subscribed'] = '1';
		$item = array('id'=>$id);
		$this->markAsSeen($item, $seen_item);
	}
	
	/**
	 * Disables the new link
	 *
	 * @author Phil Calder
	 * @return boolean false
	 */      
	function getNewLink() {
		return false;
	}
	
	/**
	 * Disables the Add Bookmark link
	 *
	 * @author Phil Calder
	 * @return string 'off'
	 */
	function getAddBookmarkLink() {
		return 'off';
	}
}
?>