<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * This class handles the database functions for the item table
 * 
 * This module is an extended verison of WebMATDatabaseTable class
 * 
 * @package Tasks
 */
class itemTasksDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemTasksDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemTasksDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * Set the data definition array
	 *
	 * Performs some Tasks specific pre processing, then calls parent function
	 *
	 * @param array $data_definition An array containing the data definition (fields and options)
	 */
	function setDataDefinition($data_definition){
		global $SERVICES,$L;
		$SERVICES->getService('Language');
		$SERVICES->getService('Preferences');
		$SERVICES->getService('DateLib');
		
		$this->mod =& $SERVICES->modules->getModule('Tasks');
		
		if (is_array($data_definition['fields'])) { // Phil 20051107 - it should be, but better to check
			
			$get_priority = -1;
			if (isset($_GET['priority'])) {
				if ($_GET['priority'] == 1) {
					$get_priority = 1;
				} else {
					$get_priority = 0;
				}
			}
			
			foreach($data_definition['fields'] as $k => $v) {
				//TODO - Form values
				if (!isset($this->form_values['submit']) || !$this->form_values['submit']) {
					if ($get_priority == 1) {
						if ($v['field_key'] == 'task_datetime') {
							// bcrm_unset($data_definition, 'fields', $k);
							// print '<div>db functions unset task_datetime</div>';
						} else if ($v['field_key'] == 'priority_id') {
							$data_definition['fields'][$k]['notnull'] = 1;
						}
					} else { // if ($get_priority == 0) {
						if ($v['field_key'] == 'priority_id') {
							// bcrm_unset($data_definition, 'fields', $k);
							// print '<div>db functions unset priority_id</div>';
							
						}
					}
					
					if ($v['field_key'] == 'task_datetime') {
						if ($get_priority != 1) {
							// today plus user preference offset
							$data_definition['fields'][$k]['default_value'] = time() + ($SERVICES->preferences->getUserPref('tasksDefaultDate') * 24 * 3600);
						} else {
							// today
							$data_definition['fields'][$k]['default_value'] = time();
							$data_definition['fields'][$k]['hidden'] = 1;
							$data_definition['fields'][$k]['label'] = '';
						}
					} else {
						
					}
				}
				switch ($v['field_key']) {
					case 'alert_in_advance':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('tasksDefaultAlertAdvance');
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_in_advance');
					break;
					case 'alert_end':
						$data_definition['fields'][$k]['default_value'] = $SERVICES->preferences->getUserPref('tasksDefaultAlertEnd');					
						$data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('alert_end');
					break;
					case 'task_datetime':
						
						if (!$get_priority) {
							$time_advance = time() + 86400 * $SERVICES->preferences->getUserPref('tasksDefaultDate'); // 86400 is 1 day in seconds
							if ($SERVICES->preferences->getUserPref('tasksDefaultTime') == 2) {
								$time_advance = $SERVICES->offsetTimeZoneForWrite(mktime(23,59,0, date("m", $time_advance), date("d", $time_advance), date("Y", $time_advance)));
							}
							//echo "<br>after: "; print_r($time_advance);	
							//$SERVICES->preferences->getUserPref('tasksDefaultTime');
							if (isset($data_definition['fields'][$k])) {
								$data_definition['fields'][$k]['default_value'] = $time_advance;	
								// $data_definition['fields'][$k]['dropdown_values'] = $this->mod->getSharedVars('task_datetime');
							}
						}
						
					break;
					case 'is_closed':
						$data_definition['fields'][$k]['list_values'] = array($L['Open'],$L['Closed']);
					break;
					case 'content':
						//trim content field as neccessary(taken from user pref)
						$data_definition['fields'][$k]['list_detail_length'] = $SERVICES->preferences->getUserPref('tasksSummaryDescriptionLength');
					break;
				}
			}
		}
		parent::setDataDefinition($data_definition);
	}
	
	/**
	 * Returns true if the task is closed
	 */ 
	function isClosed($id) {
		if (!empty($id)) {
			$item = $this->getItem($id);
			if ($this->db->hasError()) {
				$this->db->printError();
			}
			if ($item['is_closed'] == '1') {
				return true;
			}
		}
		return false;
	}
}
?>