<?php
require_once("../../prepend.inc");
/**
 * Edit a user group.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package UserAdmin
 * @author Phil Calder
 */

$ua =& $SERVICES->getService('Authentication');
$ua->adminRestrict();

$SERVICES->getService('Language');
$SERVICES->language->getLanguage('UserAdmin');

$output =& $SERVICES->getService('Output');

$SERVICES->getService('Modules');

/**
 * returns error messages
 */
function doSubmit() {
	global $SERVICES, $L, $ua, $item, $reset_perms, $id;
	// Reset to default permissions
	if ($reset_perms && $id) { // GET variables
		// Reset the Everyone group permissions to their default values
		$default_mod = $SERVICES->modules->getDefaultModules();
		// print '<div>default mod:</div>'; print_r($default_mod); print '<hr>';
		if (is_array($default_mod) && sizeof($default_mod)) {
			$list_of_modules = $SERVICES->modules->getModules();
			if (is_array($list_of_modules)) {
				foreach($list_of_modules as $k => $v) {
					$ua->groups->setPrivileges($id, $k, 0);
				}
			}
			
			$module_ids = array_flip($list_of_modules);
			foreach($default_mod as $def_priv) {
				if (isset($module_ids[$def_priv['name']])) {
					$ua->groups->setPrivileges(
						$id, 
						$module_ids[$def_priv['name']], 
						$def_priv['def_everyone_privs']
					);
				}
			}
		}	
		return; 
	}
	
	if (!isset($item['visible']) || !$item['visible']) {
		$item['visible'] = '0';
	} else {
		$item['visible'] = '1';
	}
	
	$item['name'] = (isset($item['name']) ? trim(stripslashes($item['name'])) : '');

    if (empty($item['name'])) {
		return $L['useradminYouMustSpecifyNameGroup'];
    } else if ($item['name']) {
		$id = $item['id'];
		$priv = $item['priv'];
		
		bcrm_unset($item, 'id');
		bcrm_unset($item, 'priv');
		
		// print '<div>setGroup(' . $id . ','; print_r($item); print ')</div>';
		if (isset($id) && $id > 0) {
			$retval = $ua->groups->setGroup($id, $item);  // Update name, visible, editable
		} else {
			$id = $ua->groups->addGroup($item);
		}
		
		// New or existing group
		if (is_array($priv) && $id) {
			foreach ($priv as $module_id => $privs) {
				$level = 0;
				if (isset($privs['read']) &&  $privs['read']) {
					$level |= $ua->getReadLevel();
				}
				
				if (isset($privs['write']) &&  $privs['write']) {
					$level |= $ua->getWriteLevel();
				}
				
				if (isset($privs['admin']) &&  $privs['admin']) {
					$level |= $ua->getAdminLevel();
				}
				
				$ua->groups->setPrivileges($id, $module_id, $level);
			}
		}
	}
}

function main() {
	global $SERVICES, $L, $ua, $id, $output, $item, $reset_perms, $PHP_SELF;
	
	if (is_array($item) || $reset_perms) {
		$message = doSubmit();
		if (!$message) {
			header('Location: groups.php');
			exit();
		}
	}
	
	$name = '';
	$visible = 1;
	$editable = 1;
	
	$display_message = ''; // $L['useradminDoNotChangeSettingsUnlessSkilled'];
	
	if ($id) {
		$group = $ua->groups->getGroup($id);
		// print '<div>group ' . $id . '</div>'; print_r($group); print '<hr>';
		if (is_array($group)) {
			$name = $group['name'];
			$visible = $group['visible'];
			$editable = $group['editable'];
			$type = getGroupType($group['type']);			
		} else {
			header('Location: groups.php');
			exit();
		}
		
		if ($editable) {
			$display_message = $L['useradminDoNotChangeSettingsUnlessSkilled'];
		}
	} else {
		$display_message = $L['useradminNewGroupSettings'];	
	}
	
	$list_of_modules = $SERVICES->modules->getModules();
	$db =& $SERVICES->getService('Database');
	
	$content = '';
	$content .= $output->getBeginForm($PHP_SELF);
	$content .= $output->getBeginTable();
	// $content .= $output->getTitleRow($L['useradminSetPermissionsForGroup']);
	
	$content .= $output->getBeginNestedEditTable();
	$content .= $output->getTextRow($display_message);
	
	if (!empty($message)) {
		// $content .= $output->getTextRow($output->getAlert($message));
		$content .= $output->getTextRow($message);
	}
	
	// $content .= $output->getFieldSpacerRow();
	
	$content .= $output->getHtmlFieldRow(array('label'=>'', 'type'=>'hidden', 'field_key'=>'id', 'value'=>$id));
	$content .= $output->getHtmlFieldRow(array('label'=>'', 'type'=>'hidden', 'field_key'=>'editable', 'value'=>$editable));
	
	if ($editable == 1) {
		$visible_type = 'checkbox';
		$content .= $output->getHtmlFieldRow(array('label'=>$L['useradminGroupName'], 'type'=>'text', 'field_key'=>'name', 'value'=>bcrm_htmlspecialchars($name)));
	} else {
		$visible_type = 'free';
		$content .= $output->getHtmlFieldRow(array('label'=>$L['useradminGroupName'], 'type'=>'hidden', 'field_key'=>'name', 'value'=>bcrm_htmlspecialchars($name), 'text'=>bcrm_htmlspecialchars($name)));
	}
	
	$content .= $output->getHtmlFieldRow(array('label'=>$L['useradminVisible'], 'type'=>$visible_type, 'field_key'=>'visible', 'value'=>( ($visible_type == 'checkbox') ? $visible : ($visible ? $L['Yes'] : $L['No'])) ) );
	
	if (!$id && ($group['type'] = $_GET['item']['type'])) {
		$type = getGroupType($group['type']);
		$content .= $output->getHtmlFieldRow(array('label'=>$L['useradminGroupType'], 'type'=>'hidden', 'field_key'=>'type', 'value'=>$group['type'], 'text'=>$type));
	} else {
		$content .= $output->getHtmlFieldRow(array('label'=>$L['useradminGroupType'], 'type'=>'hidden', 'field_key'=>'type', 'value'=>$group['type'], 'read_only'=>1, 'text'=>$type));
	}
	
	$content .= $output->getEndNestedTable();
	
	$content .= $output->getFieldSpacerRow();
	
	if ($group['type'] == 0) { // Permissions group
		$content .= $output->getTextRow($L['useradminModulePrivileges']);
		
		if ($id && ($ua->groups->getGroupName($id) != 'Administrators')) {
			$content .= $output->getTextRow('<a href="javascript:if (confirm(\'' . $L['useradminAreYouSureResetPermissions'] . '\')) { window.location.replace(\''.$PHP_SELF.'?id='.$id.'&reset_perms=1\');}">' . $L['useradminResetGroupPermissionsDefaultEveryoneGroupValues'] . '</a>');
		}
		
		// module permissions table
		
		$content .= $output->getBeginNestedListTable();
		
		$header = array(
			'module' => array('label' => $L['useradminModule']), 
			'read'   => array('label' => $L['useradminRead']), 
			'write'  => array('label' => $L['useradminWrite']), 
			'admin'  => array('label' => $L['useradminAdministrator'])
		);
		
		$content .= $output->getHeaderRow($header);
		
		foreach ($list_of_modules as $k => $v) {
			$read = $write = $see_all = $admin = 0;
		
			if (isset($id) && $id) {
				if ( is_object($ua) && ($group = $ua->groups->getPrivileges($id, $k)))  {
					if (isset($group['priv']) && ($group['priv'] & $ua->getReadLevel())) $read = 1;
					if (isset($group['priv']) && ($group['priv'] & $ua->getWriteLevel())) $write = 1;
					if (isset($group['priv']) && ($group['priv'] & $ua->getAdminLevel())) $admin = 1;
				}
			}
			
			$read_box  = $output->getHtmlField(array('label'=>'', 'type'=>'checkbox', 'field_key'=>'priv][' . $k . '][read', 'value'=>$read));
			$write_box = $output->getHtmlField(array('label'=>'', 'type'=>'checkbox', 'field_key'=>'priv][' . $k . '][write', 'value'=>$write));
			$admin_box = $output->getHtmlField(array('label'=>'', 'type'=>'checkbox', 'field_key'=>'priv][' . $k . '][admin', 'value'=>$admin));
			$filler = $output->getHtmlField(array('label'=>'', 'type'=>'hidden', 'field_key'=>'priv][' . $k . '][filler', 'value'=>1));
			$content .= $output->getTableRow(array($v, $read_box, $write_box, $admin_box . $filler));
			
		}	
		
		$content .= $output->getEndNestedTable();
	}
	
	if ($editable || ($group['type'] == 0) ) {
		$content .= $output->getSubmitRow();
	}
	$content .= $output->getEndTable();
	$content .= $output->getEndForm();
	
	// $output->addToBody($content);
	// $output->display();
	// exit();
	
	$window_layout = $output->getBeginTable('mainTbl', true);
	$window_layout .= $output->getTitleRow($L['useradminSetPermissionsForGroup'], array('bookmark'=>'off'));
	$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
	$window_layout .= $content;
	$window_layout .= '</td>' . $output->getEndRow(true);
	$window_layout .= $output->getEndTable(true);
	
	$output->addToBody($window_layout);
	$output->display();
	exit();
}


function getGroupType($type_id = 0) {
	global $L;
	
	switch ($type_id) {
		case '2': 
			$type = $L['useradminGroupLabelTeams'];
		break;
		case '1':
			$type = $L['useradminGroupLabelPrivate'];
		break;
		default:
			$type = $L['useradminGroupLabelPermissions'];
		break;
	}
	
	return $type;
}

main();

?>