<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
//it is a violation of the terms of the licence agreement to alter 
// or modify the licence mechanisms and user count reporting

$licence_id = '17de344cb9c7cabb4aae3f7e85f5d612';
$licensed_users = 2;
$renewal_date = '';

/**
 * This service manager handles the creation of services for this application.
 *
 * Services are single instance, provided calls to {@link getService()} are returned by reference.
 * 
 * @author Phil Calder, Matthew Ross
 * @package Services
 */
class Services {
	
	/**
	 * @var licence_properties
	 * @access private
	 */
	var $licence_properties = false;
	
	/**
	 * @var class_debug
	 * @access private
	 */
	var $class_debug = false; // should be false when shipping

	/**
	 * @var start_token
	 * @access private
	 */
	var $start_token = '------ LICENCE FILE DATA -------';
	/**
	 * @var end_token
	 * @access private
	 */
	var $end_token =   '--------------------------------';  //32
	
	function Services($config_file = '') {
		$this->config = $this->getService('Config', $config_file);
		$db_config = $this->config->lookup('database');
		
		$this->database = $this->getService(
			'Database', 
			array(
				'username' => $db_config['username'], 
				'password' => $db_config['password'], 
				'database' => $db_config['dbname'], 
				'host'     => (isset($db_config['host']) ? $db_config['host'] : false), 
				'port'     => (isset($db_config['port']) ? $db_config['port'] : false)
			)
		);
		
		$this->licence_file = getLicenceDir() . '/licence.txt';
		
		// $this->log = $this->getService('Log');
		// $this->datelib = $this->getService('DateLib');
		// $this->lang = $this->getService('Language');
		// $this->prefs = $this->getService('Preferences');
		// $this->screen = $this->getService('Screen');
		// $this->output = $this->getService('Output');
		
		$this->setLicenceProperties();
	}
	
	function init() {
		// Example to override the Output class
		// includeModule('ModuleName', 'inc/custom_output.inc');
		// $my_page =& $this->getService('Output', '', 'CustomOutput');
		// $this->checkDbVersion( $this->config->lookup('app.version') );
		$this->checkDbVersion( $this->getCodeVersion() );
		$this->checkSecureOnly();
	}
	
	/**
	 * Loads and returns a service. Services are only loaded once.
	 * 
	 * The service can be accessed through the return variable or through the SERVICES object, as $SERVICES->type
	 *
	 * @param string $type Case sensitive Service name
	 * @param array $args This optional array will be sent as the first argument to the new service.
	 * @param string $override_name Optional class name to use instead of using $type. 
	 * $override_name should be the name of a class that extends class $type
	 * Include this extended class in the same file as class $type to ensure it is loaded when necessary.
	 * @return object $type, or false. The given Service object. Returns by reference ie use =& when getting this returned variable.
	 */
	function &getService($type, $args = array(), $override_name = '') {
		
		if (!empty($override_name)) {
			$var_type = strtolower($override_name);	
		} else {
			$var_type = strtolower($type);
		}
		
		if ($type == 'Output') { // see function init to override the Output class
			$var_type = strtolower($type);
		}
		
		if (isset($this->$var_type) && is_object($this->$var_type)) {
			// $this->servicesPrint('Services::getService(' . $type . ') - Already exists', false);
		} else {
			
			// $this->servicesPrint('Services::getService(' . $type . ') ' . $var_type, false);
			// $this->getService('Log');
			// $this->log->logAction('getService: ' . $type);
			
			if (!includeService($type)) {
				// $this->servicesPrint('Services::getService(' . $type . ') returning false', false);
				return false; // files dont exist or couldnt be loaded
			}
			
			// Override for Custom Database functionality
			if ( ($type == 'Database') && class_exists('CustomDatabase')) {
				$type = 'CustomDatabase';	
			}
			
			$class_name = (!empty($override_name) ? $override_name: $type);
			
			if (sizeof($args)) {
				// $this->$var_type = new $type($args);
				$this->$var_type = new $class_name($args);
			} else {
				// $this->$var_type = new $type();
				$this->$var_type = new $class_name();
			}
		}
		return $this->$var_type;
	}

	/**
	 * Check if the Service type is loaded
	 *
	 * @param string $type Case sensitive Service name
	 * @return boolean True or false if the service is already loaded
	 * @author Brenton Bartel 20050812
	 */
	function hasService($type){
		$var_type = strtolower($type);
		if ($type == 'Output') { // see function init to override the Output class
			$var_type = strtolower($type);
		}
		if (isset($this->$var_type) && is_object($this->$var_type)) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Use to create a separate Database object, eg to connect to an external database.
	 * @param array $args. Required keys: username, password, database, host. Optional: port
	 * @access private
	 */
	function getStandaloneDatabase($args) {
		if (!is_array($args)) {
			// print '<div>ERROR: no args</div>';
			return false; // wrong args
		}
		$args['standalone'] = true;
		
		if (!includeService('Database')) {
			// print '<div>ERROR: cant include service</div>';
			return false; // files dont exist or couldnt be loaded
		}
		if (!class_exists('Database')) {
			// print '<div>ERROR: cannot create standalone database - class does not exist</div>';
			return false; 
		}
		// print '<div>Creating standalone database</div>';
		$standalone_database = new Database($args);
		if (!is_object($standalone_database)) {
			// print '<div>ERROR: could not initialise standalone database</div>';
			return false; 
		} 
		return $standalone_database;
	}

	function getGlobal($key) {
		
		$key = strtolower($key);
		$val = '';
		$failed = false;
		
		switch($key) {
			case 'php_self':
				global $PHP_SELF;
				//val should be just filename with no query string
				$val = ($PHP_SELF ? $PHP_SELF : $_SERVER['PHP_SELF']);
				//verify that there is no query string
				if (strpos($val, '?') == true) {
					$val = explode('?', $val);
					$val = $val[0]; 
				}
			break;
			
			case 'query_string':
				//val should be query string with no leading ?
				$val = $_SERVER['QUERY_STRING'];
				//verify no leading ?
				if (substr($val, 0, 1) == '?') {
					$val = substr($val, 1);
				}
			break;
			
			case 'http_host':
				$val = $_SERVER['HTTP_HOST'];
			break;
			
			case 'http_referer':
			case 'http_referrer':
				$val = ($_SERVER['HTTP_REFERER'] ? $_SERVER['HTTP_REFERER'] : '');
			break;
			
			case 'user_agent':
				$val = $_SERVER['HTTP_USER_AGENT'];
			break;
			
			case 'script_filename':
				$val = $_SERVER['SCRIPT_FILENAME'];
			break;
			
			default:
				$failed = true;
			break;
			
		}
		
		// For debugging
		// if ($failed || empty($val)) {
			// $message = 'Global request for ' . $key . ' is empty';
			// if ($failed) {
				// $message .= ' (key not found)';
			// }
			// $this->getService('Log');
			// $this->log->logAction($message);
		// }
		
		return $val;
	}
	
	/**
	 * servicesPrint
	 *
	 * A simple 'debug' type output function. Disabled by default.
	 */
	function servicesPrint($msg, $force = true) {
		return; // ====== DISABLE HERE =====
		// Note: This requires the inc/service.inc printDebug method.
		// This will be disabled by default.
		if (is_object($this->service)) {
			$this->service->printDebug($msg, $force);
		} else if (is_object($this->config)) { // Phil 20060529
			$this->config->printDebug($msg, $force);
		}
	}
	
	// ------------------------------------------------
	
	/**
	 * @access private
	 */
	function isLicenceFileExisting() {
		if (is_file($this->licence_file)) {
			return true;
		}
		return false;
	}
	
	/**
	 * @access private
	 */
	function setLicenceProperties() {
		if ($this->isLicenceFileExisting()) {
			$fh = fopen($this->licence_file, 'r');
			$lic_body = fread($fh, 8192);
			
			$lic_body = base64_decode(base64_decode(str_replace(array("\r\n", "\n", "\r", $this->end_token), '', substr($lic_body, strpos($lic_body, $this->start_token) + strlen($this->start_token)))));
			foreach(explode(',', $lic_body) as $k => $v) {
				$v = explode('=', $v);
				if (strlen(trim($v[0])) > 4 && substr(trim($v[0]),0,4) == 'lic_') {
					$this->licence_properties[trim($v[0])] = trim($v[1]);
				}
			}
		}
	}
	
	/**
	 * @access private
	 */
	function getLicenceProperties() {
		return $this->licence_properties;
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function isValidLicence() {
		if ($this->isExpired()) {
			if (defined('LICENCE_PAGE') && LICENCE_PAGE) {
				
			} else {
				global $BCRM_DOCUMENT_ROOT, $L;
				$lang =& $this->getService('Language');
				$lang->getLanguage('Licence');
				
				$message_title = $L['licenceGetNewKey'];
				$message_body = $L['licenceIsExpired'];
				
				global $base_dir, $PHP_SELF;
				$base_dir = trim(substr($PHP_SELF, 0, strpos($PHP_SELF, 'setup')), '/'); // This is the sub directory we were in
				
				define('MESSAGE_SHOW_HELP', 1);
				$show_lic_upload = true;
				include($BCRM_DOCUMENT_ROOT . '/message.inc.php');
				exit();
				//exit('Your licence has expired');
			}
		}
	}
	
	/**
	 * @access private
	 */
	function showPromoMessages() {
		if ($this->isLicenceFileExisting()) {
			if ($this->licence_properties['lic_reg_only']) {
				return true;
			} else if (isset($this->licence_properties['lic_hidden_show_header_promo']) 
			&& $this->licence_properties['lic_hidden_show_header_promo']) {
				return true;
			} else {
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * deprecated
	 * @access private
	 */
	function getBuildDate() {
		if (isset($file_properties['buildBuildDate'])) {
			return $file_properties['buildBuildDate'];
		}
		return time();
	}
	
	/**
	 * deprecated
	 * @access private
	 */
	function getBuildId() {
		if (isset($file_properties['buildId'])) {
			return $file_properties['buildId'];
		}
	}
	
	/**
	 * @access private
	 */
	function getLicenceId() {
		global $licence_id;
		if ($this->licence_properties['lic_id']) {
			return $this->licence_properties['lic_id'];
		} else {
			return $licence_id;
		}
	}
	
	/**
	 * @access private
	 */
	function getRenewalDate() {
		global $renewal_date;
		if ($this->licence_properties['lic_renewal_date']) {
			$temp = explode('-', $this->licence_properties['lic_renewal_date']);
			return mktime(0,0,0, $temp[1],$temp[2],$temp[0]);
		} else {
			return $renewal_date;
		}
	}
	
	/**
	 * @access private
	 */
	function getExpiryDate() {
		if (isset($this->licence_properties['lic_expiry_date']) && $this->licence_properties['lic_expiry_date']) {
			$temp = explode('-', $this->licence_properties['lic_expiry_date']);
			return mktime(0,0,0, $temp[1],$temp[2],$temp[0]);
		} else {
			return false;
		}
	}
	
	/**
	 * @access private
	 */
	function isExpired() {
		if ($this->getExpiryDate() &&  $this->getExpiryDate() < (time() + 24*3600)) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * @access private
	 */
	function getContactUrlRoot() {
		return 'http://www.browsercrm.com';
	}
	
	/**
	 * @access private
	 */
	function getNewsUrlRoot() {
		return 'http://news5.browsercrm.com';
		//return 'https://www.browsercrm.com/news5/app'; 
	}
	
	// Phil 20061116 Added optional qt and notes parameters
	function getContactUrlQueryString($qt = false, $notes = false) {
		global $SERVICES;
		
		$qs = '';
		if (is_array($this->licence_properties) && count($this->licence_properties)) {
			foreach($this->licence_properties as $k => $v) {
				$qs .= '&' . $k . '=' . $v;
			}
		} else {
			global $licence_id, $licensed_users, $renewal_date;
			$qs .= '&lic_file=none';
			$qs .= '&lic_id=' . $licence_id;
			$qs .= '&lic_users=' . $licensed_users;
			$qs .= '&lic_renewal_date=' . $renewal_date;
		}
		
		$lang = $SERVICES->preferences->getUserPref('cfgLanguage');
		if (empty($lang)) {
			$lang = 'en'; // default
		}
		$qs .= '&user_lang=' . $lang;
		
		$qs .= '&installed_users=' . $this->getCurrentUsersCount();
		$qs .= '&ver=' . str_replace('.', '_', $this->getCodeVersion());
		$qs .= '&http_host=' . $this->getGlobal('http_host');
		$qs .= '&http_referer=' . $this->getGlobal('http_referer');
		
		if ($qt) {
			$qs .= '&qt=' . $qt;
		}
		
		if (!$notes) {
			$notes = 'php' . phpversion();
		} else {
			$notes .= ' | php' . phpversion();
		}
		$qs .= '&nt=' . $notes;
		
		return 'l=' . base64_encode($qs);
	}
	
	function getContactUrl($function_name) {
		return $this->getContactUrlRoot() . '/app_redirect.php?fn=' . $function_name . '&' . $this->getContactUrlQueryString();
	}
	
	function getNewsUrl($page_name = 'index', $qt = false, $notes = false) {
		return $this->getNewsUrlRoot() . '/' . $page_name . '.php?' . $this->getContactUrlQueryString($qt, $notes);
	}
	
	/**
	 * @access private
	 */
	function getNumLicensedUsers() { 
		//it is a violation of the terms of the licence agreement to alter or modify the 
		//user count reporting
		$licensed_users = 2;
		
		if ($this->licence_properties['lic_users']) {
			return $this->licence_properties['lic_users'];
		} else {
			return $licensed_users;
		}
	}
	
	/**
	 * @access private
	 */
	function getCurrentUsers() {
		$ua =& $this->getService('Authentication');
		return $ua->users->getCurrentUsers();
	}
	
	/**
	 * @access private
	 */
	function getCurrentUsersCount() {
		$ua =& $this->getService('Authentication');
		return count($ua->users->getUserIDs()) -1; // Allow for Admin user
	}
	
	// ------------------------------------------------
	
	function getOffsetFromTimeZone() {
		$this->getService('Preferences');
		$server_offset = date("Z");
		
		//timezones list is hardcoded, as offset from GMT
		$user_offset_from_gmt = $this->preferences->getUserPref('cfgTimeOffsetHours') * 3600;
		$offset = $server_offset - $user_offset_from_gmt;
		
		//test if current date (on server) is in daylight savings time
		if (date('I', time()) == 1) {
			$offset -= 3600;
		}
		return $offset;
	}
	
	function offsetTimeZoneForRead($timestamp, $offset=0) {
		if (!$offset) {
			$offset = $this->getOffsetFromTimeZone();
		}
		return ($timestamp - $offset);
	}
	
	function offsetTimeZoneForWrite($timestamp, $offset=0) {
		if (!$offset) {
			$offset = $this->getOffsetFromTimeZone();
		}
		return ($timestamp + $offset);
	}
	
	/**
	 * Uses curl functions if they exist, or fopen.
	 */
	function getDataFromUrl($url, $timeout = 10, $show_errors = false) {
		if (function_exists('curl_init')) {
			return $this->getDataFromUrlWithCurl($url, $timeout, $show_errors);
		}
		
		return $this->getDataFromUrlWithFopen($url, $timeout);
	}
	
	/**
	 * CURL function to retrieve data from a URL. 
	 * Phil - 20040626 optional timeout and show_errors parameters added
	 * (timeout may need to be increaded in some cases and showing errors 
	 * prevents any header calls because the errors are printed here)
	 */
	function getDataFromUrlWithCurl($url, $timeout = 10, $show_errors = false) { 
		$ch = curl_init();
		$agent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)";
		curl_setopt($ch, CURLOPT_USERAGENT, $agent);
	
		curl_setopt ($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch,CURLOPT_URL,$url);
		curl_setopt($ch,CURLOPT_RETURNTRANSFER,1); // so it will return data into a variable instead of printing out
		curl_setopt($ch,CURLOPT_TIMEOUT,$timeout); // give it a time in seconds to reply
		//curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false); //dont validate SSL cert
		$result = curl_exec($ch);
	
		if ($show_errors && curl_error($ch)) {
			printf("Curl error %s: %s", curl_errno($ch), curl_error($ch));
			print(' <a href="' . $url . '" target="_blank">This is the url</a><br>');
		}
	
		curl_close($ch);
		//echo $url . '<br>';	die('cURL says: ' . $result); //mcr
		return $result;
	}
	
	/**
	 * Use getDataFromUrl which will call this function if curl not available
	 * Note: requires php version >= 4.3.0
	 */
	 function getDataFromUrlWithFopen($url, $timeout = 10) {
		//fopen version ; needs php > 4.3.0 for fopen over ssl
		// Phil 20060720 Added fopen timeout and stream_* calls
		$old = @ini_set('default_socket_timeout', $timeout); // PHP >= 4.3.0
		if ($handle = @fopen($url, "rb")) {
			if ($old !== false) {
				@ini_set('default_socket_timeout', $old);
			}
			@stream_set_timeout($handle, $timeout);
			@stream_set_blocking($handle, 0);
			
			$contents = '';
			//while (!feof($handle)) {
				$contents .= @fread($handle, 8192);
			//	}
			@fclose($handle);
		}
		
		
		return $contents;
	}
	
	function getCodeVersion() {
		// Phil 20060927 Now using separate file to determine code version
		global $BCRM_DOCUMENT_ROOT;
		$version_file = $BCRM_DOCUMENT_ROOT . '/modules/Front/version.inc.php';
		@include_once($version_file);
		$config_version = $this->config->lookup('app.version');
		// print '<div>front version: ' . $front_version . '</div>';
		if (isset($front_version) && $front_version) {
			if ($config_version && ($config_version != $front_version)) {
				// print '<div>rebuild config - front version: ' . $front_version . ', config_version: ' . $config_version . '</div>';
				$this->config->removeCachedConfig();
				$this->config->init();
			}
			return $front_version;
		}
		return $this->config->lookup('app.version');
	}
	
	//MCR
	function checkDbVersion($code_version) {
		if (isset($GLOBALS['auth_no_version_check']) && $GLOBALS['auth_no_version_check']) {
			return;
		}
		if ($code_version) {  //only test where there is a config value
			$db_structure = $this->database->quickGet('version','System__version');
			// Phil 20050309 we are now using eg version x.xx.xx
			// but only want the version number x.xxx (the part used to determine whether a database upgrade is required)
			$db_structure = substr($db_structure,0,5); 
			if ($db_structure) {
				if (substr($code_version,0,5) != $db_structure) {	//consider 4.xxx part; forget suffix eg 4.xxx.xx for minor code upgrades
					global $L;
					$this->getService('Language');
					$this->language->getLanguage('Setup');
					// Phil
					$setup_href = ($this->config->lookup('site.websubdir') ? '/' . $this->config->lookup('site.websubdir') : '') . '/setup/index.php';
					$setup_link = '<br><a href="' . $setup_href . '" target="_top">' . (strlen($L['setupTitleSetup']) ? $L['setupTitleSetup'] : 'Setup') . '</a>';
					$this->displayMessage(
						$L['errDbAndCodeVersionsDiffer'] . ' 
						<br>' . $L['errDbStructureVersion'] . ': ' . $db_structure . '
						<br>' . $L['errCodeVersion'] . ': ' . substr($code_version,0,5) . '
						<br>' . $L['errRunUpgradesToVersion'] . ' ' .  substr($code_version,0,5) . '
						<br>' . $setup_link
					);
				}
			} else {
				global $L;
				$this->getService('Language');
				$this->language->getLanguage('Setup');
				// Phil
				$setup_href = ($this->config->lookup('site.websubdir') ? '/' . $this->config->lookup('site.websubdir') : '') . '/setup/index.php';
				$setup_link = '<br><a href="' . $setup_href . '" target="_top">' . $L['setupTitleSetup'] . '</a>';
				// print '<div>DEBUG, code_version: ' . $code_version . '</div>';
				$this->displayMessage($L['errCannotReadDbVersion'] . '<br>' . $setup_link);
			}
		}
	} // checkDbVersion
	
	/** 
	 * @access private
	 */
	function checkSecureOnly() {
		global $L, $BCRM_DOCUMENT_ROOT;
		$this->getService('Language');
		if ($this->config->lookup('site.force_secure')) {
			if (!isset($_SERVER['HTTPS']) || ($_SERVER['HTTPS'] != 'on')) {
				
				$message_title = '';
				$message_body = $L['SecureModeRequired'];
				if (!defined('MESSAGE_SHOW_HELP')) {
					define('MESSAGE_SHOW_HELP', 0);
				}
				include($BCRM_DOCUMENT_ROOT . '/message.inc.php');
				exit();
				// quickPage($L['SecureModeRequired']);
				// exit();
			}
		}
	}
	
	function displayMessage($message_body, $message_title = '', $show_help = false) {
		global $BCRM_DOCUMENT_ROOT;
		if (!$show_help && !defined('MESSAGE_SHOW_HELP')) {
			define('MESSAGE_SHOW_HELP', 0);
		}
		include($BCRM_DOCUMENT_ROOT . '/message.inc.php');
		exit();
		// quickPage($message);
		// exit();
	}
}

/**
 * Uses the Output service to display the given content.
 *
 * Note: It is standard practise to call exit() after this function.
 *
 * @param $content A string to display.
 * @param $page_type Optional, defaults to 'main'. This is the parameter sent to the Output service.
 * @param $page_class Optional, for classes which extend the standard Output service (eg some WebForms in the 'pub' folder).
 */
function quickPage($content, $page_type = 'main', $page_class = '') {
	// Phil 20061205: In many cases, 
	// we are now using $SERVICES->displayMessage rather than quickPage
	global $SERVICES;
	$tmp =& $SERVICES->getService('Output'); // Loads the relevant file(s)
	
	if ($page_class && class_exists($page_class)) {
		$output = new $page_class($page_type);
	} else {
		$output =& $SERVICES->getService('Output');
	}
	
	$content = $output->getBeginTable('mainTbl', true) . $output->getBeginDataCell(true) . $content;
	$content .= $output->getEndDataCell() . $output->getEndTable(true);
	$output->addToBody($content);
	$output->display();
}
	
/**
 * A customised call to the php function htmlspecialchars
 *
 * @param $param1 The string parameter sent to htmlspecialchars
 * @param $param2 Optional, Deprecated (defaults to ENT_QUOTES)
 * @param $param3 Optional, the character set to use, defaults to the value retrieved by the Config Service.
 */
function bcrm_htmlspecialchars($param1, $param2 = '', $param3 = '') {
	if (empty($param3)) { // Phil 20050204
		global $SERVICES;
		$SERVICES->getService('Config');
		$param3 = $SERVICES->config->lookup('charset');
	}
	// Phil 20070605 removed
	// if (strtolower($param3) == 'utf-8') { // Phil 20070503
		// return $param1;
	// }
	
	return @htmlspecialchars($param1, ENT_QUOTES, $param3);
}

/**
 * A customized call to the php function unset
 *
 * @param $param The variable to attempt to unset, must be an array.
 * @param $offset1 The first offset in the array - required
 * @param $offset2 The second offset - optional
 */
function bcrm_unset(&$param, $offset1, $offset2="") {
	// check first if it's an array, if not, jump out
	if (!is_array($param)) {
		return;
	}

	// General things: since this function can accept either string offset or
	// index offset, we should check that (cause if the index doesn't even exist
	// then the 'isset' function will return false)
	// since it may be a string, see if the array key even exists.
	// and finally just unset if it is set
	if ($offset2 != "") {
		// if neither is a string, just unset, otherwise will need to do a check
		if (!is_string($offset1) && !is_string($offset2)) {
			unset($param[$offset1][$offset2]);
		} else if (array_key_exists($offset1, $param) && array_key_exists($offset2, $param[$offset1])) { // need to test for both dimensions
			unset($param[$offset1][$offset2]);
	
		} else if (isset($param[$offset1][$offset2])) {
			unset($param[$offset1][$offset2]);
		}
	} else {
		// if it's not a string, just unset, otherwise will need to do a check
		if (!is_string($offset1)) {
			unset($param[$offset1]);
		} else if (array_key_exists($offset1, $param)) {
			unset($param[$offset1]);
		} else if (isset($param[$offset1])) {
			unset($param[$offset1]);
		}
	}
}

/**
 * A customized call to the php function array_merge
 *
 * @param mixed $args A mixed arguement that will be merged, note: this function can take any number of params.
 */
function bcrm_array_merge() {
	$return_array = array();

	$num_args = func_num_args();
	// loop through all the arguements
	for ($i=0; $i<$num_args; $i++) {
		$arg = func_get_arg($i);

		if (is_array($arg)) {
			$return_array = array_merge($return_array, $arg);
		} else {
			$return_array = array_merge($return_array, (array)$arg);
		}
	}

	return $return_array;
}
?>