<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Used for date/time input and display.
 * @package Services
 */
class DateLib extends Service {
	
	/**
	 * @access private
	 */
	var $show_errors = false;

	function DateLib() {
		global $SERVICES;
		$SERVICES->getService('Preferences');
	}
	
	function getDateInputMask() {
		global $SERVICES;
		$mask = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
		$mask = str_replace('j', 'd', $mask);
		$mask = str_replace('Y', 'y', $mask);
		return $mask;
	}

	// function getUserPrefDateFormatNoYear(){
		// return trim($SERVICES->preferences->getUserPref('cfgShortDateFormat'), '/Y'.'Y/'.'/y'.'y/'); // does this work???
	// }
	
	
	// user_pref: Override user pref with this param
	function getJsDateMaskFromUserPref($user_pref = ''){
		global $SERVICES;
		
		if ($user_pref == '') {
			$user_pref = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
		}
		
		// $date_format = explode('/', $SERVICES->preferences->getUserPref('cfgShortDateFormat'));
		$date_format = explode('/', $user_pref);
		$return_format = '';

		
		foreach($date_format as $key => $index){
			switch($index){
					
				// Single digit or multidigit day
				case 'j':
					$return_format .= 'd';
					break;
				case 'd':
					$return_format .= 'dd';
					break;
				
				// Single or multidigit month
				case 'n':
					$return_format .= 'm';
					break;
				case 'm':
					$return_format .= 'mm';
					break;
				
				// SHORT text month
				case 'M':
					$return_format .= 'MMM';
					break;
				
				// LONG text month
				case 'F':
					$return_format .= 'MMM';
					break;
				
				// Two digit year
				case 'y':
					$return_format .= 'yy';
					break;
					
				// Four digit year
				case 'Y':
					$return_format .= 'yyyy';
					break;
				
				default:
					// $this->error("Unrecognised date format: $in_date. Unable to continue.");
					// $this->error = 'Unrecognised date format: ' . (isset($in_date) ? $in_date : '') . ' Unable to continue.';
					$this->error = 'Unrecognised date format. Unable to continue.';
					return;
			}
			$return_format .= $SERVICES->preferences->getUserPref('cfgDateSeparator');
		}
		return trim($return_format, $SERVICES->preferences->getUserPref('cfgDateSeparator'));
	}
	
	
	function getUserPrefDateFromTimestamp($in_timestamp, $mask = '', $separator = ''){
		global $SERVICES;
		
		if ($in_timestamp <= 0) {
			
			return '';
			
		} else {
			
			if($mask == '') {
				$mask = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
			}
			
			if($separator == '') {
				$separator = $SERVICES->preferences->getUserPref('cfgDateSeparator');
			}
		
			return date(str_replace('/', $separator, $mask), $in_timestamp);
			
		}
		
	}
	
	
	function getUserPrefTimeFromTimestamp($in_timestamp, $mask = '', $separator = ''){
		global $SERVICES;
		if($mask == '')
				$mask = $SERVICES->preferences->getUserPref('cfgTimeFormat');
			
		if($separator == '')
			$separator = $SERVICES->preferences->getUserPref('cfgTimeSeparator');
				
		return date(str_replace(':', $separator, $mask), $in_timestamp);
	}
	
	
	function getTimestampFromUserPrefDateAndTime($in_date, $in_time){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setDateFormat($SERVICES->preferences->getUserPref('cfgShortDateFormat'), $SERVICES->preferences->getUserPref('cfgDateSeparator'));
		$dateConvert->setDate($in_date);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		
		$dateConvert->setTimeFormat($SERVICES->preferences->getUserPref('cfgTimeFormat'), $SERVICES->preferences->getUserPref('cfgTimeSeparator'));
		$dateConvert->setTime($in_time);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		// print '<div>getTimestampFromUserPrefDateAndTime</div>';
		return $dateConvert->getTimestamp();
	}
	
	
	function getTimestampFromUserPrefDate($in_date, $mask = '', $separator = ''){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setDateFormat($mask, $separator);
		$dateConvert->setDate($in_date);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		// print '<div>getTimestampFromUserPrefDate</div>';
		return $dateConvert->getTimestamp();
	}
	
	
	function getTimestampFromUserPrefTime($in_time, $mask = '', $separator = ''){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setTimeFormat($mask, $separator);
		$dateConvert->setTime($in_time);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		
		// print '<div>getTimestampFromUserPrefTime ' . $in_time . ' | ' . $mask . ' | ' . $separator . '</div>';
		return $dateConvert->getTimestamp();
	}
	
	
	function getMySqlDateFromUserPrefDate($in_date, $mask = '', $separator = ''){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setDateFormat($mask, $separator);
		$dateConvert->setDate($in_date);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		
		$dateConvert->setDateFormat('Y/m/d', '-');
		$return = $dateConvert->getDate();
		
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		} else {
			return $return;
		}
	}
	
	
	function getMySqlTimeFromUserPrefDate($in_time, $mask = '', $separator = ''){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setTimeFormat($mask, $separator);
		$dateConvert->setTime($in_time);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		
		$dateConvert->setTimeFormat('H:i', ':');
		$return = $dateConvert->getTime();
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		} else {
			return $return;
		}
	}
	
	
	function getUserPrefDateFromMySqlDate($in_date){
		global $SERVICES;
		if ($in_date == 0) {
			return '';
		} else {
			$dateConvert = new DateConverter();
			
			$dateConvert->setDateFormat('Y/m/d', '-');
			$dateConvert->setDate($in_date);
			if($dateConvert->getError()){
				$this->printString($dateConvert->getError());
				return;
			}
			
			$dateConvert->setDateFormat($SERVICES->preferences->getUserPref('cfgShortDateFormat'), $SERVICES->preferences->getUserPref('cfgDateSeparator'));
			$return = $dateConvert->getDate();
			if($dateConvert->getError()){
				$this->printString($dateConvert->getError());
				return;
			} else {
				return $return;
			}
		}
	}
	
	
	function getUserPrefTimeFromMySqlDate($in_time){
		global $SERVICES;
		$dateConvert = new DateConverter();
		
		$dateConvert->setTimeFormat('H:i', ':');
		$dateConvert->setTime($in_time);
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		}
		
		$dateConvert->setTimeFormat($SERVICES->preferences->getUserPref('cfgTimeFormat'), $SERVICES->preferences->getUserPref('cfgTimeSeparator'));
		$return = $dateConvert->getTime();
		if($dateConvert->getError()){
			$this->printString($dateConvert->getError());
			return;
		} else {
			return $return;
		}
	}
	
	function getTimestampFromMysqlDate($mySQL) {
		return strtotime($mySQL);
	}
	
	function getMysqlDateFromTimestamp($timestamp) {
		return date('Y-m-d',$timestamp);
	}
	
	/**
	 * Wrapper function for printDebug
	 *
	 * Standard functionality is not to print this to screen.
	 * Will call {@link printDebug} (Commented out)
	 * Will also print string if show errors is set in this class
	 * Logs the string through the Logger service (as an error)
	 *
	 * @author Phil Calder
	 * 20051109
	 */
	function printString($str = '') {
		if(!empty($str) && is_string($str)) {
			$this->logError($str);
			if ($this->show_errors) {
				print $str;
			}
			// $this->printDebug($str);
		}
	}
}


/**
 * Standalone Date Converter class
 */
class DateConverter {

	function DateConverter() {
		global $SERVICES;
		
		// SET the initial values
		$this->year = '1970';
		$this->month = '01';
		$this->day = '01';
		$this->hour = '00';
		$this->minute = '00';
		$this->second = '00';
		$this->error = '';
		
		// SET the formats
		$this->date_separator = $SERVICES->preferences->getUserPref('cfgDateSeparator');
		$this->date_format = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
		$this->time_separator = $SERVICES->preferences->getUserPref('cfgTimeSeparator');
		$this->time_format = $SERVICES->preferences->getUserPref('cfgTimeFormat');
	}
	
	function setDateFormat($in_format = '', $in_separator = ''){
		// SET the format values if we need to
		if($in_format != '')
			$this->date_format = $in_format;
		
		if($in_separator != '')
			$this->date_separator = $in_separator;
	}
	
	function setTimeFormat($in_format = '', $in_separator = ''){
		// SET the format values if we need to
		if($in_format != '')
			$this->time_format = $in_format;
		
		if($in_separator != '')
			$this->time_separator = $in_separator;
	}
	
	function setDate($in_date){
		// Initalise the error
		$this->error = '';
		
		//SET the date values
		$format = explode('/', $this->date_format);
		$dates = explode($this->date_separator, $in_date);
		
		foreach($format as $key => $index){
			$date_value = $dates[$key];
			switch($index){
					
				// Single digit or multidigit day
				case 'j':
				case 'd':
					$inval = intval($date_value);
					if(0 > $inval || $inval > 32){
						$this->error = "Day value: $date_value outside bounds.";
						return;
					} elseif($date_value != (string)$inval){
						$this->error = "Day value: $date_value not a number.";
						return;
					} else {
						$this->day = (strlen($date_value) == 1 ? '0'.$date_value : $date_value);
					}
					break;
				
				// Single or multidigit month
				case 'n':
				case 'm':
					$inval = intval($date_value);
					if(0 > $inval || $inval > 12){
						$this->error = "Month value: $date_value outside bounds.";
						return;
					} elseif($date_value != (string)$inval){
						$this->error = "Month value: $date_value not a number.";
						return;
					} else {
						$this->month = (strlen($date_value) == 1 ? '0'.$date_value : $date_value);
					}
					break;
				
				// SHORT text month
				case 'M':
					$loc = array_search(ucfirst($date_value), array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'));
					if($loc === false){
						$this->error = "Month value: $date_value not in recogonised format.";
						return;
					} else {
						$this->month = $loc+1;
						$this->month = (strlen($this->month) == 1 ? '0'.$this->month : $this->month);
					}
					break;
				
				// LONG text month
				case 'F':
					$loc = array_search(ucfirst($date_value), array('January','February','March','April','May','June','July','August','September','October','November','December'));
					if($loc === false){
						$this->error = "Month value: $date_value not in recogonised format.";
						return;
					} else {
						$this->month = $loc+1;
						$this->month = (strlen($this->month) == 1 ? '0'.$this->month : $this->month);
					}
					break;
				
				// Two digit year
				case 'y':
					$inval = intval($date_value);
					if(-1 > $inval || $inval > 100){
						$this->error = "Year value: $date_value outside bounds.";
						return;
					} elseif($date_value != (string)$inval){
						$this->error = "Year value: $date_value not a number.";
						return;
					} else {
						$this->year = ($date_value > 49 ? '19'.$date_value : '20'.$date_value);
					}
					break;
					
				// Four digit year
				case 'Y':
					$inval = intval($date_value);
					if(0 > $inval || $inval > 9999){
						$this->error = "Year value: $inval outside bounds.";
						return;
					} elseif($date_value != (string)$inval){
						$this->error = "Year value: $date_value not a number.";
						return;
					} else {
						$this->year = $date_value;
					}
					break;
				
				default:
					$this->error = "Unrecognised date format: $in_date.";
					return;
			}
		}
	}
	
	function setTime($in_time){  //should be H:i I think , MCR where : is the UserPref separator
		// Initalise the error
		$this->error = '';
		
		//SET the date values
		$format = explode(':', $this->time_format);
		$times = explode($this->time_separator, $in_time);
		
		//print_r($format);
		//print_r($times);
		
		foreach($format as $key => $index){
			
			$time_value = (isset($times[$key]) ? $times[$key] : '');
			
			if ($time_value) {  //added MCR
				switch($index){
					// Single digit or Multidigit hours
					case 'g':
					case 'G':
					case 'h':
					case 'H':
						$inval = intval($time_value);
						//if(-1 > $inval || $inval > 59){
						if(-1 > $inval || $inval > 24){
							$this->error = "Hour value: $time_value outside bounds.";
							return;
						} elseif($time_value != (string)$inval){
							$this->error = "Hour value: $time_value not a number.";
							return;
						} else {
							$this->hour = (strlen($time_value) == 1 ? '0'.$time_value : $time_value);
						}
						break;
						
					// Single digit or Multidigit minutes
					case 'i':
						$inval = intval($time_value);
						if(-1 > $inval || $inval > 59){
							$this->error = "Minute value: $time_value outside bounds.";
							return;
						} elseif($time_value != (string)$inval){
							$this->error = "Minute value: $time_value not a number.";
							return;
						} else {
							$this->minute = (strlen($time_value) == 1 ? '0'.$time_value : $time_value);
						}
						break;
					
					// Multidigit seconds
					case 's':
						$inval = intval($time_value);
						if(-1 > $inval || $inval > 59){
							$this->error = "Second value: $time_value outside bounds.";
							return;
						} elseif($time_value != (string)$inval){
							$this->error = "Second value: $time_value not a number.";
							return;
						} else {
							$this->second = (strlen($time_value) == 1 ? '0'.$time_value : $time_value);
						}
						break;
					
					default:
						$this->error = "Unrecognised time format: $in_time.";
						return;
				}
			} else {
				//return;
			}
		}
	}
	
	function getDate(){
		// Initalise the error
		$this->error = '';
		
		// print("Day:$this->day, Month:$this->month, Year:$this->year");
		
		//Get the date values
		$format = explode('/', $this->date_format);
		$return_date = '';
	
		foreach($format as $key => $index){
			switch($index){
					
				// Single digit or multidigit day
				case 'j':
					// $return_date .= trim($this->day, '0');
					$return_date .= ltrim($this->day, '0');
					// print '<div>j = ' . $return_date . '</div>';
					break;
				case 'd':
					$return_date .= $this->day;
					break;
				
				// Single or multidigit month
				case 'n':
					// $return_date .= trim($this->month, '0');
					$return_date .= ltrim($this->month, '0');
					break;
				case 'm':
					$return_date .= $this->month;
					break;
				
				// SHORT text month
				case 'M':
					$loc = array('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
					// $return_date .= $loc[trim($this->month, '0')-1];
					$return_date .= $loc[ltrim($this->month, '0')-1];
					break;
				
				// LONG text month
				case 'F':
					$loc = array('January','February','March','April','May','June','July','August','September','October','November','December');
					// $return_date .= $loc[trim($this->month, '0')-1];
					$return_date .= $loc[ltrim($this->month, '0')-1];
					break;
				
				// Two digit year
				case 'y':
					$return_date .= substr($this->year, 2, 2);
					break;
					
				// Four digit year
				case 'Y':
					$return_date .= $this->year;
					break;
				
				default:
					$this->error = "Unrecognised date format: $this->date_format.";
					return;
			}
			$return_date .= $this->date_separator;
		}
		//print("Date:$return_date");
		return trim($return_date, $this->date_separator);
	}
	
	function getTime(){
		// Initalise the error
		$this->error = '';
		
		//print("Hours:$this->hour, Minutes:$this->minute");
		
		//Get the time values
		$format = explode(':', $this->time_format);
		$return_time = '';
		
		
		foreach($format as $key => $index){
			switch($index){
					
				// Single digit or multidigit hours
				case 'g':
				case 'G':
					// $return_time .= trim($this->hour, '0');
					$return_time .= ltrim($this->hour, '0');
					break;
				case 'h':
				case 'H':
					$return_time .= $this->hour;
					break;
				
				// Multidigit minutes
				case 'i':
					$return_time .= $this->minute;
					break;
					
				// Multidigit seconds
				case 's':
					$return_time .= $this->second;
					break;
				
				default:
					$this->error = "Unrecognised time format: $this->time_format.";
					return;
			}
			$return_time .= $this->time_separator;
		}
		//print("Time:$return_time");
		return trim($return_time, $this->time_separator);
	}
	
	function getTimestamp(){
		// return the created timestamp
		// Phil 20051109 added ampersand to mktime
		// echo ' [' . $this->hour. ' | ' . $this->minute. ' | ' . $this->second. ' | ' . $this->month. ' | ' . $this->day. ' | ' . $this->year . '] ';
		return @mktime($this->hour, $this->minute, $this->second, $this->month, $this->day, $this->year);
	}
	
	function getError(){
		// return the error, if we have one
		if($this->error != '')
			return $this->error;
	}
		
}
?>