<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */
 
/**
 * This Language class handles storage and retrieval of language specific text.
 *
 * @author Phil Calder
 * @package Services
 */
class Language extends Service {
	/**
	 * @access private
	 */
	var $files = '';
	/**
	 * @access private
	 */
	var $languages = array();
	/**
	 * @access private
	 */
	var $modules_loaded = false; // Module Service may not have been loaded
	/**
	 * @access private
	 */
	var $lang_dir;
	/**
	 * @access private
	 */
	var $modules_dir;
	/**
	 * @access private
	 */
	var $modules_ext_dir;
	/**
	 * @access private
	 */
	var $modules_custom_dir;
	/**
	 * @access private
	 */
	var $custom_lang_dir;
	/**
	 * The language constructor initialises the object and loads the global language file.
	 */
	function Language() {
		global $SERVICES;
		$this->config =& $SERVICES->getService('Config');
		$this->filesystem =& $SERVICES->getService('FileSystem');
		$this->lang_dir = $this->config->lookup('site.langdir');
		$this->modules_dir = $SERVICES->config->lookup('modules.basedir');
		$this->modules_ext_dir = $SERVICES->config->lookup('modules.extension_basedir');
		$this->modules_custom_dir = $SERVICES->config->lookup('modules.custom_basedir');
		$this->custom_lang_dir = $this->config->lookup('site.customlangdir');
		
		$this->getLanguage('_Global');
		$this->languages[] = '_Global';
	}
	
	/**
	 * Attempts to load the language file for the specified resource.
	 *
	 * The default language is loaded first ('en'), then the users preferred language.
	 * There are varoius language file locations, all of which are loaded if the exist.
	 * eg: 
	 * The main language file (or the module-based language file if the main file is not found).
	 * The custom 'system' language file
	 * The custom 'per installation' file.
	 * Major changes in made in the way order the various language files are loaded - 20050404 - Phil
	 * @param $name The resource name - This is (usually) a module name. It is NOT the name of the language (eg 'en', 'fr').
	 */
	function getLanguage($name) {
		// print '<div>getLanguage(' . $name . ')</div>';
		if (in_array($name, $this->languages)) {
			// $this->printDebug('Language::getLanguage(' . $name . ') - already loaded');
			//print('Language::getLanguage(' . $name . ') - already loaded');
			
			return; // already loaded this language
		}
		
		$lang = $this->getUserPrefLang();
		
		if ($this->cacheNeedsRebuilding($name, $lang)) {
			$this->rebuildCache($name, $lang);
		}
		global $BCRM_DOCUMENT_ROOT;
		$this->languages[] = $name;
		$this->_load($this->getCacheDir() . '/' . $lang . '/' . $name . '.inc');
		
		if ($name == 'Contact') {
			// print '<div>loading person and company</div>';
			$this->getLanguage('Person');
			$this->getLanguage('Company');
		} elseif ($name == 'Person') {
			// print '<div>loading person and company</div>';
			$this->getLanguage('Contact');
			$this->getLanguage('Company');
		}
			
	}
	
	/**
	 * @access private
	 */
	function removeCache() {
		$cache_dir = $this->getCacheDir();
		$this->filesystem->recursiveDelete($cache_dir);
	}
	
	/**
	 *
	 * @param string $name The module name
	 * @param string $lang The language name (Optional, default 'en')
	 * @return boolean Whether the cache file for the given language needs to be rebuilt
	 * @access private
	 */
	function cacheNeedsRebuilding($name, $lang = 'en') {
		if ($this->filesystem->checkFileExists($name . '.inc', $this->getCacheDir() . '/' . $lang)) {
			// print '<p>compiled lang file exists - no need to rebuild (' . $name . ', ' . $lang . ')</p>';
			return false;
		}
		// print '<p>compiled lang file does not exist - need to rebuild (' . $name . ', ' . $lang . ') <br>[' . $this->getCacheDir() . '/' . $lang. '/' . $name . '.inc]</p>';
		return true;
	}
	
	/**
	 * @return string Full path to the cahce directory
	 * @access private
	 */
	function getCacheDir() {
		global $BCRM_DOCUMENT_ROOT;
		return $BCRM_DOCUMENT_ROOT . '/cache/lang';
	}
	
	/**
	 * @param string $name The module name
	 * @param string $lang The language name (Optional, default 'en')
	 * @access private
	 */
	function rebuildCache($name, $lang = 'en') {
		// $this->printDebug('Language::rebuildCache(' . $name . ', ' . $load . ') - loading');
		// print '<div>Language::rebuildCache(' . $name . ', ' . $lang . ')</div>';
		global $L;
		$old_L = (array)$L; // retain current language keys/values
		$L = array(); // empty language array
		
		$dir_name = $name; // Phil 20051011
		$name = $this->_getSubstituteName($name); // Phil 20051011
		
		// English (we develop in English. Its possible there are terms in 
		// this file that are not in the users preferred language)
		if ($lang != 'en') {
			// Standard English Language file
			$lang_file = $this->lang_dir . '/en/' . $name . '.inc';
			$this->_load($lang_file);
			
			// Custom English Language file
			$lang_file = $this->custom_lang_dir . '/en/' . $name . '.inc';
			$this->_load($lang_file);
		}
		
		// Custom Module Language Extension file
		$lang_file = $this->modules_custom_dir . '/' . $dir_name . '/_language.inc'; // Phil 20051011
		$this->_load($lang_file);
		
		// Standard Module Language file
		$lang_file = $this->modules_dir . '/' . $dir_name . '/_language.inc'; // Phil 20051011
		$this->_load($lang_file);
		
		// Module Language Extension file
		$lang_file = $this->modules_ext_dir . '/' . $dir_name . '/_language.inc';
		
		$this->_load($lang_file);
		
		// Standard Language file - users preferred language
		$lang_file = $this->lang_dir . '/' . $lang . '/' . $name . '.inc';
		$this->_load($lang_file);
		
		// Custom Language file - users preferred language
		$lang_file = $this->custom_lang_dir . '/' . $lang . '/' . $name . '.inc';
		$this->_load($lang_file);
		
		$result = false;
		if (function_exists('var_export')) {
			
			// Make sure the real cache dir exists
			$real_cache_dir = dirname($this->getCacheDir());
			if ($real_cache_dir && !$this->filesystem->checkDirExists(basename($real_cache_dir), dirname($real_cache_dir))) {
				$this->filesystem->createDir(basename($real_cache_dir), dirname($real_cache_dir));
			}
		
			// Make sure the cache/lang dir exists
			if (!$this->filesystem->checkDirExists(basename($this->getCacheDir()), dirname($this->getCacheDir()))) {
				// print '<div>Need to create lang dir: ' . $this->getCacheDir() . '/' . $lang . '</div>';
				$this->filesystem->createDir(basename($this->getCacheDir()), dirname($this->getCacheDir()));
			}
			// Make sure the cache/lang/[lang] dir exists
			if (!$this->filesystem->checkDirExists($lang, $this->getCacheDir())) {
				// print '<div>Need to create lang dir: ' . $this->getCacheDir() . '/' . $lang . '</div>';
				$this->filesystem->createDir($lang, $this->getCacheDir());
			} // else {
				// print '<div>NO need to create lang dir: ' . $this->getCacheDir() . '/' . $lang . '</div>';
			// }
			
			$lang_content = '<?php' . "\n" . '$L=';
			$lang_content .= var_export($L, true);
			$lang_content .= "\n" . '?>';
			$newfile = $this->getCacheDir() . '/' . $lang . '/' . $name . '.inc';
			
			$result = $this->filesystem->writeFileContents(false, false, $lang_content, true, $newfile);
			// if ($file = @fopen($newfile, "w")) {
				// @fwrite($file, $lang_content);
				// @fclose ($file);
			// }
			
		}
		
		if ($result === false) {
			$L = bcrm_array_merge($old_L, $L); // Populate current lang array
		} else {
			$L = $old_L; // restore current language keys/values
		}
	}
	
	/**
	 * Used to substitute one language file (set) for another
	 *
	 * A module may contain text specific to one domain. Some installations may need to 
	 * select a different language file to make the langauage file more relevant for their needs.
	 * eg The 'Company' module may be more appropriately named 'Organistaion' for some installations.
	 *
	 * @param string name - The param as passed to {@link getLanguage}
	 * @return string substiute name (concatenated Existing with Substiitute eg ExistingSubstitute
	 * @access private
	 */
	function _getSubstituteName($name) {
		if (is_array($substitutes = $this->config->lookup('services.language.substitute'))) {
			if (array_key_exists($name, $substitutes) && $substitutes[$name]) {
				$name = $name . $substitutes[$name];
				// print '<div>name: ' . $name . '</div>';
			}
		}
		return $name;
	}
	
	/**
	 * This is an internal (private) function
	 * @access private
	 */
	function _load($file) {
		global $L;
		// if (!empty($file) && file_exists($file) && is_readable($file)) {
		if (!empty($file) && $this->filesystem->checkFileExists(basename($file), dirname($file))) {
			// $this->printDebug('Language loading: ' . $file );
			// print '<div>loading: ' . $file . '</div>';
			$old_L = $L;
			@require_once($file);
			$L = array_merge((array)$old_L, (array)$L);
			// $this->printDebug('Language loaded: ' . $file );
			return true;
		}
		// $this->printDebug('Language NOT loading: ' . $file);
		// print '<div>NOT loading: ' . $file . '</div>';
		return false;
	}
	
	/**
	 * checks if the value passed is a language item if its not it removes the module name and returns as text
	 * 
	 * @param string $language_item the language key
	 * @param string $module_name Optional module name (defaults to the current module)
	 * @return string the language item of the text
	 *
	 * @author Matthew Corich, 20040815
	 * @author Phil Calder, 20040922 - module_name optional (defaults to current module)
	 */
	function getLanguageText($language_item, $module_name = '') {
		global $L;
		// print '<div>language_item: [' . $language_item . '] module_name: ' . $module_name . ' lang: ' . $L[$language_item] . '</div>';
		if ( isset($L[$language_item])) { //  &&  $L[$language_item]) {
			$temp_label = $L[$language_item];
			// print '<div>temp 1: ' . $temp_label . '</div>';
		} else {
			$this->printDebug('Language::getLanguageText(' . $language_item . ', ' .  $module_name . ')');
			if (empty($module_name)) {
				$module_name = $this->getCurrentModuleName();
				// $this->printDebug('Language::getLanguageText(' . $language_item . ', ' .  $module_name . '), loading current module');
			}
			
			$this->getLanguage($module_name);
			// print '<div>language_item: ' . $language_item . ' module_name: ' . $module_name . ' lang: ' . $L[$language_item] . '</div>';
			
			global $L;
			if ( isset($L[$language_item])) { // &&  $L[$language_item]) {
				$temp_label = $L[$language_item];
				// print '<div>temp 2: ' . $temp_label . '</div>';
			} else {	//allow use of label direct from module config file with no language file present
				$temp_label = $language_item;
				if ( substr($temp_label, 0, strlen($module_name)) == strtolower($module_name) ) {
					$temp_label = substr($temp_label, strlen($module_name)); //strip module name from front
					// print '<div>temp 3: ' . $temp_label . '</div>';
				}
			}
		}
		//print '<div>temp 4: ' . $temp_label . '</div>';
		return $temp_label;
	}
	
	function getCurrentModuleName() {
		global $SERVICES;
		if (!$this->modules_loaded) {
			$SERVICES->getService('Modules');
			$this->modules_loaded = true;
		}
		$module_name = $SERVICES->modules->getCurrent();
		return $module_name;
	}
	
	/**
	 * @access private
	 */
	function getUserPrefLang() {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		$lang = $SERVICES->preferences->getUserPref('cfgLanguage');
		if (empty($lang)) {
			$lang = 'en'; // default
		}
		return $lang;
	}
} // class Language
?>