<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Logs messages to a given medium (defaults to database).
 *
 * Note: There is also a Logger module which gets the usage stats from the same database table
 * Eventually that should be incorporated with this service
 * @author Phil Calder
 * @package Services
 */
class Log extends Service {
	
	/** 
	 * @access private
	 */
	var $db = '';
	/** 
	 * @access private
	 */
	var $errors_table = '';
	/** 
	 * @access private
	 */
	var $usage_table = '';
	/** 
	 * @access private
	 */
	var $actions_table = '';
	
	/**
	 * Initialises the Log object
	 */
	function Log() {
		global $SERVICES;
		
		$this->errors_table = 'Logger__errors';
		$this->usage_table = 'Logger__usage';
		$this->actions_table = 'Logger__actions';
		
		$this->facilities_to_skip = array('Front'); // ie Modules we dont log usage for
			
		// $this->db = &$db;
		$this->db =& $SERVICES->getService('Database');
		// $this->db =& $SERVICES->getService('Authentication');
		// $this->db =& $SERVICES->getService('Database');
		
	}
	
	/**
	 * Logs the given string to the system error log (database table).
	 *
	 * @param $str The error message.
	 * @param $facility Optional facility name, eg the current module.
	 * @param $priority Optional priority level string. Can be any value (eg 'Debug', 'Development'). Used for filtering results.
	 */
	function logError($str, $facility = '', $priority = '') {
		global $PHP_SELF, $USER;
		
		$current_error_status = $this->db->errorStatus(false); // Turn error tracking off
		
		if (empty($facility)) {
			
			$facility = $this->getFacility();
			
		}
		
		$str = $this->arrayToString($str);
		
		// $this->printDebug('<strong>logError</strong>: ' . $str);
		
		$fields = array(
			'timestamp' => time(), 
			'facility'  => $facility, 
			'priority'  => $priority, 
			'message'   => $str,
			'page'      => $PHP_SELF, 
			'user_id'   => (isset($USER['user_id']) ? $USER['user_id'] : 0)
		);
		
		// $this->printDebug($fields);
		
		$this->db->insert($this->errors_table, $fields);
		
		$this->db->errorStatus($current_error_status); // Turn error tracking back to what it was
		
	}
	
	/**
	 * Logs to the system usage log (database table).
	 *
	 * @param $facility Optional facility name, default to the current module.
	 */
	function logUsage($facility = '') {
		global $PHP_SELF, $USER;
		
		// $this->printDebug('<strong>logUsage</strong>: ' . $PHP_SELF . ', ' . $USER['user_id'] . ', ' . session_id() . '');
		$current_error_status = $this->db->errorStatus(false); // Turn error tracking off
		
		if (empty($facility)) {
			
			$facility = $this->getFacility();
			
		}
		
		if (!empty($facility) && !in_array($facility, $this->getFacilitiesToSkip())) {
		
			$fields = array(
				'timestamp'   => time(), 
				'module_name' => $facility, 
				'session_id'  => session_id(), 
				'page'        => $PHP_SELF, 
				'user_id'     => $USER['user_id'],
				'remote_addr' => $_SERVER['REMOTE_ADDR']
			);
			
			// print_r($fields);
			// $this->printDebug('Log::logUsage:');
			// $this->printDebug($fields);
			
			$this->db->insert($this->usage_table, $fields);
			
		}
		
		// Update last load time for user
		// (Note: place this in the block above to exclude this update for certain modules)
		if (!empty($USER['user_id'])) {
			
			// $this->printDebug('Log::logUsage calling Authentication::users::setLastLoad');
			
			global $SERVICES;
		
			$SERVICES->getService('Authentication');
			
			$SERVICES->authentication->users->setLastLoad();
			
		}
		
		$this->db->errorStatus($current_error_status); // Turn error tracking back to what it was
		
	}
	
	/**
	 * Logs the given string to the system action log (database table).
	 *
	 * Useful for recording user actions such as editing or deleting an item in a module.
	 * @param $str The 'action' message.
	 * @param $facility Optional facility name, defaults to the current module.
	 * @param $priority Optional priority level string. Can be any value (eg 'Debug', 'Development'). Used for filtering results.
	 */
	function logAction($str, $facility = '', $table='', $action=0, $item_id='', $priority = '') {
		global $PHP_SELF, $USER;
		
		// Phil 20070619 mainly for automated scripts that we dont want to log for
		if (isset($USER['no_log']) && $USER['no_log']) {
			return;
		}
		
		// $this->db->log_sql = false; // Turn logging off (see Database->logSql)
		$current_error_status = $this->db->errorStatus(false); // Turn error tracking off
		
		if (empty($facility)) {
			
			$facility = $this->getFacility();
			
		}
		
		$str = $this->arrayToString($str);
		
		// $this->printDebug('<strong>logAction</strong>: ' . $str . '');
		
		$fields = array(
			'`timestamp`' => time(), 
			'facility'  => $facility, 
			'priority'  => $priority, 
			'message'   => $str, 
			'page'      => $PHP_SELF,
			'module'    => $facility,
			'table_name'=> $table,
			'item_id'   => $item_id,
			'action'    => $action,
			'user_id'   => $USER['user_id']
		); //Syed 20050824 | added fields module, etc
		
		// $this->printDebug($fields);
		
		$this->db->insert($this->actions_table, $fields);
		
		// $this->db->log_sql = $current_log_status; // Turn logging back to what it was
		$this->db->errorStatus($current_error_status); // Turn error tracking back to what it was
		
	}
	
	/**
	 * Retrieves errors from the system error log (database table).
	 *
	 * @param $where An optional MySQL where clause, defaults to '1' (Note: do not include the word 'WHERE')
	 * @param $limit An optional MySQL limit clause, defaults to 'LIMIT 1, 50' (Note: do include the word 'LIMIT')
	 * @param $order Optional, defaults to 'timestamp DESC'
	 * @return An array of error items.
	 */
	function getErrors($where = '1', $limit = 'LIMIT 1, 50', $order = 'timestamp DESC') {
		
		if (is_array($order)) {
			
			$order = implode(' ', $order);
			
		}
		
		$result = $this->db->select(
			'*',
			$this->errors_table, 
			$where,
			$order,
			'', 
			$limit
		);
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$row['message'] = addslashes($row['message']); // Added Phil 20050202
			
			$items[] = $row;
			// print_r($row);
			
		}
		
		return $items;
		
	}
	
	/**
	 * @return The number of error items in the system error log (database table).
	 */
	function countErrors() {
		
		return $this->db->quickGet(
			'count(*)',
			$this->errors_table
		);
		
	}
	
	/**
	 * Not implemented. See the 'Logger' module for getting usage statistics.
	 */
	function getUsage() {
		// See the Logger module
	}
	
		
	/**
	 * Retrieves logged 'actions' from the system action log (database table).
	 * 
	 * @param $where An optional MySQL where clause, defaults to '1' (Note: do not include the word 'WHERE')
	 * @param $limit An optional MySQL limit clause, defaults to 'LIMIT 1, 50' (Note: do include the word 'LIMIT')
	 * @param $order Optional, defaults to 'timestamp DESC'
	 * @return An array of logged 'action' items.
	 */
	function getActions($where = '1', $limit = 'LIMIT 1, 50', $order = 'timestamp DESC') {
		
		if (is_array($order)) {
			
			$order = implode(' ', $order);
			
		}
		
		$result = $this->db->select(
			'*',
			$this->actions_table, 
			$where,
			$order,
			'', 
			$limit
		);
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			
			$items[] = $row;
			
		}
		
		return $items;
		
	}
	
	/**
	 * @return The number of 'action' items in the system action log (database table).
	 */
	function countActions() {
		
		return $this->db->quickGet(
			'count(*)',
			$this->actions_table
		);
		
	}
	
	/**
	 * @return array An array of facility names to skip (eg modules not to log against).
	 */
	function getFacilitiesToSkip() {
		
		return $this->facilities_to_skip;
		
	}
	
	/**
	 * Returns the current facility (module) if there is one.
	 */
	function getFacility() {
		global $SERVICES;
			
		$SERVICES->getService('Modules');
		
		// $this->printDebug('Log::getFacility: $SERVICES->modules->getCurrent());
		
		return $SERVICES->modules->getCurrent();
		
	} 
	
	/**
	 * Internal (private) function for converting an array to a string
	 *
	 * @access private
	 */
	function arrayToString($msg) {
		
		if (is_array($msg)) {
				
			ob_start();
			
			print_r($msg);
			
			$msg = nl2br(ob_get_contents());
			
			ob_end_clean();
			
		}
		
		return $msg;
	}

}

?>