<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Service is the base class for the Services package.
 * 
 * A Service object provides basic application functionality and can be extended for more specific functionality.
 *
 * @author Phil Calder
 * @package Services
 * @abstract
 */
class Service {
	
	function Service() {
		$GLOBALS['debug'] = true; 
	}
	
	/**
	 * @access private
	 */
	function getPageExecutionTime() {
		global $BCRM_TIME_START, $L;
		
		$time_end = microtimeFloat();
		$time = $time_end - $BCRM_TIME_START;
		
		return $time;
	}
	
	/**
	 * Logs the given string to the error log via the Logger Service
	 *
	 * @param $str The error message.
	 * @param $facility Optional facility name, eg the current module.
	 * @param $priority Optional priority level string. Can be any value (eg 'Debug', 'Development'). Used for filtering results.
	 */
	function logError($str, $facility = '', $priority = '') {
		global $SERVICES;
		if (is_object($SERVICES) && method_exists($SERVICES, 'getService')) {
			$log =& $SERVICES->getService('Log');
			if (is_object($log) && method_exists($log, 'logError')) {
				$log->logError($str, $facility, $priority);
			}
		}
	}
	
	/**
	 * Logs the given string to the system action log (database table).
	 *
	 * Useful for recording user actions such as editing or deleting an item in a module.
	 * @param $str The 'action' message.
	 * @param $facility Optional facility name, defaults to the current module.
	 * @param $priority Optional priority level string. Can be any value (eg 'Debug', 'Development'). Used for filtering results.
	 */
	function logAction($str, $facility = '', $table='', $action=0, $item_id='', $priority = '') {
		global $SERVICES;
		if (is_object($SERVICES) && method_exists($SERVICES, 'getService')) {
			$log =& $SERVICES->getService('Log');
			if (is_object($log) && method_exists($log, 'logError')) {
				$log->logAction($str, $facility, $table, $action, $item_id, $priority);
			}
		}
	}
	
	/**
	 * Encode the given parameter into JSON notation
	 *
	 * @author Phil Calder
	 */
	function jsonEncode($value) {
		if (function_exists('json_encode')) {
			return json_encode($value);
		} 
		includeExternal('services_json', 'JSON.php');
		$json = new Services_JSON();
		// print '<div>class: ' . get_class($json) . '</div>';
		return $json->encode($value);
	}
	
	/**
	 * Decode the given JSON encoded value
	 *
	 * @param mixed $value The JSON encoded value to decode
	 * @param boolean $assoc When true, returns an associative array. When false, returns a stdClass.
	 * @return mixed The decoded data
	 *
	 * @author Phil Calder
	 */
	function jsonDecode($value, $assoc = true) {
		if (function_exists('json_decode')) {
			return json_decode($value, $assoc);
		}
		includeExternal('services_json', 'JSON.php');
		if ($assoc) {
			$json = new Services_JSON(SERVICES_JSON_LOOSE_TYPE);
		} else {
			$json = new Services_JSON();
		}
		// print '<div>class: ' . get_class($json) . '</div>';
		return $json->decode($value);
	}
	
	/**
	 * This function can be customised to print debugging information for select services.
	 *
	 * Generally disabled in production versions. This is used internally to debug development code.
	 * 
	 * @param string $msg The debug message
	 * @access private
	 */
	function printDebug($msg, $force = false) {
		global $USER, $PHP_SELF;
		return; // ====== DISABLE HERE =====
		
		// comment out the services you dont want to print debugging info for
		$classes = array(
			// 'authentication', 'users', 'groups', 
			// 'config', 
			'database', 'customdatabase', 
			// 'datadefinition', 'admindatadefinition', 
			// 'datelib', 
			// 'filesystem', 
			// 'language', 
			// 'log',
			// 'message', 
			// 'modules', 
			// 'output', 
			// 'preferences', 
			// 'screen', 
			// 'service'
		);
		
		if (
			($force === true)
			|| 
				(
				1 
				&& (stristr($PHP_SELF, 'frame') === false) 
				&& (( (stristr($PHP_SELF, 'Contact/menu.php') === false)) )
				// && (isset($GLOBALS['debug']) && ($GLOBALS['debug'] === true))  
				)
		) {
			// print get_class($this) . ', ' . $PHP_SELF . '<hr>';
			if (($force === true) || in_array(get_class($this), $classes)) { // Limit to certain classes
				if (is_array($msg)) {
					ob_start();
					print_r($msg);
					$msg = nl2br(ob_get_contents());
					ob_end_clean();
				}
				
				// Could create a Log service to log these messages
				// global $SERVICES;
				// $log =& $SERVICES->getService('Log');
				// $log->logAction($msg); // optional 'facility' and 'priority' args here
				// Can leave facility blank, and specify a debugging priority, 
				// (facility will default to the current module)
				// eg: $log->logAction($msg, '', 'Debugging'); 
				
				// $usage = (function_exists('memory_get_usage') ? '[' . memory_get_usage() . ']' : '');
				$usage = $this->getMemoryUsage(true);
				// or just print to screen
				print '<p>' . round($this->getPageExecutionTime(), 3) . ': ' . $msg . '' . (strlen($usage) ? '<br>' . $usage : '') . '</p>';
				@ob_flush();
				@flush();
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function getMemoryUsage($formatted = false) {
		if (function_exists('memory_get_usage')) {
			$usage = memory_get_usage();
			if ($formatted) {
				$usage = (float) ($usage / 1024 / 1024);
				// print '<div>usage: ' . $usage . '</div>';
				$usage =  '<b style="color: red;">memory usage:</b>' . number_format((float)$usage, 3) . ' Mb ';
			}
			return $usage;
		} 
		return '';
	}

}

?>