<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */
 
includeService('Authentication');

/**
 * The Users class provides function for getting information on a user or group of users.
 *
 * This class can be accessed directly through {@link Authentication}
 * @author Phil Calder
 * @package Services
 */
class Users extends Service {
	
	/**
	 * @var $last_load_done
	 * @access private
	 */
	var $last_load_done = false;
	
	/**
	 * @access private
	 */
	var $usernames = '';
	
	/**
	 * @access private
	 */
	var $user_lists = array(
		'standard' => false, 
		'extended' => false
	);
	
	function Users() {
		global $SERVICES;
		
		$this->db =& $SERVICES->getService('Database'); // Must already exist since we dont know the args	
		
		$this->current_user_time_offset = 180;
		$this->new_user_time_offset = 300;
		
		$this->ghost_user = '_bcrm'; // Was: config->lookup('admin.ghostusername')
		$this->webform_user = '_pub';
		$this->system_users = array($this->ghost_user, $this->webform_user);
	}
	
	function getAllUsersList() {
		return $this->getUsersList(true, false, true, true);
	}
	
	function getArchivedUsersList($show_all = false) {
		global $USER, $SERVICES, $L;
		
		$authentication =& $SERVICES->getService('Authentication');
		$lang =& $SERVICES->getService('Language');
		$lang->getLanguage('_Global');
		
		$where = '';
		
		// if ( ($USER['username'] == $this->ghost_user)  || ($show_all && $authentication->adminAccess($USER['user_id'], 'UserAdmin')) ) {
		if ( $this->isGhostUser() || ($show_all && $authentication->adminAccess($USER['user_id'], 'UserAdmin')) ) {	
			// where stays the same
		} else {
			if (is_array($ignore = $this->getSystemUsers()) && sizeof($ignore)) {
				$where .= 'username NOT IN ("' . implode('","', $ignore) . '")';
			}
		}
		
		$where .=  (strlen($where) ? ' AND ' : '') . '`status`=2'; // Archived users
		
		$result = $this->db->select(
			'id as user_id, username, status', // Phil 20060814 added status
			'System__users',
			$where, 
			'username', 
			'', 
			$limit
		);
			
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			$items[$row['user_id']] = $row['username'];
		}
		
		// print '<div>Returning archived users list</div>';
		return $items;		
	}
	
	function getUsersList($show_all = false, $limit = '', $incl_deleted = false, $incl_resources = false) {
		global $USER, $SERVICES, $L;
		
		$cache_standard = $cache_extended = false;
		if (!$show_all && !$limit && !$incl_deleted && !$incl_resources) {
			if (is_array($this->user_lists['standard'])) {
				// print '<div>CACHED returning standard users list</div>';
				return $this->user_lists['standard'];
			}
			// print '<div>Need to CACHE standard users list</div>';
			$cache_standard = true;
		} else if (!$show_all && !$limit && $incl_deleted && $incl_resources) {
			if (is_array($this->user_lists['extended'])) {
				// print '<div>CACHED returning extended users list</div>';
				return $this->user_lists['extended'];
			}
			// print '<div>Need to CACHE extended users list</div>';
			$cache_extended = true;
		}
		
		$authentication =& $SERVICES->getService('Authentication');
		$lang =& $SERVICES->getService('Language');
		$lang->getLanguage('_Global');
		
		$label_deleted = ' (' . $L['LabelArchived'] . ')';
		$label_resource = ' (' . $L['LabelResource'] . ')';
		$label_deleted_resource = ' (' . $L['LabelDeletedResource'] . ')';
		$where = '';
		
		// if ( ($USER['username'] == $this->ghost_user)  || ($show_all && $authentication->adminAccess($USER['user_id'], 'UserAdmin')) ) {
		if ( $this->isGhostUser() || ($show_all && $authentication->adminAccess($USER['user_id'], 'UserAdmin')) ) {	
			// where stays the same
		} else {
			if (is_array($ignore = $this->getSystemUsers()) && sizeof($ignore)) {
				$where .= 'username NOT IN ("' . implode('","', $ignore) . '")';
			}
		}
		
		if ($incl_deleted && $incl_resources) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,2,3)';
		} else if ($incl_deleted) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,2)';
		} else if ($incl_resources) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,3)';
		} else {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status`=1';
		}
		
		$result = $this->db->select(
			'id as user_id, username, status', // Phil 20060814 added status
			'System__users',
			$where, 
			'username', 
			'', 
			$limit
		);
			
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			// $items[$row['user_id']] = $row['username'];
			// Phil 20060814
			switch ($row['status']) {
				case 1: // active User
					$items[$row['user_id']] = $row['username'];
				break;
				case 2: // deleted User
					$items[$row['user_id']] = $row['username'] . $label_deleted;
				break;
				case 3: // active Resource
					$items[$row['user_id']] = $row['username'] . $label_resource;
				break;
				case 4: // deleted Resource
					$items[$row['user_id']] = $row['username'] . $label_deleted_resource;
				break;
				
				case 0: // unknown
				default:
				break;
			}
		}
		
		if ($cache_standard) {
			$this->user_lists['standard'] = $items;
		} else if ($cache_extended) {
			$this->user_lists['extended'] = $items;
		}
		// print '<div>Returning users list</div>';
		return $items;
	}
	
	function getSystemUsers() {
		return $this->system_users;
	}
	
	/**
	 * @access private
	 */
	function isWebFormUser() {
		global $USER;
		// print_r($USER);
		if ($USER['username'] == $this->webform_user) {
			return true;
		}
		return false;
	}
	
	/**
	 * @access private
	 */
	function isGhostUser() {
		global $USER;
		if ($USER['username'] == $this->ghost_user) {
			return true;
		}
		return false;
	}
	
	/**
	 * @param string $contact_id The string to match against the users client id 
	 * @return boolean Whether the given 'contact_id' matches the currect users 'client contact id'
	 * @access private
	 */
	function isClient($contact_id) {
		// $this->printDebug('Users::isClient(' . $contact_id . ')');
		return ( ($contact_id == $this->getUsersContactId()) ? true : false);
	}
	
	function getUsersContactId() {
		global $USER;
		// print '<div>USER:</div>'; print_r($USER); print '<hr>';
		return (isset($USER['client_contact_id']) ? $USER['client_contact_id'] : false);
	}
	
	function getCurrentUsers() {
		global $USER;
		$ignore_users = '1';
		
		if (is_array($ignore = $this->getSystemUsers()) && sizeof($ignore)) {
			$ignore_users = 'username NOT IN ("' . implode('","', $ignore) . '")';
		}
		
		$result = $this->db->select(
			'id as user_id, username', 
			'System__users',
			$ignore_users . ' AND last_load >= ' . (time() - $this->current_user_time_offset) . ' AND `status`=1', 
			'last_login'
		);		
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			$items[$row['user_id']] = $row['username'];
		}
		return $items;
	}
	
	function getNewUsers() {
		global $USER;
		$ignore_users = '1';
		
		if (is_array($ignore = $this->getSystemUsers()) && sizeof($ignore)) {
			$ignore_users = 'username NOT IN ("' . implode('","', $ignore) . '")';
		}
		
		$count = $this->db->quickGet(
			'count(*)', 
			'System__users',
			$ignore_users . ' AND last_login >= ' . (time() - $this->new_user_time_offset) . ' AND id <> ' . $USER['user_id'] . ' AND `status`=1'
		);
		return $count;
	}
	
	function getUserIDs($show_all = false, $incl_deleted = false, $incl_resources = false) {
		global $USER;
		// Phil 200605 Added $show_all param
		$where = '';
		
		// if (!isset($USER['username']) || ($USER['username'] != $this->ghost_user)) { $this->isGhostUser()
		if (!$show_all ||  !isset($USER['username']) || !$this->isGhostUser() ) {	
			if (is_array($ignore = $this->getSystemUsers()) && sizeof($ignore)) {
				$where = 'username NOT IN ("' . implode('","', $ignore) . '")';
			}
		}
		
		if ($incl_deleted && $incl_resources) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,2,3)';
		} else if ($incl_deleted) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,2)';
		} else if ($incl_resources) {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status` IN (1,3)';
		} else {
			$where .=  (strlen($where) ? ' AND ' : '') . '`status`=1';
		}
		
		$result = $this->db->select(
			'id',
			'System__users',
			$where
		);
		
		$items = array();
		
		while ($row = $this->db->getRow($result)) {
			$items[] = $row['id'];
		}
		return $items;
	}

	function getUser($user_id) {
		if (empty($user_id)) {
			return false;
		}
		return $this->db->quickGet(
			'*',
			'System__users',
			'id=' . $user_id
		);
	}
	
	function getUserFromContact($contact_id) {
		return $this->db->quickGet(
			'*',
			'System__users',
			'contact_id = ' . $contact_id
		);
	}
	
	function getUsername($user_id) {
		if (empty($user_id)) {
			return false;
		}
		
		if (is_array($this->usernames)) {
			if (isset($this->usernames[$user_id]) && $this->usernames[$user_id]) {
				return $this->usernames[$user_id];
			}
		} else {
			$this->usernames = array();
		}
		 $this->usernames[$user_id] = $this->db->quickGet(
			'username',
			'System__users',
			'id=' . $user_id
		);
		return  $this->usernames[$user_id];
	}
	
	function getPrivateGroup($user_id) {
		if (empty($user_id)) {
			return false;
		}
		
		if (is_array($this->private_groups)) {
			if (isset($this->private_groups[$user_id]) && $this->private_groups[$user_id]) {
				return $this->private_groups[$user_id];
			}
		} else {
			$this->private_groups = array();
		}
		
		 $this->private_groups[$user_id] = $this->db->quickGet(
			'private_group_id',
			'System__users',
			'id=' . $user_id
		);
		return  $this->private_groups[$user_id];
	}
	
	function setPrivateGroup($user_id, $group_id) {
		if ($user_id && $group_id) {
			return $this->db->update(
				'System__users', 
				array(
					'private_group_id'=>$group_id
				), 
				"id=$user_id"
			);
		}
		return false;
	}
	
	function getSessionID($user_id = '') {
		if (empty($user_id)) {
			global $USER;
			$user_id = $USER['user_id'];
		}
		if (!empty($user_id)) {
			return $this->db->quickGet(
				'session_id',
				'System__users',
				'id=' . $user_id
			);
		}
		return false;
	}
	
	function getContactID($user_id) {
		if (empty($user_id)) {
			return false;
		}
		return $this->db->quickGet(
			'contact_id',
			'System__users',
			'id=' . $user_id
		);
	}
	
	function getContactIDs() {
		return $this->db->quickGetAllRows(
			'contact_id',
			'System__users'
		);
	}
	
	/**
	 * Extended Add User functionality to allow specifying additional user info.
	 *
	 * Used by importer / updater scripts, not part of BrowserCRM core functionality.
	 *
	 * @param array $user_info Passes extended user_info to {@link AaddUser}
	 * @see addUser
	 * @author Phil Calder
	 * @access private
	 */
	function _addUser($user_info) {
		$username    = $user_info['username'];
		$password    = $user_info['password'];
		$skin        = $user_info['skin'];
		$contact_id  = $user_info['contact_id'];
		$groups      = $user_info['groups'];
		$team_groups = $user_info['team_groups'];
		$user_id     = $user_info['id'];
		$status      = (int)(isset($user_info['status']) ? $user_info['status'] : 1);
		return $this->addUser($username, $password, $skin, $contact_id, $groups,  $team_groups, $user_id, $status);
	}
	
	/**
	 * Administrators can add a new user to the system
	 *
	 * @param $username The new users username
	 * @param $password The new users password
	 * @param $skin The skin for the new user
	 * @param $contact_id The contact id to associate with this user
	 * @param $groups An array of group ids which the new user will be a member of
	 * @return false, or the user users user id
	 */
	function addUser($username, $password, $skin = false, $contact_id = false, $groups = array(),  $team_groups = array(), $user_id = false, $status = 1) {
		global $SERVICES, $USER;
		
		$ua =& $SERVICES->getService('Authentication');
		
		if (!$ua->adminAccess($USER['user_id'], 'UserAdmin')) {
			$this->logAction('WARNING: Attempt to add a user without required UserAdmin access', 'Users');
			return false;
		}
		
		if (!$this->isValidUsername($username)) {
			$this->logAction('WARNING: Attempt to add a user with invalid username', 'Users');
			return false;
		}
		
		if (!$this->isValidPassword($password)) {
			$this->logAction('WARNING: Attempt to add a user with invalid password', 'Users');
			return false;
		}
		
		$fn_prefix = $this->db->fn_prefix;
		// $item = array( 
			// 'username' => ucfirst($username), 
			// 'password' => $fn_prefix . $ua->getSqlPasswordFunctionName() . "('$password')"
		// );
		// Phil 20070115 new password format
		$item = array( 
			'username' => ucfirst($username), 
			'password' => $ua->generateHash($password)
		);
		
		if (isset($contact_id) && $contact_id) {
			$item['contact_id'] = $contact_id;
		}
		
		if (isset($skin) && $skin) {
			$item['skin'] = $skin;
		} else {
			$item['skin'] = $USER['skin'];
		}
		
		if ($user_id) {
			$item['id'] = (int)$user_id;
		}
		
		$users = $this->getUsersList(true);
		
		if (is_array($users) && in_array(ucfirst($username), $users)) {
			$this->logAction('WARNING: Attempt to add a user with existing username: ' . $username . '', 'Users');
			return false; // username already exists
		}
		
		if ( (isset($item['id']) && $item['id']) && (is_array($users) && array_key_exists($item['id'], $users)) ) {
			$this->logAction('WARNING: Attempt to add a user with existing id: ' . $item['id'] . '', 'Users');
			return false; // specified user id already exists
		}
		
		if (isset($status)) {
			$item['status'] = (int)$status;
		} else {
			$item['status'] = 1; // Standard user (not archived or resource)
		}
		
		// $original_field_checking_status = $this->db->setFieldChecking(true); // because we have a password(..) in our item
		$this->db->insert('System__users', $item);
		// $this->db->setFieldChecking($original_field_checking_status);
		
		$user_id = $this->db->getInsertId();
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->logAction('User added: ' . $this->getUsername($user_id));
		}
		
		if (!is_array($groups)) {
			$groups = array();	
		}
					
		if (!sizeof($groups)) {
			$groups[] = $ua->groups->getGroupID('Everyone'); // User has to be in at least one group
		}
		
		$private_group = array(
			'name'=>'[' . $this->getUsername($user_id) . ']',
			'visible'=>1,
			'editable'=>1, 
			'type'=>1 // private
		);
		$private_group_id = $ua->groups->addGroup($private_group);
		
		$groups[] = $private_group_id;
		
		if ($private_group_id) {
			// print '<p>adding private_group_id ' . $private_group_id . ' for user_id ' . $user_id . '</p>';
			$retval = $this->setPrivateGroup($user_id, $private_group_id);
			// $this->db->printError();
			// print '<p>returned ' . $retval . '</p>';
		}
		
		foreach($groups as $k => $v) {
			$ua->groups->setUser( $user_id, $v, 1 );
		}
		
		if (!is_array($team_groups)) {
			$team_groups = array();	
		}
		
		foreach($team_groups as $k => $v) {
			$ua->groups->setUser( $user_id, $v, 1 );
		}
		
		return $user_id;
	}
	
	/**
	 * The 'Front' function where a user can change their own password (see also changeUserPassword below)
	 * 
	 * @param $old The existing password
	 * @param $new The users new password
	 * @return true or an error message
	 * @author Phil Calder
	 */
	function changePassword($old, $new) {
		// Phil 20070115 Updated to new BrowserCRM password format
		global $SERVICES, $USER, $L;
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('UserAdmin');
		$ua =& $SERVICES->getService('Authentication');
		
		if (!$this->isValidPassword($new)) {
			return $L['useradminInvalidPassword'];
		}
		
		// $name = $this->db->quickget(
			// 'username', 
			// 'System__users', 
			// 'id=' . $USER['user_id'] . ' AND password=' . $ua->getSqlPasswordFunctionName() . '(\'' . $old . '\')'
		// );
		// 
		$old_hash = $this->db->quickGet(
			'password', 
			'System__users', 
			'`id` = "' . $USER['user_id'] . '" AND `status` = 1'
		);
		$old_hash_check = $ua->generateHash($old, $old_hash);
		// if ($old_hash == $old_hash_check) {
			// print '<p>They match</p>';
		// }
		$name = $this->db->quickget(
			'username', 
			'System__users', 
			'id=' . $USER['user_id'] . ' AND password=\'' . $old_hash . '\''
		);
		$this->db->printError();
		// print '<p>old: ' . $old . ' new: ' . $new . ' old_hash: ' . $old_hash . '  old_hash_check: ' . $old_hash_check . '</p>'; exit();
		if (!$name) { 
			return $L['useradminIncorrectPassword']; 
		} else {
			// $fn_prefix = $this->db->fn_prefix;
			// $original_field_checking_status = $this->db->setFieldChecking(true); // because we have a password(..) in our item
			// $this->db->update( 
				// 'System__users', 
				// array( 
					// // 'i_password' => "PASSWORD('$new')"
					// 'password' => $fn_prefix . $ua->getSqlPasswordFunctionName() . '("' . $new . '")'
				// ), 
				// 'id=' . $USER['user_id'] . ' AND password=' . $ua->getSqlPasswordFunctionName() . '(\'' . $old . '\')' 
			// );
			// $this->db->setFieldChecking($original_field_checking_status);
			$new_hash = $ua->generateHash($new);
			$this->db->update( 
				'System__users', 
				array( 
					'password' => $new_hash
				), 
				'id=' . $USER['user_id'] . ' AND password=\'' . $old_hash . '\'' 
			);
			return true;
		}
	}
	
	/**
	 * Administrators can change user passwords.
	 *
	 * @param $user_id The users id
	 * @param $new The new password for the given user
	 * @return true or an error message
	 * 2author Phil calder
	 */
	function changeUserPassword($user_id, $new) {
		// Phil 20070115 Updated to new BrowserCRM password format
		global $USER, $L, $SERVICES;
		$ua =& $SERVICES->getService('Authentication');
		if (!$ua->adminAccess($USER['user_id'], 'UserAdmin')) {
			return $L['useradminChangePassword'] . ' - ' . $L['useradminDenied'];
		}
		
		if (!$this->isValidPassword($new)) {
			return $L['useradminInvalidPassword'];
		}
		
		$name = $this->db->quickGet( 
			'username', 
			'System__users', 
			'id=' . $user_id
		);
		
		if (!$name) { 
			return $L['useradminIncorrectUserId']; 
		} else {
			
			// $fn_prefix = $this->db->fn_prefix;
			// $original_field_checking_status = $this->db->setFieldChecking(true); // because we have a password(..) in our item
			// if ($this->db->update( 
				// 'System__users', 
				// array( 'password' => $fn_prefix . $ua->getSqlPasswordFunctionName() . '(\'' . $new . '\')') , 'id=' . $user_id)) {
				// $this->db->setFieldChecking($original_field_checking_status);
				// return true;
			// }
			// $this->db->setFieldChecking($original_field_checking_status); 
			$new_hash = $ua->generateHash($new);
			$this->db->update( 
				'System__users', 
				array( 
					'password' => $new_hash
				), 
				'id=' . $user_id . '' 
			);
			return true;
		}
		return false;
	}
	
	function changeOwnPasswordIsAllowed() {
		global $SERVICES;
		$SERVICES->getService('Config');
		if ($SERVICES->config->lookup('modules.UserAdmin.user_change_own_password')) {
			return true;
		}
		return false;
	}

	/**
	 * Administrators can change an existing system username
	 * 
	 * @param $old The existing user id
	 * @param $new The users new username
	 * @return true or an error message
	 *
	 */
	function changeUsername($id, $new) {
		global $SERVICES, $USER, $L;
		
		$ua =& $SERVICES->getService('Authentication');
		
		if ( ($id != $USER['user_id']) && !$ua->adminAccess($USER['id'], 'UserAdmin')) {
			if ( ($id == $USER['user_id']) && $this->changeOwnPasswordIsAllowed()) {
				// see if users are allowed to change their username in this installation
			} else {
				return $L['useradminChangeUsername'] . ' - ' . $L['useradminDenied'] . '';
			} 
			// return $L['useradminChangeUsername'] . ' - ' . $L['useradminDenied'] . '1';
		}
		
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('UserAdmin');
		
		if (!$this->isValidUsername($new)) {
			return $L['useradminInvalidUsername'];
		}
		
		$name = $this->db->quickget(
			'username', 
			'System__users', 
			'id=' . $id
		);
		
		if (!$name) { 
			return $L['useradminIncorrectUserId']; 
		} else if ($id == $this->getAdminUserId()) {
			return $L['useradminIncorrectUserId']; 
		} else {
			$retval = $this->db->update( 
				'System__users', 
				array('username' => ucfirst($new)), 
				'id=' . $id
			);
			
			// update users private group
			if ( $retval && ($private_group_id = $this->getPrivateGroup($id)) ) {
				global $SERVICES;
				$SERVICES->groups->setGroup($private_group_id, array('name'=>'['.ucfirst($new).']'));
			}
			// return $L['useradminUsernameSuccessfullyChanged'];
			return true;
		}
	}	
	
	/**
	 *
	 */
	function switchUser($newuser = '') {
		global $USER;
		
		if ( !isset($USER['orig_id']) ) {
			$USER['orig_id'] = $USER['user_id'];
		}
	
		if (empty($newuser)) {
			$USER['user_id'] = $USER['orig_id'];
			bcrm_unset($USER, 'orig_id');
			return;
		}
		
		if ( ((int)$newuser) == $newuser ) {
			$USER['user_id'] = $newuser;
		} else {
			$USER['user_id'] = $this->db->quickGet(
				'System__users.id as user_id',
				'System__users,System__user_group',
				"username = '$newuser' and System__user_group.group_id = {$USER['group_id']}"
			);
		}
	}	 
	
	function refresh() {
		global $USER, $SERVICES, $BCRM_DOCUMENT_ROOT;
		if (!empty($USER['user_id'])) {
			$ua =& $SERVICES->getService('Authentication');
			$USER['document_root'] = $BCRM_DOCUMENT_ROOT;
			$USER['groups'] = array('0'=>'1') + $ua->groups->getGroupsForUser($USER['user_id']);
			$USER['group_string'] = implode(',', array_keys($USER['groups']));
			// if ($this->isWebFormUser()) {
				// $USER['client_contact_id'] = $this->getUsersContactId();
			// }
		}
	}
	
	function checkUser() {
		// global $USER;
		// $this->printDebug('Users::checkUser: username is ' . $USER['username'] . '');
		// Phil 20050908. Allows multiple '_pub' users to be logged in simultaneously
		if ($this->isWebFormUser()) { 
			return true;
		}
		return ($this->getSessionID() == session_id());
	}
	
	/**
	 * Updates user login stats
	 *
	 * Sets session id
	 * Optionally sets last login time
	 *
	 * @param $current_session Optional, default false
	 */
	function setLogin($current_session = false) {
		global $USER;
		if (!empty($USER['user_id'])) {
			if ( $current_session && ($current_session == session_id()) ) {
				// $this->printDebug($current_session . ': ' . session_id() . '');
				// No need to update 
				return true;
			}
			
			$last_login = time();
			$item = array(
				'session_id' => session_id(), 
				'last_login' => $last_login
			);
			$result = $this->db->update(
				'System__users', 
				$item,
				'id= ' .$USER['user_id']
			);
			if ($result) {
				$USER['last_login'] = $last_login;
				return true;
			}
		}
		return false;
	}
	
	function setLastLoad() {
		global $USER;
		if (!$this->last_load_done 
		&& !empty($USER['user_id'])
		) {
			$this->db->update(
				'System__users', 
				array('last_load' => time()),
				'id= ' .$USER['user_id']
			);
			
			$this->last_load_done = true;
			return true;
		}
		return false;
	}
	
	function setSessionSkin($skin) {
		global $USER, $SERVICES;
		
		$SERVICES->getService('Config');
		$websubdir = $SERVICES->config->lookup('site.websubdir');
		
		//path to skin as seen by the browser
		
		$USER['skin_web_path'] = ($websubdir ? '/' . $websubdir : '') . $skin; //has trailing slash
		//path to /_base skin which always loads
		$USER['base_skin_web_path'] = ($websubdir ? '/' . $websubdir : '') . '/skins/_base/';
		
		//MCR 20060106; js is now in root /js
		$USER['js_web_path'] = ($websubdir ? '/' . $websubdir : '') . '/js/';
		$site_root = $SERVICES->config->lookup('site.root');
		
		//path to skin as seen by the file system
		$USER['skin_file_path'] = $site_root . $skin;
		//path to /_baseskin as seen by the file system
		$USER['base_skin_file_path'] = $site_root . '/skins/_base/';
		
		//ext theme
		//$USER['ext_theme'] = $SERVICES->config->lookup('skin.ext_theme');
		$SERVICES->getService('Screen');
		$USER['ext_theme'] = $SERVICES->screen->getSkinExtTheme($skin);
		
	}
	
	/**
	 * Sets user session value for javascript web path
	 *
	 * Javascript dir is located in the js sub folder of the site.
	 * Handles installations installed in a sub directory
	 */
	function setSessionJsPath() {
		global $USER, $SERVICES;
		$SERVICES->getService('Config');
		$websubdir = $SERVICES->config->lookup('site.websubdir');
		//path to js dir as seen by the browser
		$USER['js_web_path'] = ($websubdir ? '/' . $websubdir : '') . '/js/';
		// Not needed (yet)
		// $site_root = $SERVICES->config->lookup('site.root');
		//path to js dir as seen by the file system
		// $USER['js_file_path'] = $site_root . '/js/';
	}
	
	function setShowWelcomePage($value = true) {
		global $USER;
		if (isset($USER['id']) && $USER['id']) {
			// boolean, stored as 0/1 in db
			if (!$value) {
				$value = 0;
			} else {
				$value = 1;	
			}
			
			// Set the User welcome to be the value given
			$this->db->update( 
				'System__users', 
				array('welcome' => $value), 
				'id='. $USER['id']
			);
			$USER['welcome'] = $value;
		}
	}
	
	function getShowWelcomePage() {
		global $USER, $SERVICES;
		$SERVICES->getService('Config');
		if ($dev_value = $SERVICES->config->lookup('dev.welcome_value')) {
			return $dev_value;
		}
		
		$welcome_value = 0;
		if (isset($USER['id']) && $USER['id']) {
			$welcome_value = $this->db->quickGet( 
				'welcome', 
				'System__users', 
				'id='.$USER['id']
			);
		}
		return $welcome_value;
	}
	
	function removeUser($id) {
		global $SERVICES, $USER;
		$log =& $SERVICES->getService('Log');
		$ua =& $SERVICES->getService('Authentication');
		
		if (!$ua->adminAccess($USER['user_id'], 'UserAdmin')) {
			$log->logAction('User ' . $ua->users->getUsername($USER['user_id']) . ' attempted to remove user ' . $ua->users->getUsername($id));
			return false;
		}
		
		// Remove the users groups
		$this->db->delete(
			'System__user_group', 
			'user_id = ' . $id
		);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
		
		$users_name = $ua->users->getUsername($id);
		
		if ($users_name) {
			// Remove the users saved cookies
			$this->db->delete(
				'System__cookies', 
				'username = "' . $users_name . '"'
			);
		}
		
		// if ($this->db->hasError()) {
			// $this->db->printError();
		// }
		
		// Phil 20060814 - no longer 'deleting user', just set status to 2 (deleted user)
		// // Delete the user
		// $this->db->delete(
			// 'System__users', 
			// 'id = ' . $id
		// );
		// $this->db->update(
			// 'System__users', 
			// array(
				// 'status' => 2, // deleted user
				// 'password' => 'password(' . $users_name . ')', // only because password cannot be empty
				// 'session_id' => ''
			// ), 
			// 'id = "' . $id . '"'
		// );
		$this->db->update(
			'System__users', 
			array(
				'status' => 2, // deleted user
				'password' => $ua->generateHash($users_name), // only because password cannot be empty
				'session_id' => ''
			), 
			'id = "' . $id . '"'
		);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$log->logAction('User ' . $ua->users->getUsername($USER['user_id']) . ' removed user ' . $users_name);
		}
		
		$this->db->delete(
			'System__prefs_user', 
			'user_id = ' . $id
		);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
	}
	
	function restoreUser($id) {
		global $SERVICES, $USER;
		$log =& $SERVICES->getService('Log');
		$ua =& $SERVICES->getService('Authentication');
		
		$users_name = $ua->users->getUsername($id);
		
		if (!$ua->adminAccess($USER['user_id'], 'UserAdmin')) {
			$log->logAction('User ' . $ua->users->getUsername($USER['user_id']) . ' attempted to restore user ' . $users_name);
			return false;
		}
		
		$this->db->update(
			'System__users', 
			array(
				'status' => 1, // active user
				'password' => $ua->generateHash($users_name), // only because password cannot be empty
				'session_id' => ''
			), 
			'id = "' . $id . '" AND status = 2'
		);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$log->logAction('User ' . $ua->users->getUsername($USER['user_id']) . ' restored user ' . $users_name);
		}
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
	}
	
	/**
	 * The beginnings of a username validation function.
	 *
	 * @param $string The proposed new username
	 * @return boolean true or false
	 *
	 */
	function isValidUsername($string) {
		// length in range 1 - 49
		if (! (strlen($string) > 0 && strlen($string) < 50) ) {
			return false;
		}
		return true;
	}
	
	/**
	 * The beginnings of a password validation function.
	 *
	 * @param $string The proposed new password
	 * @return boolean true or false
	 *
	 */
	function isValidPassword($string) {
		// length in range 1 - 49
		if (! (strlen($string) > 0 && strlen($string) < 50) ) {
			return false;
		}
		return true;
	}
	
	function getAdminUserId() {
		$result = $this->db->select(
			'id', 
			'System__users', 
			'username=\'Admin\' and status=1'
		);
		
		if($this->db->hasError()) {
			$this->db->printError();
		}
		
		while ($row = $this->db->getRow($result)) {
			$id = $row['id'];
		}
		
		return $id;
	}
}
?>