<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('edit.inc');
includeWebMAT('advanced_filters.inc');

/**
 * Advanced Search Filters
 * 
 * A extended version of the edit page that enables you define an advanced search filter for a module
 *
 * @package WebMAT
 * @author Phil Calder
 */
class WebMATStandardAdvancedFilters extends WebMATEdit {
	
	// function init($module, $data_option, $post, $get, $table_name = false, $language_overwrite = false) {
		// parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
	// }
	
	/**
	 * returns the WebMat type
	 */
	function getWebmatType() {
		return 'advanced_filters';
	}
	
	/**
	 * This function runs after the DataDictionary has been set up
	 */
	function afterDataDictionary() {
		// print '<div>afterDataDictionary</div>';
		if (is_array($this->dd->data_dictionary)) {
			foreach ($this->dd->data_dictionary as $k => $v) {
				if ($v['type'] == 'email') { // Phil 20060531 avoid validation for eamil addresses
					// print '<div>field: ' . $k . '</div>';
					$this->dd->changeAttributes(array('field_key' => $v['field_key'], 'type' => 'text'));
				}
			}
		}
	}
	
	/**
	* Calls {@link doSubmit} where appropriate
	 */
	function run() {
		if (isset($this->form_values['submit'])) {
			// $this->db_functions->setLogAction('edit');
			$this->doSubmit();
		}
	}
	
	/**
	 * Gets title row links
	 * 
	 * Removes bookmark link
	 * 
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinks() {
		$title_links = parent::getTitleRowLinks();
		$title_links['bookmark'] = 'off';
		return $title_links;
	} 
	
	/**
	 * Overridden to handle checkboxes with a false value
	 */
	function doSubmitBegin() {
		// print '<div>doSubmitBegin this->form_values:</div>'; print_r($this->form_values); print '<hr>';
		// print '<div>doSubmitBegin this->item:</div>'; print_r($this->item); print '<hr>';
		
		$item = $this->item;
		// Includes fields having value as false of type checkbox 
		foreach ($this->dd->data_dictionary as $k => $v) {
			// print '<div>dd k=' . $k . ' (type=' . $v['type'] . ')</div>';
			if ( ($v['type'] == 'checkbox') && isset($item['_changed_' . $k]) && !isset($item[$k])) {
				// print '<div>setting checkbox ' . $k . ' to 0</div>';
				$this->item[$k] = 0;
				// $this->form_values['item'][$k] = 0;
				// $_POST['item'][$k] = 0;
			} else if ( (($v['type'] == 'dropdown') || ($v['type'] == 'multi_select')) ) { // && isset($item['_changed_' . $k]) && (!isset($item[$k]) || !is_array($item[$k]))) {
				// print '<div>setting dropdown/multi_select ' . $k . ' to [empty string]</div>';
				// print '<div>dropdown/multi_select ' . $k . ' ' . (!is_array($v) ? $v : '' ) . '</div>'; if (is_array($v)) { print_r($v); } print '<hr>';
				$field_key = $k;
				if (isset($v['foreign']) && $v['foreign']) {
					$field_key = '_' . $field_key;
				} 
				if (isset($item['_changed_' . $field_key]) && (!isset($item[$field_key]) || !is_array($item[$field_key])) ) {
					// print '<div>dropdown/multi_select ' . $k . ' item[' . $field_key . '] = ' . $item[$field_key] . '</div>'; print_r($item[$field_key]); print '<hr>';
					// $this->item[$field_key] = array(0 => '');
					 $this->item[$field_key] = array();
				}
				
			}
		}
		
		foreach ($this->item as $k => $v) {
			// print '<div>======item k=' . $k . ' ' . $v . '</div>';
			if ((strpos($k, '_changed_') !== false) || (strpos($k, '_options_') !== false)) {
				// print '<div>skipping ' . $k . ' value=[' . $v . ']</div>'; // value=[' . $v . '] item[k]= ' . $item[strpos($k, 9)] . '</div>';
			} else if (!$item['_changed_' . $k]) {
				// print '<div>unsetting item ' . $k . ' (no item[_changed_' . $k . '])</div>';
				bcrm_unset($this->item, $k);
			} // else {
				// print '<div>including ' . $k . ' (' . $v . ')</div>';
			// }
			// print '<hr>';
		}
		$this->form_values['item'] = $this->item; // Save our changes in form values
		
	}
	
	/**
	 * Sets data dictionary include fields based on the item parameter keys
	 * 
	 * @param array $item the item array
	 * @return array containing the item and the error message if an error message is present
	 */
	function validateFields(& $item) {
		$this->dd->setDictionaryIncludeFields(array_keys($item));
		// return parent::validateFields(& $item);
		return parent::validateFields($item);
	}
	
	function saveFormValues() {
		global $USER;
		// print '<div>' . get_class($this) . '::saveFormValues</div>'; print_r($this->form_values); print '<hr>';
		$USER['adv_filter'][$this->getModuleName()][$this->getTableName()] = $this->form_values;
	}
	
	function doSubmitProcess() {
		// print '<div>dosubmitProcess</div>'; 
		// print '<div>this->form_values:</div>'; print_r($this->form_values); print '<hr>';
		// Save form values
		$this->saveFormValues();
		$this->redirectToList();
	}
	
	/**
	 * gets the form fields
	 * 
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string and table row for the form
	 */
	function getFieldRows($item, $error_msg) {
		// print '<div>' . get_class($this) . '::getFieldRows item:</div>'; print_r($item); print '<hr>';
		// print '<div>' . get_class($this) . '::getFieldRows data structure with data:</div>'; print_r($this->dd->getDataStructureWithData($item)); print '<hr>';
		$content = '';
		if ($error_msg) {
			$checked = 1;
		} else {
			$checked = 0;
		}
		foreach ($this->dd->getDataStructureWithData($item) as $k => $v) {
			$field_row = false;
			$checkbox = false;
			if ($v['field_key'] != 'id') {
				// print '<div>key: ' . $v['field_key'] . ' type: ' . $v['type'] . '</div>';
				bcrm_unset($v, 'notnull');
				$this->output->setUncheckIngore(true);
				 
				$v['tag_extras'] = ' onChange="uncheckIgnore(\'_changed_' . $v['field_key'] . '\')" ';
				if ($v['type'] == 'dropdown' || $v['type'] == 'multi_select') {
					$v['tag_extras'] .= ' onFocus="uncheckIgnore(\'_changed_' . $v['field_key'] . '\')" ';
				}
				$checkbox = $this->output->getHtmlField(
					array('field_key' => '_changed_' . $v['field_key'], 'type' => 'checkbox', 'value' => $checked)
				);
				$field_row = $this->getFieldRow($v, $error_msg[$v['field_key']], $checkbox);
				$content .= $field_row;
			}
		}
		return $content;
	}
	
	/**
	 * Wrapper function for {@link Output::getHtmlFieldRow}
	 * @param mixed $checkbox Optional, default false. Can be a string checkbox, passed to {@link Output::getHtmlFieldRow}
	 * @author Phil Calder
	 */
	function getFieldRow($field, $error_msg, $checkbox = false) {
		// return $this->output->getHtmlFieldRow($field, $error_msg, $checkbox);
		// print '<div>' . get_class($this) . '::getFieldRow</div>'; print_r($field); print '<hr>';
		
		if ($checkbox) {
			$checkbox = '<th style="text-align: center">' . $checkbox . '</th>';	
		}
		
		$html_field = '';
		$col_name = $field_key = $field['field_key'];
		if ( (!isset($field['hidden']) || !$field['hidden']) && ($field['type'] != 'hidden') ) {
			$html_field = $this->output->getSearchFilterCell($field_key, $field, $col_name);
		}
		
		$content = '';
		if (strlen($html_field)) {
			$content =  '<tr>';
			$content .= $checkbox . '';
			$content .= '<th>' . $field['label'] . '</th>';
			$content .= '<td>' . $html_field . ($error_msg ? '<br>' . $this->output->getValidationWarning($error_msg) : '') . '</td>';
			$content .= '</tr>';
		}
		return $content;		
	}
	
	// function getDescriptionContent() {
	// }
	
	/**
	 * builds the title row for the page
	 * 
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		global $L;
		$content = $L['titleAdvancedSearchFilter'];
		$suffix = WebMATDisplay::getTitleText($prefix);
		if (strlen($suffix)) {
			$content .= ': ' . $suffix;
		}
		return $content;
	}
	
	/**
	 * Overridden to remove next page row, permissions controls
	 *
	 * @return string The mass edit submit row
	 * @see Output::getSubmitRowMassEdit
	 * @author Phil Calder
	 */
	function getSubmitRow() {
		return $this->output->getSubmitRowMassEdit();
	}
	
	/**
	 * Disabled in this class
	 */
	function addPermissionControls(&$id, &$item, &$error_msg) {
	}
	
	/**
	 * Disabled in this class
	 */
	function getNextPageRow() {
		return false;
	}
	
	/**
	 * Disabled in this class
	 */
	function getExtraItemLinks() {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 */
	function getDuplicateLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 */
	function getHideUnhideLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 */
	function getDeleteLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return boolean false
	 */
	function getViewLink($id = false) {
		return false;
	}
	
	/**
	 * Disabled in this class
	 *
	 * @author Phil Calder
	 * @param array $default_overrides Not used
	 */
	function getDefaultItem($default_overrides = false) {
		// return $this->dd->getDefaultItem($default_overrides);
		return array();
	}
	
	/**
	 * disables the read only fields for a new item
	 */
	function disableReadOnly() {
		if (!$this->form_values['id']) {
			$this->dd->setNoReadOnly(false); // foreign fields not (automatically) treated as read only
		}
	}
	
	/**
	 * Uses this modules list page as the return page
	 *
	 * @param string $page The relative page (Unused in this class)
	 */
	function setReturnPage($page = false) {
		global $SERVICES;
		$SERVICES->getService('Config');
		$_SESSION['return_page'] = $SERVICES->config->lookup('modules.Front.modulelink.' . $this->module_name . '.link');
	}
	
	/**
	 * redirects back to the list page
	 *
	 * Adds an advanced filters key and value to the url
	 *
	 * @see getListPageUrl()
	 *
	 * @author Phil Calder
	 */
	function redirectToList() {
		$list_page = $this->getListPageUrl() . '?adv_filter=1';
		header('Location: ' . $list_page);
		exit();
	}
	
	/**
	 * @deprecated version 4.150
	 * @see addPermissionControls
	 */
	function addMakePermissionsPrivate(&$id, &$item, &$error_msg) {
		return false;
	}
	
	/**
	 * @deprecated version 4.150 see {@link getFieldRows}
	 */
	function getHtmlFieldRows($item, $error_msg) {
		return $this->getFieldRows($item, $error_msg);
	}
	
}
?>