<?php
/**
 * Class for displaying dashboard content
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 * @author Phil Calder
 **/

includeWebMAT('dashboard.inc');

/**
 * @abstract
 */
class WebMATStandardBookmarkDashboard  extends WebMATDashboard {
	
	/**
	 * Sets the design time filter used to display the dashboard piece
	 */
	function setPieceFilter() {
		if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
			global $USER, $SERVICES;
			$module_name = $this->mod->getModuleName();
			if (!isset($this->db_functions)) {
				$this->db_functions =& $this->mod->getDatabaseTable('item');
			}
			if (!$this->dashboard_bookmark) {
				$table_name = $this->db_functions->getTableName();
				
				global $SERVICES;
				if ($SERVICES->modules->exists('SystemBookmarks')) {
					$system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks');
					$this->dashboard_bookmark = $system_bookmarks_mod->getDesktopBookmark($module_name, $table_name);
				}
				
			}
			if ($bookmark = $this->dashboard_bookmark) {
				if ($bookmark['extra_values']['filters']['dtf']) {
					$this->db_functions->resetDesignTimeFilters();
					$this->db_functions->setDesignTimeFilters($bookmark['extra_values']['filters']['dtf']);
				}
				if ($bookmark['extra_values']['filters']['filter_info']) {
					$this->db_functions->setConfigFilterInfo($bookmark['extra_values']['filters']['filter_info']);
				}
				if ($bookmark['extra_values']['sort']['field']) {
					$field = $bookmark['extra_values']['sort']['field'];
					if ($bookmark['extra_values']['sort']['order']) {
						$order = strtoupper($bookmark['extra_values']['sort']['order']);
					} else {
						$order = 'DESC';
					}
					$this->setSortField($field, $order);
				}
				
			} else {
				$this->getDefaultPieceFilter();
			}
		}
	}
	
	/**
	 * Sets the default piece filter
	 */
	function getDefaultPieceFilter() {
		global $USER;
		$this->db_functions->resetDesignTimeFilters();
		$this->db_functions->setDesignTimeFilters( array('user_id' => $USER['user_id']) );
	}
	
	/**
	 * Sets which fields to display in the dashboard if no field is specified 
	 * then the display_name will be used.
	 */
	function setContentFields($name = false) {
		if (!$name) {
			if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
				if (!isset($this->db_functions)) {
					$this->db_functions =& $this->mod->getDatabaseTable('item');
				}
				$name = $this->db_functions->getDisplayNameField();
			}
			if ($name == '__none__') {
				$name = 'content';
			}
			$this->content_field_name = $name;
		}
		$this->content_field_name = $name;
		
	}
	
	/**
	 * processed before a table row to remove and empty fields from the display
	 * 
	 * @param array $item the item array (passed by reference)
	 * @param string $style_flag the style normal used for alerts (passed by reference)
	 * 
	 */
	function beforeTableRow(&$item, &$style_flag) {
		if (is_array($contact_fields = $this->getContentFields())) {
			foreach ($contact_fields as $k => $v) {
				if (!$item[$v]) {
					bcrm_unset($item, $v);
				}
			}
		} else {
			if (!$item[$contact_fields]) {
				unset($contact_fields);
			}
		}
		if (!$this->getContentLinks()) {
			bcrm_unset($item, 'links');
		}
		if (!$item['label'] || $item['label'] == ' ') {
			bcrm_unset($item, 'label');
		}
	}
	
	/**
	 * over written to remove the view and edit links
	 */
	function getContentLinks($id = '') {}
	
	/**
	 * Used to make the display work as a link
	 */
	function calculatePiecesValues(& $item) {
		if (is_array($contact_fields = $this->getContentFields())) {
			$display_name = current($contact_fields);
		} else {
			$display_name = $contact_fields;
		}
		
		$name = $this->getTrimmedContentField($item[$display_name]);
		if (!$name) {
			global $L;
			$name = $L['labelNoTitle'];
			$title = $L['labelNoTitle'];
		}
		$href = $this->getModulePath() . str_replace('%id%', $item['id'], $this->getFileSetHref('view'));
		$item[$display_name] = $this->output->getLinkHref(array('href' => $href, 'text' => $name, 'title' => $item[$display_name]));
	}
}

?>