<?php
/**
 * Handles the database functions for WebMAT
 *
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * This class handles the database functions for WebMAT
 *
 * @package WebMAT
 * @author Phil Calder, Matthew Corich, Matthew Ross, Simon Cook, Brenton LeTrabb
 * @abstract
 */
class WebMATStandardDatabaseTable extends WebMAT {
	
	/**
	 * @var string the module name
	 * @access private
	 */
	var $module_name = false;
	
	/**
	 * @var string the name of the table you are currently editing
	 * @access private
	 */
	var $table_name = false;
	
	/**
	 * @var array the data definition for use with DataDictionary
	 * @access private
	 */
	var $data_definition = array();
	
	/**
	 * @var array containing the database colums
	 * @access private
	 */
	var $columns = array();
	
	/**
	 * @var string the name used to append user pref
	 * @access private
	 */
	var $pref_name = false;
	
	/**
	 * @var object the database object
	 * @access private
	 */
	var $db = false;
	
	/**
	 * @var array the design_time_filters array
	 * @access private
	 */
	var $design_time_filters = array();
	
	/**
	 * @var array the fixed_where array
	 * @access private
	 */
	var $fixed_where = array();
	
	/**
	 * @var string check for active used in the where statment
	 * @access private
	 */
	var $active = 1;
	
	/**
	 * @var object the UserAuthentication object
	 * @access private
	 */
	var $ua = false;
	
	/**
	 * @var bool $disable_filter used to disable the module.config filter
	 * @access private
	 */
	var $disable_filter = false;
	
	
	/**
	 * @var integer Stores the contact_id field name (usually contact_id)
	 * @access private
	 */
	var $contact_id_field = '';
	
	/**
	 * @var integer Stores the fields part of the count query, usually '*'
	 * @access private
	 */
	var $count_items_fields = '*';
	
	/**
	 * @var integer Stores the item count value (cached)
	 * @access private
	 */
	var $count_items = -1;
	
	/**
	 * @var integer Stores the alert count value (cached)
	 * @access private
	 */
	var $count_alerts = -1;
	
	/**
	 * @var mixed Stores the alert query parts in an array, or false.
	 * @access private
	 */
	var $count_alerts_query = false;
	
	/**
	 * @var Placeholder to store data option from referring class, for browse query.
	 * @access private
	 */
	var $data_option = '';
	
	/**
	 * @var Placeholder to store qfilter from referring class.
	 * @access private
	 */
	var $qfilter = '';
	
	/**
	 * @var Placeholder to store qfilter from referring class.
	 * @access private
	 */
	var $log_action = '';
	
	/**
	 * The constructor method for WebMATDatabaseTable
	 * 
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 * @todo Move most of this to an init method
	 */
	function WebMATStandardDatabaseTable($module_name, $data_definition) {
		// $this->printDebug('WebMAT constructor for : ' . get_class($this) . '');
		global $USER, $SERVICES;
		$this->module_name = $module_name;
		// print '<div>data_definition</div>'; print_r($data_definition); print '<hr>';
		$this->setDataDefinition($data_definition);
		$this->setActive(1);
		
		$SERVICES->getService('Config');
		$SERVICES->getService('Preferences');
		$SERVICES->getService('DateLib');
		$SERVICES->getService('Modules');
		$this->db =& $SERVICES->getService('Database');
		
		if (!$this->table_name) {
			$this->setTable($this->module_name . '__item');
		} else {
			$this->setTable($this->table_name);
		}
		
		$this->setFixedWhere();
		$php_self = $SERVICES->getGlobal('php_self');
		$this->error = array();
		$this->module_name_lower = strtolower( substr($module_name, 0, 1) ) . substr($module_name, 1);
		$this->setPrefName( $this->module_name_lower . str_replace(array(dirname($php_self), '.php', '/'), '', $php_self) );
		$this->setContactIdField(); // sets default contact id field name (contact_id)
		$this->setQueryType(); // set blank query type
	}
	
	/**
	 * Set the data definition array
	 *
	 * @param array $data_definition An array containing the data definition (fields and options)
	 * @author Phil Calder
	 */
	function setDataDefinition($data_definition) {
		// print '<div>' . get_class($this) . '::setDataDefinition</div>'; print_r(array_keys($data_definition['fields'])); print '<hr>';
		$this->resetWhereStatement(); // Phil 20060209
		if (is_array($data_definition)) {
			$this->data_definition = $data_definition;
		} else {
			$this->data_definition = array();
		}
	}
	
	/**
	 * set the user pref name for the module
	 * 
	 * @param string $pref_name the user preference name
	 */
	function setPrefName($pref_name) {
		 $this->pref_name = $pref_name;
	}
	
	/**
	 * set the table name for the module
	 * 
	 * @param string $table_name the table name
	 */
	function setTable($table_name) {
		$this->table_name = $this->module_name . '__' . str_replace($this->module_name . '__', '', $table_name);
		$this->columns =  $this->db->getColumns($this->table_name);
	}
	
	/**
	 * gets the table name
	 * 
	 * @return string The table name
	 */
	function getTableName() {
		return $this->table_name;
	}
	
	/**
	 * returns the current data_definition
	 * 
	 * @return array the current data definition
	 */
	function getDataDefinition() {
		return $this->data_definition;
	}
	
	/**
	 * returns the current data definition fields
	 * 
	 * @param boolean $field_keys Optional (default false) When true, returns an array of field keys
	 * @return array the current data_definition, or an array of data definition field keys (depending on parameter)
	 * @author Phil Calder
	 */
	function getDataDefinitionFields($field_keys = false) {
		if (is_array($this->data_definition['fields'])) {
			if ($field_keys) {
				if (!$this->dd_field_keys) {
					$this->dd_field_keys = array();
					foreach ($this->data_definition['fields'] as $v) {
						$this->dd_field_keys[] = $v['field_key'];
					}
				}
				return $this->dd_field_keys;
			} else {
				return $this->data_definition['fields'];
			}
		}
		return array();
	}
	
	/**
	 * returns the current data definition options
	 * 
	 * @param string $key Option (default ''). Get just the specified key (if supplied)
	 * @return array the current data definition
	 * @author Phil Calder
	 */
	function getDataDefinitionOptions($key = '') {
		// print '<div>' . get_class($this) . '::getDataDefinitionOptions(' . $key . ')</div>';
		if (is_array($this->data_definition['data_options'])) {
			if (!empty($key)) {
				if (is_array($this->data_definition['data_options'][$key])) {
					return $this->data_definition['data_options'][$key];
				}
				return array();
			} else {
				return $this->data_definition['data_options'];
			}
		}
		return array();
	}
	
	/**
	 * Sets the given field as not read only (temporarily)
	 * @param string $field_key The data definition field key
	 *
	 */
	function setNotReadOnly($field_key) {
		foreach ($this->data_definition['fields'] as $k => $v) {
			if ( ($field_key = '__all__') || ($v['field_key'] == $field_key)) {
				$this->data_definition['fields'][$k]['read_only'] = 0;
			}
		}
	}
	
	/**
	 * 
	 * @param string $key The data definition name
	 * @return boolean true or false if the data definition option exists
	 * @author Phil Calder
	 */
	function dataDefinitionOptionExists($key) {
		// print '<div>' . get_class($this) . '::dataDefinitionOptionExists(' . $key . ')</div>';
		if (is_array($this->data_definition['data_options'])) {
			return (array_key_exists($key, $this->data_definition['data_options']));
		}
		return false;
	}
	
	/**
	 * returns the current data dictionary fields
	 * 
	 * @param boolean $field_keys Optional (default false) When true, returns an array of field keys
	 * @return array the current data dictionary fields, or an array of data dictionary field keys (depending on parameter)
	 * @author Phil Calder
	 */
	function getDataDictionaryFields($field_keys = false) {
		// print '<div>' . get_class($this) . '::getDataDictionaryFields, getDataOption: ' . $this->getDataOption() . '</div>'; print '<hr>';
		if (!is_object($this->dd) || !is_array($this->dd_fields)) {
			$this->dd = getWebMATObject(
				'WebMATDataDictionary', 
				$this->module_name, 
				array(
					'data_definition' => $this->getDataDefinition(), 
					'data_option' => $this->getDataOption()
				)
			);
			$this->dd_fields = $this->dd->getDataDictionary();
		}
		
		if (is_array($this->dd_fields)) {
			if ($field_keys) {
				if (!$this->dd_field_keys) {
					$this->dd_field_keys = array();
					foreach ($this->dd_fields as $v) {
						$this->dd_field_keys[] = $v['field_key'];
					}
				}
				return $this->dd_field_keys;
			} else {
				return $this->dd_fields;
			}
		}
		return array();
	}
	
	/**
	 * returns the fileset config name
	 * 
	 * @return string the Path for the current fileset
	 */
	function getFilesetConfigPath() {
		if (!isset($this->fileset_config_path) || !$this->fileset_config_path) {
			$this->fileset_config_path = 'modules.' .  $this->module_name . '.file_set.' . 
				str_replace($this->module_name . '__', '', $this->getTableName()) . '.';
		}
		return $this->fileset_config_path;
	}
	
	/**
	 * Sets the design time filters
	 * 
	 * @param array the an array with 
	 * @author Phil Calder
	 */
	function setDesignTimeFilters($filter) {
		// print '<div>setDesignTimeFilters, table: ' . $this->getTableName() . ' before:</div>'; print_r($filter); print '<hr>';
		if (is_array($filter)) {
			$this->design_time_filters = $this->arrayMergeReplace($this->design_time_filters, $filter);
		}
		// print '<div>setDesignTimeFilters, table: ' . $this->getTableName() . ' after:</div>'; print_r($filter); print '<hr>';
	}
	
	/**
	 * resets the design time filters
	 * 
	 * @param array the filter value you wish to unset if null will reset all.
	 */
	function resetDesignTimeFilters($filter = false) {
		// print '<div>resetDesignTimeFilters(' . $filter . ') for ' . $this->getTableName() . ' [count=' . (is_array($this->design_time_filters) ? count($this->design_time_filters) : ' not an array') . '</div>'; 
		// print_r($this->design_time_filters); print '<hr>';
		if ($filter) {
			if (isset($this->design_time_filters[$filter])) {
				bcrm_unset($this->design_time_filters, $filter);
			}
		} else {
			// print '<div>reset dtf for module</div>';
			$this->design_time_filters = array();
		}
		$this->resetWhereStatement(); // Phil 200704
	}
	
	/**
	 * sets the active value that is required - false for no active required
	 * 
	 * @param string of values to check in active with false to do no check
	 */
	function setActive($active) {
		$this->active = $active;
	}
	
	/**
	 * sets the display name class variable
	 * 
	 * @param string $display_name the display name (optional)
	 * Phil 20051007 Renamed from setDisplayName
	 */
	function setDisplayNameField($display_name = false) {
		if ($display_name) {
			$this->display_name = $display_name;
		}
		if (is_array($this->data_definition['fields'])) {
			foreach ($this->data_definition['fields'] as $k => $v) {
				if ($v['display_name']) {
					$this->display_name = $v['field_key'];
					return;
				}
			}
		}
		if (!$this->display_name) {
			$this->display_name = '__none__';
		}
	}
	
	/**
	 * Gets the display name class variable
	 * 
	 * if there is no display name it will return '__none__'
	 * 
	 * @param string the display name 
	 * @return string The display name variable
	 */
	function getDisplayNameField() {
		if (!$this->display_name) {
			$this->setDisplayNameField();
		}
		return $this->display_name;
	}
	
	/**
	 * Counts the number of alerts
	 *
	 * 20050822 - value is cached
	 * 20060104 - Added execute count query
	 *
	 * @param boolean $execute_count Optional, default true. 
	 * When false, will populate class variables to store the query parts without running the query.
	 * @return integer the number of alerts
	 * @author Phil Calder
	 */
	function countAlerts($execute_count = true) {
		if ( ($this->count_alerts == -1) || ($execute_count !== true) ) {
			// print '<div>' . get_class($this) . '::countAlerts [mod: ' . $this->getModuleName() . ']</div>';
			if ($alert_where = $this->getAlertWhere()) {
				$where_array = $this->getDisplayByGroupWhere(array());
				$where_array = $this->getFixedWhere($where_array);
				$where_array[] = $alert_where; // $this->getAlertWhere();
				
				// -- added for contact and foreign filtering
				$tables = $fields = '';
				$contact_fields = $this->processGetItemsContactFields($tables, $fields); // Phil - Alters the where statement for contact fields
				$foreign_fields = $this->processGetItemsForeignFields($tables, $fields); // Phil - Alters the where statement for foreign fields
				
				if (count($contact_fields)) {
					$where_array = $this->getDesignTimeFiltersWhere($where_array, false);
				} else {
					$tables =  $this->getTableName();
				}
				
				$this->processGetItemsItemUserTable($tables, $fields); // Phil - Alters the where statement for item_user fields (ie extended seen table)
				
				// Storing query parts in class variables
				// see getCountAlertsQuery
				$this->count_alerts_query = array();
				$this->count_alerts_query['fields'] = 'count(' .  $this->getTableName() . '.id)';
				$this->count_alerts_query['tables'] = $tables;
				$this->count_alerts_query['where'] = implode(" AND ", $where_array);
				
				$result = $this->db->quickGet(
					$this->count_alerts_query['fields'], 
					$this->count_alerts_query['tables'], 
					$this->count_alerts_query['where']
				);
				
				if ($this->db->hasError()) {
					$this->db->printError();
					$this->count_alerts = false;
				} else {
					$this->count_alerts = $result;
				}
			} else {
				$this->count_alerts = false;
			}
		}
		return $this->count_alerts;
	}
	
	/**
	 * counts the number of unseen items
	 * 
	 * @return the number of unseen items, or false
	 * @author Phil Calder
	 */
	function countUnseen() {
		if ($this->getHasSeenTable()) {
			global $SERVICES, $USER;
			$contact_fields = array();
			// Phil - set in processGetItemsContactFields, processGetItemsForeignFields, processGetItemsItemUserTable
			$tables = $fields = '';	
			$this->processGetItemsContactFields($tables, $fields);
			$this->processGetItemsForeignFields($tables, $fields);
			$this->processGetItemsItemUserTable($tables, $fields);
			
			$this->disableFilter(true);
			$where = $this->getWhereStatement();
			$this->disableFilter(false);
			
			$where .= ' AND (' . $this->table_name . '_seen.seen IS NULL OR ' . $this->table_name . '_seen.seen = 0)';
			return $this->db->quickGet('count(' . $this->table_name . '.id)', $tables, $where);
		}
		return false;
	}
	
	/**
	 * counts the number of items according to a preset filter.
	 * The default setting is to count all items assigned to a group the current user is in.
	 *
	 * @return mixed Either an integer - the number of items 'assigned to' the current user - or false
	 * @author Phil Calder
	 * @todo Implement config based lookup per module. This method is not finished yet.
	 */
	function countMine() {
		global $USER;
		$fields = $this->getTableName() . '.id';
		$tables = $this->getTableName();
		
		$mine = false;
		$where = $this->getMineWhere();
		if (empty($where)) {
			//Phil 20051006, TODO: probably add active=1
			if (in_array('owner_group_id', $this->columns)) {
				$where = array( 
					'owner_group_id' => array(
						'where' => 'owner_group_id IN (' . $USER['group_string'] . ')'
					)
				);
				$where = $where['owner_group_id']['where'];
			}
		}
		if ($where) {	
			$mine = $this->db->quickGet('count(' . $fields . ')', $tables, $where);
			if ($this->db->hasError()) {
				$this->db->printError();
			}
		}
		return $mine;
	}
		 
	/**
	 * Returns the number of items in the database visible to the current user
	 * 
	 * @return integer the number of items
	 * @author Phil Calder
	 */
	function countItems() {
		// print '<div>' . get_class($this) . '::countItems [mod: ' . $this->getModuleName() . ']</div>';
		global $SERVICES, $USER;
		$tables = $fields = '';
		$this->processGetItemsContactFields($tables, $fields); // Phil - Alters the where statement for contact fields
		$this->processGetItemsForeignFields($tables, $fields); // Phil - Alters the where statement for foreign fields
		$this->processGetItemsItemUserTable($tables, $fields); // Phil 20050809 
		// Added mainly for cases where we have extra functionality in the seen table,
		// but also to be consistent (ie we do this for getItem, getItems etc)
		$where = $this->getWhereStatement();
		// print '<div>' . get_class($this) . '::countItems [mod: ' . $this->getModuleName() . '] where:<br> ' . $where . '</div>';
		$this->count_items = $this->db->quickGet('count(' . $this->getCountItemsFields() . ')', $tables, $where);
		return $this->count_items;
	}
	
	/**
	 * 
	 * @return mixed Array containing query parts (keys: fields, tables, where,..), or false
	 * @author Phil Calder
	 */
	function getCountAlertsQuery() {
		if (!is_array($this->count_alerts_query)) {
			$this->countAlerts(false); // Populates the count_alerts_query var
		}
		return (is_array($this->count_alerts_query) ? $this->count_alerts_query :  false);
	}
		
	/**
	 * Sets an item seen status
	 * 
	 * @param integer $id the item id
	 * @param integer $status the seen status of the item:
	 *	1 = seen, 0 = unseen, 2 = edited, 3 = hidden
	 * @param bool $safe this will do various checks first (deals primarily
	 *	with hide/unhide). if 'false' then it doesn't care and just updates
	 * @param mixed $extra_values Ignored if false. Otherwise, will update other values in seen table from item array (Phil 20050727)
	 */
	function setItemSeenStatus($id, $status, $safe = true, $extra_values = false) {
		// Phil 20070619 mainly for automated scripts that we dont want to log for
		global $USER;
		if (isset($USER['no_log']) || $USER['no_log']) {
			return;
		}
		$id = $this->cleanId($id); // Phil 20070305
		global $SERVICES;
		if ($this->getHasSeenTable()) {
			global $USER;
			// Phil 20050727
			$fields = array();
			if (is_array($extra_values)) {
				$fields = $extra_values;
			}
			$seen_table = $this->getTableName() . '_seen';
			$where['item_id'] = 'item_id = \'' . $id . '\'';
			if ($status == 2) {
				// if it's edited do we want to 'unhide it'?
				//   (perhaps this var can be an arg in the future)
				$unhide_edited = true;
				if ($unhide_edited) {
					$where['seen'] = '(seen = \'1\' OR seen = \'3\')';
				} else {
					$where['seen'] = 'seen = \'1\'';
				}
				$where['user_id'] = 'user_id <> \'' . $USER['user_id'] . '\'';
				// $this->db->update($seen_table, array('seen' => $status), implode(' AND ', $where));
				$fields['seen'] = $status; // Phil 20050727
				$this->db->update($seen_table, $fields, implode(' AND ', $where));
				bcrm_unset($where, 'seen');
				$status =  1;
			}
			$where['user_id'] = 'user_id = \'' . $USER['user_id'] . '\'';
			
			$action = 'none';
			$seen_value = $this->db->quickGet('seen', $seen_table, implode(' AND ', $where));
			// print '<div>status: ' . $status . ' seen_value: [' . $seen_value . '] is false: ' . ($seen_value == false ? 'yes' : 'no') . '</div>'; 
			// print '<div>extra_values: ' . count($extra_values) . ' [is_array: ' . (is_array($extra_values) ? 'yes' : 'no')  . ']</div>'; print_r($extra_values); print '<hr>';
			
			if ($seen_value === false) { 
				$action = 'insert'; // No entry, perform an insert
			} else if ($safe && ($status == 1) && ($seen_value == 3)) {
				$action = 'none'; // seen is 3 and we dont want to change this to 1
			} else if ( ($status == $seen_value) && (!is_array($extra_values) || !count($extra_values)) ) {
				$action = 'none'; // No need to update
			} else {
				$action = 'update'; // Update the record
			}
			
			// $this->db->printError(); // DEBUG
			// print '<div>' . get_class($this) . ' - setItemSeenStatus action: ' . $action . ', seen_value: ' . $seen_value . '</div>'; print '<hr>';
			// print '<div>' . get_class($this) . ' - setItemSeenStatus</div>'; print_r($where); print '<hr>';
			
			switch ($action) {
			case 'insert':
				$fields['item_id'] = $id;
				$fields['user_id'] = $USER['user_id'];
				$fields['seen'] = $status;
				$this->db->insert($seen_table, $fields);
				break;
			case 'update':
				$fields['seen'] = $status; // Phil 20050727
				$this->db->update($seen_table, $fields, implode(' AND ', $where));
				break;
			case 'none':
			default:
				break;
			}
			// $this->db->printError(); // DEBUG
		}
	}
	
	function getHasAlerts() {
		$alert_id = $this->getAlertIds(false, false, true);
		if (is_array($alert_id) && count($alert_id)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Get an array of alert ids
	 * 
	 * @param string $limit the paging limit
	 * @param array $item_ids Optional array of item ids to filter on
	 * @return array an array of alert ids
	 * @todo Extend (or split) to optionally get the alert items, not just the ids {@link getAlertWhere}
	 */
	function getAlertIds($limit = '', $item_ids = false, $boolean_result = false) {
		// print '<div>getAlertIds(' . $limit . ',..)</div>';
		if ($where = $this->getAlertWhere()) {
			if ($item_ids) {
				$ids = implode(',', $item_ids);
				$items_where = $this->processDesignTimeFilter('id', array('where' => 'id IN (' . $ids . ')'));
			} else {
				// echo 'getAlertIds() missing second param'; //TODO remove
				$items_where = '';
			}
			$tables = $this->getTableName();
			if ($this->processGetItemsItemUserTable($tables, $fields)) {
				$where .= '';
			}
			// print '<div>limit: ' . $limit . '</div>';
			$result = $this->db->select(
				'id', 
				$tables, 
				$where . (strlen($items_where) ? (' AND ' . $items_where) : ''), // Phil 20051201
				'', 
				($boolean_result === true ? 'LIMIT 0, 1' : false)
				
			);
			if ($this->db->hasError()) { $this->db->printError(); } // Phil 20051201
			$items = array();
			while ($row = $this->db->getRow($result)) {
				$items[] = $row['id'];
			}
			return $items;
		} else {
			return array();
		}
	}
	
	/**
	 * Returns an item from the database
	 * 
	 * @param integer $id the item id (If false, will set param $include_where automatically to true)
	 * @param mixed $order_by Optional, defaults to false. No order by processing done here. 
	 * Value is passed to database class.
	 * @param boolean $include_where Optional, default false - Applies only when $id is set. 
	 * Allows where clause to be processed as well as id
	 * @param boolean $refresh Bypass the cache. Default false
	 * @return array an array containing the data from the database
	 */
	function getItem($id, $order_by = false, $include_where = false, $refresh = false) {
		// print '<div>[db tables] ' . get_class($this) . '::getItem(' . $id . ')</div>';
		$id = $this->cleanId($id); // Phil 20070305
		
		// Phil 20070305 - check cache before isViewable
		// Phil 20050517 - caching item
		if (!$refresh && $id && isset($this->item_cache[$this->module_name][$this->getTableName()][$id]) 
		&& $this->item_cache[$this->module_name][$this->getTableName()][$id]) {
			// print '<div>cached</div>'; print_r($item); print '<hr>';
			return $this->item_cache[$this->module_name][$this->getTableName()][$id];
		}
		
		if ($id && !$this->isViewable( $id )) {
			// print '<div>not viewable ' . $id . '</div>'; print '<hr>';
			return false;
		}
		
		$tables = $fields = '';
		
		$this->processGetItemsContactFields($tables, $fields);
		$this->processGetItemsForeignFields($tables, $fields);
		$this->processGetItemsItemUserTable($tables, $fields);
		
		$where = '';
		
		if (isset($id) && ($id !== false)) {  // id can be 0
			$where .= $this->getTableName() . '.id = "' . $id . '"';
		} else if ($id === false) { // Phil 20060626
			$include_where = true;
		} else {
			// print '<div>getitem(' . $id . ')</div>';
			return false;	
		}
		
		if ($include_where) {
			$this->disableFilter(true);
			
			if (strlen($where)) {
				$where .= ' AND ';	
			}
			$where .= $this->getWhereStatement();
			if ($id) {
				bcrm_unset($this->design_time_filters, 'id');
			}
		}
		
		// print '<div>fields: ' .$fields . '</div>';
		// print '<div>tables: ' .$tables . '</div>';
		// print '<div>where: ' .$where . '</div>';
		// print '<hr>';
		
		$item = $this->db->quickGet($fields, $tables, $where, $order_by);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
		
		if (isset($item['id']) && $item['id']) { // Phil 20050517 - caching item
			$this->item_cache[$this->module_name][$this->getTableName()][$id] = $item;
		}
		return $item;
	}
	
	/**
	 * Returns an array of items from the database
	 * 
	 * @param string $order_by the deleted status 1 = undeleted, 0 = deleted
	 * @param string $limit limit from paging
	 * @param mixed $where A where statement (string or array)
	 * @param boolean $return_result Optional, default false. When true this method returns a result resource.
	 * @param boolean $simple_query Optional, default false. When true this method does not proccess certain complex queries.
	 * @return mixed Either an array containing the items from the database, or a result resource (depending on param $return_resource)
	 */
	function getItems($order_by = false, $limit = false, $where = false, $return_result = false, $simple_query = false) {  //get all
		global $SERVICES, $USER;
		$contact_fields = array();
			
		$tables = $fields = '';	// set in processGetItemsContactFields and processGetItemsForeignFields
		if (!$simple_query) {
			$this->contact_fields = $this->processGetItemsContactFields($tables, $fields);
			$this->foreign_fields = $this->processGetItemsForeignFields($tables, $fields);
			$this->processGetItemsItemUserTable($tables, $fields);
		}
		
		$sort = $this->getOrderBy($order_by);
		if ($where) {
			$where = $where;
		} else {
			$where = $this->getWhereStatement();
		}
		
		// If the hidden/unhidden is to be more system wide, then probably
		// will need to add some more where statements in here!
		// Otherwise, it's currently used as a design time filter.
		
		// print '<div>' . get_class($this) . ' database table getItems query:</div>'; 
		// print_r(array('fields'=>$fields, 'tables'=>$tables, 'where'=>$where, 'sort'=>$sort, 'limit' => $limit)); 
		// print '<hr>'; 
		$result = $this->db->select($fields, $tables, $where, $sort, '',$limit);
		
		if ($this->db->hasError())	$this->db->printError();
		
		if (!$simple_query) {
			$this->storeBrowseQuery($this->getModuleNameLower(), $this->getTableName(), $this->getDataOption(), $fields, $tables, $where, $sort, $limit);
		
			// Phil 20051201 see WebMATList::getItemsResource
			if ($return_result) {
				return $result;	
			}
			
			while ($row = $this->db->getRow($result)) {
				$items[$row['id']] = $this->processRow($row['id'], $row);
			}
		}
		
		return $items;
	}
	
	/**
	 * @author Phil Calder
	 */
	function processRow($id, $row) {
		// print '<div>'  . get_class($this) . '::processRow(' . $id . ', ' . ($row === false ? 'false' : 'item') . ') calling isViewable</div>'; print '<hr>'; 
		if ($id && $this->isViewable( $id, $row )) { // Phil
			$this->addBrowseQueryIndex($id);
			$item = $this->processGetRowContactFields($row, $this->contact_fields);
			$item = $this->processGetRowForeignFields($row, $this->foreign_fields);
		}
		return $item;
	}
	
	/**
	 * get the items from the db_function object
	 *
	 * @return array the array of items
	 * @author Phil Calder
	 */
	function getItemsResource($sort = false, $limit = false, $where = false) {
		return $this->getItems($sort, $limit, $where, true);
	}
	
	/**
	 * Specify whether to use an iterator for getItems, or process and return the entire items array
	 *
	 * Added 200512xx. Extend this method for modules which rely on bulk processing 
	 * of items in the {@link afterGetItems} method
	 *
	 * @see WebMATList::useGetItemsIterator
	 *
	 * @return boolean true or false
	 * @author Phil Calder
	 */
	function useGetItemsIterator() {
		return true;
	}
	
	/**
	 * Useful for getting a 'label' for the item 
	 * 
	 * (ie for a link)
	 *
	 * @param integer $id The item id
	 * @return mixed string display name for the item, or false
	 * @author Phil Calder
	 */
	function getItemDisplayName($id) {
		$id = $this->cleanId($id); // Phil 20070305
		// print '<div>getItemDisplayName(' . $id . ') ' . get_class($this) . ' [' . $this->module_name . ']</div>';
		if ( $id && ($field = $this->getDisplayNameField()) != '__none__') {
			// Phil - Use cached value if possible (see function getItem)
			// unlikey that its cached - this functionality generally called before getItem/getItems
			if ($id && isset($this->item_cache[$this->module_name][$this->getTableName()][$id][$field]) ) {
				// print '<div>getItemDisplayName(' . $id . ') cached: ' . $this->item_cache[$this->module_name][$this->getTableName()][$id][$field] . '</div>'; print '<hr>';
				return $this->item_cache[$this->module_name][$this->getTableName()][$id][$field];
			} else {
				$table = $this->getTableName();
				$where = $this->getFixedWhere(array('id' => 'id=' . $id));
				// print '<div>where (array):</div>'; print_r($where); print '<hr>';
				$where = implode(' AND ', $where);
				// print '<div>SELECT ' . $field . ' FROM ' . $table . ' WHERE ' . $where . '</div>';
				if ($result = $this->db->quickGet(
					$field, 
					$table,
					$where
				)) {
					return $result;
				}
			}
		}
		return false; // Cant determine what the display name is, or there isn one
	}

	/**
	 * Stores the data option from the referring class (for browse query)
	 * @author Phil Calder
	 */
	function setDataOption($data_option) {
		$this->data_option = $data_option;
	}
	
	/**
	 * Gets the data option from the referring class (for browse query)
	 * @author Phil Calder
	 */
	function getDataOption() {
		return $this->data_option;
	}
	
	/**
	 * Determines whether to use the webmat list/view browse query functionality
	 *
	 * @return boolean Whether to use/update the browse query functionality
	 * @see storeBrowseQuery
	 * @see clearBrowseQuery
	 * @author Phil Calder
	 */
	function isUsingBrowseQuery() {
		if (!isset($this->is_using_browse_query)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			if ($SERVICES->config->lookup('modules.Front.webmat.database_table.is_using_browse_query')) {
				$this->is_using_browse_query = true;
			} else {
				$this->is_using_browse_query = false;
			}
		}
		return $this->is_using_browse_query;
	}
	
	function addBrowseQueryIndex($id) {
		$id = $this->cleanId($id); // Phil 20070305
		if ($this->current_browse_query) {
			// print '<div>addBrowseQueryIndex(' . $id . ')</div>';
			// print '<div>limit: ' . $this->current_browse_query['limit'] . '<div>';
			$pos = 0;
			if (!isset($this->current_browse_query['cache']) || empty($this->current_browse_query['cache']) ) {
				$pos = $this->current_browse_query['start_pos'];
				$this->current_browse_query['cache'][$pos] = $id;
			} else {
				$this->current_browse_query['cache'][] = $id;
			}
			// // print_r($this->current_browse_query['cache']); print '<hr>';
		}
	}
	
	/**
	 * Stores getItems select query in Session
	 * @author Bosco 20050728
	 * Phil 20050822 - Only stores if there is a data option
	 * Phil 20051104 - see {@link isUsingBrowseQuery}
	 */
	function storeBrowseQuery($module, $table, $data_option, $fields, $tables, $where, $sort, $limit) {
		if (!$this->isUsingBrowseQuery()) { // Phil 20051104
			return;
		}
		if (!empty($data_option)) { // Phil see WebMATList::getItems, WebMATDatabaseTable::setDataOption, WebMATDatabaseTable::getDataOption
			global $USER;
			// print '<div>store browse query for ' . $module. ', ' . $table . ', ' . $data_option . '</div>';
			$start_pos = 0;
			if ($limit) {
				$start_pos = ltrim($limit, 'LIMIT ');
				$start_pos = intVal($start_pos);
			}
			$USER['browse_query'][$module][$table][$data_option] = array(
				'module' => $module,
				'table'  => $table,
				'data_option' => $data_option,
				'fields' => $fields,
				'tables' => $tables,
				'where'  => $where,
				'sort' 	 => $sort,
				'limit'  => $limit,
				// Bosco 20050817 | Added to refine the browse limit
				// 'browse_limit' => $limit, // Phil - removed
				'start_pos' => $start_pos
			);
			$this->current_browse_query =& $USER['browse_query'][$module][$table][$data_option];
			// print "<div>stored browse query for  $module, $table, $data_option</div>";
			// $this->getBrowseQuery($module, $table, $data_option); // Just to populate the current_query_index
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function updateBrowseQuery($module, $table, $data_option, $values) {
		global $USER;
		$query = $this->getBrowseQuery($module, $table, $data_option);
		if (is_array($query) && is_array($values)) {
			// print "<div>query</div>"; print_r($query); print '<hr>';
			// print "<div>update browse query $module, $table, $data_option</div>"; print_r($values); print '<hr>';
			$new_query = $this->arrayMergeReplace($query, $values);
			$USER['browse_query'][$module][$table][$data_option] = $new_query;
		}
	}
	
	function getBrowseQuery($module, $table, $data_option) {
		global $USER;
		// print "<div>getBrowseQuery($module, $table, $data_option)</div>";
		if (@is_array($USER['browse_query'][$module][$table][$data_option])) {
			$this->current_browse_query =& $USER['browse_query'][$module][$table][$data_option];
			return $this->current_browse_query;
		}
		return false;
	}
	
	/**
	 * Clears the getItems select Query stored in Session
	 * @author Bosco 20050728
	 * @todo (Phil 20050810) should be able to clear a particular browse query
	 */
	function clearBrowseQuery($module = '', $table = '', $data_option = '') {
		global $USER;
		// print "<div>clearBrowseQuery($module, $table, $data_option)</div>";
		bcrm_unset($USER, 'browse_query', $module, $table, $data_option);
		// $this->current_browse_query = false;
	}
	
	/**
	 * @param string $type Option query type (eg mine, alert, items)
	 * @author Phil Calder
	 */
	function setQueryType($type = '') {
		$this->qfilter = $type;
	}
	
	/**
	 * @author Phil Calder
	 * @todo Integrate with other WebMAT classes (esp. filters) - WIP 20050526
	 * @see getWhereStatement
	 */
	function getQueryType() {
		$type = $this->qfilter; // $_GET['qfilter'];
		switch ($type) {
			case 'alert':
			case 'items':
			case 'mine':
			case 'unseen':
			case 'modified':
				return $type;
				break;
			default: // standard getItems functionality (ie get all)
				return 'items';
				break;
		}
	} 
	
	/**
	 * Updates tables and fields parameters
	 * 
	 * @param string $tables Tables string for database query
	 * @param string $fields Field list for database query
	 * @return array An array of contact fields
	 * @author Phil Calder
	 */
	function processGetItemsContactFields(&$tables, &$fields) {
		// print '<div>processGetItemsContactFields, ' . get_class($this) . ' ' . $this->getTableName() . '</div>'; print '<hr>'; // print_r($this->getDataDictionaryFields()); print '<hr>';
		$contact_fields = array();
		
		// Handles contact fields
		// foreach ($this->getDataDefinitionFields() as $k => $v) {
		foreach ($this->getDataDictionaryFields() as $k => $v) { // Phil 20051107
			// Phil 20051029 Added second condition (thanks Brenton)
			// Phil 20051031 Can be 0, 1, or 2 - we skip 1 and 2
			// if ( ($v['type'] == 'contact') && (!$v['field_is_custom']) ) { 
			// Phil 20051220 Added 3rd condition	
			if ( ($v['type'] == 'contact') && !$v['field_is_custom'] && ($this->getTableName() != $this->getContactTable()) ) { 
				
				$contact_fields[] = $v['field_key'];
				
				if ($v['field_key'] == $this->getContactIdField()) { // keep for permissions
					
					global $USER;
					
					$contact_is_viewable = array(
						'_contact_id' => array( 
							'where' => "" . 
							$this->getWebFormString() . "
							(  
								(
									(" . $this->getContactTable() . ".view_group_id IS NULL) 
									OR (" . $this->getContactTable() . ".view_group_id IN (" . $USER['group_string'] . ")) 
									OR (" . $this->getContactTable() . ".edit_group_id IN (" . $USER['group_string'] . ")) 
									OR (" . $this->getContactTable() . ".owner_group_id IN (" . $USER['group_string'] . ")) 
									OR (" . $this->getContactTable() . ".admin_group_id IN (" . $USER['group_string'] . ")) 
								)
							)"
						)
					);
					
					// print '<div>$contact_is_viewable:</div>'; print_r($contact_is_viewable); print '<hr>';
					$this->setDesignTimeFilters($contact_is_viewable);
				}
			}
		}
		if (count($contact_fields)) {
			
			$columns = array();
			$columns[] = $this->getTableName() . '.*';
			$tables = $this->getTableName();
			
			foreach ($contact_fields as $k => $v) {
				$tables .= ' LEFT JOIN Contact__item _' . $v . ' ON ' . $this->getTableName() . '.' . $v . ' = _' . $v . '.id' ;
				$columns[] = '_' . $v . '.display_name' . ' AS _' . $v . '_display_name';
			}
			$fields = implode(', ', $columns);
		} else {
			$tables = $this->getTableName();
			$fields = $this->getTableName() . '.*';
		}
		
		return $contact_fields;
	}
	
	/**
	 * Handles foreign key definitions 
	 * 
	 * Note: not SQL foreign keys, our data definitions for values in other tables. 
	 * Handles standard and custom foreign fields
	 * 
	 * Updates $tables and $fields parameters
	 * @param string $tables Tables string for database query
	 * @param string $fields Field list for database query
	 * @return array Foreign fields
	 * @author Phil Calder
	 */
	function processGetItemsForeignFields(&$tables, &$fields) {
		// print '<div>' . get_class($this) . '::processGetItemsForeignFields - getDataDefinitionFields</div>'; print_r($this->getDataDefinitionFields() ); print '<hr>';
		$foreign_fields = $custom_fields = array();
		foreach ($this->getDataDictionaryFields() as $k => $v) { // Phil 20051107
			if (isset($v['foreign']) && ($v['foreign'] == 1)) {
				$foreign_fields[] = $v['field_key'];
				// Phil 20051031 Added second condition - some custom foreign fields are from standard tables (2)
				if (isset($v['field_is_custom']) && ($v['field_is_custom'] == 1) ) {
					$custom_fields[] = $v['field_key'];
				} 
			}
		}
		if (count($foreign_fields)) {
			// print '<div>' . get_class($this) . '::processGetItemsForeignFields - foreign fields:</div>'; print_r($foreign_fields); print '<hr>';
			$columns = array();
			$tables_used = array(); // Phil 20050222, so we can have multiple foreign fields from the same table
			$primary_field_name = $this->getPrimaryFieldName(); // 'primary_record'; // this (possibly) needs to be data or config driven
			foreach ($foreign_fields as $k => $v) {
				// print '<div>' . get_class($this) . '::processGetItemsForeignFields -  foreign field: ' . $v . '</div>';
				$info = explode('.', $v);
				$table = $info[0]; 
				$field_name = $info[1];
				
				// Phil 20061010 is_contact_field - Handle custom foreign Contact fields differently
				$is_contact_field = false;
				if (substr($v, 0, strlen('Contact__item')) == 'Contact__item') {
					$is_contact_field = true;
				}
				
				if ($is_custom = in_array($v, $custom_fields)) {
					// print '<div>is_custom: ' . $v . '</div>';
					if (strpos($table, "_custom") === false) {
						$table .= '_custom';
					}
					$table_alias = '_' . $table;
					$parent_table = substr($table, 0, strpos($table, "_custom"));
					$parent_table_alias = '_' . $parent_table;
					//--
					
					if (!in_array($parent_table, $tables_used)) { // Phil 20051027
						if (!$is_contact_field && $this->getPrimaryFieldName()) {
							$tables .= ' LEFT JOIN ' . $parent_table . ' ' . $parent_table_alias .
							' ON 
							(
							' . $parent_table_alias . '.' . $this->getPrimaryFieldName() . ' =1 
							AND
							' . $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $parent_table_alias . '.' . $this->getContactIdField('foreign') . ' 
							)
							';
							$tables_used[] = $parent_table; 
						} else {
							$tables .= ' LEFT JOIN ' . $parent_table . ' ' . $parent_table_alias .
							' ON 
							(
							' . $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $parent_table_alias . '.' . 'id' . ' 
							)
							';
							$tables_used[] = $parent_table; 
						}
					}
					//--
				} else {
					$table_alias = '_' . $table; // May be changed if this is a custom foreign field
					// print '<div>alias: ' . $table_alias . '</div>';
				}
				
				if ( !in_array($table, $tables_used)) { //  || $is_custom ) { // 20050322
					if ($is_custom) {
						$tables .= ' LEFT JOIN ' . $table . ' ' . $table_alias .
							' ON 
							(
							 ' . $table_alias . '.custom_id = ' . $parent_table_alias . '.id
							)
							';
					} else {
						
						if ($is_contact_field) {
							$tables .= ' LEFT JOIN ' . $table . ' ' . $table_alias .
							' ON 
							(
							' . $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $table_alias . '.' . 'id' . ' 
							)
							';
							$tables_used[] = $table; 
							
							
						} else {
						
							$tables .= ' LEFT JOIN ' . $table . ' ' . $table_alias .
								' ON 
								(
								' . $table_alias . '.' . $this->getPrimaryFieldName() . ' =1 
								AND
								' . $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $table_alias . '.' . $this->getContactIdField('foreign') . ' 
								)
								';
						}
					}
					$tables_used[] = $table; // 20050322
				}	
				$columns[] = '_' . $table . '.' . $field_name . ' AS _' . $table . '_' . $field_name;
			}
			$foreign_fields_str = implode(', ', $columns);
			$fields = $fields . ', ' . $foreign_fields_str;
			$fields = trim($fields, ' ,');
		}	
		return $foreign_fields;
	}
	
	/**
	 * Returns true and updates tables and fields parameters, or returns false if there is no seen table
	 * 
	 * @param string The tables string (gets updated if there is a seen table)
	 * @param string The fields string (gets updated if there is a seen table)
	 * @return boolean Whether or not there is a seen table
	 * @author Phil Calder
	 */
	function processGetItemsItemUserTable(&$tables, &$fields) {
		global $SERVICES;
		if ($this->getHasSeenTable()) {
			global $USER;
			
			$seen_table = $this->table_name . '_seen';
			
			$tables .= ' 
			LEFT JOIN ' . $seen_table . ' ON (' . $this->table_name . '.id = ' . $seen_table . '.item_id ' . ' AND ' . $seen_table . '.user_id = \'' . $USER['user_id'] . '\')';
			
			$fields .= ', '. $seen_table . '.seen AS seen';
			
			return true;
		}
		
		return false;
	}
	
	/**
	 * Processes a row for contact fields
	 *
	 * @param array $contact_fields The contact fields to look for
	 * @return array The processed row
	 * @author Phil Calder
	 */
	function processGetRowContactFields($row, $contact_fields) {
		// contact fields
		if (is_array($contact_fields) && count($contact_fields)) {
			foreach ($contact_fields as $k => $v) {
				$edited_row = $row;
				$edited_row[$v] = array('id' => $row[$v], 'display_name' => $row['_' . $v. '_display_name']);
				bcrm_unset($edited_row, '_' . $v. '_display_name');
			}
			return $edited_row;
		}
		return $row;
	}
	
	/**
	 * Processes a row for foreign fields
	 *
	 * @param array $foreign_fields The contact fields to look for
	 * @return array The processed row
	 * @author Phil Calder
	 */
	function processGetRowForeignFields($row, $foreign_fields) {
		// foreign fields
		if (count($foreign_fields)) {
			$edited_row1 = $row;
			foreach ($foreign_fields as $k => $v) {
				$info = explode('.', $v);
				$table = $info[0];
				$field_name = $info[1];
				
				$edited_row = $edited_row1;
				
				$edited_row[$v] = $edited_row['_' . $table. '_' . $field_name];
				bcrm_unset($edited_row, '_' . $table. '_' . $field_name);
				$edited_row1 = $edited_row;
			}
			return $edited_row;
		}
		return $row;
	}
	
	/**
	 * Returns order by statement without 'ORDER BY'
	 * 
	 * @param string $order_by the order by array from the sorting functions
	 * @return string the order by statement without 'ORDER BY'
	 */
	function getOrderBy($order_by = '') {
		if ($order_by == 'none') {
			$sort = '';
		}
		
		if (strpos($order_by['field'], '__') && strpos($order_by['field'], '.')) { 
			// Phil, Foreign fields, eg ModuleName__table.fieldname becomes _ModuleName__table_fieldname
			//$order_by['field'] = '_' . str_replace('.', '_', $order_by['field']);
			$order_by['field'] = str_replace('.', '_', $order_by['field']); //syed :: 20050628
		}
		
		if ( ($order_by) && (is_array($order_by)) && ((in_array($order_by['field'], $this->columns)) || 
			substr($order_by['field'], 0, 1) == '_') ) {
			$sort = $order_by['field'] . ' ' .$order_by['order'];
		}
		return $sort;
	}
	
	/**
	 * Returns the fields to use in countItems()
	 * 
	 * @return string countItems fields
	 * @author Phil Calder
	 */
	function getCountItemsFields() {
		return $this->count_items_fields;
	}
	
	/**
	 * Sets the fields to use in countItems()
	 * 
	 * It is not necessary to call this method, defaults to star
	 * @param string $str - the countItems fields
	 * @author Phil Calder
	 */
	function setCountItemsFields($str) {
		$this->count_items_fields = $str;
	}
	
	//=== The following function are used to create the where string ===========
	
	// * @param boolean $disable_filter disables the getFilterWhere function
	/**
	 * Creates the where string
	 * 
	 * @return string the where statement
	 */
	function getWhereStatement() {
		if ($this->where_statement) {
			return $this->where_statement;
		}
		
		// Phil 20050526 - will update to handle other query-type filters
		// Phil 20050816 - No longer return qfilter here - 
		// will continue processing other factors which contribute to the where clause 
		// (eg config and search filters, fixed conditions)
		$type = $this->getQueryType();
		// $type = 'modified';
		// print '<div>query type: ' . $type . '</div>';
		switch ($type) {
			case 'alert':
				$where_str = $this->getAlertWhere();
			break;
			case 'mine':
				$where_str = $this->getMineWhere();
			break;
			case 'unseen':
				$where_str = $this->getUnseenWhere();
			break;
			case 'modified':
				$where_str = $this->getModifiedWhere();
				// print '<div>modified: ' . $where_str . '</div>';
			break;
			case 'items':
			default: // standard where statement functionality (ie get all)
				// print '<div>getWhereStatement - switch-default</div>';
			break;
		}
		
		// Phil 20050816
		if (isset($where_str) && is_string($where_str)) {
			$where_array = (array)$where_str;
		} else if (!isset($where_str) || !is_array($where_array)) { 
			$where_array = array();
		}
		$where_array = $this->getFixedWhere($where_array);
		$where_array = $this->getDesignTimeFiltersWhere($where_array);
		if (! ($type == 'mine') ) { // Phil 20051006 Dont call getDisplayByGroupWhere if filtering on 'mine'
			$where_array = $this->getDisplayByGroupWhere($where_array);
		}
		if (!$this->disable_filter) {
			$where_array = $this->getFilterWhere($where_array);
		}
		foreach ($where_array as $k =>$v) {
			if (trim($v) == '') {
				bcrm_unset($where_array, $k);
			}
		}
		
		$this->where_statement = trim( implode(" AND ", $where_array) ); // Phil 20050526
		return $this->where_statement;
	}
	
	/**
	 * Unset the cached where statement
	 *
	 * @author Phil Calder
	 */
	function resetWhereStatement() {
		$this->where_statement = false;
	}
	
	/**
	 * Sets the fixed where clause
	 */
	function setFixedWhere() {
		if ($this->active) {
			$this->fixed_where = $this->arrayMergeReplace($this->fixed_where, $this->getTableName() . '.active=' . $this->active );
		}
	}
	
	/**
	 * Returns fixed_where array merged with current where array
	 * 
	 * @param array the current where statement
	 * @return array the current where statement with the new where statement added to the end
	 */
	function getFixedWhere($where_array) {
		$where_array = $this->arrayMergeReplace($this->fixed_where, $where_array);
		return $where_array;
	}
	
	/**
	 * returns the display by group flag for the current file set
	 *
	 * Checks global WebMAT override and per module setting
	 *
	 * @return integer Display by group flag (0,1,2)
	 * @author Phil Calder 20050204
	 */
	function getDisplayByGroupFlag() {
		// Note: could return 0 here to always disable group permission filtering
		if ($this->isDisplayByGroupFlagDisabled()) { // Phil 20051104
			return 0;
		}
		global $SERVICES;
		if (!isset($this->display_by_group_flag)) {
			$this->display_by_group_flag = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'display_by_group');
		}	
		return $this->display_by_group_flag;
	}
	
	/**
	 * returns the display by group flag for the current file set
	 *
	 * Checks global WebMAT override and per module setting
	 *
	 * @return integer Display by group flag (0,1,2)
	 * @author Phil Calder 20050204
	 */
	function getContactRequiredFlag() {
		global $SERVICES;
		if (!isset($this->contact_required_flag)) {
			$this->contact_required_flag = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'contact_required');
		}	
		return $this->contact_required_flag;
	}
	
	/**
	 * A WebMAT setting to disable the display by group functionality
	 *
	 * @return boolean When true, means display by group flag should be set to 0.
	 * @author Phil Calder
	 * @access private
	 * @see getDisplayByGroupFlag
	 */
	function isDisplayByGroupFlagDisabled() {
		global $SERVICES;
		if (!isset($this->display_by_group_flag_disabled)) {
			if ($SERVICES->config->lookup('modules.Front.webmat.database_table.display_by_group_flag_disabled')) {
				return ($this->display_by_group_flag_disabled = true);
			}
		}
		return ($this->display_by_group_flag_disabled = false);
	}
	
	/**
	 * @return boolean Whether the current file set contains a 'seen' table
	 * @author Phil Calder 20051007
	 */
	function getHasSeenTable() {
		// Note: could return false here to always disable seen table processing
		// Phil 20070619 mainly for automated scripts that we dont want to log for
		global $USER;
		if (isset($USER['no_log']) && $USER['no_log']) {
			return false;
		}
		global $SERVICES;
		if (!isset($this->has_seen_table)) {
			$this->has_seen_table = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'seen_table');
		}	
		return $this->has_seen_table;
	}
	
	function getWebFormString() {
		$web_form_string = '';
		if (is_object($this->ua) && ($this->ua->isWebFormUser() && $this->ua->isWebFormUserAllowedToViewOwnContact()) && ($users_contact_id = $this->ua->users->getUsersContactId())) {
			// $web_form_string = "OR " . $this->getContactTable() .".id = '" . $this->ua->users->getUsersContactId() . "' "; 
			$web_form_string = "" . $this->getContactTable() .".id = '" . $users_contact_id . "' OR "; 
			// print '<p>==== ' . $web_form_string . ' =====</p>';
		}
		return $web_form_string;
	}	
	
	/**
	 * Adds the display by group filters to the where array
	 * 
	 * As of 20050916 Handles Web Form user contact permissions
	 * NOTE: this can be turned off in the module.config
	 * 
	 * @param the current where statement
	 * @return the current where statement with the new where statement added to the end
	 * @author Phil Calder, Matthew Ross
	 */
	function getDisplayByGroupWhere($where_array) {
		global $SERVICES, $USER;
		// print '<p>getDisplayByGroupWhere</p>';
		
		// lock_to_contact (if there is one. contact required is a subset of this)
		// contact module:
		// lock_permissions (all items for this contact use the contacts permissions)
		$permissions_flag = $this->getDisplayByGroupFlag();
		
		if ($permissions_flag == 0) {
			// Do not display by group (contact and foreign filtering will still apply elsewhere)
		} else if ($permissions_flag && in_array('view_group_id', $this->columns)) {
			
			// Phil 20050916 Added $web_form_string to query
			$web_form_string = $this->getWebFormString();
			
			$assigned_to_string = '';
			if ( in_array('assigned_to_group_id', $this->columns) ) {
				$assigned_to_string = " OR " . $this->prefixString('assigned_to_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") ";
			}
			
			if (in_array($this->getContactIdField(), $this->columns)) {
				
				if ( ($permissions_flag == 2)) {
					// print '<br>2<hr>'; print_r($where_array); print '<hr>';
					// Lock permissions to the contact (if there is one)
					$where_array[] = "
					(
						" . $web_form_string . "
						( 
							(" . $this->getContactTable() . ".view_group_id IS NULL) 
							AND 
							(
								(" . $this->prefixString('view_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
								OR " . $this->prefixString('edit_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
								OR " . $this->prefixString('owner_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
								" . $assigned_to_string . " 
								OR " . $this->prefixString('admin_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
								)	
							)
						)
						OR 
						( 
							(
							" . $this->getContactTable() .".view_group_id" . " IN (" . $USER['group_string'] . ") 
							OR " . $this->getContactTable() .".edit_group_id" . " IN (" . $USER['group_string'] . ") 
							OR " . $this->getContactTable() .".owner_group_id" . " IN (" . $USER['group_string'] . ") 
							" . $assigned_to_string . " 
							OR " . $this->getContactTable() .".admin_group_id" . " IN (" . $USER['group_string'] . ") 
							)
						)
					)
					";
				} else {
					// print '<br>NOT 2<hr>'; print_r($where_array); print '<hr>';
					// -- test // if ( !($this->module_name == 'Contact') ) {
						// Using the current modules permissions for this item (contact and foreign field filtering will still apply elsewhere)
						$where_array[] = "
						(
							" . $web_form_string . "
							(
								( 
									(" . $this->getContactTable() . ".lock_permissions = 1) 
									AND 
									( 
										(
										" . $this->getContactTable() .".view_group_id" . " IN (" . $USER['group_string'] . ") 
										OR " . $this->getContactTable() .".edit_group_id" . " IN (" . $USER['group_string'] . ") 
										OR " . $this->getContactTable() .".owner_group_id" . " IN (" . $USER['group_string'] . ") 
										" . $assigned_to_string . " 
										OR " . $this->getContactTable() .".admin_group_id" . " IN (" . $USER['group_string'] . ") 
										)
									) 
								)
							)
							OR 
							(  
								( (" . $this->getContactTable() . ".lock_permissions IS NULL) OR ((" . $this->getContactTable() . ".lock_permissions = 0)) ) 
								AND 
								(
									(
									" . $this->prefixString('view_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
									OR " . $this->prefixString('edit_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
									OR " . $this->prefixString('owner_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
									" . $assigned_to_string . " 
									OR " . $this->prefixString('admin_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ")
									)	
								)
							)
						)
						";
				}
			
			} else { 
				$where_array[] = "
				( 
					(" . $this->prefixString('view_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
					OR " . $this->prefixString('edit_group_id',  $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
					OR " . $this->prefixString('owner_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ") 
					" . $assigned_to_string . " 
					OR " . $this->prefixString('admin_group_id', $this->getTableName() . ".") . " IN (" . $USER['group_string'] . ")
					)
				)
				";
				
			}
				
		}
		
		return $where_array;
	}
	
	/**
	 * Adds the design time filters to the where array
	 *
	 * @param the current where statement
	 * @return the current were statement with the new where statement added to the end
	 */
	function getDesignTimeFiltersWhere($where_array) {
		$data_definition = $this->getDataDefinition(); // Phil 20060523
		$data_dictionary_fields = $this->getDataDictionaryFields(); // Phil 20051111
		
		foreach ($this->design_time_filters as $k => $v) {
			$where = false;
			$key = $k;
			// Phil 20051115 Added __special__ key to force inclusion in design time filter regardless of whether the key is in the data definition
			if ( 
				!in_array($k, $this->columns) && 
				!($key == $this->getContactIdField()) && 
				!($key == '__special__') && 
				(!in_array(trim($key, '_'), array_keys($data_dictionary_fields))) 
			) {
				// Check this is not a contact field (eg _field.display_name)
				$tmp = explode('.', $key);
				$tmp_key = trim($tmp[0], '_');
				if (!in_array($tmp_key, array_keys($data_dictionary_fields))) {
					bcrm_unset($this->design_time_filters, $k);
					break; // break the foreach loop
				} 
			}
			
			// standard/custom fields
			if ( in_array($k, $this->columns) ) {
				// Phil 20051111 Using data dictionary fields instead of data definition
				// Phil 20051031 - check field_is_custom == 1 (ie not 2)
				// Phil 20060523 - To enable advanced bookmarks (ie filter on fields not in the current data option field list)
				if (isset($data_definition['fields'][$k]['field_is_custom']) && ($data_definition['fields'][$k]['field_is_custom'] == 1)) { 
					$where = $this->processDesignTimeFilter($k, $v, "_custom");	
				} else {
					$where = $this->processDesignTimeFilter($k, $v);
				}
				
			} else if (substr($k, 0, 1) == '_') { // foreign fields, or __special__
				if ( is_array($v) ) {
					if (isset($v['where'])) { // Phil: Lets not have an array of where statements here. Use the last entry.
						if (is_array($v['where'])) {
							$count = count($v['where']);
							$v['where'] = $v['where'][$count-1]; // use the last one
						}
					}
					$where = $v['where'];
				} else {
					$where = $k . "='" . $v . "'";
				}
			}
			if ($where) {
				$where_array[] = $where;
			}
		}
		return $where_array;
	}
	
	/**
	 * processes a design time filter value and returns the correct where
	 * 
	 * @param sting the field that is being processed
	 * @param string the design time filter value
	 * @param string the field from custom table that is being processed ::  Syed :: 20050626
	 * @return string the where statement$this->getTableName
	 */
	function processDesignTimeFilter($field, $value,$custom='') {
		if ( is_array($value) ) {
			if (strstr($value['where'], $this->prefixString($field, $this->getTableName() .$custom. '.'))) {
				$where = $value['where'];
			} else {
				 $where = str_replace($field, $this->prefixString($field, $this->getTableName() . $custom.'.'), $value['where']);
			}
		} else {
			// print '<div>field: ' . $field . ' value: ' . $value . '</div>';
			$where = $this->prefixString($field, $this->getTableName() .$custom. '.') . "='" . $value . "'";
		}
		if ($where) {
			return $where;
		} else {
			return false;
		}
	}
	
	/**
	 * Add the filter options to the where array
	 * 
	 * @param the current where statement
	 * @return the current were statement with the new where statement added to the end
	 */
	function getFilterWhere($where_array) {
		//print '<div>getFilterWhere</div>'; print_r($where_array);
		$filter_options = false;
		if (isset($this->filter_info) && $this->filter_info) {
			//$this->filter_info['item_field'] = 'Contact__item.acc_type_id';
			//print_r(($this->filter_info));
			$filter_value = 'f' . $this->filter_info['value'];
			if ($this->filter_info['type'] == 'sql') {
				$filter_options = $this->filter_info['options'];
			} else {
				$filter_options = $this->filter_info['options'];
				if (is_array($this->filter_info['options'])) { // Phil 20050722 - added if
					if (isset($this->filter_info['item_field']) && $this->filter_info['item_field']) {
						$item_table_field = $this->filter_info['item_field'];
					} else {
						$item_table_field = rtrim($this->filter_info['table'], '_dropdown') . '_id';
					}
					
						foreach ($this->filter_info['options'] as $k => $v) {
							//20071119 fix for new foreign key Person/Company filters
							if (strpos($item_table_field, '.') !== false) {
								$filter_options[$k] = array('where' => $item_table_field . ' = ' . $k);
							} else {
								$filter_options[$k] = array('where' => $this->getTableName() . '.' . $item_table_field . ' = ' . $k);
								//$filter_options[$k] = array('where' => $this->getTableName() . '.' . $item_table_field . ' = ' . $k);
							}
						}
					}
				// print_r($filter_options); print '<hr>';
			}
		} else {
			global $SERVICES;
			$filter_config = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'filter');
			if ($filter_config) {
				$filter_options = array();
				$filter_value = false;
				if (isset($filter_config[0]) && is_array($filter_config[0])) {
					if (sizeof($filter_config)) {
						$filter_options = $filter_config;
					} else {
						return $where_array;
					}
				} else if (isset($filter_config[0]) && strstr($filter_config[0], '__table(')) {
					$temp =  explode('.', str_replace(array('__table(', ')__'), '', $filter_config[0]) );
					if (is_array($temp) && count($temp) > 1) {
						if (!$temp[2]) {
							$temp[2] = false;
						}
						$dropdown = getWebMATObject(
						'WebMATDropdown', 
						$temp[0], // module
						array(
							'module' => $temp[0], 
							'table' => $temp[1], 
							'field_name' => $temp[2]
							)
						);
						$dropdown_values = $dropdown->getValues();
						$item_table_field = rtrim($temp[1], '_dropdown') . '_id';
						foreach ($dropdown_values as $k => $v) {
							$filter_options[$k] = array('where' => $this->getTableName() . '.' . $item_table_field . ' = ' . $k);
						}
						// print_r($filter_options); print '<hr>';
						unset($dropdown);
					} else {
						return $where_array;
					}
				}
			}
		}
		if ($filter_options) {
			//echo '*********GETTING*****'; print_r($filter_options); echo $filter_value;
			$where_array[] = $this->getFilter($filter_options, $this->module_name, $this->pref_name, false, $filter_value);
		}
		return $where_array;
	}
	
	/**
	 * sets the class variable used to disable the filter from the module.config
	 * 
	 * @param bool $disable_filter
	 */
	function disableFilter($disable_filter = true) {
		$this->disable_filter = $disable_filter;
	}
	
	/**
	 * @deprecated 20060119
	 * @see setConfigFilterInfo
	 */
	function setFilter($filter_info = false) {
		$this->setConfigFilterInfo();
	}
	
	/**
	 * sets config filter info, used in getFilterWhere
	 */
	function setConfigFilterInfo($filter_info = false) {
		if (is_array($filter_info)) {
			$this->filter_info = $filter_info;
		} else {
			$this->filter_info = false;
		}
	}
	
	/**
	 * creates a where string to add filtering
	 * 
	 * @param the current where statement
	 * @return the current were statement with the new where statement added to the end
	 */
	function getFilter($filter_options, $module_name = false, $pref_name = false, $where = '', $filter = false) {
		// Phil 20061129 Dont check if we are on a contact view page
		if (substr($pref_name, (strlen($pref_name) - strlen('contact_view'))) == 'contact_view') {
			return false;
		}
		
		if (!$filter) {
			global $SERVICES;
			$filter = $SERVICES->preferences->getUserPref($pref_name . 'Filter', 0);
		}
		if ($filter && $filter_options && $filter != 'none') {
			if (!trim($where) == '') {
				$where .= ' AND ';
			}
			if ($filter_options[ltrim($filter, 'f')]['where']) {
				$where .= $filter_options[ltrim($filter, 'f')]['where'] . ' ';
			} else {
				$where = rtrim($where, ' AND ');
			}
		}
		return $where;
	}
	//==========================================================================
	
	/**
	 * Soft deletes an item
	 * 
	 * @param integer $id the item id
	 */
	function deleteItem($id) {
		$id = $this->cleanId($id); // Phil 20070305
		global $SERVICES, $L;
		$this->setPrimaryRecordDelete($id); // SEC 20/1/05
		//soft delete - set active to 0
		$this->db->update($this->getTableName(), array('active' => '0'), "id='$id'"); 
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($this->log_action,'id=' . $id, $id); //Syed 20050824
		}
	}
	
	/**
	 * Counts the number of (soft) deleted items
	 * 
	 * @return mixed Either an integer (number of items), or false
	 * @author Phil Calder
	 */
	function countDeleted() {
		$result = false;
		if ($this->active) { // see setActive
			$result = $this->db->quickGet(
				'count(*)', 
				$this->getTableName(), 
				'active=0'
			); 
			
			if ($this->db->hasError()) {
				$this->db->printError();
			}
		}
		return $result;
	}
	
	function getSoftDeletedItemIds() {
		$item_ids = false;
		// For security, only Admin module Administrators can hard delete items
		global $USER;
		$this->setUserAuthenticationModule();
		if (!$this->ua->adminAccess($USER['user_id'], 'Admin')) {
			return false;
		}
		$id_field = 'id';
		if (in_array($id_field, $this->columns)) {
			if ($this->active) { // see setActive
				if ($result = $this->db->select(
					$id_field, 
					$this->getTableName(), 
					'active=0'
				)) {
					$item_ids = array();
					while ($row = $this->db->getRow($result)) {
						$item_ids[] = $row[$id_field];
					}
				}
			}
		}
		return $item_ids;
	}
	
	/**
	 * hard deletes an item
	 * 
	 * @param integer $id the item id
	 * @return boolean false if unsuccessful, true is successfully deleted
	 * @author Phil Calder
	 */
	function hardDeleteItem($id) {
		$id = $this->cleanId($id); // Phil 20070305
		$result = false;
		// For security, only Admin module Administrators can hard delete items
		global $USER;
		$this->setUserAuthenticationModule();
		if (!$this->ua->adminAccess($USER['user_id'], 'Admin')) {
			return false;
		}
		
		if ($id && $this->isDeleted($id)) { // Must be soft-deleted to begin with
			// $this->printDebug('item ' . $id . ' isDeleted (soft delete)');
			$result = $this->db->delete(
				$this->getTableName(), 
				"id='$id'", 
				'LIMIT 1'
			);
			if ($this->db->hasError()) {
				$this->db->printError();
			} else {
				if ($result !== false) {
					$result = $this->db->affectedRows($result);
				}
			}
		}
		return $result;
	}
	
	/**
	 * hard deletes all soft deleted items
	 * 
	 * @return boolean false if unsuccessful, true if successfully deleted
	 * @author Phil Calder
	 */
	function hardDeleteItems() {
		$result = false;
		// For security, only Admin module Administrators can hard delete items
		global $USER;
		$this->setUserAuthenticationModule();
		if (!$this->ua->adminAccess($USER['user_id'], 'Admin')) {
			return $result;
		}
		// print '<div>' . get_class($this) . '</div>';
		if (is_array($soft_deleted_items = $this->getSoftDeletedItemIds()) && count($soft_deleted_items)) { // Must be some soft-deleted items to begin with
			// $this->printDebug('hardDeleteItems: There are  ' . count($soft_deleted_items) . ' soft deleted items to purge');
			// print '<div>hardDeleteItems: There are  ' . count($soft_deleted_items) . ' soft deleted items to purge</div>';
			$result = $this->db->delete(
				$this->getTableName(), 
				"active=0"
			);
			if ($this->db->hasError()) {
				$this->db->printError();
			} else {
				if ($result !== false) {
					$result = $this->db->affectedRows($result);
					// print '<div>affected rows: ' . $result . '</div>'; print_r($result);
				}
			}
		} else {
			// $this->printDebug('No soft deleted items - cannot hard deleted');
			// print '<div>no items to delete</div>';
			return true;
		}
		return $result;
	}
	
	/**
	 * Checks and fixes any primary field issues
	 * 
	 * @param array $item the item array to be set/updated
	 * @author Simon Cook
	 */
	function checkPrimaryRecord(&$item) {
		global $USER;
		$primary_field = $this->getPrimaryFieldName();
		if (in_array($primary_field, $this->columns) && $item['contact_id']) {
			// not using getPrimaryRecord function because something may have broken and we might have more than 1 primary
			// this will fix that if it somehow happened.
			$items = $this->getItems(false, false, $this->getTableName() . '.contact_id = ' . $item['contact_id'] . ' AND ' . $this->getTableName() . '.' . $primary_field . ' = 1');
			
			// if we are updating an item (not making a new one)
			if (isset($item['id'])) {
				if (!isset($item['contact_id'])) {
					$edit_item = $this->getItem($item['id']);
					$item['contact_id'] = $edit_item['contact_id'];
				}
				//$this->setDesignTimeFilters(array('contact_id' => $item['contact_id'], $primary_field => 1));			
				
				if (is_array($items)) {
					foreach ($items as $k => $v) {
						if ($v['id'] == $item['id']) {
							// the item we are editing is the current primary
							$item[$primary_field] = 1;
							break;
						}
					}
				}
			}
			if ( (count($items) == 0) && ($item[$primary_field] == 0) ) {
				// currently no primary, set one
				return true; //$this->setPrimaryRecord($item['id'], $item['contact_id']);
			} else if ($item[$primary_field] == 1) {
				// setting this record to the primary
				$this->unsetPrimaryRecords($item['contact_id']);
				return false;
			} else if (count($items) > 1) {
				// more than 1 primary, unset all, reset 1				
				return true; //$this->setPrimaryRecord($item['id'], $item['contact_id']);
			} else {
				return false;
			}
		}
	}
	
	/**
	 * Makes all primary fields 0
	 * 
	 * @param $contact_id the contact id to unset all primarys
	 * @author Simon Cook
	 */
	function unsetPrimaryRecords($contact_id) {
		if ($contact_id) {
			$primary_field = $this->getPrimaryFieldName();
			// $this->setDesignTimeFilters(array('contact_id' => $contact_id, $primary_field => 1));
			// $result = $this->db->update($this->getTableName(), array($primary_field => 0), 'contact_id=' . $contact_id);
			$this->setDesignTimeFilters(array($this->getTableName() . '.contact_id' => $contact_id, $this->getTableName() . '.' . $primary_field => 1));
			$result = $this->db->update($this->getTableName(), array($primary_field => 0), $this->getTableName() . '.contact_id=' . $contact_id);
		}
	}
	
	function getSecondaryRecords($contact_id, $field, $limit_count = 5) {
		$primary_field = $this->getPrimaryFieldName();
		$where = $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $contact_id . '';
		$where .= ' AND ' . $this->getTableName() . '.' . $primary_field . ' <> 1';
		$where .= ' AND active=1';
		$limit_count = intVal($limit_count);
		$limit = 'LIMIT 0, ' . $limit_count . '';
		// $order = array('field' => 'priority', 'order' => 'ASC');
		$order = 'priority ASC';
		return $this->db->quickGetAllRows($field, $this->getTableName(), $where, $order, '', $limit);
	}
	
	function getSecondaryRecord($contact_id, $field, $offset) {
		$primary_field = $this->getPrimaryFieldName();
		$where = $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $contact_id . '';
		$where .= ' AND ' . $this->getTableName() . '.' . $primary_field . ' <> 1';
		$where .= ' AND active=1';
		$offset = intVal($offset);
		$limit = 'LIMIT ' . $offset . ', 1';
		// $order = array('field' => 'priority', 'order' => 'ASC');
		$order = 'priority ASC';
		return $this->db->quickGet($field, $this->getTableName(), $where, $order, '', $limit);
	}
	
	/**
	 * Returns the primary record (if one exists) for this contact
	 * 
	 * @param $contact_id the contact id
	 * @param boolean $one Whether to limit to one result (default true)
	 * @return $item the item array for the primary record
	 * @author Simon Cook, Phil Calder
	 */
	function getPrimaryRecord($contact_id, $one = true) {
		if ($contact_id) {
			$primary_field = $this->getPrimaryFieldName();
			// if this module has a primary record
			if (in_array($primary_field, $this->columns)) {
				$where = $this->getTableName() . '.' . $this->getContactIdField() . ' = ' . $contact_id . '';
				$where .= ' AND ' . $this->getTableName() . '.' . $primary_field . ' = 1';
				$order = false;
				if ($one) {
					$this->resetDesignTimeFilters();
					$limit = 'LIMIT 1';
					
					$items = $this->getItems($order, $limit, $where);
					if (is_array($items) && count($items)) {
						$item = current($items);
					}
				} else {
					$limit = false;
					$item = $this->getItems($order, $limit, $where);
				}
				// print '<div>getPrimaryRecord(' . $contact_id . ') returning item:</div>'; print_r($item); print '<br>';
				return $item;
			}
		}
		return array();
	}
	
	/**
	 * Sets an arbitrary record to be the primary
	 * 
	 * @param $contact_id the contact id to set a primary record for
	 * @author Simon Cook
	 */
	function setPrimaryRecord($contact_id) {
		$debug = false;
		$primary_field = $this->getPrimaryFieldName();
		if (in_array($primary_field, $this->columns)) {	
			if ($debug) { print '<div>primary_field: ' . $primary_field . ' contact_id: ' . $contact_id . '</div>'; }
			$this->resetDesignTimeFilters(); // Phil - must reset this (may not be the first query run)
			$this->setDesignTimeFilters(array('contact_id' => $contact_id));
			// no items therefore can be no primary
			$count = $this->countItems();
			if ($debug) { print '<div>count:</div>'; print_r($count); }
			if (!$count) {
				if ($debug) { print '<div>no items, returning</div>'; }
				return;
			}
			$this->unsetPrimaryRecords($contact_id);
			$result = $this->db->update($this->getTableName(), array($primary_field => 1), $this->getTableName() . '.contact_id=' . $contact_id . ' AND ' . $this->getTableName() . '.active=1', 'LIMIT 1');
			if ($debug) { print '<div>result: ' . ($result ? 'true' : 'false') . ' ' . $result . '</div>'; }
		}
	}
	
	/**
	 * Checks if the item being deleted is the primary and sets another
	 * 
	 * @param $id the id of the item being deleted
	 * @author Simon Cook
	 */
	function setPrimaryRecordDelete($id) {
		$primary_field = $this->getPrimaryFieldName();
		if (in_array($primary_field, $this->columns)) {
			// print '<div>setPrimaryRecordDelete</div>';
			$item = $this->getItem($id);
			if ($item[$primary_field] == 0) {
				return;
			}
			$contact_id = $item['contact_id'];			
			if ($contact_id) {
				$this->setDesignTimeFilters(array('contact_id' => $contact_id));
				// no items therefore can be no primary
				$count = $this->countItems();
				if ((!$count) || ($count == 1)) {
					return;
				}
				$this->unsetPrimaryRecords($contact_id);
				
				// $result = $this->db->update($this->getTableName(), array($primary_field => 1), 'contact_id=' . $contact_id . ' AND active=1 AND id <>' . $id, 'LIMIT 1');
				$result = $this->db->update($this->getTableName(), array($primary_field => 1), $this->getTableName() . '.contact_id=' . $contact_id . ' AND ' . $this->getTableName() . '.active=1 AND ' . $this->getTableName() . '.id <>' . $id, 'LIMIT 1');
			}
		}
	}
	
	/**
	 * Returns the field in the database relating to primary records
	 *
	 * @return string The primary record field name (default: 'primary_record')
	 * @author Phil Calder
	 */
	function getPrimaryFieldName() {
		return 'primary_record';
	}
	
	/**
	 * Creates/Updates an item in the DB
	 * Depending on if an 'id' already exists or not. (then appropriate function called)
	 * 
	 * @param array $item the item as an array
	 */
	function setItem($item, $set_seen = true) {
		global $SERVICES, $USER, $L;
		$primary = $this->checkPrimaryRecord($item); // SEC 20/1/05

		if ($item['id']) {
			//update current record
			$id = $this->updateItem($item);
			// if a non-numeric is returned, it's because there was an error
			// so we have to jump out of this as well.
			if (!is_numeric($id)) {
				return $id;
			}
		} else {
			// insert new record
			$id = $this->insertItem($item, $set_seen);
		}
		
		// check if we need to set a primary record
		if ($primary) {
			$this->setPrimaryRecord($item['contact_id']);
		}
		
		// Phil 20050831 - Update contact if this item is from a locked-to-contact module
		$this->updateLockedContact($item['contact_id'], $item['created'], $item['last_update']);
		
		return $id;
	}
	
	/**
	 * Updates an item in the DB
	 * Split off from setItem ~ BB 20050915
	 * 
	 * @param array $item the item as an array
	 * @return int $id id of the updated item, or string if error.
	 */
	function updateItem(& $item) {
		$id = $item['id'];
		$last_update = '';
		$last_update_str = '';
		// print '<div>last_update: ' . $item['last_update'] . ', time: ' . time() . '</div>'; print_r($item);
		if ( array_key_exists('last_update', $item) ) {
			$last_update = $item['last_update'];
			$item['last_update'] = time();
			$last_update_str = ' AND last_update=' . $last_update;
		}
		// print '<div>last_update is now: ' . $item['last_update'] . '</div>';
		
		// Phil - added this to prevent items being set as 'alerts' in some cases
		$item = $this->setAlertFields($item);
		
		$item = $this->setPermissionsFields($item);
		
		$result = $this->db->update($this->getTableName(), $item, "id='$id' AND active=1" . $last_update_str);
		// i don't think this is doing what it's intended to. ~BB
		if ($this->db->affectedRows() == 0) {
			// print '<div>updateItem</div>';
			$row = $this->getItem($id, $active=1);
			
			if ($this->db->hasError()) {
				$this->db->printError();
				// should have another return here, eg return 'Error'
			}
			
			if ($row['active'] == 0) {
				return 'Deleted';
			}
			
			if ( (array_key_exists('last_update', $item)) && ($row['last_update'] != $last_update) ) {
				return 'Updated';
			}
			
		} else {
			//$this->setLog($L['logUpdate'], 'id=' . $id);
			$this->setLog($this->log_action,'id=' . $id, $id); //Syed 20050824
			$this->setItemSeenStatus($id, 2);
		}

		return $id;
	}


	/**
	 * Inserts an item in the DB
	 * Split off from setItem ~ BB 20050915
	 * 
	 * @param array $item the item as an array
	 * @return int $id item of the new item
	 */
	function insertItem(& $item, $set_seen = true) {
		global $USER; // Phil 20051110 This needs to be here
		$id = 0;

		// print_r($item); print '<hr>';
		//set any default values here
		$item = $this->setPermissionsFields($item);
		
		if (in_array('created', $this->columns) && !$item['created']) {
			$item['created'] = time();
		}
		if ( in_array('user_id', $this->columns) ) {
			if (!$item['user_id']) {
				$item['user_id'] = $USER['user_id'];
			}
		}
		if ( in_array('creator_id', $this->columns) ) {
			if (!$item['creator_id']) {
				$item['creator_id'] = $USER['user_id'];
			}
		}
		
		if (in_array('active', $this->columns) && !$item['active']) {
			$item['active'] = 1;
		}
		
		// Phil - added this to prevent items being set as 'alerts' in some cases
		$item = $this->setAlertFields($item);
		
		foreach ($item as $k => $v) { // Phil 20050509 - for multi_select
			if (is_array($v)) {
				$item[$k] = implode(',', $v);
			}
		}
		
		//insert new record
		$this->db->insert($this->getTableName(), $item);
		// print '<div>insertItem - table: ' . $this->getTableName() . ' item:</div>'; print_r($item); print '<hr>'; $this->db->printError(); // DEBUG
		$this->last_insert_id = $id = $this->db->getInsertId();
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			//$this->setLog($L['logCreate'], 'id=' . $id);
			$this->setLog($this->log_action,'id=' . $id, $id); //Syed 20050824
			if ($set_seen) {
				$this->setItemSeenStatus($id, 1);
			}
		}

		return $id;
	}


	/**
	 * Update the last modified field for the linked contact, if any
	 *
	 * Will use greater of created or last update date
	 *
	 * @param integer $created The created date (timestamp)
	 * @param integer $last_update The last update date (timestamp)
	 * @author Phil Calder
	 * @todo Phil - after version 4.122, extend to handle last_reach
	 */
	function updateLockedContact($contact_id, $created, $last_update) {
		if ($contact_id && ($created || $last_update)) { // item has a contact
			global $SERVICES;
			$SERVICES->getService('Modules');
			if (is_object($contact =& $SERVICES->modules->getModule('Contact'))) {
				$timestamp = ($last_update > $created ? $last_update : $created); // last update can be either the created (new item) or last update field
				if ($this->getDisplayByGroupFlag() == 2) { // item is locked to contact
					$contact->updateLastUpdate($contact_id, $timestamp);
				} else {
					$contact->updateLastReach($contact_id, $timestamp);
				}
			}
		}
	}
	
	/**
	 * Sets permissions fields for contact related items
	 *
	 * @param array $item The current item
	 * @param array The current item with permissions fields set
	 * @author Phil Calder
	 */
	function setPermissionsFields($item) {
		global $SERVICES;
		
		if (!is_array($item)) {
			$item = array();	
		}
		// set default permissions fields
		$permissions_flag = $this->getDisplayByGroupFlag();
		$contact_id_field = $this->getContactIdField();
		if ($permissions_flag && $contact_id_field && $item[$contact_id_field]) {
			if ($SERVICES->modules->exists('Contact')) {
				if ($this->module_name != 'Contact') {
					$contact_lock_permissions = false;
					if ($item[$contact_id_field]) {
						$contact_mod =& $SERVICES->modules->getModule('Contact');
						$contact_db_functions =& $contact_mod->getDatabaseTable('item');
						// print '<div>setPermissionsFields</div>';
						$contact_item = $contact_db_functions->getItem($item[$contact_id_field]);
						$contact_lock_permissions = $contact_item['lock_permissions'];
					}
					if (
						( ($permissions_flag == 2) && $item[$contact_id_field] && $contact_item) 
						|| $contact_lock_permissions
					) {
						if (in_array('view_group_id', $this->columns)) {
							$item['view_group_id'] = $contact_item['view_group_id'];
						}
						if (in_array('edit_group_id', $this->columns)) {
							$item['edit_group_id'] = $contact_item['edit_group_id'];
						}
						if (in_array('owner_group_id', $this->columns)) {
							$item['owner_group_id'] = $contact_item['owner_group_id'];
						}
						if (in_array('admin_group_id', $this->columns)) {
							$item['admin_group_id'] = $contact_item['admin_group_id'];
						}
						
					}
					
				}
			}
		}	
		return $item;
	}
	
	/**
	 * Replaces certain placeholders in given string with user variables and time calculations.
	 *
	 * @param string $str The string to replace values in
	 * @param boolean $extended. Optional, default false. When true, will also replace additional placeholders (those not surrounded by percentage signs)
	 * @author Phil Calder
	 */
	function replacePlaceholders($str, $extended = false) {
		if (strlen($str)) {
			global $SERVICES;
			$groups =& $SERVICES->getService('Groups');
			$administrators_group_id = $groups->getAdministratorsGroupId();
			$everyone_group_id = $groups->getEveryoneGroupId();
			global $USER;
			$search = array('%user_id%', '%groups%', '%private_group_id%', '%offset_time%', '%current_date%');
			$time = time();
			$replace = array($USER['user_id'], $USER['group_string'], $USER['private_group_id'], $time, $time); 	
			$str = str_replace($search, $replace, $str);
			
			if ($extended === true) {
				$search = array('user_id', 'groups', 'private_group_id', 'offset_time', 'current_date', 'administrators_group_id', 'everyone_group_id');
				$replace = array($USER['user_id'], $USER['group_string'], $USER['private_group_id'], $time, $time, $administrators_group_id, $everyone_group_id); 	
				$str = str_replace($search, $replace, $str);
			}
		}
		return $str;
	}
	
	/**
	 * Calculates alert where from config settings.
	 *
	 * Phil: 
	 * This value is cached. 
	 * Bookmarks not used for alerts. 
	 * Handles seen table.
	 * Replaces placeholders. {@link replacePlaceholders}
	 * @return string the alert where statement
	 */
	function getAlertWhere() {
		
		if (!$this->alert_where_processed) {
			global $SERVICES, $USER;
			
			$this->alert_where_processed = array();
			
			$alert_array = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'alert');
			
			if (is_array($alert_array)) { 
				$has_seen = $this->getHasSeenTable(); // Phil 20051007
				$where = array();
				foreach($alert_array as $k => $v) {
					if (in_array($k, $this->columns) || ($has_seen && $k == 'seen')) {
						$where_piece = $this->processDesignTimeFilter($k, $v);
						if ($has_seen && $k == 'seen') {
							$where_piece = str_replace($this->getTableName() . '.', $this->getTableName() . '_seen.', $where_piece);
							$where_piece = '(' . $where_piece . ' OR ' . str_replace('=\'0\'', ' IS NULL', $where_piece) .')';
						}
						$where[] = $where_piece;
					}
				}
			}
			
			if (!is_array($where)) {
				$where = explode(' AND ', $where);
			}
			
			//Might be able to be removed
			foreach ($where as $k => $v) {
				if ($v && (substr($v, 0, strlen($this->getTableName())) != $this->getTableName()) && (substr($v, 0, 1) != '(') ) {
					$where[$k] = $this->getTableName() . '.' . trim($v);
				}
			}
			
			$where = implode (' AND ', $where);
			
			$this->alert_where_processed = $this->replacePlaceholders($where); // Phil 20051007
		}
		
		return $this->alert_where_processed;
		
	}
	
	/**
	 * Returns the 'mine' where statement (for 'My Items')
	 * @return string The where statement for 'My Items'
	 * @author Phil Calder
	 */
	function getMineWhere() {
		
		if (!$this->mine_where_processed) {
			global $SERVICES;
			
			$this->mine_where_processed = array();
			
			// $has_seen = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'seen_table');
			$has_seen = $this->getHasSeenTable();
			$mine_array = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'mine');
			
			if (is_array($mine_array)) {
				$mine = array();
				foreach($mine_array as $k => $v) {
					if (in_array($k, $this->columns) || ($has_seen && $k == 'seen')) {
						$mine_piece = $this->processDesignTimeFilter($k, $v);
						if ($has_seen && $k == 'seen') {
							$mine_piece = str_replace($this->getTableName() . '.', $this->getTableName() . '_seen.', $mine_piece);
							$mine_piece = '(' . $mine_piece . ' OR ' . str_replace('=\'0\'', ' IS NULL', $mine_piece) .')';
						}
						$mine[] = $mine_piece;
					}
				}
			}
			if (!is_array($mine)) {
				$mine = explode(' AND ', $mine);
			}
			
			//Might be able to be removed
			foreach ($mine as $k => $v) {
				if ($v && (substr($v, 0, strlen($this->getTableName())) != $this->getTableName()) && (substr($v, 0, 1) != '(') ) {
					$mine[$k] = $this->getTableName() . '.' . trim($v);
				}
			}
			
			$mine = implode (' AND ', $mine);
			
			$this->mine_where_processed = $this->replacePlaceholders($mine); // Phil 20051007
		}
		
		return $this->mine_where_processed;
		
		
	}
	
	/**
	 * Calculates and returns the unseen where string
	 *
	 * Only if the current file set contains a seen table.
	 * Return value is cached
	 *
	 * @return string The 'unseen' where string
	 * @author Phil Calder
	 * @see getUnseenWhereString
	 */
	function getUnseenWhere() {
		if (!$this->unseen_where_processed) {
			$this->unseen_where_processed = array();
			$has_seen = $this->getHasSeenTable();
			$where = '';
			
			if ($has_seen) {
				$where = $this->getUnseenWhereString(); // Phil 20051007
			}
			$this->unseen_where_processed = $where;
		}
		return $this->unseen_where_processed;
	}
	
	/**
	 * Returns the sql string (where clause) required for this {@link getUnseenWhere} function
	 *
	 * The is usually the only part of getUnseenWhere which might need to be changed per module
	 *
	 * @see getUnseenWhere
	 * @author Phil Calder
	 */
	function getUnseenWhereString() {
		$where = $this->getTableName() . '.active=1 AND ( (' . $this->getTableName() . '_seen' . '.seen = 0) OR (' . $this->getTableName() . '_seen' . '.seen IS NULL) )';
		return $where;
	}
	
	/**
	 * Calculates and returns the modified where string
	 *
	 * Only if the current file set contains a seen table.
	 * Return value is cached
	 *
	 * @return string The 'modified' where string
	 * @author Phil Calder
	 * @see getModifiedWhereString
	 */
	function getModifiedWhere() {
		if (!$this->modified_where_processed) {
			$this->modified_where_processed = array();
			$has_seen = $this->getHasSeenTable();
			$where = '';
			
			if ($has_seen) {
				$where = $this->getModifiedWhereString();
			}
			$this->modified_where_processed = $where;
		}
		return $this->modified_where_processed;
	}
	
	/**
	 * Returns the sql string (where clause) required for this {@link getModifiedWhere} function
	 *
	 * The is usually the only part of getModifiedWhere which might need to be changed per module
	 *
	 * @see getModifiedWhere
	 * @author Phil Calder
	 */
	function getModifiedWhereString() {
		$where = $this->getTableName() . '.active=1 AND ( (' . $this->getTableName() . '_seen' . '.seen <> 1) OR (' . $this->getTableName() . '_seen' . '.seen IS NULL) )';
		return $where;
	}
	
	/**
	 * Sets the alert where statement
	 *
	 * @param string $alert_where The alert statement
	 * @param boolean $concat Whether to concatenate this with the existing alert where statement 
	 * (Optional, false by default)
	 * @deprecated since version 4
	 */
	function setAlertWhere($where, $concat = false) {
		if ($concat) {
			if (is_string($where) && strlen($where)) {
				if (is_array($this->alert_where)) {
					$this->alert_where[] = $where;
				} else {
					$this->alert_where .= (strlen(trim($this->alert_where)) ? ' AND ' : '') . $where;
				}
			}
		} else {
			$this->alert_where = $where;
		}
		unset($this->alert_where_processed);
	}

	/**
	 * Updates the given item to ensure it is correctly set as an alert (or not) when added to the database.
	 *
	 * @param array $item The item being updated
	 * @return array $item The item with any fields altered to make this an 'alert'
	 * eg: In many cases, altering the 'seen' key will make an item an alert
	 * @author Phil Calder
	 */
	function setAlertFields($item) {
		global $USER;
		if ($this->getAlertWhere() && is_array($item) && isset($item['seen']) && isset($item['user_id'])) {
			if ($USER['user_id'] == $item['user_id']) {
				// dont set an alert for an item the user has obviously already seen
				$item['seen'] = 1;
			} else {
				$item['seen'] = 0;
			}
		}
		return $item;
	}
	
	/**
	 * Reassigns items to a different contact
	 * 
	 * @param integer $contact_id the contact id you want to replace
	 * @param integer $new_contact_id the new contact id
	 */
	function reassignItemForContact($contact_id, $new_contact_id) {
		if ( is_array($this->columns) && in_array($this->getContactIdField(), $this->columns) ) {
			global $SERVICES;
			$this->db->update($this->getTableName(), array($this->getContactIdField() => $new_contact_id), $this->getContactIdField() . '=' . $contact_id);
		}
	}
	
	/**
	 * Deletes all the items for a contact id
	 * 
	 * @param integer $contact_id the contact id
	 * @deprecated version 4.130 use {@link contactCascadingDelete}
	 */
	function crmCascadingDelete($contact_id) {
		$this->contactCascadingDelete($contact_id);
	}
	
	/**
	 * Deletes all the items for a contact id
	 * 
	 * @param integer $contact_id the contact id
	 */
	function contactCascadingDelete($contact_id) {
		if ($contact_id && in_array($this->getContactIdField(), $this->columns)) {
			global $L;
			$where = $this->getTableName() . '.' . $this->getContactIdField() . '=' . $contact_id;
			$items = $this->getItems(false, false, $where);
			if ($items && is_array($items)) {
				$this->setLogAction('cascade_delete');
				foreach ($items as $k => $v) {
					$this->deleteItem($v['id']);
					//$this->setLog($L['logCascadingDelete'], 'Contact=' . $contact_id);
					$this->setLog($this->log_action, 'Contact=' . $contact_id); //Syed 20050824			
				}
			}
		}
	}
	
	/**
	 * Adds a new log event
	 *
	 * @param string $option the event log type
	 * @param string $msg the message for the log event
	 * modified by syed
	 */
	function setLog($option ='', $msg='', $id='', $include_table_info = true) {
		if (empty($option)) { // Phil 20060117
			$option = $this->log_action;
		}
	
		global $SERVICES,$L;
		$SERVICES->getService('Log');
		switch ($option) {
			case 'create':
				$title = $L['logCreate'];
				$action = 1;
				break;
			case 'list':
				$title = $L['logList'];
				$action = 2;
				break;
			case 'view':
				$title = $L['logView'];
				$action = 3;
				break;
			case 'edit':
				$title = $L['logEdit'];
				$action = 4;
				break;
			case 'update':
				$title = $L['logUpdate'];
				$action = 5;
				break;
			
			case 'delete':
				$title = $L['logDelete'];
				$action = 6;
				break;
			case 'cascade_delete':
				$title = $L['logCascadingDelete'];
				$action = 7;
				break;
			case 'mass_update':
				$title = $L['logMassUpdate'];
				$action =  8;
				break;
			case 'multi_update':
				$title = $L['logMultiUpdate'];
				$action =  9;
				break;
			case 'mass_delete':
            	$title = $L['logMassDelete'];
				break;
			case 'selector':
				$title = $L['logSelector'];
				$action =  11;
				break;
			
		}
		
		$msg = $title . ': ' . $msg;
		$table =str_replace($this->module_name . '__', '', $this->getTableName());
		if ($include_table_info) {
			$msg .= ' [' . $table . ']';
		}
		$SERVICES->log->logAction($msg,$this->module_name,$table,$action,$id);
	}
	
	/**
	 * Performs non-database checking to see if an the current user has admin rights to the item
	 * 
	 * To enable/disable this functionality see module.config
	 * 
	 * @param integer $id the item id
	 * @param mixed $item the optional item array (including permission fields if they exist)
	 * @return bool false if the user is not in the admin group, true if they are - based on the information we have
	 * @author Phil Calder
	 */
	function isAdmin( $id, $item = false ) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER, $SERVICES;
		
		if (isset($this->cache[$this->module_name][$this->getTableName()][$id]['admin']) 
		&& $this->cache[$this->module_name][$this->getTableName()][$id]['admin']) {
			return $this->cache[$this->module_name][$this->getTableName()][$id]['admin'];
		}
		
		$this->setUserAuthenticationModule();
		if (!$id || $this->getDisplayByGroupFlag() == 0 ) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = true);
		}
		if (!$this->ua->writeAccess($USER['user_id'], $this->module_name)) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = false);
		}
		
		// ----------
		if (is_array($item)) {
			
			if (isset($item['active']) && in_array('active', $this->columns)) {
				if ($item['active']) {
					$status = true;
				} else {
					return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = false);
				}
			} 
			
			
			$perms_check = false; // Do we need to check permissions
			$perms_status = false; // Did the permissions check pass
			
			if (isset($item['admin_group_id']) && in_array('admin_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['admin_group_id']) && in_array($item['admin_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if ($perms_check && !$perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = false);
			}
			
			if ($perms_check && $perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = true);
			}
		
		}
		
		if ( in_array('admin_group_id', $this->columns) ) {
			$check = $this->db->quickGet(
				'id',
				$this->getTableName(),
				"id = '$id' AND admin_group_id IN (" . $USER['group_string'] . ")" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = false);
			} else {
				if ($check != $id) {
					
					return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = false);
				}
			}
		}
		
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['admin'] = true);		
	}
	
	/**
	 * Performs non-database checking to see if an the current user has owner rights to the item
	 * 
	 * To enable/disable this functionality see module.config
	 * 
	 * @param integer $id the item id
	 * @param mixed $item the optional item array (including permission fields if they exist)
	 * @return bool false if the user is not in the owners or admin group, true if they are - based on the information we have
	 * @author Phil Calder
	 */
	function isOwner( $id, $item = false ) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER, $SERVICES;
		
		if (isset($this->cache[$this->module_name][$this->getTableName()][$id]['owner']) 
		&& $this->cache[$this->module_name][$this->getTableName()][$id]['owner']) {
			
			return $this->cache[$this->module_name][$this->getTableName()][$id]['owner'];
			
		}
		
		$this->setUserAuthenticationModule();
		if (!$id || $this->getDisplayByGroupFlag() != 1 ) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = true);
		}
		if (!$this->ua->writeAccess($USER['user_id'], $this->module_name)) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = false);
		}
		
		// ----------
		if (is_array($item)) {
			
			if (isset($item['active']) && in_array('active', $this->columns)) {
				if ($item['active']) {
					$status = true;
				} else {
					return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = false);
				}
			}
			
			$perms_check = false; // Do we need to check permissions
			$perms_status = false; // Did the permissions check pass
			
			if (isset($item['owner_group_id']) && in_array('owner_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['owner_group_id']) && in_array($item['owner_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['admin_group_id']) && in_array('admin_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['admin_group_id']) && in_array($item['admin_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if ($perms_check && !$perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = false);
			}
			
			if ($perms_check && $perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = true);
			}
		
		}
		
		if ( in_array('owner_group_id', $this->columns) ) {
			$check = $this->db->quickGet(
				'id',
				$this->getTableName(),
				"id = $id 
				AND 
				(
					(
						owner_group_id IN (" . $USER['group_string'] . ") OR 
						admin_group_id IN (" . $USER['group_string'] . ")
					)
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = false);
			} else {
				if ($check != $id) {
					return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = false);
				}
			}
		}
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['owner'] = true);		
	}
	
	/**
	 * Performs non-database checking to see if an item is editable by the current user
	 * 
	 * To enable/disable this functionality see module.config
	 * 
	 * @param integer $id the item id
	 * @param mixed $item the optional item array (including permission fields if they exist)
	 * @return bool false if the item is not editable, true if it is editable - based on the information we have
	 * @author Phil Calder
	 */
	function isEditable( $id, $item = false ) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER, $SERVICES;
		
		if (isset($this->cache[$this->module_name][$this->getTableName()][$id]['editable']) 
			&& $this->cache[$this->module_name][$this->getTableName()][$id]['editable']
		) {
			return $this->cache[$this->module_name][$this->getTableName()][$id]['editable'];
		}
		
		$this->setUserAuthenticationModule();
		if (!$this->ua->writeAccess($USER['user_id'], $this->module_name)) {
			// print '<div>returning false</div>';
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
		}
		
		if (!$id || ($this->getDisplayByGroupFlag() < 1) ) { // No group permissions
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
		}
		
		
		if ( ($this->ua->isWebFormUser() && $this->ua->isWebFormUserAllowedToEditOwnContact()) && ($contact_id_field = $this->getContactIdField()) ) {
			
			if ( ($contact_id_field == 'id') && $this->ua->users->isClient($id) ) {	
				return true;
			} else if ($item[$contact_id_field] && $this->ua->users->isClient($item[$contact_id_field])) {
				return true;
			} else {
				if (!isset($item[$contact_id_field])) {
					$check_contact = $this->db->quickGet(
						$this->getContactIdField(),
						$this->getTableName(), 
						"id='$id'"
					);
					if ($check_contact && $this->ua->users->isClient($check_contact)) {
						return true;
					}
				}
			}
		}
		// ----------
		if (is_array($item)) {
			if (in_array('active', $this->columns)) {
				if (isset($item['active'])) {
					if ($item['active']) {
						$status = true;
					} else {
						// print '<div>not active</div>';
						return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
					}
				}
			} 
			
			$perms_check = false; // Do we need to check permissions
			$perms_status = false; // Did the permissions check pass
			
			if (isset($item['edit_group_id']) && in_array('edit_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['edit_group_id']) && in_array($item['edit_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			// Phil 20051005
			if (isset($item['assigned_to_group_id']) && in_array('assigned_to_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['assigned_to_group_id']) && in_array($item['assigned_to_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['owner_group_id']) && in_array('owner_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['owner_group_id']) && in_array($item['owner_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['admin_group_id']) && in_array('admin_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['admin_group_id']) && in_array($item['admin_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if ($perms_check && $perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
			}
		}
		
		$assigned_to_string = '';
		if ( in_array('assigned_to_group_id', $this->columns) ) {
			$assigned_to_string = "assigned_to_group_id IN (" . $USER['group_string'] . ") OR ";
		}
		
		if ( in_array('edit_group_id', $this->columns) ) {
			// print '<p>calling quickGet</p>'; print_r($item); print '<hr>';
			$check = $this->db->quickGet(
				'id',
				$this->getTableName(),
				"id = $id 
				AND 
				(
					(
						edit_group_id IN (" . $USER['group_string'] . ") OR  
						$assigned_to_string
						owner_group_id IN (" . $USER['group_string'] . ") OR 
						admin_group_id IN (" . $USER['group_string'] . ")
					)
				)" 
			);
			if ($this->db->hasError()) {
				$this->db->printError();
				
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
			} else {
				if ($check != $id) {
					
					return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = false);
				}
			}
		}
		
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['editable'] = true);
	}
	
	/**
	 * Performs non-database checking to see if an item is viewable
	 * 
	 * To enable/disable this functionality see module.config
	 * 
	 * @param integer $id the item id
	 * @param array $item Optional item array
	 * @param mixed $item the optional item array (including key group_id if it exists)
	 * @return bool
	 * @author Phil Calder
	 */
	function isViewable( $id, $item = false) {
		// print '<div>' . get_class($this) . '::isViewable(' . $id . ', ' . ($item === false ? 'false' : 'item'). ')</div>'; print_r($item); print_r($this->columns); print '<hr>';
		$id = $this->cleanId($id); // Phil 20070205
		
		global $USER, $SERVICES;
		if (
			isset($this->cache[$this->module_name][$this->getTableName()][$id]['viewable']) 
			&& $this->cache[$this->module_name][$this->getTableName()][$id]['viewable']
		) {
			return $this->cache[$this->module_name][$this->getTableName()][$id]['viewable'];
		}
		// print '<div>NOT CACHED ' . get_class($this) . '::isViewable(' . $id . ', ' . ($item === false ? 'false' : 'item'). ')</div><hr>';
		
		$this->setUserAuthenticationModule();
		if (!$this->ua->readAccess($USER['user_id'], $this->module_name)) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = false);
		}
		
		if (!$id || ($this->getDisplayByGroupFlag() < 1) ) {
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = true);
		}
		
		if ( ($this->ua->isWebFormUser() && $this->ua->isWebFormUserAllowedToViewOwnContact()) && ($contact_id_field = $this->getContactIdField()) ) {
			if ( ($contact_id_field == 'id') && $this->ua->users->isClient($id) ) {	
				return true;
			} else if ($item[$contact_id_field] && $this->ua->users->isClient($item[$contact_id_field])) {
				return true;
			} else {
				if (!isset($item[$contact_id_field])) {
					$check_contact = $this->db->quickGet(
						$this->getContactIdField(),
						$this->getTableName(), 
						"id='$id'"
					);
					
					if ($check_contact && $this->ua->users->isClient($check_contact)) {
						return true;
					}
				}
			}
		}
		
		// ----------
		if (is_array($item)) {
			if (in_array('active', $this->columns)) {
				if (isset($item['active'])) {
					if ($item['active']) {
						$status = true;
					} else {
						return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = false);
					}
				}
			} 
			
			$perms_check = false; // Do we need to check permissions
			$perms_status = false; // Did the permissions check pass
			
			if (isset($item['view_group_id']) && in_array('view_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['view_group_id']) && in_array($item['view_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['edit_group_id']) && in_array('edit_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['edit_group_id']) && in_array($item['edit_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			// Phil 20041005
			if (isset($item['assigned_to_group_id']) && in_array('assigned_to_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['assigned_to_group_id']) && in_array($item['assigned_to_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['owner_group_id']) && in_array('owner_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['owner_group_id']) && in_array($item['owner_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if (isset($item['admin_group_id']) && in_array('admin_group_id', $this->columns)) { 
				$perms_check = true;
				if (isset($item['admin_group_id']) && in_array($item['admin_group_id'], array_keys($USER['groups']))) {
					$perms_status = true;
				}
			}
			
			if ($perms_check && !$perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = false);
			}
			
			if ($perms_check && $perms_status) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = true);
			}
		
		}
		
		$where[] = "id = $id";
		
		$assigned_to_string = '';
		if ( in_array('assigned_to_group_id', $this->columns) ) {
			$assigned_to_string = "assigned_to_group_id IN (" . $USER['group_string'] . ") OR ";
		}
		
		if ( in_array('view_group_id', $this->columns) ) {
			$where[] = "
				(
					(	
						view_group_id IN (" . $USER['group_string'] . ") OR 
						edit_group_id IN (" . $USER['group_string'] . ") OR 
						$assigned_to_string
						owner_group_id IN (" . $USER['group_string'] . ") OR 
						admin_group_id IN (" . $USER['group_string'] . ")
					)
				) 
			";
			
		}
		
		if ( in_array('active', $this->columns) ) {
			$where[] = "active = 1";
		}
		
		$where = implode(' AND ', $where);
		
		$check = $this->db->quickGet(
			'id',
			$this->getTableName(), 
			$where
		);
		if ($this->db->hasError()) {
			$this->db->printError();
			return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = false);
		} else {
			if ($check != $id) {
				return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = false);
			}
		}
		return ($this->cache[$this->module_name][$this->getTableName()][$id]['viewable'] = true);
	}
	
	/**
	 * Performs non-database checking to see if an item is deletable by the current user. 
	 * Default functionality is to chack whether the item is editable. Override this function to change this.
	 * 
	 * @param integer $id the item id
	 * @param mixed $item the optional item array (including permission fields if they exist)
	 * @return bool false if the item is not deletable, true if it is deletable - based on the information we have
	 * @author Phil Calder
	 */
	function isDeletable( $id, $item = false ) {
		if ($this->getDisplayByGroupFlag() == 2) { // item is locked to contact
			if ($this->getContactRequiredFlag() || (isset($item['contact_id']) && $item['contact_id'])) {
				global $SERVICES;
				$SERVICES->getService('Preferences');
				if (! ($allow_delete = $SERVICES->preferences->getUserPref('contactAllowContactDelete'))) {
					return false;
				}
			}
		}
		
		return $this->isEditable($id, $item);
	}
	
	/**
	 * checks if an item is deleted
	 * 
	 * @param integer $id the item id
	 * @return  Returns TRUE if item is deleted, FALSE  otherwise.
	 */
	function isDeleted($id, $item = false) {
		$id = $this->cleanId($id); // Phil 20070305
		$soft_delete = in_array('active', $this->columns);
		
		if (is_array($item) && $soft_delete) {
			if (isset($item['active'])) {
				if ($item['active'] == 0) {
					return true;
				} else {
					return false;
				}
			}
		}
		
		$where = $this->getTableName() . '.id = "' . $id . '"';
		if ($soft_delete) {
			$where .= ' AND ' . $this->getTableName() . '.active = 0';
		}
		
		if ($item = $this->db->quickGet('*', $this->getTableName(), $where)) {
			if ($soft_delete) {
				return true;
			}
		} elseif (!$soft_delete) {
			return true;
		}
		return false;
	}
	
	/**
	 * Returns the int value of the specified id
	 *
	 * @return int $id
	 * @author Phil Calder
	 */
	function cleanId($id = false) { // Phil 20070205
		if ($id) {
			$id = (int)$id;	// Assumes id is an int
		}
		// print '<p>cleanId returning: ' . $id . '</p>';
		return $id;
	}
	
	/**
	 * Returns the UserAuthentication object
	 * 
	 * @return object the UserAuthentication object
	 */
	function setUserAuthenticationModule() {
		global $SERVICES;
		if (!is_object($this->ua)) {
			$this->ua =& $SERVICES->getService('Authentication');
		}
	}
	
	/**
	 * Calculates a calculated value
	 * 
	 * @param array $dd_item the items with dd structure
	 */
	function calculateValues(& $dd_item, $item) {
		//$dd_item['field_key']['value'] = $dd_item['field_key']['value'] + $dd_item['field_key']['value'];
	}
	
	/**
	 * @param string $current Optional, default 'contact_id'
	 * @param $foreign Optional, default 'contact_id'
	 * @author Phil Calder
	 */
	function setContactIdField($current = 'contact_id', $foreign = 'contact_id', $custom = 'custom_id') {
		$this->contact_id_field = array(
			'current' => $current, 
			'foreign' => $foreign, 
			'custom' => $custom
		);
	}
	
	/**
	 * @param string $type Optional, default 'current'
	 * @return string Contact id type {@link setContactIdField()}
	 * @author Phil Calder
	 */
	function getContactIdField($type = 'current') {
		switch ($type){
			case 'foreign':
				return $this->contact_id_field['foreign'];
			break;
			case 'custom':
				return $this->contact_id_field['custom'];
			break;
			default:
				return $this->contact_id_field['current'];
			break;
		}
	}
	
	/**
	 * @return string $table The name or alias of the Contact item table
	 */
	function getContactTable() {
		return '_contact_id';
	}
	
	/**
	 * Cascades permissions for the given contact
	 *
	 * Only if this is a contact-locked module, or the contact itself has locked permissions.
	 * 
	 * @param string $permissions 
	 * @param integer $contact_id 
	 * @author Phil Calder
	 */
	function cascadePermissions($permissions, $contact_id) {
		global $SERVICES;
		$config =& $SERVICES->getService('Config');
		
		// if (phpversion() >= 4.2) { // in_array can handle needle as an array
		$fields = array_keys($permissions);
		$fields[] = 'contact_id';
		// } else { // in_array does not handle needle as an array
		// 	$fields = array('contact_id');
		// }
		
		$permissions_type = $this->getDisplayByGroupFlag();
		
		$db =& $SERVICES->getService('Database');
		
		
		if (is_array($this->columns)) {
			foreach ($fields as $k => $v) {
				if (!in_array($v, $this->columns)) {
					bcrm_unset($permissions, $v);
				}
			}
		}
		
		if ($permissions_type 
			&& $contact_id 
			&& $this->getTableName() 
			&& $db->tableExists($this->getTableName())  
			&& is_array($this->columns) 
			&& (sizeof($permissions) && in_array('contact_id', $permissions))
			// && in_array($fields, $this->columns)
		) {
			$result = $this->db->update(
				$this->getTableName(), 
				$permissions, 
				'contact_id=' . $contact_id
			);
			
		}
	}
	
	/**
	 * returns the recent viewed pages details
	 * @author Syed
	 */
	function getLastViewedPages() {
		global $USER, $SERVICES;
		// $this->printDebug(get_class($this) . '::getLastViewedPages() start');
		$rows = array();
		// Bosco 20051129 | To set where according to user preference
		$where = '';
		$pref = $SERVICES->preferences->getUserPref('lastViewedScope');
		if ($pref == '3') {
			return array(); // Phil 20060917 ie none
		} else if ($pref == '2') {
			$where = 'module in (\'Contact\', \'Company\', \'Person\', \'Accounts\') AND ';
		}
		
		// Phil 20060224 - Provide a way to select for a given module
		if (isset($this->show_last_viewed_for_module) && $this->show_last_viewed_for_module) {
			$where = 'module = \'' . $this->show_last_viewed_for_module . '\' AND ';
		}
		
		$limit = $SERVICES->preferences->getUserPref('lastViewedListSize');
		if (!$limit) {
			$limit = 10;
		}
		
		//$where .= "(`action` in(1,2,3,5)) AND user_id ='" . $USER['user_id'] . "'AND active =1";
		//Phil, MCR 20060221
		// $where .= "(`action` in(3,5)) AND user_id ='" . $USER['user_id'] . "' AND active =1";
		// Phil 20060523 Only looking for viewed items (saves MySQL filesort)
		$where .= " `action`=3 AND `user_id` ='" . $USER['user_id'] . "' AND `active`=1";
		
		// Phil 20060523 altered query
		$rows = $this->db->quickGetAllRows(
			'id, item_id, module, page, `action`, table_name', // 'max( id ) AS db_id, item_id,module, page, `action`,table_name',
			'Logger__actions',
			$where , 
			'id DESC', // 'db_id DESC' , 
			// 'GROUP BY item_id, module, page, `action`',
			'',
			'Limit 0,' . ($limit * 3) // 'Limit 0,' . ($limit * 2) // Limit multiplied to handle duplicates
		);
		// $this->printDebug(get_class($this) . '::getLastViewedPages() got rows');
		$processed_rows = array();
		$check_array = array();
		$counter = 0;
		foreach ($rows as $k => $v) {
			
			// if (!array_search($v['module'] . $v['item_id'], $check_array)) {
			$key = $v['module'] . $v['item_id'];
			if (array_search($key, $check_array) === false) {
				$processed_rows[$k] = $v;
				//add uniqueness flag to check array 
				$check_array[] = $key;  
				$counter++;
				if ($counter >= $limit) {
					break; //continue; // Phil 20060523 changed to break
				}
			}
		}
		// $this->printDebug(get_class($this) . '::getLastViewedPages() returning processed rows');
		return $processed_rows;
	}
	
	function setLogAction($input){
		$this->log_action = $input;
	}
}
?>