<?php
/**
 *  This is the top level WebMAT display class
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * This is the top level WebMAT display class
 *
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardDisplay extends WebMAT {
	
	/**
	 * @var string the name of the container div which becomes the css class for the div
	 * @access public
	 */
	var $container_name = 'item-container';
	
	/**
	 * @var string the switch case for function getBeginMainContent()
	 * @access public
	 */
	var $begin_main_content_case = 'default';
	
	/**
	 * @var array used to overwrite the paging and sorting options
	 * @access private
	 */
	var $paging_overwrite; //normally array('limit' => '10' 'sort' => array('field' => 'name', 'order' => 'DESC') ) for desktop
	/**
	 * @var string the edit link name
	 * @access private
	 */
	var $edit_link;
	/**
	 * @var bookmark_filename
	 * @access private
	 */
	var $bookmark_filename = false;
	
	/**
	 * @var new flag to set style of link generated - string if false, array if true for ext toolbar
	 * @access private
	 */
	var $toolbar_link_mode = false;
	
	
	
	/**
	 * init function sets all the class variables
	 *
	 * Should only be called by the contstructor
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->setReturnPage();
		$this->setEditLinkVar();
		$this->output->req_main_js = true;
	}
	
	/**
	 * Sends content to an Output display class or returns as a string
	 * 
	 * @param mixed $content A string or array containing content to display
	 * @author Phil Calder
	 * @see getReturnString
	 */
	function display($content = '') {
		// print '<div>' . get_class($this) . '</div>';
		if ($this->output->getHasTemplate()) {
			$this->output->assignToTemplate($content);
			$this->output->display($this->getOutputDisplayParam());
			exit();
		} else if (!$this->getReturnString()) {
			$this->output->addToBody($content);
			$this->output->display($this->getOutputDisplayParam());
			exit();
		} else {
			if (is_array($content)) {
				$content = implode(' ', $content);
			}
			return $content;
		}
	}
	
	/**
	 * This function runs after the DataDictionary has been set up
	 */
	function afterDataDictionary() {
		//Runs after the Data Dictionary is created
	}
	
	/**
	 * Returns whether or not to pass content back to the caller as a string
	 *
	 * @return boolean $return_string 
	 * @author Phil Calder
	 */
	function getReturnString() {
		return ($this->return_string ? true : false);
	}


	function getBeginContainer() {
		return $this->output->getBeginContainer($this->container_name);
	}
	function getEndContainer() {
		return $this->output->getEndContainer();
	}
	
	/**
	 * @author Phil Calder
	 */
	function getBeginMainContent($add_size_limit = false) {
		
		//new setting from webmat_item for edit/views
		if ($this->begin_main_content_case == 'item') {
			return $this->output->getBeginTable('itemTopTbl', true);
		}
		
		//older mechanism
		if ($add_size_limit) {
			$this->output->addTagContent('mainTbl', 'class', ' normal');
		}
		$content = $this->output->getBeginTable('mainTbl', true);
		
		if ($add_size_limit) {
			$this->output->resetTagContent('mainTbl');
		}
		return $content;
	}
	
	function getBeginDataCell($padding = true) {
		return $this->output->getBeginDataCell($padding);
	}
	function getEndDataCell() {
		return $this->output->getEndDataCell();
	}
	
	/**
	 * @author Phil Calder
	 */
	function getEndMainContent() {
		return $this->output->getEndTable(true);
	}
	
	/**
	 * Used to redirect (eg: after data is added to the database)
	 *
	 * Redirects to the return page and exits
	 *
	 * @param integer $id Optional, only used for highlighting edited item on return page.
	 * @see getReturnPage()
	 * @author Phil Calder
	 */
	function redirectBack($id = '') {
		$return_page = $this->getReturnPage();
		// Phil 20060310 dont include highlight_id if it is an array (bulk/multi edit)
		if (!$this->item['id'] && !is_array($id)) { 
			
			if (strstr($return_page, "?")) {
				$return_page .= '&';
			} else {
				$return_page .= '?';
			}
			// Phil 20060623 rename previous highlight_id (TODO could remove previous entries)
			if (strpos($return_page, 'highlight_id=') !== false) {
				$return_page = str_replace('highlight_id=', 'prev_highlight_id=', $return_page);
			}
			$return_page .= 'highlight_id=' . $id;
			
		}
		header('Location: ' . $return_page);
		exit();
	}
	
	/**
	 * Returns the javascript to redirect back
	 * 
	 * @return string Back URL (javascript)
	 * @see getTitleRowLinks()
	 * @author Phil Calder
	 */
	function getBackUrl() {
		return 'javascript:history.go(-1)';
		// global $L;
		// $link = '';
		// if ($this->getReturnPage()) {
			// $link = $this->getReturnPage();
		// } else {
			// $link = $this->getListPageUrl();
		// }
		// return $link;	
	}
	
	// function getPrintViewUrl() {
		// return 'todo';
	// }
	
	/**
	 * returns the 'new' menu content
	 *
	 * @return string the 'new' menu content
	 * @author Phil Calder
	 */
	function getNewMenuContent() {
		if (is_array($this->new_menu_links)) {
			foreach ($this->new_menu_links as $k => $v) {
				if (!$v) {
					bcrm_unset($this->new_menu_links, $k);
				}
			}
			if (count($this->new_menu_links)) {
				return $this->output->getMenuRow( implode($this->getMenuSeparator(), $this->new_menu_links), 2, 'lNewMenuCell' );
			}
		}
		return false;
	}
	
	/**
	 * Returns the title row array
	 * @return array An array of links to display in the title row. Default keys are 'help', 'back', 'bookmark'
	 * @author Phil Calder
	 */
	function getTitleRowLinks() {
		$title_links = array(
			'help' => $this->getHelpLink(), 
			'back' => $this->getBackUrl(), 
			'print_view' => $this->getBackUrl(), 
			'bookmark' => $this->getAddBookmarkLink(), 
			'settings' => $this->getModuleName() // Phil 20060131
		);
		return $title_links;
	}
	
	
	
	/**
	 * sets the return page
	 * @param string $page The relative page (Optional, added 20051025, Phil)
	 */
	function setReturnPage($page = false) {
		// print '<div>setReturnPage(' . $page . ')</div>';
		if ($page) {
			$_SESSION['return_page'] = $page;
		} else {
			global $SERVICES;
			$SERVICES->getService('Config');
			if ( (strstr($_SESSION['return_page'], 'Front/menu.php')) || (strstr($_SESSION['return_page'], 'Front/refresh.php')) ) {
				$_SESSION['return_page'] = $SERVICES->config->lookup('modules.Front.modulelink.' . $this->module_name . '.link');
			}
			//if (!strpos($_SESSION['return_page'], 'modules')) {
			//	print '<div>WARNING - not a module page</div>';
			//}
		}
		//print '<div>session return_page: ' . $_SESSION['return_page'] . '</div>';
	}
	
	/**
	 * sets the return page
	 * @author Phil Calder
	 */
	function setRefererAsReturnPage() {
		if (isset($_SERVER["HTTP_REFERER"]) && $_SERVER["HTTP_REFERER"]) {
			$_SESSION['return_page'] = $_SERVER["HTTP_REFERER"];
		}
	}
	
	/**
	 * Sets the return string variable for display pages
	 *
	 * @param boolean $return_string 
	 * @author Phil Calder
	 */
	function setReturnString($return_string = false) {
		$this->return_string = ($return_string ? true : false);
	}
	
	/**
	 * returns the return page
	 * 
	 * @return string the url of the return page
	 */
	function getReturnPage() {
		// return $_SESSION['return_page'];
		
		// Phil 20070817 removed - need to ensure we dont load the index page (frameset) here
		//if (isset($_SESSION['return_page']) && $_SESSION['return_page']) {
		//	return $_SESSION['return_page'];
		//}
		
		return $list_page = $this->getListPageUrl();
	}
	
	/**
	 * Sets the edit link name
	 * 
	 * @param string the edit link name
	 */
	function setEditLinkVar($name = false) {
		if ($name && is_string($name)) {
			$this->edit_link = $name;
		} else {
			$this->edit_link = 'edit';
		}
	}
	
	/**
	 * Returns the new link for the current module
	 *
	 * @author Phil Calder
	 * @todo Extend to include file set specific new links for adding an item and a Contact on one form
	 */
	function getNewLinks() {
		// Modules can override this method to include more than one new link
		return array('new' => $this->getNewLink());
	}
	
	/**
	 * Generates a link to the edit page of the current module
	 *
	 * Substitutes the current contact id in the link query
	 *
	 * @return string A link to the edit page for a new item, or an empty string.
	 * @author Phil Calder
	 */
	function getNewLink() {
		if ($this->getFileSetHref($this->edit_link)) {
			global $L;
			$contact_id = $this->getContactId();
			if ($this->isDataOption('edit')) {
				$link_href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $contact_id ,$this->getFileSetHref($this->edit_link))) );
				return array(
					 'href' => $link_href,
					 'text' => $L['LinkNew'],
					 //'icon' => 'page_add.png',
					 'icon' => 'add.png',
					 'title' => $L['LinkNew']
				);
			}
		}
		return false;
	}
	
	/**
	 * gets the data option to use for browse queries
	 *
	 * @return array the array of items
	 * @author Phil Calder
	 */
	function getBrowseQueryDataOption() {
		// Phil 20060105 Moved from WebMATList
		return 'list';
	}
	
	
	function getPastLinksTable() {
		// $this->printDebug(get_class($this) . '::getPastLinksTable start');
		$content .= '<table' . $this->output->getTagContent('linksTbl') . '>';
		// LastViewed
		$content .= $this->getLastViewedContent();
		// $this->printDebug(get_class($this) . '::getPastLinksTable got last viewed content');
		// Bookmarks
		$content .= $this->getBookmarksContent();
		// $this->printDebug(get_class($this) . '::getPastLinksTable got bookmarks content');
		$content .= '</table>';
		// $this->printDebug(get_class($this) . '::getPastLinksTable return');
		return $content;
	}
	
	
	
	/**
	 * returns the last viewed row
	 *
	 * @param boolen $is_sidebar Links created and displayed differently if viewing recent items from the sidebar
	 * @author Syed, Phil Calder
	 */
	function getLastViewedContent($is_sidebar = false) {
		$this->is_sidebar = (boolean)$is_sidebar;
		$links = array();
		$links = $this->getLastViewedItems();
		if (!empty($links)) {
			if ($this->is_sidebar) {
				return $this->output->getLastViewedMenuList($links);
			} else {
				return $this->output->getLastViewedMenu($links,'2','lastViewedRow');
			}
		}
		return '';
	}
	
	//Links
	
	/**
	 * Gets the table links
	 * 
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		$id = $item['id'];
		bcrm_unset($item, 'id');
		
		// $links = bcrm_array_merge($this->getExtraItemLinks(), $this->getStandardLinks($id, $item));
		// Phil 20060221 Changed link order - standard links first
		$links = bcrm_array_merge($this->getStandardLinks($id, $item), $this->getExtraItemLinks());
		// print '<div>WebMATDisplay::getTableLinks ' . get_class($this) . ' table links</div>'; print_r($links); print '<hr>';
		$this->editTableLinks($links, $id, $item);
		
		return $this->replaceLinkVariables($id, $item, $links);
	}
	
	/**
	 * An empty function that can be used to edit links before they are processed
	 */
	function editTableLinks(&$links, $id, $item) {}
	
	/**
	 * returns an array of standard links
	 * 
	 * @param integer $id the current item id
	 * @param array the item array
	 * @return array the array of standard links
	 */
	function getStandardLinks($id, $item) {
		global $SERVICES;
		$links = array();
		if ($this->getFileSetHref('list')) {
			$links['list'] = $this->getListLink($id);
		}
		if ($this->getFileSetHref('view')) {
			$links['view'] = $this->getViewLink($id);
		}
		if ($this->getFileSetHref($this->edit_link)) {
			$links['edit'] = $this->getEditLink($id);
		}
		if (($this->getFileSetHref('hide') || $this->getFileSetHref('unhide')) && $this->getFileSetHref('view')) {
			if (is_array($item)) {
				if (array_key_exists('seen', $item)) {
					$links['hide_unhide'] = $this->getHideUnhideLink($id, $item['seen']);
				} else {
					$links['hide_unhide'] = $this->getHideUnhideLink($id);
				}
			} else {
				$links['hide_unhide'] = $this->getHideUnhideLink($id);
			}
		}
		if ($this->getFileSetHref('duplicate')) {
			$links['duplicate'] = $this->getDuplicateLink($id, $item);
		}
		if ($this->getFileSetHref($this->edit_link . '_delete')) {
			$links['delete'] = $this->getDeleteLink($id);
		}
		
		// Phil 20051007 - use wrapper function if it exists, otherwise use db_functions
		if (method_exists($this, 'isEditable')) {
			$is_editable = $this->isEditable( $id, $item );
		} else {
			$is_editable = $this->db_functions->isEditable( $id, $item );
		}
		
		// Phil 20051007 - use wrapper function is it exists, otherwise use db_functions
		if (method_exists($this, 'isDeletable')) {
			$is_deletable = $this->isDeletable( $id, $item );
		} else {
			$is_deletable = $this->db_functions->isDeletable( $id, $item );
		}
		
		// if (!$is_editable || !$is_deletable) {
			// foreach ($links as $type => $v) {
				// if ( ($type == 'edit') && !$is_editable) {
					// $links[$type] = '';
				// } else if ( ($type == 'delete') && !$is_deletable) {
					// $links[$type] = '';
				// }
			// }
		// }
		// Phil 20060117
		if (!$is_editable && array_key_exists('edit', $links)) {
			bcrm_unset($links, 'edit');
		}
		if (!$is_deletable && array_key_exists('delete', $links)) {
			bcrm_unset($links, 'delete');
		}
			
		return $links;
	}
	
	/**
	 * replaces the links variable place holders with the correct variable
	 * 
	 * @param integer $id the current item id
	 * @param array the item array
	 * @param array the array of links
	 * @return array the array of links with replaced values
	 */
	function replaceLinkVariables($id, $item, $links) {
		if ($item['contact_id']) {
			if (is_array($item['contact_id'])) {
			$contact_id = $item['contact_id']['id'];
			} else {
			$contact_id = $item['contact_id'];
			}
		} else {
			$contact_id = '';
		}	 
		
		bcrm_unset($item, 'contact_id');
		
		$replace_values = false;
		if (is_array($links)) {
			foreach ($links as $k => $v) {
				$v = str_replace(array('%id%', '%contact_id%'), array($id, $contact_id) , $v);
				if (strpos($v, '%') && is_array($item)) {
					if (!$replace_values) {
						$replace_values = array();
						foreach ($item as $kk => $vv) {
							if (is_array($vv)) {
								$replace_values['search'][] = '%' . $kk .'%';
								$replace_values['replace'][] = $vv['id'];
							} else {
								$replace_values['search'][] = '%' . $kk .'%';
								$replace_values['replace'][] = $vv;
							}
						}
					}
					$v = str_replace($replace_values['search'], $replace_values['replace'] , $v);
				}
				$links[$k] = $v;
			}
		}
		return $links;
	}
	
	//===================================
	
	/**
	 * Returns the charater used as a menu item separator
	 * @return string $menu_separator
	 * @author Phil calder
	 * @see setMenuSeparator()
	 */
	function getMenuSeparator() {
		if (empty($this->menu_separator)) {
			$this->setMenuSeparator();
		}
		return $this->menu_separator;
	}
	
	/**
	 * Sets the charater used as a menu item separator
	 * @param string $menu_separator An empty string will result in the default separator being set
	 * @author Phil calder
	 * @see getMenuSeparator()
	 */
	function setMenuSeparator($separator = '') {
		if (empty($separator)) {
			$separator = ' | ';	
		}
		$this->menu_separator = $separator;
	}
	
	/**
	 * returns the second menu row
	 * 
	 * @return string the second menu row
	 */
	function getExtraItemLinks() {
		
		$links = $this->getExtraConfigLinks();
		
		if (count($this->item_menu_links)) {
			foreach ($this->item_menu_links as $k => $v) {
				$links[] = $v;
			}
		}
		
		if (count($links)) {
			return $links;
		}
	}
	
	/**
	 * Adds the extra links from the module.config
	 *
	 * @param boolean $as_array Whether to return each link as a processed href (html a tag) or an array (eg for inclusion in a toolbar)
	 * 
	 * @return array An array of extra config links
	 */
	function getExtraConfigLinks($as_array = false) {
		if (!isset($this->extra_config_links)) { // Phil - need to check not set - may be set but not have any extra links
			global $SERVICES;
			$config =& $SERVICES->getService('Config');
			$language =& $SERVICES->getService('language');
			$table_name = str_replace($this->module_name . '__', '', $this->table_name);
			$extra_links = $config->lookup('modules.' .  $this->module_name . '.file_set.' . $table_name . '.extra_links');
			$links = array();
			
			if (is_array($extra_links)) {
				$module_path = $this->getModulePath();
				foreach ($extra_links as $k => $v) {
					if (!$href = $this->getFileSetHref($v)) {
						$href = $v;
					}
					// Phil 20050906 - Objects  can be part of combined view pages such as desktop and contact view
					// Need to link to correct module
					$href = $module_path . $href; 
					
					$text = $language->getLanguageText($k, $this->module_name);
					
					// Phil 20070507 - return in one of two formats
					// see WebMATItem - default $as_array = true
					if (!$as_array && !$this->toolbar_link_mode) {
						$links[$k] = $this->output->getLinkHref(array('text' => $text, 'href' => $href));
					} else {
						$links[$k] = array(
							'href' => $href,
							'text' => $text,
							//'icon' => 'page_misc.png',
							'title' => $text
						);
					}
					//todo?
				}
			}
			$this->extra_config_links = $links;
		}
		return $this->extra_config_links;
	}
	
	//===================================
	
	/**
	 * Builds the edit link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getEditLink($id = false) {
		global $L;
		// print '<div>' . get_class($this) . '::getEditLink piece: ' . $this->getModuleName() . ' edit_link: ' . $this->edit_link . '</div>';
		if ($link_href = $this->getFileSetHref($this->edit_link)) {
			if ($this->toolbar_link_mode) {
				return array(
				'href' => $link_href,
				'text' => $L['LinkEdit'],
				'icon' => 'page_edit.png',
				'title' => $L['LinkEdit']
				);
			} else {
				return $this->output->getLinkEdit($this->getModulePath() . $link_href);
			}
		}
		return false;
	}
	
	/**
	 * Builds the list link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getListLink($id = false) {
		global $L;
		$link_href = $this->getFileSetHref('list');
		$list_text = $this->output->getImageTag('img/list.gif', false, 'width=12 height=12') . $L['ListView'];
		//return $this->output->getLinkList($this->getModulePath() . $link_href, $list_text);
		if ($this->toolbar_link_mode) {
			return array(
			'href' => $link_href,
			'text' => $L['ListView'],
			'icon' => 'table.png',
			'title' => $L['ListView']
			);
		} else {
			return $this->output->getLinkList($this->getModulePath() . $link_href, $list_text);
		}

	}
	
	/**
	 * Builds the Delete link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getDeleteLink($id = false) {
		if ($link_href = $this->getFileSetHref($this->edit_link . '_delete')) {
			if ($this->toolbar_link_mode) {
				// Phil 20070507 Returns an array for use in a toolbar. Complete with 'handler1' key for confirm message.
				return $this->output->getLinkDelete($this->getModulePath() . $link_href, false, true);
			} else {
				return $this->output->getLinkDelete($this->getModulePath() . $link_href);
			}
		} 
		return false;
	}
	
	/**
	 * Builds the View link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getViewLink($id = false) {
		global $L;
		$link_href = $this->getFileSetHref('view');
		if ($link_href) {
			if ($this->form_values['contact_id']) {
				if (!strstr($link_href, '%contact_id%')) {
					$link_href .= '&contact_id=%contact_id%';
				}
			}
			//return $this->output->getLinkView($this->getModulePath() . $link_href);
			if ($this->toolbar_link_mode) {
				return array(
						 'href' => $link_href,
						 'text' => $L['LabelView'],
						 'icon' => 'page_edit.png',
						 'title' => $L['LabelView']
						 );
			} else {
				return $this->output->getLinkView($this->getModulePath() . $link_href);
			}
		}
		return '';
	}
	
	/**
	 * Builds the Hide or Unhide link, depending on the situation
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @param integer $seen_status Is the status of the item
	 * @return string containing the link
	 */
	function getHideUnhideLink($id = false, $seen_status = false) {
		
		// seen status not provided, need it.
		// (3 = because status could be 0, which is also false)
		if (($seen_status === false) && ($id > 0)){
			// print '<div>getHideUnhideLink</div>';
			$temp_item = $this->db_functions->getItem($id);
			if (is_array($temp_item)) {
				if (array_key_exists('seen', $temp_item)) {
					$seen_status = $temp_item['seen'];
				}
			}
			unset($temp_item);
		}
		
		// if it's already hidden, go for the unhide
		if ($seen_status == 3) {
			$link_href = $this->getFileSetHref('unhide');
			return $this->output->getLinkUnhide($link_href);
		} else {
			$link_href = $this->getFileSetHref('hide');
			return $this->output->getLinkHide($link_href);
		}
	}

	/**
	 * Builds the Duplicate link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getDuplicateLink($id = false) {
		global $L;
		$link_href = $this->getFileSetHref('duplicate');
		//return $this->output->getLinkHref(array('href' => $link_href, 'text' => $L['LinkDuplicate']));
		//return array('href' => $link_href, 'text' => $L['LinkDuplicate']);
		return array(
			'href' => $link_href,
			'text' => $L['LinkDuplicate'],
			'icon' => 'page_copy.png',
			'title' => $L['LinkDuplicate']
		);
	}
	//====
	
	
	/**
	 * Creates a bookmark session and returns the session name
	 * 
	 * @return string the session name
	 */
	 function createBookmarkSession() {
		global $SERVICES;
		$module_name = $this->getModuleName();
		// $php_self = $SERVICES->getGlobal('php_self');
		// $file = basename($php_self);
		$file = $this->getBookmarkFilename(); // Phil 20060530
		//$session_name = md5($file); // used different sessions for each page
		$session_name = 'bookmark'; // use the same session for all bookmarks
		$bookmark = array();
		$bookmark['module'] = $module_name;
		$bookmark['filename'] = $file;
		$bookmark['querystring'] = $_SERVER['QUERY_STRING'];
		$bookmark['page_type'] = $this->getWebmatType();
		if ($this->getTableName()) {
			$bookmark['table_name'] = $this->getTableName();
		}
		switch ($bookmark['page_type']) {
			case 'list':
				bcrm_unset($bookmark, 'querystring');
				
				$bookmark['name'] = $this->getTitleText();
				
				$bookmark['extra_values']['view_all'] = $this->form_values['view_all'];
				$bookmark['extra_values']['page_num'] = $this->form_values['page_num'];
				$bookmark['extra_values']['sort'] = $this->sortTable($this->form_values['sort']);
				if (is_object($this->filters)) {
					if ($bookmark['extra_values']['filters']['contact'] = $this->filters->getContactFilter()) {
						$bookmark['name'] .= $this->getContactNameForBookmark();
					}
					$bookmark['extra_values']['filters']['qfilter'] = $this->form_values['qfilter']; // Phil 20050824 - now stores qfilter in bookmark
					$bookmark['extra_values']['filters']['search_filter'] = $this->filters->getSearchFilter();
					$bookmark['extra_values']['filters']['filter'] = $this->filters->getFilter();
					$bookmark['extra_values']['filters']['filter_info'] = $this->filters->getConfigFilterInfo();
					if (count($this->filters->getIdsFilter())) {
						$bookmark['extra_values']['filters']['ids'] = $this->filters->getIdsFilter();
					} else {
						$bookmark['extra_values']['filters']['ids'] = 'off';
					}
					$bookmark['extra_values']['filters']['dtf'] = $this->filters->getDesignTimeFilter();
				}
			break;
			case 'edit':
				$bookmark['name'] = $this->getTitleText();
				if ($this->form_values['contact_id'] && !$this->form_values['id']) {
					$bookmark['name'] .= $this->getContactNameForBookmark();
				}
				if (!$this->form_values['id']) {
					global $L;
					$bookmark['name'] .= ' - ' . $L['LinkNew'];
				} else {
					// $bookmark['name'] .= $this->getDisplayNameForBookmark(); // Phil - not needed  -see WebMATItem::gettitleText
					$bookmark['querystring'] = 'id=' . $this->form_values['id'];
				}
			break;
			
			case 'view':
				$bookmark['name'] = $this->getTitleText(); // . $this->getDisplayNameForBookmark();  // Phil - not needed  -see WebMATItem::gettitleText
			break;
			
			case 'multi_edit':
				$bookmark['name'] = $this->getTitleText();
				if ($this->form_values['method'] = 'session') {
					if (is_array($_SESSION[$this->form_values['values']])) {
						$ids = implode(',', $_SESSION[$this->form_values['values']]);
					} else {
						$ids = $_SESSION[$this->form_values['values']];
					}
					$bookmark['querystring'] = 'method=ids&values=' . $ids;
				}
			break;
		}
		// Phil - This is one of the places where we require magic_quotes_gpc = On in php.ini
		if (isset($bookmark['extra_values'])) {
			$bookmark['extra_values'] = serialize($bookmark['extra_values']);
		}
		$_SESSION[$session_name] = $bookmark;
		return $session_name;
	}
	
	/**
	 * @author Phil Calder
	 */
	function setBookmarkFilename($filename = false) {
		if ($filename === false) {
			global $SERVICES;
			$php_self = $SERVICES->getGlobal('php_self');
			$this->bookmark_filename = basename($php_self);
		} else {
			$this->bookmark_filename = $filename;
		}
	}
	/**
	 * @author Phil Calder
	 */
	function getBookmarkFilename() {
		if (!isset($this->bookmark_filename) || !$this->bookmark_filename) {
			$this->setBookmarkFilename();
		}
		return $this->bookmark_filename;
	}
	
	/**
	*
	* @return an array containing the bookmark items for the current module
	* @author Syed, Phil Calder
	*/
	function getModuleBookmarkLinks() {
		global $SERVICES;
		if (is_object($system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks'))) {
			
			$links = $system_bookmarks_mod->getBookmarkLinks(false, false, $this->getModuleName());
			if (is_array($links)) {
				if (!empty($this->form_values['bookmark'])) { // Phil 20060116 Only loop through the links if there is a current bookmark
					foreach ($links as $k => $v) {
						if (!empty($v['id']) && ($v['id'] == $this->form_values['bookmark'])) {
							$links[$k]['selected']=1;
						}
					}
				}
				// print '<div>module [' . $this->getModuleName() . ']:</div>'; print_r($links); print '<hr>';
				return $links; // Phil 20060116 returns an array
			}
			
		}
		return false;
	}
	
	/**
	*
	* @return an array containing the bookmark items for the current module
	* @author Syed, Phil Calder
	*/
	function getGlobalBookmarkLinks($exclude_module = false) {
		global $SERVICES;
		if (is_object($system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks'))) {
			
			$links = $system_bookmarks_mod->getBookmarkLinks(false, true, false, $exclude_module);
			if (is_array($links)) {
				if (!empty($this->form_values['bookmark'])) { // Phil 20060116 Only loop through the links if there is a current bookmark
					foreach ($links as $k => $v) {
						if (!empty($v['id']) && ($v['id'] == $this->form_values['bookmark'])) {
							$links[$k]['selected']=1;
						}
					}
				}
				// print '<div>global:</div>'; print_r($links); print '<hr>';
				return $links; // Phil 20060116 returns an array
			}
		}
		return false;
	}
	
	
	
	
	
	/**
	 *
	 * @return mixed false or the bookmarks content
	 * @see Output::getBookmarkMenu
	 * @author Phil Calder
	 */
	function getBookmarksContent() {
		global $SERVICES, $L;
		$content = '';
		
		if (is_object($prefs =& $SERVICES->getService('Preferences'))) {
			if (!$prefs->getUserPref('systemBookmarksShowInPages')) {
				return false;
			}
		}
		
		// If we are showing module bookmarks, exclude the current module from global bookmarks
		$exclude_module = $this->getModuleName();
		
		// Bookmarks for the current module
		$global_links = (array)$this->getGlobalBookmarkLinks($exclude_module);
		
		if (!empty($global_links)) {
			$content .= $this->output->getBookmarkMenu($global_links, 2, 'bookmarkRow');
		}
		
		// Bookmarks for the current module
		$module_links = (array)$this->getModuleBookmarkLinks();
		
		if (!empty($module_links)) {
			$content .= $this->output->getBookmarkMenu($module_links, 2, 'bookmarkRow', true); // true=use 'Module Bookmarks' label
		}
		
		if (strlen($content)) {
			return $content;
		}
		
		return false;
	}
	
	/**
	 * Gets the contact name for the current bookmark (based on form values id)
	 *
	 * @return string the formatted contact name for the bookmark
	 */
	function getContactNameForBookmark() {
		global $SERVICES, $L;
		if ($SERVICES->modules->exists('Contact')) {
			
			$crm_module =& $SERVICES->modules->getModule('Contact');
			if (!$contact = $crm_module->getName($this->form_values['contact_id'])) {
				$contact = $this->form_values['contact_id'];
			}
			return ' - ' . $L['Contact'] . ': ' . $contact;
		}
		
		return '';
	}
	
	/**
	 * Gets the display name for the current bookmark (based on form values id)
	 *
	 * @return string the formatted display name ffor the bookmark
	 */
	function getDisplayNameForBookmark() {
		$bookmark_title = false;
		if (is_object($this->db_functions) && $this->form_values['id']) { // Phil 20051006, only if we have an id
			$item = $this->db_functions->getItem($this->form_values['id']);
			$display_name = $this->db_functions->getDisplayNameField();
			if ($display_name != '__none__') {
				$bookmark_title .= $item[$display_name];
			}
			if (!$bookmark_title) {
				$bookmark_title .= $this->form_values['id'];
			}
		}
		if (!$bookmark_title) {
			$bookmark_title = $this->form_values['id'];
		}
		return ' - ' . $bookmark_title;
	}
	
	/**
	 * returns the morelink for last viewed row
	 * @author Syed
	 */
	function getMoreLink() {
		global $L;
		$text  = $L['LabelMore'];
		$href  = '../../modules/Logger/index.php';
		$link = $this->output->getLinkHref(array('text' => $text, 'href' => $href));
		return $link;
	
	}	
	
	/**
	 * returns the last viewed items links
	 * @author Syed, Phil Calder
	 */
	function getLastViewedItems() {
		global $SERVICES, $L;
		$links =array();
		// $this->printDebug(get_class($this) . '::getLastViewedItems() start');
		$items = $this->getLastViewedPages(); // Phil 20050908
		// $this->printDebug(get_class($this) . '::getLastViewedItems() got last viewed pages');
		
		$tag_extras = '';
		
		if ($this->is_sidebar) {
			$target_frame = 'contentframe';
			$tag_extras = ' onClick="parent.window.Layout.focusContentPanel()"';
		} else {
			$target_frame = '';
		}
		
		foreach ($items as $k => $v) { 
			// print '<div>k=' . $k . '</div>'; print_r($v); print '<hr>';
			// $this->printDebug($v);
			$text = $label = ''; // Phil 20051007 reset these (see if(empty(..)) checks in switch below)
			
			// Phil - calculate the text label to use
			// Phil 20051006 added item_id check (so we dont call getItem(0) for eg 'list' pages)
			// Phil 20060626 Added Contact module check for 'View' pages (action 3) - we get the display name for Contacts through Output	 
			if (isset($v['item_id']) && $v['item_id'] && ( ! (($v['module'] == 'Contact') && $v['action'] == 3)) ) { 
				if (is_object($mod =& $SERVICES->modules->getModule($v['module']))) {
					// $this->printDebug(get_class($this) . '::getLastViewedItems() got mod for module: ' . $v['module']);
					$db_func =& $mod->getDatabaseTable($v['table_name']);
					// $this->printDebug(get_class($this) . '::getLastViewedItems() got db_func for module: ' . $v['module'] . ' table: ' . $v['table_name']);
					if (is_object($db_func)) { // Phil 20051006 check this is an object
						if ($text = $db_func->getItemDisplayName($v['item_id'])) { // Phil 20051104
							$text = strip_tags($text);
							// Phil 20051104
							$text = ( (strlen($text) > 15) ? (substr($text,0,13) . '..') : $text); 
						}
					}
				}
			} else {
				$SERVICES->language->getLanguage($v['module']); // Phil 20051007 Make sure the language is loaded
			}
			
			$filename = basename($v['page']);
			
			// if ($SERVICES->modules->isCustom($v['module'])) {
				// $path = '../../modules_custom/' . $v['module'];
			// } else {
				// $path = '../../modules/' . $v['module'] ;
			// }
			$path = $SERVICES->modules->getModulePath($v['module'], true);
			
			$tmp_label = substr_replace($v['module'],strtolower(substr($v['module'],0,1)),0,1) . 'ModuleLabel';
			$module = $L[$tmp_label];
			
			if (empty($module)) {
				$module = $v['module'];
			}
			
			$contact_link = false;
			$text_only = false;
			
			switch ($v['action']) {
				case 1:
					if (empty($text)) {
						$title = $L['LabelCreated'];
						$text = $module . ':' . $title ;
					}
					$href = $path . '/' . $filename .'?id=' .$v['item_id'];
					$text_only = false;
					// $links[] = $this->output->getLinkHref(array('text' => $text, 'href' => $href,'tag_extras' => 'class="LastViewCreate"')); 
					break;
				case 2:
					if (empty($text)) {
						$text = $module;
					}
					$class = '';
					$href = $path . '/' . $filename;
					$text_only = false;
					// $links[] = $this->output->getLinkHref(array('text' => $text, 'href' => $href,'tag_extras' => 'class="LastViewList"')); 
					break;
				case 3: // View
					// if ($module == 'Contacts') {  //MCR 20060221 (removed Phil 20060625)
					if ($v['module'] == 'Contact') {  // Phil 20060626 (cant use $module, it can be a language item, $L[$tmp_name])
						// $links[] = $this->output->getLinkContact(array('id' => $v['item_id']));
						$contact_link = $this->output->getLinkContact(array('id' => $v['item_id']), $target_frame, ($this->is_sidebar? 2 : 0), $tag_extras); // Phil 20060504
					} else {
						if (empty($text)) {
							$text = $module . ':' . $L['LabelView'] ;
						}
						$class = 'LastViewView';
						$href = $path . '/' . $filename . '?id=' . $v['item_id'];
						$text_only = false;
						// $links[] = $this->output->getLinkHref(array('text' => $text,'href' => $href,'tag_extras' => 'class="LastViewView"'));
					} 
					break;
				
				case 5:
					if (strpos($v['page'],'dashboard.php') === false) {
						if (empty($text)) {
							$title = $L['LabelEdit'];
							$text = $module . ':' . $title ;
						}
						$class = 'LastViewEdit';
						$href = $path . '/'.  $filename . '?id=' .$v['item_id'];
						$text_only = false;
						// $links[] = $this->output->getLinkHref(array('text' => $text, 'href' => $href,'tag_extras' => 'class="LastViewEdit"')); 
					}
					break;
				case 6:
					if (empty($text)) {
						$text = $module . ':' . $L['LabelDeleted'] ;
					}
					$class = 'LastViewDelete';
					$href ='';
					$text_only = true;
					// $links[] = $this->output->getLinkHref(array('text' => $text, 'href' => $href,'tag_extras' => 'class="LastViewDelete"','text_only' => true )); 
					break;
			}
			
			if ($contact_link) {
				$links[] = $contact_link;
			} else if (!empty($text)) {
				if ($this->is_sidebar) {
					$href = str_replace('../', '', $href);
					$href = str_replace('..\\', '', $href);
				} else {
					$target_frame = '';
				}
				
				$links[] = $this->output->getLinkHref(
					array(
						'text' => $text, 
						'href' => $href,
						'tag_extras' => ($class ? 'class="' . $class . '"' : '') . ($tag_extras ? ' ' . $tag_extras : ''),
						'text_only' => ($text_only ? true : false), 
						'target' => $target_frame
					)
				); 	
			}
			// unset($mod);  // Phil 20051104 Dont unset object which has been returned by reference
			// unset($db_func); // Phil 20051104 Dont unset object which has been returned by reference
		}
		// Phil 20060625 uncomment this block to add more link
		// $ua =& $SERVICES->getService('Authentication');
		// if ($ua->readAccess($USER['user_id'], 'Logger') && count($links) > 6) {
			// if ($this->getMoreLink()) {
				// //MCR 20060221
				// //$links[] = $this->getMoreLink();
			// }
		// }
		return $links;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getLastViewedPages() {
		return $this->db_functions->getLastViewedPages();
	}
	
	/**
	 * Returns the help link 
	 * 
	 * This method needs to be overridden in child classess. Return -1 to disable help button.
	 * @return mixed (nothing) or the help link, or -1
	 */
	function getHelpLink() {
		//return -1;
	}
	
	/**
	 * builds the title row for the page
	 * 
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		global $SERVICES,$L;
		if ($this->language_overwrite['title']) {
			$title_text = $SERVICES->language->getLanguageText($this->language_overwrite['title'], $this->getModuleName());
		} else {
			// $php_self = $SERVICES->getGlobal('php_self');
			$module_lower = $this->getModuleNameLower();
			// $title_text = $module_lower . $prefix . substr(basename($php_self), 0, strpos(basename($php_self), '.'));
			// eg $L['moduleNameTitle_itemlist']
			$title_text = $this->getModuleNameLower() . $prefix . str_replace($this->getModuleName() . '__', '', $this->getTableName()) . $this->getWebmatType();
			// print '<div>title_text: ' . $title_text . ' module: ' . $this->module_name . '</div>';
			if (!$L[$title_text]) {
				$title_text = $this->getModuleNameLower() . 'ModuleTitle';
			}
		}
		return $SERVICES->language->getLanguageText($title_text, $this->module_name);
	}
	
	/**
	 * returns the add bookmark link
	 * 
	 * @return string the add bookmark link
	 */
	function getAddBookmarkLink() {
		return $this->createBookmarkSession();
	}
	
	
	/**
	*
	* @return an array containing the bookmark items for the current module
	* @deprecated
	* @see getModuleBookmarkLinks
	*/
	function getBookmarkLinks() {
		return $this->getModuleBookmarkLinks();
	}

}
?>