<?php
/**
 * Exports the data to the csv file
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */


includeWebMAT('webmat.inc');
includeWebMAT('data_dictionary.inc');

/**
 * This class exports the data to either csv,vcard file
 *
 * @package WebMAT
 * @author Syed 
 */
class WebMATStandardExport extends WebMAT{
	
	// var $table_name = 'Contacts__item';
	
	var $fields = array();

	var $separate_field_by;

	var $enclose_field_by;

	var $terminate_line_by;

	var $escape_fields_by;
	// Bosco 20051208 | Removed escape_chars array as it was not used anywhere
	var $strip_line_chars = array("\n","\r","\r\n");

	var $filename = '';

	var $include_header = true;

	var $header_row = array();

	var $row_items;
	
	var $replace_null_by;
	
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->setEscapeField();
		$this->setEnclosedField();
		$this->setTerminateField();
		$this->setFilename();
		// Bosco 20051201 | Uncommented
		$this->setLineTerminate();
		// Bosco 20060105 | Added
		$this->setReplaceNullBy();
	}
	
	function setFilename() {
		if (trim($this->form_values['item']['file_name'])) {
        	$this->filename =  stripslashes($this->form_values['item']['file_name']);
		} else {
			$module = substr($this->form_values['table'],0,strpos($this->form_values['table'],'__'));
			// print $this->form_values['table'] . '<hr>';;
			// print $this->getModuleName() . '<hr>';
			// Phil 20051118 Contact quickexport had no module name set
			if (empty($module)) {
				$module = $this->getModuleName();
			}
			$this->filename = $module . "_" . date('Ymd'); 
		}
	}
	
	/**
	 * Gets the Char array from module.config and returns the array
	 * @param String $array_name Name of the char array
	 * @return Array $chars Array of Characters
	 * @author Bosco
	 */
	function getChars($array_name = '') {
		global $SERVICES;
		$chars = array();
		if (!empty($array_name)) {
			$chars = $SERVICES->config->lookup('modules.Export.' . $array_name);
		}
		return $chars;
	}
	
	function setEscapeField() {
		global $SERVICES;
		$field_escape_chars = $this->getChars('field_escape_chars');
		if (!($index = $this->form_values['item']['field_escaped_by'])) {
			$index = $SERVICES->preferences->getUserPref('exportFieldEscaped');
		}
		if (is_array($field_escape_chars) && !empty($field_escape_chars)) {
			$this->escape_fields_by = $field_escape_chars[$index];
		}
	}
	
	function setReplaceNullBy() {
		global $SERVICES;
		$replace_null_by = $this->getChars('replace_null_by');
		if (!($index = $this->form_values['item']['replace_null_by'])) {
			$index = $SERVICES->preferences->getUserPref('exportReplaceNullBy');
		}
		if (is_array($replace_null_by) && !empty($replace_null_by)) {
			$this->replace_null_by = $replace_null_by[$index];
		}
	}

	function setEnclosedField() {
		global $SERVICES;
		$field_enclose_chars = $this->getChars('field_enclose_chars');
		if (!($index = $this->form_values['item']['field_enclosed_by'])) {
			$index = $SERVICES->preferences->getUserPref('exportFieldEnclosed');
		}
		if (is_array($field_enclose_chars) && !empty($field_enclose_chars)) {
			$this->enclose_field_by = $field_enclose_chars[$index];
		}
	}

	function setTerminateField() {	
		global $SERVICES;
		$field_terminate_chars = $this->getChars('field_terminate_chars');
		if (!($index = $this->form_values['item']['fields_terminated_by'])) {
			$index = $SERVICES->preferences->getUserPref('exportFieldSeparator');
		}
		if (is_array($field_terminate_chars) && !empty($field_terminate_chars)) {
			$this->separate_field_by = $field_terminate_chars[$index]; 
		}
		// print '<div>setTerminateField separated by: ' . $this->separate_field_by . ' [' . $index . ']</div>';
		// if ($this->separate_field_by == "\t") {
			// print '<div>is tab separated</div>';
		// }
	}
	
	function setLineTerminate() {
		global $SERVICES;
		$line_terminate_chars = $this->getChars('line_terminate_chars');
		if (!($index = $this->form_values['item']['terminate_line_by'])) {
			$index = $SERVICES->preferences->getUserPref('exportLineTerminator');
		}
		if (is_array($line_terminate_chars) && !empty($line_terminate_chars)) {
			$this->terminate_line_by = $line_terminate_chars[$index];
		}
	}
	
	function setCsvForExcelDelimiters() {
		global $SERVICES;
		$this->terminate_line_by = "\r\n";
        $this->enclose_field_by = '"';
        $this->escape_fields_by = '"';
		$field_terminate_chars_excel = $this->getChars('field_terminate_chars_excel');
        if (!($index = $this->form_values['item']['excel_version'])) {
			$index = $SERVICES->preferences->getUserPref('exportExcelVersion');
        }
		if (is_array($field_terminate_chars_excel) && !empty($field_terminate_chars_excel)) {
			if (isset($field_terminate_chars_excel[$index])) {
				$this->separate_field_by = $field_terminate_chars_excel[$index];
			} else {
				$field_terminate_chars_excel = $this->getChars('field_terminate_chars');
				$this->separate_field_by = $field_terminate_chars[$index];
			}
		}
	}

	function main() {
		$this->beforeExport();
		global $SERVICES;
		switch ($this->form_values['export']) {
			case 'vcard':
				$this->exportDataToVcard();
				break;
			default:
				// $ids =implode(',', $_SESSION[$this->form_values['ids']]);
				$ids_array = $this->getExportIds();
				if (!is_array($ids_array) || !count($ids_array)) {
					$this->displayNoItemsError();
				}
				$ids =implode(',', $ids_array);
				$ids_where = array('id' => array('where' => 'id IN (' . $ids . ')'));
				$items = $this->getWhereItems($ids_where);
				$this->setHeaderRow();
				$this->include_header = true;
				foreach ($items as $k => $v) {
					$dd_item = $this->dd->getDataStructureWithData($v);
					// Phil 20051124 Optional 2nd parameter to getRow defaults to true 
					// and will convert field values according to Output::getStringField
					$data [] = $this->getRowData($dd_item);
				}
				$this->setData($data);
				// Bosco 20060105 | Test for Csv for Excel
				$csv_for_excel_idx = 2;
				if ($SERVICES->preferences->getUserPref('exportFileType') == $csv_for_excel_idx) {
					$this->setCsvForExcelDelimiters();
				}
				$this->exportDataToCsv();
				break;
		}
		$this->afterExport();
	}
	
	/**
	 * Placeholder method
	 *
	 * Override to implement. Will be called prior to exporting data.
	 * @author Phil Calder
	 */
	function beforeExport() {
		
	}
	
	/**
	 * Placeholder method
	 *
	 * Override to implement. Will be called after exporting data.
	 * This method must not generate output
	 * @author Phil Calder
	 */
	function afterExport() {
		
	}
	
	/**
	 * @author Phil Calder
	 * @return array An array of item ids
	 */
	function getExportIds() {
		$ids = array();
		if (isset($this->form_values['ids']) && isset($_SESSION[$this->form_values['ids']])) {
			$ids = (array)$_SESSION[$this->form_values['ids']];
		}
		return $ids;
	}
	
	/**
	 * Returns the javascript to redirect back
	 * 
	 * @return string Back URL (javascript)
	 * @see displayNoItemsError()
	 * @author Phil Calder
	 */
	function getBackUrl() {
		return 'javascript:history.go(-1)';
	}
	
	/**
	 * Prints an error message and exits
	 *
	 * @author Phil Calder
	 */
	function displayNoItemsError() {
		global $L;
		$back_link = '<a href="' . $this->getBackUrl() . '">' . $L['Back'] . '</a>';
		quickPage($L['errNoItemsToExport'] . '<br /><br />' . $back_link);
		exit();
	}
	
	/**
	 * Prints an 'unknown' error message and exits
	 *
	 * @author Phil Calder
	 */
	function displayUnknownError() {
		global $L, $SERVICES;
		$back_link = '<a href="' . $this->getBackUrl() . '">' . $L['Back'] . '</a>';
		quickPage($L['msgUnknownError'] . '<br /><br />' . $back_link);
		exit();
	}
	
	function exportDataToVcard() {
		// Phil 20070528 update to version 5
		global $SERVICES;
		$modules =& $SERVICES->getService('Modules');
		$db =& $SERVICES->getService('Database');
		$contact  =& $modules->getModule('Contact');
		$contact_id = $this->form_values['id'];
		if ($is_person = $contact->isPerson($contact_id)) {
			$row = $db->quickGet(
			'Contact__item.display_name as Name,
			Person__item.first_name as Fname,
			Person__item.middle_name as Mname,
			Person__item.last_name as Lname,
			Person__item.title as Title,
			Person__item.job_title as Job_Title,
			Company__item.display_name as Company, 
			Contact__item.email1_content as Email,
			Contact__item.email2_content as Email2,
			Contact__item.email3_content as Email3,
			Contact__item.phone1_content as Phone,  
			Contact__item.phone2_content as Phone2, 
			Contact__item.phone3_content as Phone3, 
			Contact__item.fax1_content as Fax, 
			Contact__item.addrpost1_houseno as HouseNo, 
			Contact__item.addrpost1_housesuffix as HouseSuffix, 
			Contact__item.addrpost1_street as Street, 
			Contact__item.addrpost1_line2 as Line2, 
			Contact__item.addrpost1_line3 as Line3,  
			Contact__item.addrpost1_city as City, 
			Contact__item.addrpost1_state as State, 
			Contact__item.addrpost1_postcode as Postalcode, 
			Contact__item.addrpost1_country as Country,
			Contact__item.url1_content as URL', 
			'Contact__item 
 			LEFT JOIN Person__item on Contact__item.id = Person__item.contact_id and Person__item.primary_record=1 
			LEFT JOIN Company__item on Contact__item.contact_company = Company__item.contact_id 
			 ', 
			"Contact__item.id= '$contact_id' "
			);
			
			
			
		} else if ($is_company = $contact->isCompany($contact_id)) {
			$row = $db->quickGet(
			'Company__item.display_name as Name, 
			Company__item.display_name as Company, 
			Contact__item.email1_content as Email,
			Contact__item.email2_content as Email2,
			Contact__item.email3_content as Email3,
			Contact__item.phone1_content as Phone,  
			Contact__item.phone2_content as Phone2, 
			Contact__item.phone3_content as Phone3, 
			Contact__item.fax1_content as Fax, 
			Contact__item.addrpost1_houseno as HouseNo, 
			Contact__item.addrpost1_housesuffix as HouseSuffix, 
			Contact__item.addrpost1_street as Street, 
			Contact__item.addrpost1_line2 as Line2, 
			Contact__item.addrpost1_line3 as Line3,  
			Contact__item.addrpost1_city as City, 
			Contact__item.addrpost1_state as State, 
			Contact__item.addrpost1_postcode as Postalcode, 
			Contact__item.addrpost1_country as Country,
			Contact__item.url1_content as URL', 
			'Contact__item LEFT JOIN Company__item ON Contact__item.id = Company__item.contact_id and Company__item.primary_record=1 ', 
			"Contact__item.id= '$contact_id' "
			);
		}
		
		// Phil 20060404 Dont proceed if there is an error here
		if ($db->hasError()) {
			// $db->printError();
			$this->displayUnknownError();
			exit();
		}
				
		$vcard = "BEGIN:VCARD\r\n";
		$vcard .= "VERSION:3.0\r\n";
		$vcard .= "CLASS:PUBLIC\r\n";
		
		if ($row['Name']) { // Phil Using the FN type
			$vcard .= "FN:" . $row['Name']. "\r\n";
			$filename=preg_replace("/\W/","",$row['Name']);
		}
		
		if ($is_person) { // Phil Using the N type
			$vcard .= "N:" . $row['Lname']. ";" . $row['Fname']. ";" . $row['Mname']. ";" . $row['Title']. ";\r\n";
			$filename=preg_replace("/\W/","",$row['Name']);
		}
		
		if (strlen($filename) == 0) {
			$filename = "profile";	
		}
		
		// =====
		// ADR;TYPE=dom,home,postal,parcel:;;123 Main Street;Any Town;CA;91921-1234 	
		// The structured type value corresponds, in sequence, to: 
		// the post office box; 
		// the extended address; 
		// the street address; 
		// the locality (e.g., city); 
		// the region (e.g., state or province); 
		// the postal code; 
		// the country name.
		// =====
		
		// Phil 20060403 added the other address fields
		$tmp['StreetAddress'] = trim($row['HouseNo'] . ' ' . $row['HouseSuffix'] . ' ' . $row['Street'] . ' ' . $row['Line2'] . ' ' . $row['Line3']);
		
		// $vcard .= "ADR;TYPE=home:" . ";;;" . $row['City']. ";" . $row['State']. ";" . $row['Postalcode']. ";" . $row['Country'] . "\r\n";
		$vcard .= "ADR;TYPE=work:" . ";;" . $tmp['StreetAddress']. ";" . $row['City']. ";" . $row['State']. ";" . $row['Postalcode']. ";" . $row['Country'] . "\r\n";
		
		
		//TEL;TYPE=work,voice,pref,msg:+1-213-555-1234
		$phone_type = 'work';
		if ($row['Phone']) {
			$vcard .= "TEL;TYPE=work,pref:" . $row['Phone']. "\r\n";
		}
		if ($row['Phone2']) {
			$vcard .= "TEL;TYPE=work:" . $row['Phone2']. "\r\n";
		}
		if ($row['Phone3']) {
			$vcard .= "TEL;TYPE=work:" . $row['Phone3']. "\r\n";
		}
		
		if ($row['Fax']) {
			$vcard .= "TEL;TYPE=fax,work:" . $row['Fax']. "\r\n";
		}
		
		//EMAIL;TYPE=internet:fdawson@earthlink.net
		if ($row['Email'] != "") {
			$vcard .= "EMAIL;TYPE=internet,pref:" . $row['Email'] . "\r\n";
		}
		if ($row['Email2'] != "") {
			$vcard .= "EMAIL;TYPE=internet:" . $row['Email2'] . "\r\n";
		}
		if ($row['Email3'] != "") {
			$vcard .= "EMAIL;TYPE=internet:" . $row['Email3'] . "\r\n";
		}
		
		//ROLE:Programmer
		if ($row['Job_Title']) {
			$vcard .= "ROLE:" . $row['Job_Title']. "\r\n";
		}
		//ORG:ABC\, Inc.;North American Division;Marketing
		if ($row['Company']) {
			$vcard .= "ORG:" . $row['Company']. "\r\n";
		}
		// URL:http://www.swbyps.restaurant.french/~chezchic.html
		if ($row['URL'] != "") {
			$vcard .= "URL:" . $row['URL'] . "\r\n";
		}
		$vcard .="END:vCard";
		header("Content-type: text/directory");
		header("Content-Disposition: attachment; filename=" .$filename. ".vcf");
		header("Pragma: public");
		print $vcard;
	}
	
	function getWhereItems($values){
		$fields = $this->dd->getDataDictionary();
		$this->db_functions->setDesignTimeFilters($values);
		return $this->db_functions->getItems();
	}

	function getHeaderRow() {
		return $this->header_row;
	}

	function setHeaderRow($item = array()) {
		if (empty($item)) {
			$item = $this->dd->getDataDictionary();
		}
		foreach ($item as $k => $v) {
			$header[] = $k;
		}
		$this->header_row = $header;
	}
	
	
	/**
	 *
	 * @param array item The item array / row
	 * @param boolean $convert_values Optional, default true - will convert values according to {@link Output::getStringField}
	 * @author Phil Calder 
	 */
	function getRowData($item, $convert_values = true) {
		
		// Phil 20051124 - option to use Output::getStringValue
		if ($convert_values && !is_object($this->output)) {
			$this->output =& $this->getOutput();	
		}
		
		foreach ($item as $k => $v ) {
			// Phil 20051124 - option to use Output::getStringValue
			if ($convert_values) { 
				if ($item[$k]['type'] == 'hidden') {
					$item[$k]['type'] = 'text'; // Otherwise, hidden fields are displayed as '----'
				}
				if ($item[$k]['type'] == 'integer') { // Phil 20060329
					$item[$k]['type'] = 'text'; // Otherwise, integer fields are not displayed correctly
				}
				$item[$k]['list_detail_length'] = 'All'; // Prevent trimminmg
				$item[$k]['no_link'] = true; // Prevent html / links (eg for urls, email addresses, contacts)
				$item[$k]['value'] = $this->output->getStringField($item[$k]);
			}
			$row[] = $item[$k]['value'];
		}
		return $row;
	}
	
	function setData($item) {
		$this->row_items = $item;
	}

	function exportDataToCsv(){
		// Bosco 20051207 | Added code for download
		$browser_type = $_SERVER['HTTP_USER_AGENT'];
                if (!strpos($browser_type, 'MSIE') === false) {
                        $browser_type = 'IE';
                } elseif (!strpos($browser_type, 'Opera') === false) {
                        $browser_type = 'OPERA';
                }

		header('Content-Type: text/comma-separated-values');
		header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
		header('Content-Disposition: attachment; filename="' . $this->filename . ".csv" . '"');
		if ($browser_type == 'IE') {
            		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		        header('Pragma: public');
	        } else {
        	    header('Pragma: no-cache');
        	}
	
		$items = array();
		if ($this->include_header) {
			$items[] = $this->header_row;
		}
		$items = array_merge($items,$this->row_items);
		$content ="";
		foreach ($items as $k => $value) {
			$length= count($value); // count fields
			for ($i=0; $i<$length; $i++ ) {
				// Bosco 20060104 | Replace NULL Values
				if ((!isset($value[$i]) || is_null($value[$i])) && !empty($this->replace_null_by)) {
                			$value[$i] .= $this->replace_null_by;
            			}
				if ($i == ($length-1)) {
					$content .= $this->encloseFieldValue($value[$i]); // No separator for last field
				} else {
					$content .= $this->encloseFieldValue($value[$i]) . $this->separate_field_by;
				}
			}
			$content .= $this->terminate_line_by; // Add line termination
		}
		print $content;
	}
	
	function encloseFieldValue($input) {
		$input = $this->stripFieldValue($input);
		$input = $this->escapeFieldValue($input);
		$input = $this->enclose_field_by . $input . $this->enclose_field_by;
		return $input;
	}
	
	function escapeFieldValue($input) {
		if (strlen($this->separate_field_by) > 0) {
			$input = str_replace($this->separate_field_by,' ',$input);
		}
		// Bosco 20051206 | Changed to escape field terminator
		if ($this->enclose_field_by != '') {
			$input = str_replace($this->enclose_field_by, $this->escape_fields_by . $this->enclose_field_by, $input);
		}
		return $input;
	}
	
	function stripFieldValue($input) {
		$input = trim($input,implode("",$this->strip_line_chars));
		// Added to Remove <br> | <br /> tags if any
		$input = str_replace('<br>', "", $input);
		$input = str_replace('<br />', "", $input);
		return $input;
	}

}
?>