<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('edit.inc');

/**
 * Create identical items for multiple contacts on one form
 *
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardMassCreate extends WebMATEdit {

	/**
	 * Call prior to displaying the page. This method handles submit or delete functionality
	 * and sets the return page variable.
	 */
	function run() {
		if (isset($this->form_values['submit'])) {
			$this->setContactSessionId($this->form_values['contact_session_id']);
			$this->setLogAction();
			$this->doSubmit();
		}
		$this->setRefererAsReturnPage();
	}

	/**
	 * set the log actions
	 */
	function setLogAction() {
		// if ($this->form_values['submit']) {
			// $this->db_functions->setLogAction('mass_create');
		// }
	}
	
	function setContactSessionId($session_id) {
		$this->contact_session_id = $session_id;
	}
	
	function getContactSessionId() {
		return $this->contact_session_id;
	}
	
	function getContactIds() {
		if ($contact_session_ids = $this->getContactSessionId()) {
			if (isset($_SESSION[$contact_session_ids]) && is_array($_SESSION[$contact_session_ids])) {
				return $_SESSION[$contact_session_ids];
			}
		}
	}
	
	/**
	 * Suffix for the description row language text key
	 *
	 * 
	 * @return string the description row content language key suffix
	 */
	function getDescriptionContentSuffix() {
		return 'MassCreateDescription'; // TODO populate lang items
	} 
	
	/**
	 * Builds the edit form
	 *
	 * This method: 
	 * gets the item
	 * checks that it is editable
	 * set the primary record flag
	 * calls {@link beforeForm}
	 * calls {@link handlePermissions}
	 * calls its parent method
	 *
	 * @param  integer  $id  an item id
	 * @param  array  $item  the item array from a form that failed validation
	 * @param  array  $error_msg  an array of error messages.
	 */
	function main($id = '', $item = '', $error_msg = '') {
		global $SERVICES, $L;
		
		$item = $this->getItem($id, $item);
		// $this->checkEditable($item['id'], $item);

		// If this record is the primary record set the checkbox readonly
		// $this->setPrimaryReadOnly($item);
		//add any data elements not in item to the item array
		$this->beforeForm($id, $item, $error_msg);

		// Removes permissions fields that the user is not allowed to see and/or edit
		// Either because of permission locking (at a module or contact level)
		// or bcause the user is not a member of the required group to edit a permission field
		$this->handlePermissions($id, $item, $error_msg);
		
		$this->processDataDefinitionChanges();
		$this->setContactSessionId($this->form_values['contact_session_id']);
		
		// return parent::main($id, $item);
		// return parent::main($id, $item, $error_msg);
		return parent::main($item['id'], $item, $error_msg);
	}
	
	/**
	 * runs on submit to validate the data and then update the DB
	 */
	function doSubmit() {
		$this->processDataDefinitionChanges();
		
		$this->doSubmitBegin();
		$this->doSubmitPermissions();
		
		if ($this->submit) {
			$this->doSubmitValidate();
			$ids = $this->doSubmitProcess();
			$this->doSubmitPrintErrors($ids);
		}
		$this->doSubmitEnd($ids);
		$this->doRedirect($ids);
		$this->redirectBack($ids);

		return $id;
	}
	
	/**
	 * check if the user can edit the item
	 */
	function doSubmitPermissions() {
		bcrm_unset($this->item, '_permission_controls'); // Phil 20050712 @see addPermissionControls
		// $this->checkEditable($this->item['id'], $this->item);
	}

	/**
	 * validate the item
	 */
	function doSubmitValidate() {
		if ($error = $this->validateFields($this->item)) {
			$this->main( $error['item']['id'], $error['item'], $error['error_msg']);
		}
		$this->doSubmitRemoveReadOnly();
	}
	
	/**
	 * processes the item and returns the id
	 *
	 * @return integer id the item that was created/updated
	 */
	function doSubmitProcess() {
		// print '<div>dosubmitProcess contact_session_id: ' . $this->getContactSessionId() . '</div>'; 
		
		$contact_ids = (array)$this->getContactIds();
		// print '<div>dosubmitProcess contact_ids:</div>'; print_r($contact_ids); print '<hr>';
		
		$item = $this->getItemVariable();
		// print '<div>dosubmitProcess item:</div>'; print_r($item); print '<hr>';
		$ids = array();
		foreach ($contact_ids as $k => $v) {
			$item['contact_id'] = $v;
			$ids[$v] = $this->setItem($item);
		}
		return $ids;
	}

	/**
	 * print an insert time error if there is one
	 *
	 * @param array $ids An array of item ids
	 */
	function doSubmitPrintErrors($id) {
		if ($error = $this->getErrorMessage()) {
			global $L;
			quickPage( $L[$error] . '<br><br> ' . $this->getLinkReturnPage() );
		}
	}
	
	/**
	 * redirects the user to this module tables list page, filtered on the new item ids
	 * @param array $ids An array of ids to filter on
	 * @author Syed
	 */
	function doRedirect($ids='') {
		$this->setIdsSession($ids);
		$this->redirectToList($ids);
	}
	
	/**
	 * redirects back to the list page, with given ids filter appended to query
	 * @see getListPageUrl()
	 * @author Phil Calder
	 */
	function redirectToList() {
		$list_page = $this->getListPageUrl();
		$ids_filter = $this->getIdsSession();
		$list_page .= '?ids_session=' . $ids_filter;
		header('Location: ' . $list_page);
		exit();
	}
	
	function setIdsSession($ids) {
		if (is_array($ids)) {
			$this->ids_session = md5($module . time());
			$_SESSION[$this->ids_session] = $ids;
		} else {
			$this->ids_session = $this->form_values['ids_session'];
		}
	}
	
	function getIdsSession() {
		return $this->ids_session;
	}
	
	/**
	 * Disabled
	 */
	function doSubmitDelete() {
	}
	
	/**
	 * Make custom changes to the data definition 
	 *
	 * Remove contact id field and make sure it is not required
	 */
	function processDataDefinitionChanges() {
		// Remove the contact_id field
		$this->dd->setDictionaryExcludeFields(
			array(
				'contact_id'
			)
		);
		// TODO: need to ensure contact_id is not required
	}

	/**
	 * Writes the item to the database
	 *
	 * @param array $item the item array
	 * @return array containing the item id and the error message(language key) if an error message is present
	 */
	function setItem($item = false) {
		if (!$item) {
			$item = $this->getItemVariable(); // $this->item;
		}

		$this->beforeDBWrite($item, $fields, $error_msg);  //passed by reference
		$updated = '';
		$item = $this->removeForeignFields($item);
		$updated = $this->db_functions->setItem($item); //To add more cases you need to update setItem
		switch ($updated) { // Error catching for updates
			case 'Deleted': //If item has been deleted while editing
			case 'Updated': //If item has been changed while editing
				global $L;
				$this->setErrorMessage('errItem' . $updated);
				return $item['id'];
			break;
			default:
			// Do nothing
			break;
		}
		$this->afterDBWrite($item, $fields, $error_msg, $updated);
		return $updated;
	}
	
	/**
	 * Gets the table links
	 *
	 * Overridden to handle new items and to remove the edit link
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 * @author Phil Calder
	 */
	function getTableLinks($item) {
		// TODO
		$links = array();
		if ($this->form_values['id']) {	
			$links = parent::getTableLinks($item);
			bcrm_unset($links, 'edit');
		} else {
			$links = $this->replaceLinkVariables(false, $item, (array)$this->getListLink($item));
		}
		return $links;
	}
	
	/**
	 * Disabled
	 *
	 * @param integer $id the id of the item (not used)
	 * @param array $item the item array
	 * @return array the default item
	 * @see getDefaultItem
	 */
	function getItem($id = false, $item = false) {
		return $this->getDefaultItem($item);
	}
	
	/**
	 * gets default values
	 *
	 * Wrapper function
	 * @author Phil Calder
	 * @param array $default_overrides values that overwrite the data definition defaults.
	 */
	function getDefaultItem($default_overrides = false) {
		return $this->dd->getDefaultItem($default_overrides);
	}
	
	/**
	 * Gets the title text for the page
	 *
	 * CHANGED to change the prefix
	 *
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		// TODO
		if (($prefix == 'Title_') && !$this->form_values['id']) {
			$prefix = 'TitleMassCreate_'; // TODO add lang items
		}
		return parent::getTitleText($prefix);
	}
	
	/**
	 * Adds contact counts to the standard description row
	 * 
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return string the content for the form
	 * @see getContactCountContent()
	 * @see getEmailAddressCountRow()
	 */
	function getDescriptionContent() {
		$content = parent::getDescriptionContent();
		$content .= $this->getContactCountContent();
		return $content;
	}
	
	/** 
	 * Returns a table row containing the contact count
	 * @see getDescriptionContent()
	 */
	function getContactCountContent() {
		global $L;	
		$contact_ids = $this->getContactIds();
		$contact_count = count($contact_ids);
		$description = $L['NumberOfContacts'] . ': ' . $contact_count;
		if ($contact_count) {
			$description .= ' ' . $this->getContactListLink();
		}
		$padding = true;
		$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
		$content .= $description;
		$content .= '</td>' . $this->output->getEndRow(true);
		
		return $content;
	}
	
	/**
	 * Link to the Contact list page, filtered on contact ids
	 */
	function getContactListLink() {
		global $L;
		$session_id = $this->getContactSessionId();
		// TODO config lookup
		return '<a href="../../modules/Contact/index.php?ids_session=' . $session_id . '&filter=fnone&qfilter=&search_filter=off">' . $L['LabelView'] . '</a>';
	}
	
	/**
	 * any processing required before adding to the form is created
	 *
	 * Override this method to add functionality
	 */
	function beforeForm(&$id, &$item, &$error_msg) {}
	
	/**
	 * No new link
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getNewLink() {
		return false;
	}
	
	/**
	 * Returns the help link 
	 * 
	 * Overridden to append #mass_create to the link
	 * @return mixed (nothing) or the help link, or -1
	 */
	function getHelpLink() {
		if ($help_link = $this->output->getHelpId()) {
			$help_link .= '#mass_create';
			return $help_link;
		}
		return parent::getHelpLink();
	}
	
	/**
	 * Delete is not relevant for this class
	 * @access private
	 */
	function setDeleteVariable() {
		$this->delete = false;
	}

	/**
	 * Delete is not relevant for this class
	 *
	 * @return boolean false
	 * @access private
	 */
	function getDeleteVariable() {
		return false;
	}
}
?>