<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('view.inc');
// print __FILE__ . '<hr>';
/**
 * @package WebMAT
 * @author Phil Calder
 */
class WebMATStandardPublicView extends WebMATView {
	
	/**
	 * Allow/prevent permissions fields from being displayed
	 * Applied on top of data option setting 
	 * (ie will prevent permissions fields from being displayed even if they are included by the data option).
	 * @access private
	 */
	var $allow_permissions = false;
	
	/**
	 *
	 *
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		global $SERVICES;
		$this->ua =& $SERVICES->getService('Authentication');
		$this->ua->clientRestrict(); // Requires client login
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->output->setShowContactSelector(false);
		$this->output->setShowContactPopup(false); // disables contact popup link option
		$this->setContact();
		$this->setPublicExcludeFields();
	}
	
	function isBrowseable() {
		return false;
	}
	function getBrowseLinks() {
		return false;
	}
	function getNewLink() {
		return false;
	}
	function markAsSeen($item, $extra_values = false) {}
	
	/**
	 * Returns the title row array with an added Logout link
	 * @return array An array of links to display in the title row.
	 * @author Phil Calder
	 */
	function getTitleRowLinks() {
		$title_links = parent::getTitleRowLinks();
		$title_links['help'] = -1;
		$title_links['bookmark'] = 'off'; 
		$title_links['index'] = $this->getIndexLink();
		$title_links['logout'] = $this->getLogoutLink();
		return $title_links;
	}
	
	function getLastViewedPages() {
		return array();
	}
	function getLastViewedContent() {
		return '';
	}
	function getLastViewedItems() {
		return array();
	}
	
	function setPublicExcludeFields() {
		if (!$this->allow_permissions) {
			$exclude = array(
				'lock_permissions', 
				'view_group_id', 
				'edit_group_id', 
				'owner_group_id', 
				'assigned_to_group_id', 
				'admin_group_id'
			);
			$this->dd->setDictionaryExcludeFields($exclude);
		}	
	}
	
	function setContact() {
		if ($this->ua->isWebFormUser()) {
			// print '<p>setContact is web form user</p>';
			// $contact_id_field = 'id'; // 'contact_id'
			$contact_id_field = $this->db_functions->getContactIdField();
			if ($id = $this->ua->users->getUsersContactId()) {
				// print '<p>setting id = ' . $id . '</p>';
				$this->form_values[$contact_id_field] = $id;
			}
		}
		// print '<p>setContact id = ' . $this->form_values[$contact_id_field] . '</p>';
	}
	
	function checkItem() {
		$this->setContact();
		parent::checkItem();
	}
	
	function getTableLinks($item) {
		$links = parent::getTableLinks($item);
		bcrm_unset($links, 'list');
		return $links;
	}
	
	/**
	 * Alters certain fields to remove links
	 *
	 * @param array $field The field array from a data definition, required for the Output Service getStringField function.
	 * @return string representation of this field, as defined in the Output Service
	 * @author Phil Calder
	 */
	function getStringField($field) {
		if ( 
			($field['type'] == 'email') || 
			($field['type'] == 'url') || 
			($field['type'] == 'contact')
		) {
			$field['no_link'] = 1; //'text';
		}
		return parent::getStringField($field);
	}
	
	/*
	 * Builds the View link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	// function getViewLink($id = false) {
		// return $this->output->getLinkView(
			// 'contact_view.php' // todo: make this a config lookup
		// );
	// }
	
	/**
	 * Builds the Edit link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getEditLink($id = false) {
		return $this->output->getLinkEdit(
			'contact_edit.php' // todo: make this a config lookup
		);
	}
	/**
	 * Disables the delete link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string An empty string
	 */
	function getDeleteLink($id = false) {
		return '';
	}
	
	function isDeletable($id = false) {
		return false;	
	}
	
	/**
	 * @return mixed false or a logout link
	 */
	function getLogoutLink() {
		if ($this->ua->isWebFormUserLoggedIn()) {
			global $L;
			return '<a href="?logout=1">' . $L['logout'] . '</a>';
		}
		return false;
	}
	
	/**
	 * @return string a link to the index page
	 */
	function getIndexLink() {
		global $L;
		return '<a href="index.php">' . $L['LinkHome'] . '</a>';
	}
	
	/**
	 * Prefixes link_name with 'public_'  if not already done. Calls parent.
	 */
	function getFileSetHref($link_name) {
		$prefix = 'public_';
		if (substr($link_name, 0, strlen($prefix)) != $prefix) {
			$link_name = $prefix . $link_name;
		}
		return parent::getFileSetHref($link_name);
	}
	
	function getModulePath() {
		return '';
	}
	
}
?>