<?php
require_once("../../prepend.inc");
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 * @author Matthew Corich
 */

includeWebMAT('list.inc');

/**
 * A selector popup
 * @package WebMAT
 */
class WebMATStandardSelector extends WebMATList {
	/**
	 * @var string the selector link back
	 * @access private
	 */
	var $selector_link_back = false;
	/**
	 * @var string the selector link back
	 * @access private
	 */
	var $selector_set = false;

	/**
	 * init function sets all the class variables
	 * 
	 * CHANGED to add the java script required to the head
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->output->setPagingRowsSwitch(true);
		$this->output->setGraphicBlockSwitch(false);
	}

	/**
	 * returns the add bookmark link
	 * 
	 * CHANGED to disable the link
	 * 
	 * @return string the add bookmark link
	 */
	function getAddBookmarkLink() {
		return 'off';
	}

	/**
	 * Sets the Log Action
	 * @author Bosco
	 */
	function setLogAction() {
		// Bosco 20051209 | Overridden to set 'selector' action type
		$this->db_functions->setLogAction('selector');
	}

	/**
	 * gets the content for a table cell
	 * 
	 * CHANGED to return a display name only
	 *
	 * @param array $item the items array
	 * @return string the table cell content.
	 */
	function buildStringFieldRow($item) {
		$this->output->addToHead($this->getSelectorJS());

		$id = $item['id'];
		$table_row = $this->getItemCheckbox($id);

		if ($this->showCheckBoxMenuRow()) {
			$table_row['__cb'] = $this->getItemCheckbox($item['id']);
		}
		$table_row['__links'] = ''; // '[edit]'; // view, edit, etc, links
		
		$table_row['__flags'] = ''; // '[flags]'; // alert, unseen/edited, etc
		
		$dd_item = $this->dd->getDataStructureWithData($item);
		
		$display_name = $this->db_functions->getDisplayNameField();
		
		foreach ($dd_item as $k => $field) {
			if ($field['field_key'] == $display_name) {
				$table_row[] = $this->buildSelectorLink($id, $field, $item);
			} else {
				$table_row[] = $this->getStringField($field);
			}
		}
		return $table_row;
	}
	
	/**
	 * creates the selector link
	 */
	function buildSelectorLink($id, $field, $item = false) {
		$text = $this->getStringField($field);
		$href = 'javascript:setReturn(\'' . $id . '\',\'' . addslashes($text) . '\');window.close()';
		return $this->output->getLinkHref( array('href' => $href, 'text' => $text) );
	}
	
	/**
	 * Adds the edit and delete links to the display row
	 * 
	 * CHANGED to remove the item links
	 * 
	 * @param array $table_row 
	 * @param string $id  as item id
	 * @return array the table row array with the extra values.
	 */
	function getTableLinks($item){
		return array();
	}
	
	/**
	 *
	 */
	function getSelectorJS() {
		if (!$this->selector_set) {
			$selector_js = '
			<script type="text/javascript" language="JavaScript">';
			$selector_js .= $this->getJSSetReturn();
			$selector_js .= $this->getJSChangeLink();
			$selector_js .= '
			</script>';
			$this->selector_set = true;
			return $selector_js;
		}
	}
	
	/**
	 *
	 */
	function getJSSetReturn() {
		$field = $this->form_values['field'];
		$text_key = substr(rtrim($field, ']'), (strrpos($field, '[') +1));
		
		$script = '
		function setReturn(id, name) {
			window.opener.document.forms["mainform"].elements["' . $field .'"].value = id;
			';
		if ($link_back = $this->getLinkBack()) {
			$script .= 'changeLink("_' . $text_key . '_text", name, ' . $this->getLinkBack() . ');';
		} else {
			$script .= 'changeLink("_' . $text_key . '_text", name);';
		}
		$script .= '
		}';
		
		return $script;
	}
	
	/**
	 *
	 */
	function getJSChangeLink() {
		$script .= '
		function changeLink(id, text, new_href) {
			if (window.opener.document.getElementById(id)) {
				x = window.opener.document.getElementById(id);
				if (new_href) {
					x.href = new_href;
				}
				x.title = "";
				x.title = text;
				
				x.innerHTML = "";
				x.innerHTML = text;
			}
		}';
		return $script;
	}
	
	/**
	 * Creates an link pointing back to the module the selector is from
	 * 
	 * @return string the module href
	 */
	function getLinkBack() {
		return $this->selector_link_back;
	}
	
	/**
	 * Creates an link pointing back to the module the selector is from
	 * 
	 * @return string the module href
	 */
	function setLinkBack($link) {
		$this->selector_link_back = $link;
	}
	
	/**
	 * Creates an link pointing to the edit page from the module config
	 *
	 * CHANGED to remove the link
	 *
	 * @param string $contact_id the current contact id
	 * @return string an a href for the a new link
	 */
	function getNewLink($contact_id = '') {
		return '';
	}
	
	/**
	 * returns the mass edit link
	 * 
	 * CHANGED to remove the link
	 * 
	 * @return string the mass edit link
	 */
	function getMassEditItem() {
		return false;
	}
	
	/**
	 * Functionality disabled - when doing selector, don't need past links
	 *
	 * @return boolean false
	 */
	function getPastLinksTable() {
		return false;
	}
	
	/**
	 * Functionality disabled - don't need when doing just selector
	 *
	 * @return boolean false
	 */
	function getActionLinksContent() {
		return false;
	}
	
	/**
	 * returns the mass delete link
	 * 
	 * CHANGED to remove the link
	 * 
	 * @return string the mass delete link
	 */
	function getMassDeleteItem() {
		return false;
	}
	
	/**
	 * returns the multi edit link
	 * 
	 * CHANGED to remove the link
	 * 
	 * @return string the multi edit link
	 */
	function getMultiEditItem() {
		return false;
	}
	
	/** 
	 * There is no item checkbox / process multi functionality in selector popups
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getItemCheckbox() {
		return false;
	}
	
	/**
	 * returns the title row array - disable for selecting.
	 *
	 * @param string $contact_id the contact id if required
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinks() {
		$title_links = array(
			'bookmark' => 'off',
			'refresh' => 'off',
			'back' => 'off',
			'help' => '-1'
		);
		return $title_links;
	}
	
	/** 
	 * Advanced search filter link not allowed
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function allowAdvancedFilterLink() {
		return false;
	}
}
?>