<?php
/**
 *  This is the root level WebMAT class
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

/**
 * The root level WebMAT class
 *
 * @author Phil Calder
 * @abstract
 * @package WebMAT
 */
class WebMATStandard {
	/**
	 * @var array the combined post and get arrays
	 * @access private
	 */
	var $form_values = array();
	/**
	 * @var object a collection of database functions
	 * @access private
	 */
	var $db_functions;
	/**
	 * @var object the DataDictionary class
	 * @access private
	 */
	var $dd;
	/**
	 * @var object the Output class
	 * @access private
	 */
	var $output;
	/**
	 * @var output display parameter
	 * @access private
	 */
	var $output_display_param;
	/**
	 * @var string the module name with the fisrt letter converted to lower case
	 * @access private
	 */
	var $module_name_lower;
	/**
	 * @var string the module name
	 * @access private
	 */
	var $module_name;
	/**
	 * @var array with the keys title and description
	 * @access private
	 */
	var $language_overwrite;
	/**
	 * @var string the name of the table you are currently editing.
	 * @access private
	 */
	var $table_name;
	/**
	 * @var string the name used toa apend user pref
	 * @access private
	 */
	var $pref_name;
	/**
	 * @var The current module object
	 * @access private
	 */
	var $mod;
	/**
	 * @var The array data definition from the $db_functions object.
	 * @access private
	 */	
	var $data_definition = array();
	/**
	 * @var string the menu item separator
	 * @access private
	 */
	var $menu_separator = '';
	
	/**
	 * @access private
	 */
	var $restrict_level = 'read';
	
	/**
	 * Class Constructor function
	 *
	 * Calls init
	 * @see init()
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function WebMATStandard($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		$this->init($module, $data_option, $post, $get, $table_name, $language_overwrite);
	}
	
	/**
	 * init function sets all the class variables
	 *
	 * Should only be called by the contstructor
	 * 
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 *
	 * @author Phil Calder
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		// print '<div>WebMAT init (' . $data_option . ', ' . $module . ', ' . $table_name . ', [get:])</div>';
		$this->restrictAccess(); // Phil 20061122
		$this->setDataOption($data_option); // Phil 20050822
		
		if (!$post) {
			$post = $_POST;
		}
		if (!$get) {
			$get = $_GET;
		}
		
		$this->setModuleInfo($module);
		
		$this->setOutput();
		$this->setOutputOptions();
		
		$this->setFormValues($get);
		$this->setFormValues($post);
		
		$this->setContactVariable();
		
		$this->setLanguageOverwrite($language_overwrite);
		$this->setTableName($table_name);
		
		$this->setDbFunctions();
		$this->setDataDefinition($data_option);
	}
	
	/**
	 * Stores the data option
	 * @author Phil Calder
	 */
	function setDataOption($data_option) {
		$this->data_option = $data_option;
	}
	
	/**
	 * Gets the data option
	 * @author Phil Calder
	 */
	function getDataOption() {
		return $this->data_option;
	}
	
	/**
	 * sets the output object
	 */
	function setOutput() {
		global $SERVICES;
		$this->output =& $SERVICES->getService('Output');
	}
	
	/**
	 * @param mixed Either array to pass to Output display method, or false (default)
	 * @author Phil Calder
	 */
	function setOutputDisplayParam($param = false) {
		if (is_array($param)) {
			$this->output_display_param = $param;
		}
	}
	
	/**
	 * Override this function to set Output class options
	 * @author Phil Calder
	 */
	function setOutputOptions() {
		// if (is_object($this->output)) {	
			// $this->output->[etc]
		// }
	}
	
	/**
	 * returns the output object
	 * 
	 * @return object the output object
	 */
	function &getOutput(){
		return $this->output;
	}
	
	/**
	 * @return mixed Either array to pass to Output display method, or false (default)
	 * @author Phil Calder
	 */
	function getOutputDisplayParam() {
		if (is_array($this->output_display_param)) {
			return $this->output_display_param;
		}
		return false;
	}
	
	/**
	 * sets the mod object
	 *
	 * @see getModuleName()
	 * @see getModuleNameLower()
	 * @see getModule()
	 * 
	 * @param string $module the module name
	 */
	function setModuleInfo($module) {
		global $SERVICES, $USER;
		$SERVICES->getService('Modules');
		$php_self = $SERVICES->getGlobal('php_self');
		$this->module_name = $module; //sets the module name
		$this->mod =& $SERVICES->modules->getModule(trim($module)); //creates the module object
		if (is_object($this->mod)) {
			if (method_exists($this->mod, 'getModuleNameLower')) {
				$this->module_name_lower = $this->mod->getModuleNameLower(); //gets the module name with a lower case starting letter
				$this->pref_name = $this->mod->getModuleNameLower() . str_replace(array(dirname($php_self), '.php', '/'), '', $php_self);
			} else {
				$this->module_name_lower = strtolower($this->module_name);
			}
			if (method_exists($this->mod, 'setDataOptionFlag') && $this->data_option) { // Phil 20050812
				$this->mod->setDataOptionFlag($this->data_option);
			}
		}
	}
	
	/**
	 * returns the module object
	 * 
	 * @return object the module object (by reference, eg use $mod =& $this->getModule)
	 * @see setModuleInfo()
	 */
	function &getModule() {
		return $this->mod;
	}
	
	/**
	 * returns the module_name
	 * 
	 * @return string the module name
	 */
	function getModuleName() {
		return $this->module_name;
	}
	
	/**
	 * returns module name with the first letter lower case
	 * 
	 * @return string the module name
	 */
	function getModuleNameLower() {
		return $this->module_name_lower;
	}
	
	/**
	 * returns returns the pref_name class vairable
	 * 
	 * @return string the pref name
	 */
	function getPrefName() {
		return $this->pref_name;
	}
	
	/**
	 * sets the form values
	 * 
	 * @param array an array of values
	 */
	function setFormValues($value) {
		if (is_array($value)) {
			if ($this->form_values) {
				// $this->form_values = array_merge_recursive($value, $this->form_values);
				$this->form_values = $this->arrayMergeReplace($this->form_values, $value);
			} else {
				$this->form_values = $value;
			}
		}
	}
	
	/**
	 * resets the form values to an empty array
	 */
	function resetFormValues() {
		$this->form_values = array();
	}
	
	/**
	 * returns the form values
	 * 
	 * @param array the form values
	 */
	function getFormValues() {
		return $this->form_values;
	}
	
	/**
	 * sets the $contact_id class variable
	 *
	 * Uses the contact id from form variables passed to this class when initialised
	 */
	function setContactVariable() {
		if (isset($this->form_values['contact_id']) && $this->form_values['contact_id']) {
			$this->contact_id = $this->form_values['contact_id'];
		}
	}
	
	/**
	 * Gets the contact id associated with the object
	 * 
	 * @return integer the contact id
	 */
	function getContactVariable() {
		return $this->contact_id;
	}
	
	/**
	 * sets the language overwrite array
	 * 
	 * @param array $language_overwrite the language overwrite (with the keys title and description)
	 */
	function setLanguageOverwrite($language_overwrite) {
		if (is_array($language_overwrite)) {
			$this->language_overwrite = $language_overwrite;
		} else {
			$this->language_overwrite = false;
		}
	}
	
	/**
	 * returns the language overwrite class variable
	 * 
	 * @see $this::setLanguageOverwrite()
	 * @return array the language overwrite array
	 */
	function getLanguageOverwrite() {
		return $this->language_overwrite;
	}
	
	/**
	 * sets the table_name class variable
	 * 
	 * @param string $table_name the table name
	 */
	function setTableName($table_name = false) {
		if (!$table_name) {
			$this->table_name = 'item'; //sets the default table to item
		} else {
			$this->table_name = $table_name;
		}
		$this->table_name = $this->prefixString($this->table_name, $this->module_name . '__');
	}
	
	/**
	 * returns the table name
	 * 
	 * @return string the table name
	 */
	function getTableName() {
		return $this->table_name;
	}
	
	/**
	 * Instanciates the database functions object
	 *
	 * @param boolean $refresh Optional (default false) parameter to force a new db object to be created
	 * @author Phil Calder
	 */
	function setDbFunctions($refresh = false) {
		if (!is_object($this->mod)) {
			$this->setModuleInfo($this->module_name);
		} 
		if (is_object($this->mod) && method_exists($this->mod, 'getDatabaseTable')) {
			$this->db_functions =& $this->mod->getDatabaseTable($this->table_name, $refresh);
		} else {
			global $L;
			quickPage($L['errWebmatNoDbFunctions'] . ' <strong>' . $this->getModuleName() . '<br>' . $this->table_name . '</strong>'); 
			exit();
		}
	}
	
	/**
	 * gets the database functions object
	 * 
	 * @see WebMATDatabaseTable
	 * @return object the database functions object 
	 */
	function getDbFunctions() {
		return $this->db_functions;
	}
	
	/**
	 * Initialises the data definition object
	 * 
	 * @param string $data_option the data option used to set the data definition
	 * @author Phil Calder
	 */
	function setDataDefinition($data_option) {
		if (is_object($this->db_functions)) {
			$data_definition = $this->db_functions->getDataDefinition();
		} else {
			$data_definition = array('fields'=>array());	
		}
		// print '<div>' . get_class($this) . '::setDataDefinition</div>'; print_r($data_definition); print '<hr>';
		$this->dd = getWebMATObject(
			'WebMATDataDictionary', 
			$this->getModuleName(), 
			array(
				'data_definition' => $data_definition, 
				'data_option' => $data_option
			)
		);
	}
	
	/**
	 * returns the data definition object
	 * 
	 * @return object the data definition object
	 */
	function getDataDefinition() {
		return $this->dd;
	}
	
	/**
	 * returns the WebMAT type
	 * @return string WebMAT type - class name
	 * @author Phil Calder
	 */
	function getWebmatType() {
		return get_class($this);
	}
	
	/**
	 * Uses the Output Service function getStringField, after changing or adding to the existing field
	 *
	 * eg: the type of this field set to to 'hidden' if the hidden flag is set.
	 * @see DataDefinition::setDictionaryExcludeHidden
	 *
	 * @param array $field The field array from a data definition, required for the Output Service getStringField function.
	 * @return string representation of this field, as defined in the Output Service
	 * @author Phil Calder
	 */
	function getStringField($field) {
		if (!is_object($this->output)) {
			global $SERVICES;
			$this->output =& $SERVICES->getService('Output');
		}
		if (isset($field['hidden']) && $field['hidden']) {
			$field['type'] = 'hidden';
		}
		return $this->output->getStringField($field);
	}
	
	/**
	 * Prefix's a string with another string if it is not already prefixed
	 * 
	 * @param string $string the string you want to prefix
	 * @param string $prefix the prefix
	 * @return string the prefixed string
	 * @author Phil Calder
	 */
	function prefixString($string, $prefix) {
		if (strstr($string, $prefix)) {
			return $string;
		} else {
			  return $prefix . $string;
		}
	}
	
	/**
	 * Used to replace the function getModuleHref() returns an value from the module.config
	 * 
	 * @param string The file set functions link name to look for (eg 'list')
	 * @return mixed false or the href for the link
	 * @author Phil Calder
	 */
	function getFileSetHref($link_name) {
		if (!isset($this->file_set_hrefs) || !$this->file_set_hrefs) {
			global $SERVICES;
			$table_name = str_replace($this->module_name . '__', '', $this->getTableName());
			if (!$table_name || !$this->module_name) {
				// print '<div>getFileSetHref(' . $link_name . ') returning false, [1]</div>';
				return false; // Phil 20041206
			}
			$this->file_set_hrefs = $SERVICES->config->lookup('modules.' .  $this->module_name . '.file_set.' . $table_name . '.functions');
		}
		if ($this->file_set_hrefs[$link_name]) {
			return $this->file_set_hrefs[$link_name];
		}
		return false;
	}
		
	/**
	 * sets a tag content
	 * 
	 * @see Output::setTagContent()
	 */
	function setTagContent($tag, $element, $content) {
		$this->output->setTagContent($tag, $element, $content);
	}
	
	/**
	 * Checks if the value passed is a data option
	 *
	 * @param string $data_option the data option you are checking for
	 * @param boolean true if exists
	 * @author Phil Calder
	 */
	function isDataOption($data_option) {
		$data_definition = $this->db_functions->getDataDefinition();
		return isset($data_definition['data_options'][$data_option]);
	}
	
	/**
	 * @author Phil Calder
	 */
	function getContactId() {
		$contact_id = $this->form_values['contact_id'];
		return $contact_id;
	}
	
	/**
	 * Gets the list page for the current file set.
	 * 
	 * @return string The list page url (relative)
	 * @see redirectToList()
	 * @author Phil Calder
	 */
	function getListPageUrl() {
		global $SERVICES;
		$list_page = $SERVICES->config->lookup($this->db_functions->getFilesetConfigPath() . 'functions.list');
		if (!$list_page) {
			$list_page = 'index.php';
		}
		
		if ($query_pos =  strpos($list_page, '?')) {
			$list_page = substr($list_page, 0, $query_pos);
		}
		
		if ($this->form_values['contact_id']) {
			$list_page .= '?contact_id=' . $this->form_values['contact_id'];
		}
		
		return $list_page;
	}
	
	/**
	 * Merges two arrays.
	 *
	 * Values in the new array overwrite those in the old array.
	 * 
	 * @param array $old_array The existing array
	 * @param array $new_array The new array
	 * @return array The new merged array
	 * @author Phil Calder
	 */
	function arrayMergeReplace($old_array, $new_array) {
		
		if (!is_array($new_array)) {
			$new_array = array($new_array);	
		}
		
		foreach($new_array as $k => $v) {
		   if (is_array($v)) {
			   if (!isset($old_array[$k])) {
				   $old_array[$k] = array();
			   }
			   $old_array[$k] = $this->arrayMergeReplace($old_array[$k], $v);
		   } else {
			   $old_array[$k] = $v;
		   }
		}
		return $old_array;
	}
	
	/**
	 * Useful function for debugging.
	 *
	 * @see Services::printDebug
	 * @access private
	 * @author Phil Calder
	 */
	function printDebug($str) {
		return; // ====== DISABLE HERE =====
		// Note: This requires the SERVICES servicesPrint method.
		// This will be disabled by default.
		global $SERVICES;
		$SERVICES->servicesPrint($str);
		// print '<div>DEBUG: ' . $str . '</div>';
	}
	
	/**
	 * @return string The relative path to the module
	 * @author Phil Calder
	 */
	function getModulePath() {
		if (empty($this->module_path)) {
			$this->setModulePath();
		}
		return $this->module_path;
	}
	
	/**
	 * Sets the relative path to the module 
	 *
	 * @param string (optional) The relative path to the module
	 * @author Phil Calder
	 */
	function setModulePath($path = '') {
		if (empty($path)) {
			// $path = '../../modules/' . $this->getModuleName() . '/';
			// global $SERVICES;
			// $modules =& $SERVICES->getService('Modules');
			// if ($modules->isCustom($this->getModuleName())) {
				// $path = '../../modules_custom/' . $this->getModuleName() . '/';
			// }
			global $SERVICES;
			$modules =& $SERVICES->getService('Modules');
			$path = $modules->getModulePath($this->getModuleName(), true) . '/';
		}
		$this->module_path = $path;
	}
	
	/**
	 * Passes access restrict checking to Authentication service
	 *
	 * @param string $type (read, write, or admin)
	 * @author Phil Calder
	 */
	function restrictAccess($type = false) {
		if (!isset($this->ua) || !is_object($this->ua)) {
			global $SERVICES;
			$this->ua =& $SERVICES->getService('Authentication');
		}
		if (!$type) {
			$type = $this->restrict_level;
		}
		switch($type) {
			case 'read':
				$this->ua->readRestrict();
			break;
			case 'write':
				$this->ua->writeRestrict();
			break;
			default: // admin
				$this->ua->adminRestrict();
			break;
		}
	}
	
	/**
	 * Wrapper function for WebMATDataDictionary class function
	 *
	 * Called by {@link getTable} and other functions
	 * @return array Data dictionary
	 * @author Phil Calder
	 */
	function getDataDictionary() {
		if (is_object($this->dd) && method_exists($this->dd, 'getDataDictionary')) {
			return $this->dd->getDataDictionary();
		}
		return false;
	}
	
	/**
	 * Calculates a calculated value
	 *
	 * @param array $dd_item the data structure with data
	 * @param array $item the item
	 * @author Phil Calder
	 */
	function calculateValues(&$dd_item, $item) {
		if (is_object($this->db_functions) && method_exists($this->db_functions, 'calculateValues')) {
			$this->db_functions->calculateValues($dd_item, $item); 
		}
	}
}
?>